# ------------------------------------------------------------------------
# 
# Makefile for SESAME - Monte Carlo simulation of electron scattering in solids
#          the geometry conversion routine (pif2imsil) is commented out
#          for the time being.
#
# Version: SeSAME 2.2, Compaq Tru64 Unix version
#
# Author:  Christian Schiebl
#
# Date:    13-09-1994 (created)
#          12-02-1998 (last update)
#
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------

# Compiling and Linking

FC   = f77
CC   = cc
CPP  = cpp
LINK = f77
COPY = cp

# ------------------------------------------------------------------------
# Pathes

ROOTPATH = /usr/users5/faricell/sesame/v2.2
MAINNAME = sesame
OBJPATH  = $(ROOTPATH)/obj
OPTPATH  = $(OBJPATH)/opt
DBGPATH  = $(OBJPATH)/dbg
SRCPATH  = $(ROOTPATH)/src
BINPATH  = $(ROOTPATH)/bin
INCPATH  = $(SRCPATH)
LIBPATH  = $(ROOTPATH)/lib

# ------------------------------------------------------------------------
# Libraries

#VLIBS = $(VROOT)/lib/pai/g2s/.ultrix/libg2s.a \
#	$(VROOT)/lib/pai/gas/.ultrix/libgas.a \
#	$(VROOT)/lib/ptb/.ultrix/libptb.a     \
#	$(VROOT)/lib/num/.ultrix/libvnum.a    \
#	$(VROOT)/lib/pai/mat/.ultrix/libmat.a \
#	$(VROOT)/lib/pai/.ultrix/libpai.a     \
#	$(VROOT)/lib/vbs/.ultrix/libvbs.a

# ------------------------------------------------------------------------
# Flags

#FFLAGSDBG   =  -c -i8 -C -g -check bounds -check overflow -fpe0 -static -o 
FFLAGSDBG   =  -c -i8 -g -check overflow -fpe0 -static -check underflow -o 
FFLAGSSTD   =  -c -i8 -C -g -check bounds -check overflow -fpe0 -static -o 
#FFLAGSOPT   =  -c -i8 -O2 -check bounds -check overflow -fpe2 -static -check underflow -o 
###FFLAGSOPT   =  -c -i8 -O2 -check overflow -fpe2 -static -o 
FFLAGSOPT   =  -c -i8 -O -fpe1 -static -o 
CPPFLAGSDBG = -P -I$(INCPATH) -I$(ROOTPATH) -D__osf__ -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_
#CPPFLAGSDBG = -P -Isrc/ -I$(VINCPATH) -I$(VCFGPATH) -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_
CPPFLAGSSTD = -P -I$(INCPATH) -I$(ROOTPATH) -D__osf__ -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_
#CPPFLAGSSTD = -P -Isrc/ -I$(VINCPATH) -I$(VCFGPATH) -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_
CPPFLAGSOPT = -P -I$(INCPATH) -I$(ROOTPATH) -D__osf__ -DL_OPT -D_OPT_ -D_MEM_STATISTICS_
#CPPFLAGSOPT = -P -Isrc/ -I$(VINCPATH) -I$(VCFGPATH) -DL_OPT -D_OPT_ -D_MEM_STATISTICS_
CFLAGSDBG   = -I$(SRCPATH) -I$(ROOTPATH) -D__osf__ -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_ -g -c -O0 -o 
#CFLAGSDBG   = -Isrc/ -I$(VINCPATH) -I$(VCFGPATH) -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_ -g -c -o
CFLAGSSTD   = -I$(INCPATH) -I$(ROOTPATH) -D__osf__ -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_ -g -c -o
#CFLAGSSTD   = -Isrc/ -I$(VINCPATH) -I$(VCFGPATH) -DL_DEBUG -D_DBG_ -D_MEM_STATISTICS_ -D_MEASURE_TIME_ -D_DEBUG_QUADTREE_ -D_VISUALIZE_QUADTREE_ -g -c -o
CFLAGSOPT   = -I$(INCPATH) -I$(ROOTPATH) -D__osf__ -DL_OPT -D_OPT_ -D_MEM_STATISTICS_ -O -c -o
#CFLAGSOPT   = -Isrc/ -I$(VINCPATH) -I$(VCFGPATH) -DL_OPT -D_OPT_ -D_MEM_STATISTICS_ -O -c -o
LFLAGSDBG   =  -g -o 
LFLAGSOPT   =  -O -o
AFLAGS      = lqvcs

# ------------------------------------------------------------------------
# Includes

INCLS = $(ROOTPATH)/COMMON.INC $(ROOTPATH)/FILE.INC \
        $(ROOTPATH)/GLOBAL.INC $(ROOTPATH)/HIST.INC \
        $(ROOTPATH)/OUTPUT.INC $(ROOTPATH)/PARAM.INC \
        $(ROOTPATH)/PATH.INC $(ROOTPATH)/TYPE.INC \
        $(ROOTPATH)/DMA.INC $(ROOTPATH)/BREMS.INC \
        $(ROOTPATH)/BOUND.INC

# Sourcen

SRCS =  $(SRCPATH)/addstr.f          $(SRCPATH)/angle.f             \
        $(SRCPATH)/auger.f           $(SRCPATH)/backup.f            \
        $(SRCPATH)/bsfinit.f                                        \
        $(SRCPATH)/ccap.f            $(SRCPATH)/chkini.f            \
        $(SRCPATH)/chkion.f          $(SRCPATH)/chkout.f            \
        $(SRCPATH)/detect.f          $(SRCPATH)/dma.f               \
        $(SRCPATH)/dmath.f				            \
        $(SRCPATH)/dpolft.f          $(SRCPATH)/dpcoef.f            \
	$(SRCPATH)/drwaug.f					    \
        $(SRCPATH)/drwhis.f          $(SRCPATH)/drwldh.f            \
        $(SRCPATH)/drwsta.f          $(SRCPATH)/elastic.f           \
        $(SRCPATH)/elment.f          $(SRCPATH)/escape.f            \
        $(SRCPATH)/extout.f                                         \
        $(SRCPATH)/find.f            $(SRCPATH)/getdat.f            \
        $(SRCPATH)/histog.f          $(SRCPATH)/inihis.f            \
        $(SRCPATH)/iniout.f          $(SRCPATH)/inition.f           \
        $(SRCPATH)/initse.f          $(SRCPATH)/init_mac.f          \
        $(SRCPATH)/init_mod.f        $(SRCPATH)/intersect.f         \
        $(SRCPATH)/ionpot.f          $(SRCPATH)/ionval.f            \
        $(SRCPATH)/lamion.f          $(SRCPATH)/loadbk.f            \
        $(SRCPATH)/mcs.f             $(SRCPATH)/ndira.f             \
        $(SRCPATH)/onetrj.f          $(SRCPATH)/plasmon.f           \
	$(SRCPATH)/polyfit.f					    \
        $(SRCPATH)/ranno.f           $(SRCPATH)/rannum.f            \
        $(SRCPATH)/rdgeo.f           $(SRCPATH)/runge.f             \
        $(SRCPATH)/sesame.f          $(SRCPATH)/sigmacw.f           \
        $(SRCPATH)/sigmaiotot.f      $(SRCPATH)/sigmani.f           \
        $(SRCPATH)/slowd.f                                          \
        $(SRCPATH)/statcs.f          $(SRCPATH)/stopm1.f            \
        $(SRCPATH)/takedi.f          $(SRCPATH)/tstgeo.f            \
        $(SRCPATH)/wrgeo.f           $(SRCPATH)/wrhist.f            \
        $(SRCPATH)/wrout.f           $(SRCPATH)/wrpmhi.f            \
        $(SRCPATH)/zphi.f                                           \
        $(SRCPATH)/zzzdat_gen.f                                     \
        $(SRCPATH)/im2dgeo.c         $(SRCPATH)/imf2c.c             \
        $(SRCPATH)/imgetmem.c  	     $(SRCPATH)/imquad.c            \
        $(SRCPATH)/imsurfgeo.c       $(SRCPATH)/imtestquad.c        \
	$(SRCPATH)/imtime.c          $(SRCPATH)/imvisualquad.c      \
	$(SRCPATH)/xpgp_for.c          				    \
        $(SRCPATH)/zzznod_gen.c


#VSRCS = $(SRCPATH)/pif2imsil.c

#ABSRCS = $(SRCPATH)/asc2bin.f

#BASRCS = $(SRCPATH)/bin2asc.f

# ------------------------------------------------------------------------
# Objects

OBJOPT =$(OPTPATH)/addstr.o          $(OPTPATH)/angle.o             \
        $(OPTPATH)/auger.o           $(OPTPATH)/backup.o            \
        $(OPTPATH)/bsfinit.o                                        \
        $(OPTPATH)/ccap.o            $(OPTPATH)/chkini.o            \
        $(OPTPATH)/chkion.o          $(OPTPATH)/chkout.o            \
        $(OPTPATH)/detect.o          $(OPTPATH)/dma.o               \
        $(OPTPATH)/dmath.o				            \
        $(OPTPATH)/dpolft.o          $(OPTPATH)/dpcoef.o            \
	$(OPTPATH)/drwaug.o 					    \
        $(OPTPATH)/drwhis.o          $(OPTPATH)/drwldh.o            \
        $(OPTPATH)/drwsta.o          $(OPTPATH)/elastic.o           \
        $(OPTPATH)/elment.o          $(OPTPATH)/escape.o            \
        $(OPTPATH)/extout.o                                         \
        $(OPTPATH)/find.o            $(OPTPATH)/getdat.o            \
        $(OPTPATH)/histog.o          $(OPTPATH)/inihis.o            \
        $(OPTPATH)/iniout.o          $(OPTPATH)/inition.o           \
        $(OPTPATH)/initse.o          $(OPTPATH)/init_mac.o          \
        $(OPTPATH)/init_mod.o        $(OPTPATH)/intersect.o         \
        $(OPTPATH)/ionpot.o          $(OPTPATH)/ionval.o            \
        $(OPTPATH)/lamion.o          $(OPTPATH)/loadbk.o            \
        $(OPTPATH)/mcs.o             $(OPTPATH)/ndira.o             \
        $(OPTPATH)/onetrj.o          $(OPTPATH)/plasmon.o           \
        $(OPTPATH)/polyfit.o					    \
        $(OPTPATH)/ranno.o           $(OPTPATH)/rannum.o            \
        $(OPTPATH)/rdgeo.o           $(OPTPATH)/runge.o             \
        $(OPTPATH)/sesame.o          $(OPTPATH)/sigmacw.o           \
        $(OPTPATH)/sigmaiotot.o      $(OPTPATH)/sigmani.o           \
        $(OPTPATH)/slowd.o                                          \
        $(OPTPATH)/statcs.o          $(OPTPATH)/stopm1.o            \
        $(OPTPATH)/takedi.o          $(OPTPATH)/tstgeo.o            \
        $(OPTPATH)/wrgeo.o           $(OPTPATH)/wrhist.o            \
        $(OPTPATH)/wrout.o           $(OPTPATH)/wrpmhi.o            \
        $(OPTPATH)/zphi.o                                           \
        $(OPTPATH)/zzzdat_gen.o                                     \
        $(OPTPATH)/im2dgeo.o         $(OPTPATH)/imf2c.o             \
        $(OPTPATH)/imgetmem.o  	     $(OPTPATH)/imquad.o            \
        $(OPTPATH)/imsurfgeo.o       $(OPTPATH)/imtestquad.o        \
	$(OPTPATH)/imtime.o          $(OPTPATH)/imvisualquad.o      \
        $(OPTPATH)/xpgp_for.o	     $(OPTPATH)/zzznod_gen.o

#VOBJOPT = $(OPTPATH)/pif2imsil.o

#ABOBJOPT = $(OPTPATH)/asc2bin.o

#BAOBJOPT = $(OPTPATH)/bin2asc.o

OBJDBG =$(DBGPATH)/addstr.o          $(DBGPATH)/angle.o             \
        $(DBGPATH)/auger.o           $(DBGPATH)/backup.o            \
        $(DBGPATH)/bsfinit.o                                        \
        $(DBGPATH)/ccap.o            $(DBGPATH)/chkini.o            \
        $(DBGPATH)/chkion.o          $(DBGPATH)/chkout.o            \
        $(DBGPATH)/detect.o          $(DBGPATH)/dma.o               \
        $(DBGPATH)/dmath.o				            \
        $(DBGPATH)/dpolft.o          $(DBGPATH)/dpcoef.o            \
	$(DBGPATH)/drwaug.o 					    \
        $(DBGPATH)/drwhis.o          $(DBGPATH)/drwldh.o            \
        $(DBGPATH)/drwsta.o          $(DBGPATH)/elastic.o           \
        $(DBGPATH)/elment.o          $(DBGPATH)/escape.o            \
        $(DBGPATH)/extout.o                                         \
        $(DBGPATH)/find.o            $(DBGPATH)/getdat.o            \
        $(DBGPATH)/histog.o          $(DBGPATH)/inihis.o            \
        $(DBGPATH)/iniout.o          $(DBGPATH)/inition.o           \
        $(DBGPATH)/initse.o          $(DBGPATH)/init_mac.o          \
        $(DBGPATH)/init_mod.o        $(DBGPATH)/intersect.o         \
        $(DBGPATH)/ionpot.o          $(DBGPATH)/ionval.o            \
        $(DBGPATH)/lamion.o          $(DBGPATH)/loadbk.o            \
        $(DBGPATH)/mcs.o             $(DBGPATH)/ndira.o             \
        $(DBGPATH)/onetrj.o          $(DBGPATH)/plasmon.o           \
        $(DBGPATH)/polyfit.o					    \
        $(DBGPATH)/ranno.o           $(DBGPATH)/rannum.o            \
        $(DBGPATH)/rdgeo.o           $(DBGPATH)/runge.o             \
        $(DBGPATH)/sesame.o          $(DBGPATH)/sigmacw.o           \
        $(DBGPATH)/sigmaiotot.o      $(DBGPATH)/sigmani.o           \
        $(DBGPATH)/slowd.o                                          \
        $(DBGPATH)/statcs.o          $(DBGPATH)/stopm1.o            \
        $(DBGPATH)/takedi.o          $(DBGPATH)/tstgeo.o            \
        $(DBGPATH)/wrgeo.o           $(DBGPATH)/wrhist.o            \
        $(DBGPATH)/wrout.o           $(DBGPATH)/wrpmhi.o            \
        $(DBGPATH)/zphi.o                                           \
        $(DBGPATH)/zzzdat_gen.o                                     \
        $(DBGPATH)/im2dgeo.o         $(DBGPATH)/imf2c.o             \
        $(DBGPATH)/imgetmem.o  	     $(DBGPATH)/imquad.o            \
        $(DBGPATH)/imsurfgeo.o       $(DBGPATH)/imtestquad.o        \
	$(DBGPATH)/imtime.o          $(DBGPATH)/imvisualquad.o      \
        $(DBGPATH)/xpgp_for.o	     $(DBGPATH)/zzznod_gen.o

#VOBJDBG = $(DBGPATH)/pif2imsil.o

#ABOBJDBG = $(DBGPATH)/asc2bin.o

#BAOBJDBG = $(DBGPATH)/bin2asc.o

# ------------------------------------------------------------------------
# Compiling

#all: imsil pif2imsil asc2bin bin2asc include
all: sesame include

sesame: opt dbg

#pif2imsil: pif2imsilopt pif2imsildbg

#asc2bin: asc2binopt asc2bindbg

#bin2asc: bin2ascopt bin2ascdbg

include: $(INCLS)

opt: $(BINPATH)/$(MAINNAME).opt

dbg: $(BINPATH)/$(MAINNAME).dbg

#pif2imsilopt: $(BINPATH)/$(VMAIN).opt

#pif2imsildbg: $(BINPATH)/$(VMAIN).dbg

#asc2binopt: $(BINPATH)/$(ABMAIN).opt

#asc2bindbg: $(BINPATH)/$(ABMAIN).dbg

#bin2ascopt: $(BINPATH)/$(BAMAIN).opt

#bin2ascdbg: $(BINPATH)/$(BAMAIN).dbg

#$(BINPATH)/$(MAINNAME).opt: $(OBJOPT) $(INCLS)
#	$(LINK) $(LFLAGSOPT) $(BINPATH)/$(MAINNAME).opt $(OBJOPT) -lm -ldnet

$(BINPATH)/$(MAINNAME).opt: $(OBJOPT) $(INCLS)
	$(LINK) $(LFLAGSOPT) $(BINPATH)/$(MAINNAME).opt $(OBJOPT) -L$(LIBPATH) -lXpgp -lm -lX11

#$(BINPATH)/$(MAINNAME).dbg: $(OBJDBG) $(INCLS)
#	$(LINK) $(LFLAGSDBG) $(BINPATH)/$(MAINNAME).dbg $(OBJDBG) -lm -ldnet

$(BINPATH)/$(MAINNAME).dbg: $(OBJDBG) $(INCLS)
	$(LINK) $(LFLAGSDBG) $(BINPATH)/$(MAINNAME).dbg $(OBJDBG) -L$(LIBPATH) -lXpgp -lm -lX11

#$(BINPATH)/$(VMAIN).opt: $(VOBJOPT) $(VLIBS)
#	cc $(LFLAGSOPT) $(BINPATH)/$(VMAIN).opt $(VOBJOPT) $(VLIBS) -lm

#$(BINPATH)/$(VMAIN).dbg: $(VOBJDBG) $(VLIBS)
#	cc $(LFLAGSDBG) $(BINPATH)/$(VMAIN).dbg $(VOBJDBG) $(VLIBS) -lm

#$(BINPATH)/$(ABMAIN).opt: $(ABOBJOPT)
#	$(LINK) $(LFLAGSOPT) $(BINPATH)/$(ABMAIN).opt $(ABOBJOPT) -lm

#$(BINPATH)/$(ABMAIN).dbg: $(ABOBJDBG)
#	$(LINK) $(LFLAGSDBG) $(BINPATH)/$(ABMAIN).dbg $(ABOBJDBG) -lm

#$(BINPATH)/$(BAMAIN).opt: $(BAOBJOPT)
#	$(LINK) $(LFLAGSOPT) $(BINPATH)/$(BAMAIN).opt $(BAOBJOPT) -lm

#$(BINPATH)/$(BAMAIN).dbg: $(BAOBJDBG)
#	$(LINK) $(LFLAGSDBG) $(BINPATH)/$(BAMAIN).dbg $(BAOBJDBG) -lm

$(ROOTPATH)/BOUND.INC: $(SRCPATH)/BOUND.INC
	$(COPY) $(SRCPATH)/BOUND.INC $(ROOTPATH)/BOUND.INC

$(ROOTPATH)/BREMS.INC: $(SRCPATH)/BREMS.INC
	$(COPY) $(SRCPATH)/BREMS.INC $(ROOTPATH)/BREMS.INC

$(ROOTPATH)/COMMON.INC: $(SRCPATH)/COMMON.INC
	$(COPY) $(SRCPATH)/COMMON.INC $(ROOTPATH)/COMMON.INC

$(ROOTPATH)/DMA.INC: $(SRCPATH)/DMA.INC
	$(COPY) $(SRCPATH)/DMA.INC $(ROOTPATH)/DMA.INC

$(ROOTPATH)/FILE.INC: $(SRCPATH)/FILE.INC
	$(COPY) $(SRCPATH)/FILE.INC $(ROOTPATH)/FILE.INC

$(ROOTPATH)/GLOBAL.INC: $(SRCPATH)/GLOBAL.INC
	$(COPY) $(SRCPATH)/GLOBAL.INC $(ROOTPATH)/GLOBAL.INC

$(ROOTPATH)/HIST.INC: $(SRCPATH)/HIST.INC
	$(COPY) $(SRCPATH)/HIST.INC $(ROOTPATH)/HIST.INC

$(ROOTPATH)/OUTPUT.INC: $(SRCPATH)/OUTPUT.INC
	$(COPY) $(SRCPATH)/OUTPUT.INC $(ROOTPATH)/OUTPUT.INC

$(ROOTPATH)/PARAM.INC: $(SRCPATH)/PARAM.INC
	$(COPY) $(SRCPATH)/PARAM.INC $(ROOTPATH)/PARAM.INC

$(ROOTPATH)/PATH.INC: $(SRCPATH)/PATH.INC
	$(COPY) $(SRCPATH)/PATH.INC $(ROOTPATH)/PATH.INC

$(ROOTPATH)/TYPE.INC: $(SRCPATH)/TYPE.INC
	$(COPY) $(SRCPATH)/TYPE.INC $(ROOTPATH)/TYPE.INC


clean: 
	rm -f $(OBJPATH)/*/*.o
	rm -f $(OBJPATH)/*/*.f
	rm -f $(OBJPATH)/*/*.obj
	rm -f $(OBJPATH)/*/*.for
	rm -f $(BINPATH)/$(MAINNAME).opt
	rm -f $(BINPATH)/$(MAINNAME).dbg
#	rm -f $(BINPATH)/$(VMAIN).opt
#	rm -f $(BINPATH)/$(VMAIN).dbg
#	rm -f $(BINPATH)/$(ABMAIN).opt
#	rm -f $(BINPATH)/$(ABMAIN).dbg
#	rm -f $(BINPATH)/$(BAMAIN).opt
#	rm -f $(BINPATH)/$(BAMAIN).dbg
	rm -f $(ROOTPATH)/BOUND.INC
	rm -f $(ROOTPATH)/BREMS.INC
	rm -f $(ROOTPATH)/COMMON.INC
	rm -f $(ROOTPATH)/DMA.INC
	rm -f $(ROOTPATH)/FILE.INC
	rm -f $(ROOTPATH)/GLOBAL.INC
	rm -f $(ROOTPATH)/HIST.INC
	rm -f $(ROOTPATH)/OUTPUT.INC
	rm -f $(ROOTPATH)/PARAM.INC
	rm -f $(ROOTPATH)/PATH.INC
	rm -f $(ROOTPATH)/TYPE.INC

# ------------------------------------------------------------------------
# Files

# ----------------------------------------
# ---------- Optimized Version -----------
# ----------------------------------------

$(OPTPATH)/addstr.o: $(INCLS) $(SRCPATH)/addstr.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/addstr.o $(SRCPATH)/addstr.f

$(OPTPATH)/angle.o: $(INCLS) $(SRCPATH)/angle.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/angle.o $(SRCPATH)/angle.f

$(OPTPATH)/auger.o: $(INCLS) $(SRCPATH)/auger.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/auger.o $(SRCPATH)/auger.f

$(OPTPATH)/backup.o: $(INCLS) $(SRCPATH)/backup.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/backup.o $(SRCPATH)/backup.f

$(OPTPATH)/bsfinit.o: $(INCLS) $(SRCPATH)/bsfinit.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/bsfinit.o $(SRCPATH)/bsfinit.f

$(OPTPATH)/ccap.o: $(INCLS) $(SRCPATH)/ccap.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/ccap.o $(SRCPATH)/ccap.f

$(OPTPATH)/chkini.o: $(INCLS) $(SRCPATH)/chkini.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/chkini.o $(SRCPATH)/chkini.f

$(OPTPATH)/chkion.o: $(INCLS) $(SRCPATH)/chkion.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/chkion.o $(SRCPATH)/chkion.f

$(OPTPATH)/chkout.o: $(INCLS) $(SRCPATH)/chkout.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/chkout.o $(SRCPATH)/chkout.f

$(OPTPATH)/detect.o: $(INCLS) $(SRCPATH)/detect.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/detect.o $(SRCPATH)/detect.f

$(OPTPATH)/dma.o: $(INCLS) $(SRCPATH)/dma.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/dma.o $(SRCPATH)/dma.f

$(OPTPATH)/dmath.o: $(INCLS) $(SRCPATH)/dmath.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/dmath.o $(SRCPATH)/dmath.f

$(OPTPATH)/dpcoef.o: $(INCLS) $(SRCPATH)/dpcoef.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/dpcoef.o $(SRCPATH)/dpcoef.f

$(OPTPATH)/dpolft.o: $(INCLS) $(SRCPATH)/dpolft.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/dpolft.o $(SRCPATH)/dpolft.f

$(OPTPATH)/drwaug.o: $(INCLS) $(SRCPATH)/drwaug.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/drwaug.o $(SRCPATH)/drwaug.f

$(OPTPATH)/drwhis.o: $(INCLS) $(SRCPATH)/drwhis.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/drwhis.o $(SRCPATH)/drwhis.f

$(OPTPATH)/drwldh.o: $(INCLS) $(SRCPATH)/drwldh.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/drwldh.o $(SRCPATH)/drwldh.f

$(OPTPATH)/drwsta.o: $(INCLS) $(SRCPATH)/drwsta.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/drwsta.o $(SRCPATH)/drwsta.f

$(OPTPATH)/elastic.o: $(INCLS) $(SRCPATH)/elastic.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/elastic.o $(SRCPATH)/elastic.f

$(OPTPATH)/elment.o: $(INCLS) $(SRCPATH)/elment.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/elment.o $(SRCPATH)/elment.f

$(OPTPATH)/escape.o: $(SRCPATH)/escape.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/escape.o $(SRCPATH)/escape.f

$(OPTPATH)/extout.o: $(INCLS) $(SRCPATH)/extout.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/extout.o $(SRCPATH)/extout.f

$(OPTPATH)/find.o: $(INCLS) $(SRCPATH)/find.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/find.o $(SRCPATH)/find.f

$(OPTPATH)/getdat.o: $(INCLS) $(SRCPATH)/getdat.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/getdat.o $(SRCPATH)/getdat.f

$(OPTPATH)/histog.o: $(INCLS) $(SRCPATH)/histog.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/histog.o $(SRCPATH)/histog.f

$(OPTPATH)/inihis.o: $(INCLS) $(SRCPATH)/inihis.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/inihis.o $(SRCPATH)/inihis.f

$(OPTPATH)/iniout.o: $(INCLS) $(SRCPATH)/iniout.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/iniout.o $(SRCPATH)/iniout.f

$(OPTPATH)/inition.o: $(INCLS) $(SRCPATH)/inition.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/inition.o $(SRCPATH)/inition.f

$(OPTPATH)/initse.o: $(INCLS) $(SRCPATH)/initse.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/initse.o $(SRCPATH)/initse.f

$(OPTPATH)/init_mac.o: $(INCLS) $(SRCPATH)/init_mac.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/init_mac.o $(SRCPATH)/init_mac.f

$(OPTPATH)/init_mod.o: $(INCLS) $(SRCPATH)/init_mod.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/init_mod.o $(SRCPATH)/init_mod.f

$(OPTPATH)/intersect.o: $(INCLS) $(SRCPATH)/intersect.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/intersect.o $(SRCPATH)/intersect.f

$(OPTPATH)/ionpot.o: $(INCLS) $(SRCPATH)/ionpot.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/ionpot.o $(SRCPATH)/ionpot.f

$(OPTPATH)/ionval.o: $(INCLS) $(SRCPATH)/ionval.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/ionval.o $(SRCPATH)/ionval.f

$(OPTPATH)/lamion.o: $(INCLS) $(SRCPATH)/lamion.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/lamion.o $(SRCPATH)/lamion.f

$(OPTPATH)/loadbk.o: $(INCLS) $(SRCPATH)/loadbk.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/loadbk.o $(SRCPATH)/loadbk.f

$(OPTPATH)/mcs.o: $(INCLS) $(SRCPATH)/mcs.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/mcs.o $(SRCPATH)/mcs.f

$(OPTPATH)/ndira.o: $(INCLS) $(SRCPATH)/ndira.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/ndira.o $(SRCPATH)/ndira.f

$(OPTPATH)/onetrj.o: $(INCLS) $(SRCPATH)/onetrj.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/onetrj.o $(SRCPATH)/onetrj.f

$(OPTPATH)/plasmon.o: $(INCLS) $(SRCPATH)/plasmon.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/plasmon.o $(SRCPATH)/plasmon.f

$(OPTPATH)/polyfit.o: $(INCLS) $(SRCPATH)/polyfit.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/polyfit.o $(SRCPATH)/polyfit.f

$(OPTPATH)/ranno.o: $(INCLS) $(SRCPATH)/ranno.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/ranno.o $(SRCPATH)/ranno.f

$(OPTPATH)/rannum.o: $(INCLS) $(SRCPATH)/rannum.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/rannum.o $(SRCPATH)/rannum.f

$(OPTPATH)/rdgeo.o: $(INCLS) $(SRCPATH)/rdgeo.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/rdgeo.o $(SRCPATH)/rdgeo.f

$(OPTPATH)/runge.o: $(INCLS) $(SRCPATH)/runge.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/runge.o $(SRCPATH)/runge.f

$(OPTPATH)/sesame.o: $(INCLS) $(SRCPATH)/sesame.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/sesame.o $(SRCPATH)/sesame.f

$(OPTPATH)/sigmacw.o: $(INCLS) $(SRCPATH)/sigmacw.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/sigmacw.o $(SRCPATH)/sigmacw.f

$(OPTPATH)/sigmaiotot.o: $(INCLS) $(SRCPATH)/sigmaiotot.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/sigmaiotot.o $(SRCPATH)/sigmaiotot.f

$(OPTPATH)/sigmani.o: $(INCLS) $(SRCPATH)/sigmani.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/sigmani.o $(SRCPATH)/sigmani.f

$(OPTPATH)/slowd.o: $(INCLS) $(SRCPATH)/slowd.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/slowd.o $(SRCPATH)/slowd.f

$(OPTPATH)/statcs.o: $(INCLS) $(SRCPATH)/statcs.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/statcs.o $(SRCPATH)/statcs.f

$(OPTPATH)/stopm1.o: $(INCLS) $(SRCPATH)/stopm1.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/stopm1.o $(SRCPATH)/stopm1.f

$(OPTPATH)/takedi.o: $(INCLS) $(SRCPATH)/takedi.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/takedi.o $(SRCPATH)/takedi.f

$(OPTPATH)/tstgeo.o: $(INCLS) $(SRCPATH)/tstgeo.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/tstgeo.o $(SRCPATH)/tstgeo.f

$(OPTPATH)/wrgeo.o: $(INCLS) $(SRCPATH)/wrgeo.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/wrgeo.o $(SRCPATH)/wrgeo.f

$(OPTPATH)/wrhist.o: $(INCLS) $(SRCPATH)/wrhist.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/wrhist.o $(SRCPATH)/wrhist.f

$(OPTPATH)/wrout.o: $(INCLS) $(SRCPATH)/wrout.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/wrout.o $(SRCPATH)/wrout.f

$(OPTPATH)/wrpmhi.o: $(INCLS) $(SRCPATH)/wrpmhi.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/wrpmhi.o $(SRCPATH)/wrpmhi.f

$(OPTPATH)/zphi.o: $(INCLS) $(SRCPATH)/zphi.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/zphi.o $(SRCPATH)/zphi.f

$(OPTPATH)/zzzdat_gen.o: $(INCLS) $(SRCPATH)/zzzdat_gen.f
	$(CPP) $(CPPFLAGSOPT) $(SRCPATH)/zzzdat_gen.f >$(OPTPATH)/zzzdat_gen.f
	$(FC) $(FFLAGSOPT) $(OPTPATH)/zzzdat_gen.o $(OPTPATH)/zzzdat_gen.f

$(OPTPATH)/im2dgeo.o: $(SRCPATH)/im2dgeo.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/im2dgeo.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/im2dgeo.o $(SRCPATH)/im2dgeo.c

$(OPTPATH)/imf2c.o: $(SRCPATH)/imf2c.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h $(SRCPATH)/im2dgeo.h \
	$(SRCPATH)/imtestquad.h $(SRCPATH)/imvisualquad.h \
	$(SRCPATH)/xpgp.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/imf2c.o $(SRCPATH)/imf2c.c

$(OPTPATH)/imgetmem.o: $(SRCPATH)/imgetmem.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/imgetmem.o $(SRCPATH)/imgetmem.c

$(OPTPATH)/imquad.o: $(SRCPATH)/imquad.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/imquad.o $(SRCPATH)/imquad.c

$(OPTPATH)/imsurfgeo.o: $(SRCPATH)/imsurfgeo.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imsurfgeo.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/imsurfgeo.o $(SRCPATH)/imsurfgeo.c

$(OPTPATH)/imtestquad.o: $(SRCPATH)/imtestquad.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h $(SRCPATH)/im2dgeo.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/imtestquad.o $(SRCPATH)/imtestquad.c

$(OPTPATH)/imtime.o: $(SRCPATH)/imtime.c
	$(CC) $(CFLAGSOPT) $(OPTPATH)/imtime.o $(SRCPATH)/imtime.c

$(OPTPATH)/imvisualquad.o: $(SRCPATH)/imvisualquad.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h $(SRCPATH)/im2dgeo.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/imvisualquad.o $(SRCPATH)/imvisualquad.c

$(OPTPATH)/xpgp_for.o: $(INCLS) $(SRCPATH)/xpgp_for.c $(SRCPATH)/xpgp.h
	$(CC) $(CFLAGSOPT) $(OPTPATH)/xpgp_for.o $(SRCPATH)/xpgp_for.c

$(OPTPATH)/zzznod_gen.o: $(SRCPATH)/zzznod_gen.c
	$(CC) $(CFLAGSOPT) $(OPTPATH)/zzznod_gen.o $(SRCPATH)/zzznod_gen.c

#$(OPTPATH)/PIF2IMSIL.o: $(SRCPATH)/PIF2IMSIL.c $(vincpath)/VISTA.H \
#		$(vincpath)/VMATH.H $(vincpath)/VERRSTD.H $(vincpath)/VERRp.H \
#		$(vincpath)/PAI.H $(vincpath)/PAL.H $(vincpath)/G2S.H \
#		$(vincpath)/VRR.H $(vincpath)/VEXIT.H $(vcfgpath)/VISTA.cF
#	$(CC) $(CFLAGSOPT) $(OPTPATH)/PIF2IMSIL.o $(SRCPATH)/PIF2IMSIL.c

#$(OPTPATH)/ASC2BIN.o: $(INCLS) $(SRCPATH)/ASC2BIN.f
#	$(FC) $(FFLAGSOPT) $(OPTPATH)/ASC2BIN.o $(SRCPATH)/ASC2BIN.f

#$(OPTPATH)/BIN2ASC.o: $(INCLS) $(SRCPATH)/BIN2ASC.f
#	$(FC) $(FFLAGSOPT) $(OPTPATH)/BIN2ASC.o $(SRCPATH)/BIN2ASC.f

# ----------------------------------------
# ----------   dEBUG vERSION   -----------
# ----------------------------------------

$(DBGPATH)/addstr.o: $(INCLS) $(SRCPATH)/addstr.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/addstr.o $(SRCPATH)/addstr.f

$(DBGPATH)/angle.o: $(INCLS) $(SRCPATH)/angle.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/angle.o $(SRCPATH)/angle.f

$(DBGPATH)/auger.o: $(INCLS) $(SRCPATH)/auger.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/auger.o $(SRCPATH)/auger.f

$(DBGPATH)/backup.o: $(INCLS) $(SRCPATH)/backup.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/backup.o $(SRCPATH)/backup.f

$(DBGPATH)/bsfinit.o: $(INCLS) $(SRCPATH)/bsfinit.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/bsfinit.o $(SRCPATH)/bsfinit.f

$(DBGPATH)/ccap.o: $(INCLS) $(SRCPATH)/ccap.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/ccap.o $(SRCPATH)/ccap.f

$(DBGPATH)/chkini.o: $(INCLS) $(SRCPATH)/chkini.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/chkini.o $(SRCPATH)/chkini.f

$(DBGPATH)/chkion.o: $(INCLS) $(SRCPATH)/chkion.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/chkion.o $(SRCPATH)/chkion.f

$(DBGPATH)/chkout.o: $(INCLS) $(SRCPATH)/chkout.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/chkout.o $(SRCPATH)/chkout.f

$(DBGPATH)/detect.o: $(INCLS) $(SRCPATH)/detect.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/detect.o $(SRCPATH)/detect.f

$(DBGPATH)/dma.o: $(INCLS) $(SRCPATH)/dma.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/dma.o $(SRCPATH)/dma.f

$(DBGPATH)/dmath.o: $(INCLS) $(SRCPATH)/dmath.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/dmath.o $(SRCPATH)/dmath.f

$(DBGPATH)/dpcoef.o: $(INCLS) $(SRCPATH)/dpcoef.f
	$(FC) $(FFLAGSOPT) $(DBGPATH)/dpcoef.o $(SRCPATH)/dpcoef.f

$(DBGPATH)/dpolft.o: $(INCLS) $(SRCPATH)/dpolft.f
	$(FC) $(FFLAGSOPT) $(DBGPATH)/dpolft.o $(SRCPATH)/dpolft.f

$(DBGPATH)/drwaug.o: $(INCLS) $(SRCPATH)/drwaug.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/drwaug.o $(SRCPATH)/drwaug.f

$(DBGPATH)/drwhis.o: $(INCLS) $(SRCPATH)/drwhis.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/drwhis.o $(SRCPATH)/drwhis.f

$(DBGPATH)/drwldh.o: $(INCLS) $(SRCPATH)/drwldh.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/drwldh.o $(SRCPATH)/drwldh.f

$(DBGPATH)/drwsta.o: $(INCLS) $(SRCPATH)/drwsta.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/drwsta.o $(SRCPATH)/drwsta.f

$(DBGPATH)/elastic.o: $(INCLS) $(SRCPATH)/elastic.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/elastic.o $(SRCPATH)/elastic.f

$(DBGPATH)/elment.o: $(INCLS) $(SRCPATH)/elment.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/elment.o $(SRCPATH)/elment.f

$(DBGPATH)/escape.o: $(INCLS) $(SRCPATH)/escape.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/escape.o $(SRCPATH)/escape.f

$(DBGPATH)/extout.o: $(INCLS) $(SRCPATH)/extout.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/extout.o $(SRCPATH)/extout.f

$(DBGPATH)/find.o: $(INCLS) $(SRCPATH)/find.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/find.o $(SRCPATH)/find.f

$(DBGPATH)/getdat.o: $(INCLS) $(SRCPATH)/getdat.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/getdat.o $(SRCPATH)/getdat.f

$(DBGPATH)/histog.o: $(INCLS) $(SRCPATH)/histog.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/histog.o $(SRCPATH)/histog.f

$(DBGPATH)/inihis.o: $(INCLS) $(SRCPATH)/inihis.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/inihis.o $(SRCPATH)/inihis.f

$(DBGPATH)/iniout.o: $(INCLS) $(SRCPATH)/iniout.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/iniout.o $(SRCPATH)/iniout.f

$(DBGPATH)/inition.o: $(INCLS) $(SRCPATH)/inition.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/inition.o $(SRCPATH)/inition.f

$(DBGPATH)/initse.o: $(INCLS) $(SRCPATH)/initse.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/initse.o $(SRCPATH)/initse.f

$(DBGPATH)/init_mac.o: $(INCLS) $(SRCPATH)/init_mac.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/init_mac.o $(SRCPATH)/init_mac.f

$(DBGPATH)/init_mod.o: $(INCLS) $(SRCPATH)/init_mod.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/init_mod.o $(SRCPATH)/init_mod.f

$(DBGPATH)/intersect.o: $(INCLS) $(SRCPATH)/intersect.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/intersect.o $(SRCPATH)/intersect.f

$(DBGPATH)/ionpot.o: $(INCLS) $(SRCPATH)/ionpot.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/ionpot.o $(SRCPATH)/ionpot.f

$(DBGPATH)/ionval.o: $(INCLS) $(SRCPATH)/ionval.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/ionval.o $(SRCPATH)/ionval.f

$(DBGPATH)/lamion.o: $(INCLS) $(SRCPATH)/lamion.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/lamion.o $(SRCPATH)/lamion.f

$(DBGPATH)/loadbk.o: $(INCLS) $(SRCPATH)/loadbk.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/loadbk.o $(SRCPATH)/loadbk.f

$(DBGPATH)/mcs.o: $(INCLS) $(SRCPATH)/mcs.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/mcs.o $(SRCPATH)/mcs.f

$(DBGPATH)/ndira.o: $(INCLS) $(SRCPATH)/ndira.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/ndira.o $(SRCPATH)/ndira.f

$(DBGPATH)/onetrj.o: $(INCLS) $(SRCPATH)/onetrj.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/onetrj.o $(SRCPATH)/onetrj.f

$(DBGPATH)/plasmon.o: $(INCLS) $(SRCPATH)/plasmon.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/plasmon.o $(SRCPATH)/plasmon.f

$(DBGPATH)/polyfit.o: $(INCLS) $(SRCPATH)/polyfit.f
	$(FC) $(FFLAGSOPT) $(DBGPATH)/polyfit.o $(SRCPATH)/polyfit.f

$(DBGPATH)/ranno.o: $(INCLS) $(SRCPATH)/ranno.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/ranno.o $(SRCPATH)/ranno.f

$(DBGPATH)/rannum.o: $(INCLS) $(SRCPATH)/rannum.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/rannum.o $(SRCPATH)/rannum.f

$(DBGPATH)/rdgeo.o: $(INCLS) $(SRCPATH)/rdgeo.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/rdgeo.o $(SRCPATH)/rdgeo.f

$(DBGPATH)/runge.o: $(INCLS) $(SRCPATH)/runge.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/runge.o $(SRCPATH)/runge.f

$(DBGPATH)/sesame.o: $(INCLS) $(SRCPATH)/sesame.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/sesame.o $(SRCPATH)/sesame.f

$(DBGPATH)/sigmacw.o: $(INCLS) $(SRCPATH)/sigmacw.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/sigmacw.o $(SRCPATH)/sigmacw.f

$(DBGPATH)/sigmaiotot.o: $(INCLS) $(SRCPATH)/sigmaiotot.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/sigmaiotot.o $(SRCPATH)/sigmaiotot.f

$(DBGPATH)/sigmani.o: $(INCLS) $(SRCPATH)/sigmani.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/sigmani.o $(SRCPATH)/sigmani.f

$(DBGPATH)/slowd.o: $(INCLS) $(SRCPATH)/slowd.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/slowd.o $(SRCPATH)/slowd.f

$(DBGPATH)/statcs.o: $(INCLS) $(SRCPATH)/statcs.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/statcs.o $(SRCPATH)/statcs.f

$(DBGPATH)/stopm1.o: $(INCLS) $(SRCPATH)/stopm1.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/stopm1.o $(SRCPATH)/stopm1.f

$(DBGPATH)/takedi.o: $(INCLS) $(SRCPATH)/takedi.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/takedi.o $(SRCPATH)/takedi.f

$(DBGPATH)/tstgeo.o: $(INCLS) $(SRCPATH)/tstgeo.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/tstgeo.o $(SRCPATH)/tstgeo.f

$(DBGPATH)/wrgeo.o: $(INCLS) $(SRCPATH)/wrgeo.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/wrgeo.o $(SRCPATH)/wrgeo.f

$(DBGPATH)/wrhist.o: $(INCLS) $(SRCPATH)/wrhist.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/wrhist.o $(SRCPATH)/wrhist.f

$(DBGPATH)/wrout.o: $(INCLS) $(SRCPATH)/wrout.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/wrout.o $(SRCPATH)/wrout.f

$(DBGPATH)/wrpmhi.o: $(INCLS) $(SRCPATH)/wrpmhi.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/wrpmhi.o $(SRCPATH)/wrpmhi.f

$(DBGPATH)/zphi.o: $(INCLS) $(SRCPATH)/zphi.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/zphi.o $(SRCPATH)/zphi.f

$(DBGPATH)/zzzdat_gen.o: $(INCLS) $(SRCPATH)/zzzdat_gen.f
	$(CPP) $(CPPFLAGSOPT) $(SRCPATH)/zzzdat_gen.f >$(DBGPATH)/zzzdat_gen.f
	$(FC) $(FFLAGSDBG) $(DBGPATH)/zzzdat_gen.o $(DBGPATH)/zzzdat_gen.f

$(DBGPATH)/im2dgeo.o: $(SRCPATH)/im2dgeo.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/im2dgeo.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/im2dgeo.o $(SRCPATH)/im2dgeo.c

$(DBGPATH)/imf2c.o: $(SRCPATH)/imf2c.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h $(SRCPATH)/im2dgeo.h \
	$(SRCPATH)/imtestquad.h $(SRCPATH)/imvisualquad.h \
	$(SRCPATH)/xpgp.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/imf2c.o $(SRCPATH)/imf2c.c

$(DBGPATH)/imgetmem.o: $(SRCPATH)/imgetmem.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/imgetmem.o $(SRCPATH)/imgetmem.c

$(DBGPATH)/imquad.o: $(SRCPATH)/imquad.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/imquad.o $(SRCPATH)/imquad.c

$(DBGPATH)/imsurfgeo.o: $(SRCPATH)/imsurfgeo.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imsurfgeo.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/imsurfgeo.o $(SRCPATH)/imsurfgeo.c

$(DBGPATH)/imtestquad.o: $(SRCPATH)/imtestquad.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h $(SRCPATH)/im2dgeo.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/imtestquad.o $(SRCPATH)/imtestquad.c

$(DBGPATH)/imtime.o: $(SRCPATH)/imtime.c
	$(CC) $(CFLAGSOPT) $(DBGPATH)/imtime.o $(SRCPATH)/imtime.c

$(DBGPATH)/imvisualquad.o: $(SRCPATH)/imvisualquad.c $(SRCPATH)/imquad.h \
	$(SRCPATH)/imgetmem.h $(SRCPATH)/imtime.h $(SRCPATH)/im2dgeo.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/imvisualquad.o $(SRCPATH)/imvisualquad.c

$(DBGPATH)/xpgp_for.o: $(INCLS) $(SRCPATH)/xpgp_for.c $(SRCPATH)/xpgp.h
	$(CC) $(CFLAGSOPT) $(DBGPATH)/xpgp_for.o $(SRCPATH)/xpgp_for.c

$(DBGPATH)/zzznod_gen.o: $(SRCPATH)/zzznod_gen.c
	$(CC) $(CFLAGSOPT) $(DBGPATH)/zzznod_gen.o $(SRCPATH)/zzznod_gen.c

#$(DBGPATH)/PIF2IMSIL.o: $(SRCPATH)/PIF2IMSIL.c $(vincpath)/VISTA.H \
#		$(vincpath)/VMATH.H $(vincpath)/VERRSTD.H $(vincpath)/VERRp.H \
#		$(vincpath)/PAI.H $(vincpath)/PAL.H $(vincpath)/G2S.H \
#		$(vincpath)/VRR.H $(vincpath)/VEXIT.H $(vcfgpath)/VISTA.cF
#	$(CC) $(CFLAGSOPT) $(DBGPATH)/PIF2IMSIL.o $(SRCPATH)/PIF2IMSIL.c

#$(DBGPATH)/ASC2BIN.o: $(INCLS) $(SRCPATH)/ASC2BIN.f
#	$(FC) $(FFLAGSDBG) $(DBGPATH)/ASC2BIN.o $(SRCPATH)/ASC2BIN.f

#$(DBGPATH)/BIN2ASC.o: $(INCLS) $(SRCPATH)/BIN2ASC.f
#	$(FC) $(FFLAGSDBG) $(DBGPATH)/BIN2ASC.o $(SRCPATH)/BIN2ASC.f



