c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
      SUBROUTINE INIAUG()

***   INITIALIZE DATA STRUCTURES FOR AUGER SIMULATION
*
**    AUTHOR : HORST WAGNER  29-JAN-1994

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'
      
      INTEGER AUGERXYZ(3,3)
      DATA AUGERXYZ /1,4,9,3,7,10,4,8,10/
      CHARACTER*70 CD70      

C     CALCULATE SURFACE LENGTH

      SURFLEN = 0
      DO 20 I = 1,NSCORD-1
        SURFLEN = SURFLEN + 
     .              DSQRT((SCORDX(I)-SCORDX(I+1))**2 +
     .                    (SCORDZ(I)-SCORDZ(I+1))**2  )
20    CONTINUE

      DO 100 IAUG=1,NAUGER

C    CALCULATE APPROX. AUGER ENERGIES /CHUNG AND JENKINS/

      	IATNUM  = ATNUM(ELEMOFAUGER(IAUG))
        ILINE = LINEOFAUGER(IAUG)

        CALL GETAUG(1,ILINE,IATNUM,EAUG,IERR)

        ENEROFAUGER(IAUG) = EAUG 	
      
C    INITIALIZE HISTOGRAMS IF AUGMOD = 1

      IF (NAGDLOC(IAUG) .GT. 0) THEN

        CALL INIHIS(2,NAHISL,0.D0,SURFLEN,NAHISV,0.D0,AHISV,ND2,
     .		    D2,E2,0.D0,'AGD','AGD',CD70,IR)
        AGDHIS(IAUG) = NHIST

      ENDIF

      IF (NADDLOC(IAUG) .GT. 0) THEN

        CALL INIHIS(2,NAHISL,0.D0,SURFLEN,NAHISV,0.D0,AHISV,ND2,
     .              D2,E2,0.D0,'ADD','ADD',CD70,IR)
        ADDHIS(IAUG) = NHIST

      ENDIF

100   CONTINUE
      
*     INITIALIZE CROSS SECTIONS FOR AUGERLINE USING GRYZINSKI (=1)

      CALL INIT_SIGMAAUG(1)
      
      END

      SUBROUTINE INIT_SIGMAAUG(
     I                         I_MODEL)  

***   SUBROUTINE INIT_SIGMAAUG INITIALIZES AND CALCULATES 
*     THE TABLES OF CROSS SECTIONS FOR A EXPLICIT AUGER TRANSITION
*
**    AUTHOR: H.WAGNER 20-FEB-1995
*
**    INPUT:
*     I_MODEL...MODEL NUMBER (1=>GRYZINSKY(1965))
*
**    OUTPUT (IN COMMON BLOCK /SESAME/):
*     SIGMA_AUG...AUGER CROSS SECTION,
*                DEPENDS ON REGION AND ENERGY AND SHELL
*     
**    CALLS:  SIGMA_ION_U...CALCULATES G(U)
                        
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'

**    START OF EXECUTABLE CODE:
*     *************************

      INTEGER AUGERIONSHELL(10)
      DATA AUGERIONSHELL /1,2,2,2,3,3,3,3,3,4/

      IMODE=2
      IF (LTEST) IMODE=1

* MODEL=1:
      IF (I_MODEL.EQ.1) THEN 

      ESTEP_IO = (EMAX-EMIN)/(FLOAT(MAXENER-1))

* LOOP OVER REGIONS
*
      DO 100 I_REG = 1,NREG                                                

* LOOP OVER SIMULATED AUGER LINES 

         DO 200 I_AUG = 1,NAUGER

*	 DETERMINE ELEMENT AND ATOMIC FRACTION
      	 I_ELEM = ELEMOFAUGER(I_AUG)
         I_LINE = LINEOFAUGER(I_AUG)
         C = 0.
      	 DO 250 I_ELE = 1,NELE(I_REG)
      	    IF (ELEM(I_REG,I_ELE) .EQ. I_ELEM) C = ATFRAC(I_REG,I_ELE)    
250 	 CONTINUE

*        CLEAR SIGMA_AUG(*,I_REG,I_AUG)
         DO 260 I_ENER = 1,MAXENER
            SIGMA_AUG(I_ENER,I_REG,I_AUG) = 0.
260 	 CONTINUE

         DO 270 I_SHELL = 1,10
*          DOES I_SHELL CONTRIBUTE TO THE AUGER LINE ? 
           IF (AUGERIONSHELL(I_SHELL) .EQ. I_LINE) THEN

             CALL GETED(IMODE,I_SHELL,ATNUM(I_ELEM),E_CRIT,IERR)

* JUST FOR DEBUGGING
  
             IF (E_CRIT .GT. 0 .AND. EPRIM .GE. E_CRIT) THEN
       		U = EPRIM/E_CRIT
                CALL SIGMA_ION_U (
     I                         U,
     O                         G_U) 
             ELSE 
                G_U = 0.
             ENDIF
      
      	     IF (LTEST) WRITE(*,*) 'AUG : ',I_REG,I_ELEM,I_SHELL,C,
     .                             E_CRIT,G_U,tot_atdens(I_REG)
* END OF JUST FOR DEBUGGING

* LOOP OVER ENERGY
             E_ENER = EMIN
      	     DO 300 I_ENER  = 1,MAXENER        


              IF (E_CRIT.EQ.0.D0 .OR. E_ENER .LE. E_CRIT+ESTEP_IO) THEN
      		SIGMA = 0.D0
      	      ELSE
             
            	U = E_ENER/E_CRIT
                CALL SIGMA_ION_U (
     I                         U,
     O                         G_U) 
      
                SIGMA = E_SHELL(I_SHELL)/E_CRIT**2.D0 * G_U * C
 	      ENDIF 	
      	      	
      	      SIGMA_AUG(I_ENER,I_REG,I_AUG) = SIGMA +
     .					SIGMA_AUG(I_ENER,I_REG,I_AUG)
              E_ENER = E_ENER+ESTEP_IO

 300         CONTINUE 

      	   ENDIF

 270      CONTINUE 
*LOOP I_SHELL

 200     CONTINUE
*LOOP I_AUG

 100  CONTINUE
*LOOP I_REG

      ELSE 

             WRITE(*,'(2A)')' MODEL OF IONIZATION CROSS', 
     &                      ' NOT IMPLEMENTED' 
             WRITE(*,'(A)')' STOP IN SUBROUTINE INIT_ION_TOT'    
             STOP

      ENDIF 
      
      RETURN

      END  
***************************************************************
       

      SUBROUTINE INIAUGESCHIS(NPTS,NFCS)
***   CALCULATES AUGER AND PHOTOELECTRON ESCAPE PROBABILITY DISTRIBUTION
*
*     USES AUGESC
*
**    AUTHOR : HORST WAGNER  3-FEB-1995

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'


C  CALCULATE ESCAPE PROBABILITY
c
c      DO 100 IAUG=1,NAUGER
c
c      IHIST = AUGERESCHIST(IAUG)
c      NX = NVAL(IHIST,1)
c      NY = NVAL(IHIST,2)
c      IELE  = ELEMOFAUGER(IAUG)
c      ILINE = LINEOFAUGER(IAUG)
c
c      WRITE(*,*) 'AUGER ESCAPE PROB. ', ILINE, ' OF ELEMENT ', IELE
c      WRITE(*,*) 'ENERGY APPROX. ',ENEROFAUGER(IAUG)
c      WRITE(*,*) 'HISTOGRAMSIZE : ',NX,'x',NY
c
c      DO 93 I=1,NX
c        DO 94 J=1,NY
c           XL = VALMAX(IHIST,1)*(I-0.5)/NX
c            D = VALMAX(IHIST,2)*(J-0.5)/NY
c           CALL IMSURFSETDEPTH(XX,ZZ,D,XL,IER)
c           IF (IER .GT. 0) THEN
c              WRITE (*,*) 'ERROR IN IMSURFSETDEPTH : ',D,XL,IER
c           ENDIF
c           FRACTOT = 0.
c           DO 95 K=1,50
c             FRAC = 1.
c             E = ENEROFAUGER(IAUG)
c             X = XX
c             Y = 0.
c             Z = ZZ
c             CALL AUGESC(1,NPTS,NFCS,DIRX,DIRY,DIRZ,X,Y,Z,E,FRAC)
c      	     CALL ELDECT(DIRX,DIRY,DIRZ,E,FRAC)
c             FRACTOT = FRACTOT + FRAC
c95         CONTINUE
c           IABS = IVBEG(IHIST)+I+(J-1)*NX-1
c           VALHIS(IABS) = FRACTOT
c94      CONTINUE
c93    CONTINUE
c 
c100   CONTINUE 
c
      END



      SUBROUTINE AUGESC (
     V			  IFLAG,NPTS,NFCS,
     O			  DIRX,DIRY,DIRZ,
     I			   X,Y,Z,E,FRAC)
 
***   CALCULATES AUGER AND PHOTOELECTRON ESCAPE PROBABILITY
*
*     SIMULATES ELECTRON ONLY USING ELASTIC SCATTERING. INELASTIC SCATTERING
*     PROBABILITY IS CALCULATED BY THE TOTAL FLIGHT PATH
*     STARTING DIRECTION IS CHOOSEN RANDOMLY
*
**    AUTHOR : HORST WAGNER  29-DEC-1994


      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'

      DOUBLE PRECISION X,Y,Z,DIRX,DIRY,DIRZ,E,DELTA
      INTEGER IFLAG
      PARAMETER(DELTA = 2d-1)
*     ACCURACY FOR TRAJECTORY BREAK OFF POINT
      PARAMETER(ACCU = 5d-3)	

**    START OF EXECUTABLE CODE
*     ************************

      DOUBLE PRECISION LAMTOT,LAMELA,LAMION,LAMPLAS
      INTEGER IR,IER,IRNXT

CHW   NUMBER OF SCATTERINGS
      NSCAT=0

CHW   DETERMINE STARTING REGION

      IUSOLL=0      
      CALL IMGREG(IUSOLL,
     I 	          X,Z,PCOORD,
     O		  IR,IER)

CHW   ERROR IN POINT LOCATION ?
      IF (IER.NE.0) GOTO 2000

CHW   SET X-WINDOW STARTING POSITION
C      IF (LSCRN) THEN
C         CALL TRXZ(X,Z,NX,NZ)
C      	 CALL F_MOVE(NX,NZ,NS)
C      ENDIF

CHW   REGION IS AIR, ELECTRON CANNOT BE GENERATED HERE, RETURN WITH FRAC = 0. 
      IF (IR.LE.0) GOTO 1400


CHW   SELECT RANDOM STARTING DIRECTION

      DIRX = 1.
      DIRY = 0.
      DIRZ = 0.

      COSPSI = 2*RANNUM()-1
      SINPSI = SQRT(1-COSPSI*COSPSI)

      IFI= NINT(360*RANNUM())
      COSFI = COSINE(IFI)
      SINFI = SINE(IFI)
 
      CALL NDIRA(
     V           DIRX,DIRY,DIRZ,
     I           COSPSI,SINPSI,COSFI,SINFI)
 

CHW   ENTRY IN NEW REGION 

200   CONTINUE

C	 ELASTIC FREE PATH IN REGION IR

C   ELASTIC SCATTERING
 
      CALL E_LAMBDA (IR,E,LAMELA,IER)

C   PLASMON SCATTERING

      CALL P_LAMBDA(IR,E,LAMPLAS,IER)

***MAIN FLIGHT LOOP

220   CONTINUE

C   CALCULATE A STOCHASTIC FREE FLIGHT PATH
	 
      FFP = -LAMELA*LOG(RANNUM())

C   CALCULATE LOCATION OF NEXT SCATTERING EVENT
      
      XNXT = X + FFP*DIRX
      YNXT = Y + FFP*DIRY
      ZNXT = Z + FFP*DIRZ
 
C   CHECK IF LOCATION OF NEXT SCATTERING EVENT LIES IN SAME REGION

	 CALL IMGREG (
     I                IUSOLL,XNXT,ZNXT,PCOORD,
     O                IRNXT,IER)

CHW   ERROR IN POINT LOCATION ?

      IF (IER.NE.0) GOTO 2000
      
      IF (IR.NE.IRNXT) THEN
C          NEXT SCATTERING EVENT WOULD LIE IN A DIFFERENT DOMAIN
C          CALCULATE THE COORDINATES OF POINT AT THE INTERFACE

            CALL INTERSECT (
     D			  NPTS,NFCS,
     I			  PCOORD,ILINES,IENDIX,
     I			  IR,X,Z,DIRX,DIRZ,
     O			  XNXT,ZNXT,IRNXT,RLDB,IER)

      	    IF (IER.NE.0) GOTO 2200

CHW    NEW FFP AND INTERFACE LOCATION :

      	    FFP = RLDB

CHW   "BUGFIX" FOR IMENTR 

       	    XNXT = XNXT + DIRX*DELTA
      	    ZNXT = ZNXT + DIRZ*DELTA
      	    YNXT = Y + FFP*DIRY

CHW	    SET 'STOPPED AT INTERFACE' FLAG
      	    IINTER = 1      
      ELSE
      	    IINTER = 0
      ENDIF

C     TAKE CARE OF INELASTIC SCATTERING PROBABILITY OF LAST FLIGHT    
      FRAC = FRAC*EXP(-FFP/LAMPLAS)


C      IF (LSCRN .AND. LTRAJ) THEN
C      		CALL TRXZ(XNXT,ZNXT,NX,NZ)
C      	 	CALL F_FLINETO(NX,NZ,NS)
C      ENDIF

      X = XNXT
      Y = YNXT
      Z = ZNXT


CHW      CONTINUATION OF ELASTIC FLIGHT IMPROBABLE (?)
      IF (FRAC .LT. ACCU) GOTO 1400

CHW   IF ELECTRON STOPPED AT INTERFACE THEN NO SCATTERING

      IF (IINTER .GT. 0) THEN
      	 IR = IRNXT
      	 IF (IR.LE.0) GOTO 1000
      	 GOTO 200
      ENDIF

CHW   ELSE PERFORM SCATTERING EVENT

      NSCAT=NSCAT+1


C ELASTIC SCATTERING:
C ===================

 300  CONTINUE

C DETERMINE SCATTERING ANGLE

      CALL E_THETA(
     I               IR,E,RANNUM(),RANNUM(),
     O               PSI,IER)

      SINPSI = SINE(NINT(PSI))
      COSPSI = COSINE(NINT(PSI))

C     DETERMINE NEW DIRECTION:

      IFI= NINT(360*RANNUM())

      COSFI = COSINE(IFI)
      SINFI = SINE(IFI)

C NEW DIRECTION

      CALL NDIRA(
     V           DIRX,DIRY,DIRZ,
     I           COSPSI,SINPSI,COSFI,SINFI)

700   CONTINUE

C DO NEXT FLIGHT
      GOTO 220


1000  CONTINUE
******ELECTRON IS IN THE AIR REGION

      CALL IMENTR (
     I           X, Z, DIRX, DIRZ, PCOORD,
     O           INTERS, XNXT, ZNXT, IR, IER)

C    ERROR DURING POINT LOCATION ?		    

      IF (IER.NE.0) GOTO 2100

      IF (IR.EQ.0) THEN 
      	WRITE (*,'(2I4,8F8.1)') IR,NSCAT,X,Z,DIRX,DIRZ,XNXT,ZNXT,XOLD,ZOLD
        GOTO 1500
      ENDIF

C    ELECTRON LEFT SIMULATION AREA ?      

      IF (INTERS.EQ.0 .OR. IR .EQ. -1) THEN
           GOTO 1500
      ENDIF
 
C    ELECTRON ENTERS SPECIMEN

      IF (DIRY .NE. 0) THEN
        IF (DIRX .NE. 0) THEN
          Y = Y + (XNXT-X)/(DIRX*DIRY)
        ELSEIF (DIRZ .NE. 0) THEN
          Y = Y + (ZNXT-Z)/(DIRZ*DIRY)
        ENDIF
      ENDIF
          
      X = XNXT + DIRX*DELTA
      Z = ZNXT + DIRZ*DELTA

      GOTO 200 

1400  CONTINUE
******ELECTRON GENERATED IN AIR
      FRAC = 0.
      RETURN      

1500  CONTINUE
******ELECTRON LEFT SIMULATION AREA
      RETURN      

2000  CONTINUE
*****ERROR DURING POINT LOCATION
      WRITE (*,*) ' '
      WRITE (*,*) '***** ERROR: OCCURED IN IMGREG *****'
      WRITE (*,*) ' (X,Y,Z) = ',X,Y,Z
      GOTO 1500

2100  CONTINUE
*****ERROR DURING POINT LOCATION
      WRITE (*,*) ' '
      WRITE (*,*) '***** ERROR: OCCURED IN IMENTR *****'
      WRITE (*,*) ' (X,Y,Z) = ',X,Y,Z
      WRITE (*,*) ' (DIRX,DIRZ) = ',DIRX,DIRZ
      GOTO 1500

2200  CONTINUE
*****ERROR DURING POINT LOCATION
      WRITE (*,*) ' '
      WRITE (*,*) '***** ERROR: OCCURED IN INTERS *****'
      WRITE (*,*) ' (X,Y,Z) = ',X,Y,Z
      WRITE (*,*) ' (DIRX,DIRZ) = ',DIRX,DIRZ
      GOTO 1500

      END

