c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
*This file contains the following subroutines:
*CHKION - checks the specification of the 'ELEMENT,SHELL/SUBSHELL' string
*UPDSHL - writes/updates the SHELL table (ELEMOFSHELL, SHELLOFSHELL, etc.)
*CHKXCH - checks the specification of the 'ELEMENT, CHAR. X-RAY LINE' string
*UPDXCH - writes/updates the XCHAR table (ELEMOFXCHAR, XCHAROFXCHAR, etc.)
*CHKAUG - checks the specification of the 'ELEMENT, AUGER PEAK' string
*UPDAUG - writes/updates the AUGER table (ELEMOFAUGER, LINEOFAUGER, etc.)
*CHKATO - checks and parses the ATOM string specified in the input deck
*CHKMAT -   "      "    "     " ATMAT strings  "       "  "    "     "
*TRNTAB - transforms table information to appropriate output information
*CHKENER- adjusts limits for possible ionisations
******************************************************************************

* last modification: CS 27-Feb-1995

      SUBROUTINE CHKION (IMODE,NDUMMY,CBUFF,ICLEN)

***   CHKION - CHECKS THE SPECIFICATION OF ELEMENT, SHELL AND SUBSHELL
*              (FOR SCREEN OUTPUT OF IONIZATION, FOR IONIZATIONS SPECIFIED
*              BY THE 'ION' KEY IN THE '&MODEL' NAMELIST) AND UPDATES THE
*              'SHELL' ARRAY AND ALL 'SHELL' ARRAYS (ELEMOFSHELL, SHELLOFSHELL,
*              ETC.)
*
**    ARGUMENTS
*
*     IMODE  -  IMODE=1: ARRAY 'SHELLSCR' WILL BE NOT UPDATED
*               IMODE=2: ARRAY 'SHELLSCR' WILL BE UPDATED
*               IMODE=3: SHELL() WILL BE UPDATED ACCORDING TO XCHARLINE()
*               IMODE=4: SHELL() WILL BE UPDATED ACCORDING TO AUGERPEAK()
*     CBUFF  -  STRING CONTAINING INFORMATION (ELEMENT, SHELL, SUBSHELL)
*               CONCERNING THE IONIZATION OUTPUT TO SCREEN
*     ICLEN  -  ACTUAL LENGTH OF CBUFF
*     
**    AUTHOR:  C. Schiebl  22-DEC-1994
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'

      DIMENSION NDUMMY(20)
      DIMENSION IPOS(10),NSHL(MAXELE)
      DIMENSION ISHL(0:MAXELE,3*MAXSHELL),IELE(MAXELE)

      CHARACTER*70 CELEM
      CHARACTER*(*) CBUFF
CS      CHARACTER*40 CBUFF
      CHARACTER*10 CBL
      CHARACTER*3  CD(10)
      CHARACTER*2  CEABR

      LOGICAL LALL

      PARAMETER (CBL='          ')

**    START OF EXECUTABLE CODE
*     ************************

CS      DO 1 I=ICLEN+1,40
CS         CBUFF(I:I)=' '
CS 1    CONTINUE
      LALL=.FALSE.
      CELEM=CBL//CBL//CBL//CBL//CBL//CBL//CBL
      DO 2 I=1,10
         IPOS(I)=0
 2    CONTINUE
      DO 7 I=1,MAXELE
         NSHL(I)=0
         IELE(I)=0
         DO 5 J=1,3*MAXSHELL
            ISHL(I,J)=0
 5       CONTINUE
 7    CONTINUE
      DO 8 I=1,3*MAXSHELL
         ISHL(0,I)=0
 8    CONTINUE

      IF (IMODE.EQ.3) THEN
*        UPDATE SHELL() ACCORDING TO INFORMATION IN XCHARLINE()
         NIELE=0
         DO 11 I=1,NELEM
            NYESEL=0
            DO 9 J=1,MAXXCHAR
               IF ((XCHARLINE(J,I).GT.0).AND.
     &            (XCHARLINE(J,I).LT.11)) THEN
                  IF (NYESEL.EQ.0) THEN
                     NIELE=NIELE+1
                     NYESEL=1
                     IELE(NIELE)=ATNUM(I)
                  ENDIF
                  NSHL(NIELE)=NSHL(NIELE)+1
                  IF (XCHARLINE(J,I).EQ.1) ISHL(NIELE,NSHL(NIELE))=1
                  IF (XCHARLINE(J,I).EQ.2) ISHL(NIELE,NSHL(NIELE))=1
                  IF (XCHARLINE(J,I).EQ.3) ISHL(NIELE,NSHL(NIELE))=1
                  IF (XCHARLINE(J,I).EQ.4) ISHL(NIELE,NSHL(NIELE))=4
                  IF (XCHARLINE(J,I).EQ.5) ISHL(NIELE,NSHL(NIELE))=4
                  IF (XCHARLINE(J,I).EQ.6) ISHL(NIELE,NSHL(NIELE))=3
                  IF (XCHARLINE(J,I).EQ.7) ISHL(NIELE,NSHL(NIELE))=4
                  IF (XCHARLINE(J,I).EQ.8) ISHL(NIELE,NSHL(NIELE))=2
                  IF (XCHARLINE(J,I).EQ.9) ISHL(NIELE,NSHL(NIELE))=9
                  IF (XCHARLINE(J,I).EQ.10) ISHL(NIELE,NSHL(NIELE))=8
               ENDIF
 9          CONTINUE
 11      CONTINUE
*        JUMP TO UPDATE SHELL
         GOTO 500
      ENDIF

      IF (IMODE.EQ.4) THEN
*        UPDATE SHELL() ACCORDING TO INFORMATION IN AUGERPEAK()
         NIELE=0
         DO 17 I=1,NELEM
            NYESEL=0
            DO 16 J=1,MXAUGP
               IF ((AUGERPEAK(J,I).GT.0).AND.
     &            (AUGERPEAK(J,I).LT.4)) THEN
                  IF (NYESEL.EQ.0) THEN
                     NIELE=NIELE+1
                     NYESEL=1
                     IELE(NIELE)=ATNUM(I)
                  ENDIF
                  NSHL(NIELE)=NSHL(NIELE)+1
                  IF (AUGERPEAK(J,I).EQ.1) ISHL(NIELE,NSHL(NIELE))=1
                  IF (AUGERPEAK(J,I).EQ.2) ISHL(NIELE,NSHL(NIELE))=4
                  IF (AUGERPEAK(J,I).EQ.3) ISHL(NIELE,NSHL(NIELE))=9
               ENDIF
 16         CONTINUE
 17      CONTINUE
*        JUMP TO UPDATE SHELL
         GOTO 500
      ENDIF

*     DEFAULT:
CS    NO DEFAULT VALUES ARE SET
CS      NSHL(1)=1
CS      ISHL(1,1)=1
CS      NIELE=1
CS      IELE(1)=ATNUM(1)
      NIELE=0

      CALL CCAP (CBUFF(1:3),3)
      IF ((CBUFF(1:3).EQ.'ALL').OR.(CBUFF(1:1).EQ.'*')) THEN
*        ALL ELEMENTS SHOULD BE TAKEN INTO ACCOUNT
         NIELE=NELEM
         DO 18 I=1,NIELE
            IELE(I)=ATNUM(I)
 18      CONTINUE
         IF (((CBUFF(1:3).EQ.'ALL').AND.(CBUFF(4:4).NE.',')).OR.
     &       ((CBUFF(1:1).EQ.'*').AND.(CBUFF(2:2).NE.','))) THEN
*           ALL SHELLS AND SUBSHELLS SHOULD BE TAKEN INTO ACCOUNT
            DO 20 I=1,NIELE
               NSHL(I)=9
               DO 19 J=1,9
                  ISHL(I,J)=J
 19            CONTINUE
 20         CONTINUE
            LALL=.TRUE.
            GOTO 500
         ELSE
*           ONLY SPECIFIED SHELL(S) SHOULD BE TAKEN INTO ACCOUNT
            IE=IEND(CBUFF)
            CALL FIND (CBUFF,',',1,IE,IP,IFND)
            GOTO 80
         ENDIF
      ENDIF

      IF (CBUFF(1:3).EQ.'DEF') THEN
*        DEFAULT SHOULD BE TAKEN, I.E. FIRST ELEMENT AND K-LINE
CS         GOTO 500
         GOTO 900
      ENDIF

      IE=IEND(CBUFF)
      CALL FIND (CBUFF,',',1,IE,IP,IFND)

      IF (IFND.EQ.0) THEN
*        ONLY ELEMENT IS SPECIFIED
         NC=MIN(2,IE)
         CELEM(1:NC)=CBUFF(1:NC)
         CALL ELMENT(1,CELEM,IATNUM,CEABR,FRM,FDENS,IERR)
         IF (IERR.NE.0) THEN
*        SPECIFIED ELEMENT DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for ionization does',
     &                     ' not exist !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and K shell'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
         IEXIST=0
         DO 25 I=1,NELEM
            IF (IATNUM.EQ.ATNUM(I)) IEXIST=1
 25      CONTINUE
         IF (IEXIST.EQ.0) THEN
*        SPECIFIED ELEMENT IS NOT IN THE SAMPLE - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for ionization is',
     &                     ' not in the element table !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and K shell'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
*        ALL SHELLS AND SUBSHELLS SHOULD BE TAKEN INTO ACCOUNT
         NIELE=1
         IELE(1)=IATNUM
         NSHL(1)=9
         DO 45 J=1,9
            ISHL(1,J)=J
 45      CONTINUE
         GOTO 500
      ENDIF

 80   CONTINUE
      DO 90 I=1,10
         CD(I)='  '
 90   CONTINUE
      IPOS(1)=0
      I=2
      NPOS=2
      IPOS(I)=IP
 105  IST=IPOS(I)+1
      CALL FIND (CBUFF,',',IST,IE,IP,IFND)
      IF (IFND.EQ.1) THEN
         I=I+1
         IPOS(I)=IP
         NPOS=NPOS+1
         GOTO 105
      ENDIF
      NPOS=NPOS+1
      IPOS(NPOS)=IE+1
      NCOMMA=MIN(NPOS,10)
      DO 120 I=1,NCOMMA-1
         IF ((IPOS(I+1)-IPOS(I)).GT.1) THEN
            READ(CBUFF(IPOS(I)+1:IPOS(I+1)-1),'(A)',ERR=120) CD(I)
            CALL CCAP (CD(I),3)
         ENDIF
 120  CONTINUE
      IF ((CD(1)(1:3).EQ.'ALL').OR.(CD(1)(1:1).EQ.'*')) THEN
*        ALL ELEMENTS SHOULD BE TAKEN INTO ACCOUNT
         NIELE=NELEM
         DO 121 I=1,NIELE
            IELE(I)=ATNUM(I)
 121     CONTINUE
      ELSE
         CELEM(1:IEND(CD(1)))=CD(1)
         CALL ELMENT(1,CELEM,IATNUM,CEABR,FRM,FDENS,IERR)
         IF (IERR.NE.0) THEN
*        SPECIFIED ELEMENT DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for ionization does',
     &                     ' not exist !'
            WRITE(6,'(2A)')' Default value will be taken (=first',
     &                     ' element in the table and K shell'
            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
            GOTO 500
         ENDIF
         IEXIST=0
         DO 122 I=1,NELEM
            IF (IATNUM.EQ.ATNUM(I)) IEXIST=1
 122     CONTINUE
         IF (IEXIST.EQ.0) THEN
*        SPECIFIED ELEMENT IS NOT IN THE SAMPLE - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for ionization is',
     &                     ' not in the element table !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and K shell'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
           GOTO 900
         ENDIF
         NIELE=1
         IELE(1)=IATNUM
      ENDIF
*     WHAT SHELL(S) SHOULD BE TAKEN INTO ACCOUNT?
      IF ((CD(2)(1:1).EQ.'A').OR.(CD(2)(1:1).EQ.'*')) THEN
*        ALL SHELLS OF ELEMENT 1 OR ALL ELEMENTS, RESPECTIVELY
         DO 125 I=1,NIELE
            NSHL(I)=9
            DO 123 J=1,9
               ISHL(I,J)=J
 123        CONTINUE
 125     CONTINUE
         GOTO 500
      ENDIF
      DO 127 I=1,NIELE
         NSHL(I)=0
 127  CONTINUE
      DO 200 K=2,NCOMMA-1
         IF (CD(K)(1:1).EQ.'K') THEN
*           K SHELL
            DO 130 I=1,NIELE
               IF ((IELE(I).GE.EDLIM(1,1)).AND.
     &             (IELE(I).LE.EDLIM(1,2))) THEN
                  NSHL(I)=NSHL(I)+1
                  ISHL(I,NSHL(I))=1
               ELSE
*                 NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                  WRITE(6,'(A)')'       '
                  WRITE(6,'(A)')' %%% WARNING %%% '
                  WRITE(6,'(2A)')' There is no edge energy available',
     &                        ' for the specified element and shell !'
                  WRITE(6,'(A,I3,A)')' Element: ',IELE(I),' Shell: 1'
                  WRITE(6,'(2A)')' This shell will not be taken',
     &                           ' into account !'
                  WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
               ENDIF
 130        CONTINUE
         ELSE IF (CD(K)(1:1).EQ.'L') THEN
            IF (CD(K)(2:2).EQ.'1') THEN
*              THE L1 SHELL IS SPECIFIED
               DO 132 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(2,1)).AND.
     &                (IELE(I).LE.EDLIM(2,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=2
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 2'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 132           CONTINUE
            ELSE IF (CD(K)(2:2).EQ.'2') THEN
*              THE L2 SHELL IS SPECIFIED
               DO 134 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(3,1)).AND.
     &                (IELE(I).LE.EDLIM(3,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=3
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 3'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 134           CONTINUE
            ELSE IF (CD(K)(2:2).EQ.'3') THEN
*              THE L3 SHELL IS SPECIFIED
               DO 136 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(4,1)).AND.
     &                (IELE(I).LE.EDLIM(4,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=4
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 4'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 136           CONTINUE
            ELSE
*              ALL THREE L SUBSHELLS ARE SPECIFIED
               DO 138 J=1,3
                  DO 137 I=1,NIELE
                     IF ((IELE(I).GE.EDLIM(1+J,1)).AND.
     &                   (IELE(I).LE.EDLIM(1+J,2))) THEN
                        NSHL(I)=NSHL(I)+1
                        ISHL(I,NSHL(I))=1+J
                     ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no edge energy ava',
     &                    'ilable for the specified element and shell !'
                        WRITE(6,'(A,I3,A,I2)')' Element: ',IELE(I),
     &                                        ' Shell: ',1+J
                        WRITE(6,'(2A)')' This shell will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless -',
     &                                 ' ;-) SESAME'
                     ENDIF
 137              CONTINUE
 138           CONTINUE
            ENDIF
         ELSE IF (CD(K)(1:1).EQ.'M') THEN
            IF (CD(K)(2:2).EQ.'1') THEN
*              THE M1 SHELL IS SPECIFIED
               DO 152 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(5,1)).AND.
     &                (IELE(I).LE.EDLIM(5,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=5
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 5'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 152           CONTINUE
            ELSE IF (CD(K)(2:2).EQ.'2') THEN
*              THE M2 SHELL IS SPECIFIED
               DO 154 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(6,1)).AND.
     &                (IELE(I).LE.EDLIM(6,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=6
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 6'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 154           CONTINUE
            ELSE IF (CD(K)(2:2).EQ.'3') THEN
*              THE M3 SHELL IS SPECIFIED
               DO 156 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(7,1)).AND.
     &                (IELE(I).LE.EDLIM(7,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=7
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 7'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 156           CONTINUE
            ELSE IF (CD(K)(2:2).EQ.'4') THEN
*              THE M4 SHELL IS SPECIFIED
               DO 158 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(8,1)).AND.
     &                (IELE(I).LE.EDLIM(8,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=8
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 8'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 158           CONTINUE
            ELSE IF (CD(K)(2:2).EQ.'5') THEN
*              THE M5 SHELL IS SPECIFIED
               DO 160 I=1,NIELE
                  IF ((IELE(I).GE.EDLIM(9,1)).AND.
     &                (IELE(I).LE.EDLIM(9,2))) THEN
                     NSHL(I)=NSHL(I)+1
                     ISHL(I,NSHL(I))=9
                  ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                     WRITE(6,'(A)')'       '
                     WRITE(6,'(A)')' %%% WARNING %%% '
                     WRITE(6,'(2A)')' There is no edge energy ava',
     &                 'ilable for the specified element and shell !'
                     WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                  ' Shell: 9'
                     WRITE(6,'(2A)')' This shell will not be taken',
     &                              ' into account !'
                     WRITE(6,'(2A)')' Protests are useless -',
     &                              ' ;-) SESAME'
                  ENDIF
 160           CONTINUE
            ELSE
*              ALL FIVE M SUBSHELLS ARE SPECIFIED
               DO 168 J=1,5
                  DO 167 I=1,NIELE
                     IF ((IELE(I).GE.EDLIM(4+J,1)).AND.
     &                   (IELE(I).LE.EDLIM(4+J,2))) THEN
                        NSHL(I)=NSHL(I)+1
                        ISHL(I,NSHL(I))=4+J
                     ELSE
*                    NO EDGE ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND SHELL
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no edge energy ava',
     &                    'ilable for the specified element and shell !'
                        WRITE(6,'(A,I3,A,I2)')' Element: ',IELE(I),
     &                                        ' Shell: ',4+J
                        WRITE(6,'(2A)')' This shell will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless -',
     &                                 ' ;-) SESAME'
                     ENDIF
 167              CONTINUE
 168           CONTINUE
            ENDIF
         ELSE
*           SPECIFIED SHELL DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified shell for ionization does',
     &                     ' not exist !'
            WRITE(6,'(2A)')' Default value will be taken (=',
     &                     ' K shell'
            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
            DO 180 I=1,NIELE
               NSHL(I)=NSHL(I)+1
               ISHL(I,NSHL(I))=1
 180        CONTINUE
         ENDIF
 200  CONTINUE

 500  CONTINUE
      IF (IMODE.EQ.2) THEN
*        IONIZATIONS SHOULD BE VISUALIZED ON SCREEN => PREPARE
*        FOR SETTING CORRECT SCREEN NUMBER
         DO 520 J=1,NIELE
            DO 505 I=1,NELEM
               IF (IELE(J).EQ.ATNUM(I)) INDX=I
 505        CONTINUE
            DO 510 I=1,MAXSHELL
               IF (ISHL(J,I).NE.0) THEN
                  NSHELLSCR(INDX,ISHL(J,I))=
     &                      NSHELLSCR(INDX,ISHL(J,I))+1
                  SHELLSCR(ISHL(J,I),INDX,NSHELLSCR
     &                      (INDX,ISHL(J,I)))=MAXSCR+1
               ENDIF
 510        CONTINUE
 520     CONTINUE
      ENDIF
*     UPDATE SHELL
      IF (LALL) THEN
*        ALL ELEMENTS AND ALL SHELLS (AND SUBSHELLS)
         DO 540 I=1,NELEM
            DO 530 J=1,9
               SHELL(J,I)=J
 530        CONTINUE
 540     CONTINUE
      ELSE
*        MERGE WITH ALREADY EXISTING SHELL SPECIFICATION
         DO 650 J=1,NIELE 
            DO 630 I=1,NELEM
               IF (IELE(J).EQ.ATNUM(I)) INDX=I
 630        CONTINUE
*           UPDATE NDUMMY(8,9,...) [ELEMENT,SHELL,SHELL,...]
            NDUMMY(8)=INDX
            NDUMMY(20)=NSHL(J)
            DO 635 I=1,NSHL(J)
               NDUMMY(8+I)=ISHL(J,I)
 635        CONTINUE
            DO 640 I=1,MAXSHELL
               ISHL(J,NSHL(J)+I)=SHELL(I,INDX)
 640        CONTINUE
            NSHL(J)=NSHL(J)+MAXSHELL
 650     CONTINUE
*        ELIMINATE MULTIPLE SPECIFICATION OF SHELLS
         DO 690 K=1,NIELE
            DO 680 I=1,NSHL(K)
               ICOMP=ISHL(K,I)
               DO 670 J=1,NSHL(K)
                  IF ((ICOMP.EQ.ISHL(K,J)).AND.(I.NE.J).AND.
     &                (ISHL(K,J).NE.0)) ISHL(K,J)=0
 670           CONTINUE
 680        CONTINUE
 690     CONTINUE
*        SORT ISHL
         DO 730 K=1,NIELE
            NCNT=0
            DO 720 I=1,MAXSHELL
               DO 710 J=1,NSHL(K)
                  IF (ISHL(K,J).EQ.I) THEN
                     NCNT=NCNT+1
                     ISHL(0,NCNT)=ISHL(K,J)
                  ENDIF
 710           CONTINUE
 720        CONTINUE
            NSHL(K)=NCNT
            DO 725 J=1,NSHL(K)
               ISHL(K,J)=ISHL(0,J)
 725        CONTINUE
            DO 726 J=NSHL(K)+1,3*MAXSHELL
               ISHL(K,J)=0
 726        CONTINUE
 730     CONTINUE
*        UPDATE SHELL
         DO 770 I=1,NELEM
            DO 760 J=1,NIELE
               IF (IELE(J).EQ.ATNUM(I)) THEN
                  DO 750 K=1,NSHL(J)
                     SHELL(K,I)=ISHL(J,K)
 750              CONTINUE
               ENDIF
 760        CONTINUE
 770     CONTINUE
      ENDIF


 900  RETURN
      END

**************************************************************************
*last modification: CS 09-Dec-1996

      SUBROUTINE UPDSHL (IFLAG)
 
****  UPDSHL - UPDATE SHELL TABLES
*
**    ARGUMENTS
*
*     IFLAG - DUMMY INTEGER
*
**    AUTHOR: C. Schiebl 11-Jan-1995
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'
      INCLUDE 'DMA.INC'

**    START OF EXECUTABLE CODE
*     ************************

*     INITIALIZE SHELL TABLE ARRAYS
      MXINDX=NDMAREG*NDMASHELL
      IF (MXINDX.GT.0)
     &   CALL CHKBND(15,MXINDX,'UPDSHL    ','SHELLOFREG')
      MXINDX=(NDMASCRN+NDMAPS)*NDMASHELL
      IF (MXINDX.GT.0)
     &   CALL CHKBND(16,MXINDX,'UPDSHL    ','SCROFSHELL')
      MXINDX=NDMAPM*NDMASHELL
      IF (MXINDX.GT.0)
     &   CALL CHKBND(17,MXINDX,'UPDSHL    ','PSMINOFION')
      MXINDX=NDMAHIST*NDMASHELL
      IF (MXINDX.GT.0)
     &   CALL CHKBND(18,MXINDX,'UPDSHL    ','IONHIST   ')
      MXINDX=NDMAXCHAR*NDMASHELL
      IF (MXINDX.GT.0)
     &   CALL CHKBND(19,MXINDX,'UPDSHL    ','XCHAROFSHELL')
      DO 20 I=1,NDMAREG
         NSHELLOFREG(I)=0
         DO 10 J=1,NDMASHELL
            INDX=(J-1)*NDMAREG+I
CS            SHELLOFREG(I,J)=0
            SHELLOFREG(INDX)=0
 10      CONTINUE
 20   CONTINUE
      DO 30 I=1,NDMASHELL
         ELEMOFSHELL(I)=0
         SHELLOFSHELL(I)=0
         ENEROFSHELL(I)=0
         IONINTOFSHELL(I)=0
         NSCROFSHELL(I)=0
         NSCROFION(I)=0
         NXCHAROFSHELL(I)=0
         DO 23 J=1,NDMASCRN+NDMAPS
            INDX=(I-1)*(NDMASCRN+NDMAPS)+J
CS            SCROFSHELL(I,J)=0
CS            SCROFION(I,J)=0
            SCROFSHELL(INDX)=0
            SCROFION(INDX)=0
 23      CONTINUE
         NIONHIST(I)=0
         DO 24 J=1,NDMAHIST
            INDX=(I-1)*NDMAHIST+J
CS            IONHIST(I,J)=0
            IONHIST(INDX)=0
 24      CONTINUE
         DO 25 J=1,NDMAXCHAR
            INDX=(I-1)*NDMAXCHAR+J
CS            XCHAROFSHELL(I,J)=0
            XCHAROFSHELL(INDX)=0
 25      CONTINUE
         NPSMINOFION(I)=0
         DO 27 J=1,NDMAPM
            INDX=(I-1)*NDMAPM+J
CS            PSMINOFION(I,J)=0
            PSMINOFION(INDX)=0
 27      CONTINUE
         HISTOFION(I)=0
 30   CONTINUE
      DO 33 I=1,MAXSCR+MAXPST
         ILOGIONTOSCR(I)=0
 33   CONTINUE

      MXINDX=NELEM*MAXSHELL*NDMAXCHAR
      IF (MXINDX.GT.0)
     &   CALL CHKBND(24,MXINDX,'UPDSHL    ','XCHTABINDX')
      DO 50 IELE=1,NELEM
         DO 40 ISHL=1,MAXSHELL
            NXCHTABINDX(IELE,ISHL)=0
            DO 35 IXC=1,NDMAXCHAR
               IXINDX=(IXC-1)*NELEM*MAXSHELL+(ISHL-1)*NELEM+IELE
CS               XCHTABINDX(IELE,ISHL,IXC)=0
               XCHTABINDX(IXINDX)=0
 35         CONTINUE
 40      CONTINUE
 50   CONTINUE

*     UPDATE SHELL TABLE ARRAYS
      NSHELL=0
      DO 100 IREG=1,NREG
         NSHELLOFREG(IREG)=0
         DO 90 IEL1=1,NELEM
            DO 80 IEL2=1,NELE(IREG)
               IF (IEL1.EQ.ELEM(IREG,IEL2)) THEN
                  DO 70 I=1,MAXSHELL
                     IF (SHELL(I,IEL1).NE.0) THEN
                        IENTRY=0
                        DO 60 J=1,NSHELL
C                           IF ((ELEMOFSHELL(J).EQ.ATNUM(IEL1)).AND.
                           IF ((ELEMOFSHELL(J).EQ.IEL1).AND. 
     &                         (SHELLOFSHELL(J).EQ.SHELL(I,IEL1))
     &                         .AND.(SHELLOFSHELL(J).NE.0)) THEN
                              IENTRY=J
                           ENDIF
 60                     CONTINUE
                        NSHELLOFREG(IREG)=NSHELLOFREG(IREG)+1
                        IF (IENTRY.EQ.0) THEN
*                          NEW ENTRY
                           IF (NSHELL.LT.NDMASHELL) THEN
                              NSHELL=NSHELL+1
                              ELEMOFSHELL(NSHELL)=IEL1
                              SHELLOFSHELL(NSHELL)=SHELL(I,IEL1)
CS                              SHELLOFREG(IREG,NSHELLOFREG(IREG))=NSHELL
                              INDX=(NSHELLOFREG(IREG)-1)*
     &                             NDMAREG+IREG
                              CALL CHKBND(15,INDX,'UPDSHL    ',
     &                                    'SHELLOFREG')
                              SHELLOFREG(INDX)=NSHELL
                           ENDIF
                        ELSE
*                          OLD ENTRY
CS                           SHELLOFREG(IREG,NSHELLOFREG(IREG))=IENTRY
                           INDX=(NSHELLOFREG(IREG)-1)*NDMAREG+IREG
                           CALL CHKBND(15,INDX,'UPDSHL    ',
     &                                 'SHELLOFREG')
                           SHELLOFREG(INDX)=IENTRY
                        ENDIF
                     ENDIF
 70               CONTINUE
               ENDIF
 80         CONTINUE
 90      CONTINUE
 100  CONTINUE

      DO 200 I=1,NSHELL
         NSCROFSHELL(I)=NSHELLSCR(ELEMOFSHELL(I),SHELLOFSHELL(I))
         NSCROFION(I)=XCIODIS21(ELEMOFSHELL(I),SHELLOFSHELL(I),1)
         NPSMINOFION(I)=XCIODIS31(ELEMOFSHELL(I),SHELLOFSHELL(I),1)
         NIONHIST(I)=XCIODIS91(ELEMOFSHELL(I),SHELLOFSHELL(I))
         DO 170 J=1,NDMASCRN+NDMAPS
           INDX=(I-1)*(NDMASCRN+NDMAPS)+J
           CALL CHKBND(16,INDX,'UPDSHL    ','SCROFSHELL')
CS           SCROFSHELL(I,J)=SHELLSCR(SHELLOFSHELL(I),ELEMOFSHELL(I),J)
           SCROFSHELL(INDX)=SHELLSCR(SHELLOFSHELL(I),ELEMOFSHELL(I),J)
CS           SCROFION(I,J)=XCIODIS2(ELEMOFSHELL(I),SHELLOFSHELL(I),1,J)
           SCROFION(INDX)=XCIODIS2(ELEMOFSHELL(I),SHELLOFSHELL(I),1,J)
           IF (XCIODIS2(ELEMOFSHELL(I),SHELLOFSHELL(I),5,J).GT.0) THEN
            ILOGIONTOSCR(XCIODIS2(ELEMOFSHELL(I),SHELLOFSHELL(I),1,J))=
     &                   XCIODIS2(ELEMOFSHELL(I),SHELLOFSHELL(I),5,J)
           ENDIF
 170     CONTINUE
         DO 180 J=1,NDMAPM
           INDX=(I-1)*NDMAPM+J
           CALL CHKBND(17,INDX,'UPDSHL    ','PSMINOFION')
CS           PSMINOFION(I,J)=
CS     &                   XCIODIS3(ELEMOFSHELL(I),SHELLOFSHELL(I),1,J)
           PSMINOFION(INDX)=
     &                   XCIODIS3(ELEMOFSHELL(I),SHELLOFSHELL(I),1,J)
 180     CONTINUE
         DO 190 J=1,NDMAHIST
            INDX=(I-1)*NDMAHIST+J
            CALL CHKBND(18,INDX,'UPDSHL    ','IONHIST   ')
CS            IONHIST(I,J)=XCIODIS9(ELEMOFSHELL(I),SHELLOFSHELL(I),J)
            IONHIST(INDX)=XCIODIS9(ELEMOFSHELL(I),SHELLOFSHELL(I),J)
 190     CONTINUE
         HISTOFION(I)=XCIODIS1(ELEMOFSHELL(I),SHELLOFSHELL(I))
 200  CONTINUE

      DO 320 I=1,NXCHAR
         IF (XCHAROFXCHAR(I).EQ.1) IS=1
         IF (XCHAROFXCHAR(I).EQ.2) IS=1
         IF (XCHAROFXCHAR(I).EQ.3) IS=1
         IF (XCHAROFXCHAR(I).EQ.4) IS=4
         IF (XCHAROFXCHAR(I).EQ.5) IS=4
         IF (XCHAROFXCHAR(I).EQ.6) IS=3
         IF (XCHAROFXCHAR(I).EQ.7) IS=4
         IF (XCHAROFXCHAR(I).EQ.8) IS=2
         IF (XCHAROFXCHAR(I).EQ.9) IS=9
         IF (XCHAROFXCHAR(I).EQ.10) IS=8
         DO 310 J=1,NSHELL
            IF ((ELEMOFSHELL(J).EQ.ELEMOFXCHAR(I)).AND.
     &          (SHELLOFSHELL(J).EQ.IS)) THEN
               SHELLOFXCHAR(I)=J
            ENDIF
 310     CONTINUE
         K1=SHELLOFSHELL(SHELLOFXCHAR(I))
         NXCHTABINDX(ELEMOFXCHAR(I),K1)=
     &                         NXCHTABINDX(ELEMOFXCHAR(I),K1)+1
         K2=NXCHTABINDX(ELEMOFXCHAR(I),K1)
         IXINDX=(K2-1)*NELEM*MAXSHELL+(K1-1)*NELEM+ELEMOFXCHAR(I)
         CALL CHKBND(24,IXINDX,'UPDSHL    ','XCHTABINDX')
CS         XCHTABINDX(ELEMOFXCHAR(I),K1,K2)=I
         XCHTABINDX(IXINDX)=I
 320  CONTINUE

      DO 380 I=1,NSHELL
         DO 370 J=1,NXCHAR
            IF (SHELLOFXCHAR(J).EQ.I) THEN
               NXCHAROFSHELL(I)=NXCHAROFSHELL(I)+1
               INDX=(I-1)*NXCHAR+NXCHAROFSHELL(I)
               CALL CHKBND(19,INDX,'UPDSHL    ','XCHAROFSHELL')
CS               XCHAROFSHELL(I,NXCHAROFSHELL(I))=J
               XCHAROFSHELL(INDX)=J
            ENDIF
 370     CONTINUE
 380  CONTINUE

      RETURN
      END

****************************************************************************
* last modification: CS 11-Oct-1996

      SUBROUTINE CHKXCH (IMODE,NDUMMY,CBUFF,ICLEN)

***   CHKXCH - CHECKS THE SPECIFICATION OF ELEMENT AND CHARACTERISTIC X-RAY
*              LINE (FOR CHARACTERISTIC X-RAY LINES SPECIFIED BY THE
*              'XCHAR' KEY IN THE '&MODEL' NAMELIST) AND UPDATES THE
*              'XCHARLINE' ARRAY AND ALL 'XCHAR' ARRAYS (ELEMOFXCHAR,
*              XCHAROFXCHAR, ETC.)
*
*     SUPPORTED CHARACTERISTIC X-RAY LINES:
*	       Ka1=1, Ka2=2, Kb=3
*              La1=4, La2=5, Lb1=6, Lb2=7, Lb3=8
*              Ma=9,  Mb=10
*              ENTRIES 11 AND 12 ARE RESERVED FOR LATER EXTENSION(S)
*
**    ARGUMENTS
*
*     IMODE  -  IMODE=1: ARRAY 'XCHSCR' WILL BE NOT UPDATED
*               IMODE=2: ARRAY 'XCHSCR' WILL BE UPDATED
*     CBUFF  -  STRING CONTAINING INFORMATION (ELEMENT, CHAR. X-RAY LINE)
*               CONCERNING THE CHAR. X-RAY LINES SPECIFIED BY 'XCHAR'
*     ICLEN  -  ACTUAL LENGTH OF STRING
*     
**    AUTHOR:  C. Schiebl  13-JAN-1995
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'

      DIMENSION NDUMMY(20)
      DIMENSION IPOS(14),NXXCH(MAXELE)
      DIMENSION IXXCH(0:MAXELE,3*MAXXCHAR),IELE(MAXELE)

      CHARACTER*70 CELEM
CS      CHARACTER*50 CBUFF
      CHARACTER*(*) CBUFF
      CHARACTER*40 CDUMMY
      CHARACTER*10 CBL
      CHARACTER*3  CD(14)
      CHARACTER*2  CEABR

      LOGICAL LALL

      PARAMETER (CBL='          ')

**    START OF EXECUTABLE CODE
*     ************************

CS      DO 1 I=ICLEN+1,50
CS         CBUFF(I:I)=' '
CS 1    CONTINUE
      LALL=.FALSE.
      CDUMMY=CBL//CBL//CBL//CBL
      CELEM=CBL//CBL//CBL//CBL//CBL//CBL//CBL
      DO 2 I=1,14
         IPOS(I)=0
 2    CONTINUE
      DO 7 I=1,MAXELE
         NXXCH(I)=0
         IELE(I)=0
         DO 5 J=1,3*MAXXCHAR
            IXXCH(I,J)=0
 5       CONTINUE
 7    CONTINUE
      DO 8 I=1,3*MAXXCHAR
         IXXCH(0,I)=0
 8    CONTINUE

*     DEFAULT: (Ka1 & Ka2)
CS    DO NOT TAKE ANY DEFAUTL VALUES
CS      NXXCH(1)=2
CS      IXXCH(1,1)=1
CS      IXXCH(1,2)=2
CS      NIELE=1
CS      IELE(1)=ATNUM(1)
      NIELE=0

      CALL CCAP (CBUFF(1:3),3)
      IF ((CBUFF(1:3).EQ.'ALL').OR.(CBUFF(1:1).EQ.'*')) THEN
*        ALL ELEMENTS SHOULD BE TAKEN INTO ACCOUNT
         NIELE=NELEM
         DO 10 I=1,NIELE
            IELE(I)=ATNUM(I)
 10      CONTINUE
         IF (((CBUFF(1:3).EQ.'ALL').AND.(CBUFF(4:4).NE.',')).OR.
     &       ((CBUFF(1:1).EQ.'*').AND.(CBUFF(2:2).NE.','))) THEN
*           ALL LINES SHOULD BE TAKEN INTO ACCOUNT
            DO 20 I=1,NIELE
               NXXCH(I)=10
               DO 15 J=1,10
                  IXXCH(I,J)=J
 15            CONTINUE
 20         CONTINUE
            LALL=.TRUE.
            GOTO 500
         ELSE
*           ONLY SPECIFIED LINE(S) SHOULD BE TAKEN INTO ACCOUNT
            IE=IEND(CBUFF)
            CALL FIND (CBUFF,',',1,IE,IP,IFND)
            GOTO 80
         ENDIF
      ENDIF

      IF (CBUFF(1:3).EQ.'DEF') THEN
*        DEFAULT SHOULD BE TAKEN, I.E. FIRST ELEMENT AND Ka1 & Ka2
CS         GOTO 500
         GOTO 900
      ENDIF

      IE=IEND(CBUFF)
      CALL FIND (CBUFF,',',1,IE,IP,IFND)

      IF (IFND.EQ.0) THEN
*        ONLY ELEMENT IS SPECIFIED
         NC=MIN(2,IE)
         CELEM(1:NC)=CBUFF(1:NC)
         CALL ELMENT(1,CELEM,IATNUM,CEABR,FRM,FDENS,IERR)
         IF (IERR.NE.0) THEN
*        SPECIFIED ELEMENT DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for char. x-ray line',
     &                     ' does not exist !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and Ka1 & Ka2'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
         IEXIST=0
         DO 25 I=1,NELEM
            IF (IATNUM.EQ.ATNUM(I)) IEXIST=1
 25      CONTINUE
         IF (IEXIST.EQ.0) THEN
*        SPECIFIED ELEMENT IS NOT IN THE SAMPLE - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for char. x-ray',
     &                     ' line is not in the element table !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and Ka1 & Ka2'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
*        ALL LINES SHOULD BE TAKEN INTO ACCOUNT
         NIELE=1
         IELE(1)=IATNUM
         NXXCH(1)=10
         DO 45 J=1,10
            IXXCH(1,J)=J
 45      CONTINUE
         GOTO 500
      ENDIF

 80   CONTINUE
      DO 90 I=1,14
         CD(I)='   '
 90   CONTINUE
      IPOS(1)=0
      I=2
      NPOS=2
      IPOS(I)=IP
 105  IST=IPOS(I)+1
      CALL FIND (CBUFF,',',IST,IE,IP,IFND)
      IF (IFND.EQ.1) THEN
         I=I+1
         IPOS(I)=IP
         NPOS=NPOS+1
         GOTO 105
      ENDIF
      NPOS=NPOS+1
      IPOS(NPOS)=IE+1
      NCOMMA=MIN(NPOS,14)
      DO 120 I=1,NCOMMA-1
         IF ((IPOS(I+1)-IPOS(I)).GT.1) THEN
            READ(CBUFF(IPOS(I)+1:IPOS(I+1)-1),'(A)',ERR=120) CD(I)
            CALL CCAP (CD(I),3)
         ENDIF
 120  CONTINUE
      IF ((CD(1)(1:3).EQ.'ALL').OR.(CD(1)(1:1).EQ.'*')) THEN
*        ALL ELEMENTS SHOULD BE TAKEN INTO ACCOUNT
         NIELE=NELEM
         DO 121 I=1,NIELE
            IELE(I)=ATNUM(I)
 121     CONTINUE
      ELSE
         CELEM(1:IEND(CD(1)))=CD(1)
         CALL ELMENT(1,CELEM,IATNUM,CEABR,FRM,FDENS,IERR)
         IF (IERR.NE.0) THEN
*        SPECIFIED ELEMENT DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for char. x-ray line',
     &                     ' does not exist !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and Ka1 & Ka2)'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
         IEXIST=0
         DO 122 I=1,NELEM
            IF (IATNUM.EQ.ATNUM(I)) IEXIST=1
 122     CONTINUE
         IF (IEXIST.EQ.0) THEN
*        SPECIFIED ELEMENT IS NOT IN THE SAMPLE - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for char. x-ray',
     &                     ' line is not in the element table !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and Ka1 & Ka2'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
         NIELE=1
         IELE(1)=IATNUM
      ENDIF
*     WHAT LINE(S) SHOULD BE TAKEN INTO ACCOUNT?
      IF ((CD(2)(1:1).EQ.'A').OR.(CD(2)(1:1).EQ.'*')) THEN
*        ALL LINES OF ELEMENT 1 OR ALL ELEMENTS, RESPECTIVELY
         DO 125 I=1,NIELE
            NXXCH(I)=10
            DO 123 J=1,10
               IXXCH(I,J)=J
 123        CONTINUE
 125     CONTINUE
         GOTO 500
      ENDIF
      DO 127 I=1,NIELE
         NXXCH(I)=0
 127  CONTINUE
      DO 200 K=2,NCOMMA-1
         IF (CD(K)(1:1).EQ.'K') THEN
*           K LINE
            IF (CD(K)(2:2).EQ.'A') THEN
*              K ALPHA LINE
               IF (CD(K)(3:3).EQ.'1') THEN
*                 K ALPHA1 LINE
                  DO 128 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(1,1)).AND.
     &                   (IELE(I).LE.ECLIM(1,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=1
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 1'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 128              CONTINUE
               ELSE IF (CD(K)(3:3).EQ.'2') THEN
*                 K ALPHA2 LINE
                  DO 129 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(2,1)).AND.
     &                   (IELE(I).LE.ECLIM(2,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=2
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 2'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 129              CONTINUE
               ELSE
*                 K ALPHA1 AND ALPHA2 LINES
                  DO 130 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(1,1)).AND.
     &                   (IELE(I).LE.ECLIM(1,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=1
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 1'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
                     IF ((IELE(I).GE.ECLIM(2,1)).AND.
     &                   (IELE(I).LE.ECLIM(2,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=2
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 2'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 130              CONTINUE
               ENDIF
            ELSE IF (CD(K)(2:2).EQ.'B') THEN
*              K BETA LINE
               DO 131 I=1,NIELE
                  IF ((IELE(I).GE.ECLIM(1,1)).AND.
     &                (IELE(I).LE.ECLIM(1,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=3
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 3'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
 131           CONTINUE
            ELSE
*              K ALPHA1, ALPHA2 AND BETA LINES
               DO 132 I=1,NIELE
                  IF ((IELE(I).GE.ECLIM(1,1)).AND.
     &                (IELE(I).LE.ECLIM(1,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=1
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 1'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
                  IF ((IELE(I).GE.ECLIM(2,1)).AND.
     &                (IELE(I).LE.ECLIM(2,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=2
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 2'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
                  IF ((IELE(I).GE.ECLIM(3,1)).AND.
     &                (IELE(I).LE.ECLIM(3,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=3
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 3'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
 132           CONTINUE
            ENDIF
         ELSE IF (CD(K)(1:1).EQ.'L') THEN
*           L LINE
            IF (CD(K)(2:2).EQ.'A') THEN
*              L ALPHA LINE
               IF (CD(K)(3:3).EQ.'1') THEN
*                 L ALPHA1 LINE
                  DO 133 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(4,1)).AND.
     &                   (IELE(I).LE.ECLIM(4,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=4
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 4'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 133              CONTINUE
               ELSE IF (CD(K)(3:3).EQ.'2') THEN
*                 L ALPHA2 LINE
                  DO 134 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(5,1)).AND.
     &                   (IELE(I).LE.ECLIM(5,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=5
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 5'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 134              CONTINUE
               ELSE
*                 L ALPHA1 AND ALPHA2 LINES
                  DO 135 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(4,1)).AND.
     &                   (IELE(I).LE.ECLIM(4,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=4
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 4'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
                     IF ((IELE(I).GE.ECLIM(5,1)).AND.
     &                   (IELE(I).LE.ECLIM(5,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=5
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 5'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 135              CONTINUE
               ENDIF
            ELSE IF (CD(K)(2:2).EQ.'B') THEN
*              L BETA LINE
               IF (CD(K)(3:3).EQ.'1') THEN
*                 L BETA1 LINE
                  DO 136 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(6,1)).AND.
     &                   (IELE(I).LE.ECLIM(6,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=6
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 6'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 136              CONTINUE
               ELSE IF (CD(K)(3:3).EQ.'2') THEN
*                 L BETA2 LINE
                  DO 137 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(7,1)).AND.
     &                   (IELE(I).LE.ECLIM(7,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=7
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 7'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 137              CONTINUE
               ELSE IF (CD(K)(3:3).EQ.'3') THEN
*                 L BETA3 LINE
                  DO 138 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(8,1)).AND.
     &                   (IELE(I).LE.ECLIM(8,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=8
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 8'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 138              CONTINUE
               ELSE
*                 L BETA1, BETA21 AND BETA3 LINES
                  DO 139 I=1,NIELE
                     IF ((IELE(I).GE.ECLIM(6,1)).AND.
     &                   (IELE(I).LE.ECLIM(6,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=6
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 6'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
                     IF ((IELE(I).GE.ECLIM(7,1)).AND.
     &                   (IELE(I).LE.ECLIM(7,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=7
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 7'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
                     IF ((IELE(I).GE.ECLIM(8,1)).AND.
     &                   (IELE(I).LE.ECLIM(8,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=8
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 8'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 139              CONTINUE
               ENDIF
            ELSE
*              L ALPHA1, ALPHA2, BETA1, BETA2 AND BETA3 LINES
               DO 141 I=1,NIELE
                  DO 140 J=1,5
                     IF ((IELE(I).GE.ECLIM(3+J,1)).AND.
     &                   (IELE(I).LE.ECLIM(3+J,2))) THEN
                        NXXCH(I)=NXXCH(I)+1
                        IXXCH(I,NXXCH(I))=3+J
                     ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A,I3)')' Element: ',IELE(I),
     &                                     ' Line: ',3+J
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                     ENDIF
 140              CONTINUE
 141           CONTINUE
            ENDIF
         ELSE IF (CD(K)(1:1).EQ.'M') THEN
            IF (CD(K)(2:2).EQ.'A') THEN
*              THE M ALPHA LINE IS SPECIFIED
               DO 152 I=1,NIELE
                  IF ((IELE(I).GE.ECLIM(9,1)).AND.
     &                (IELE(I).LE.ECLIM(9,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=9
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 9'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
 152           CONTINUE
            ELSE IF (CD(K)(2:2).EQ.'B') THEN
*              THE M BETA LINE IS SPECIFIED
               DO 154 I=1,NIELE
                  IF ((IELE(I).GE.ECLIM(10,1)).AND.
     &                (IELE(I).LE.ECLIM(10,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=10
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 10'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
 154           CONTINUE
            ELSE
*              THE M ALPHA AND BETA LINES ARE SPECIFIED
               DO 167 I=1,NIELE
                  IF ((IELE(I).GE.ECLIM(9,1)).AND.
     &                (IELE(I).LE.ECLIM(9,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=9
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 9'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
                  IF ((IELE(I).GE.ECLIM(10,1)).AND.
     &                (IELE(I).LE.ECLIM(10,2))) THEN
                     NXXCH(I)=NXXCH(I)+1
                     IXXCH(I,NXXCH(I))=10
                  ELSE
*                    NO CHAR. ENERGY AVAILABLE FOR SPECIFIED ELEMENT AND LINE
                        WRITE(6,'(A)')'       '
                        WRITE(6,'(A)')' %%% WARNING %%% '
                        WRITE(6,'(2A)')' There is no char. energy ava',
     &                    'ilable for the specified element and line !'
                        WRITE(6,'(2A)')' (or acceleration voltage ',
     &                    'lower then corresponding edge energy)'
                        WRITE(6,'(A,I3,A)')' Element: ',IELE(I),
     &                                     ' Line: 10'
                        WRITE(6,'(2A)')' This line will not be taken',
     &                                 ' into account !'
                        WRITE(6,'(2A)')' Protests are useless - ',
     &                                 ';-) SESAME'
                  ENDIF
 167           CONTINUE
            ENDIF
         ELSE
*           SPECIFIED LINE DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified line for char. x-rays does',
     &                     ' not exist !'
            WRITE(6,'(2A)')' Default value will be taken (=',
     &                     ' Ka1 & Ka2)'
            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
            DO 180 I=1,NIELE
               IF ((IELE(I).GE.ECLIM(1,1)).AND.
     &             (IELE(I).LE.ECLIM(1,2))) THEN
                  NXXCH(I)=NXXCH(I)+1
                  IXXCH(I,NXXCH(I))=1
               ELSE

               ENDIF
               IF ((IELE(I).GE.ECLIM(2,1)).AND.
     &             (IELE(I).LE.ECLIM(2,2))) THEN
                  NXXCH(I)=NXXCH(I)+1
                  IXXCH(I,NXXCH(I))=2
               ELSE

               ENDIF
 180        CONTINUE
         ENDIF
 200  CONTINUE

 500  CONTINUE
*     UPDATE XCHARLINE
      IF (LALL) THEN
*        ALL ELEMENTS AND ALL LINES
         DO 540 I=1,NELEM
            DO 530 J=1,10
               XCHARLINE(J,I)=J
 530        CONTINUE
 540     CONTINUE
      ELSE
*        MERGE WITH ALREADY EXISTING XCHARLINE SPECIFICATION
         DO 650 J=1,NIELE 
            DO 630 I=1,NELEM
               IF (IELE(J).EQ.ATNUM(I)) INDX=I
 630        CONTINUE
*           UPDATE NDUMMY(8,9,...) [ELEMENT,SHELL,SHELL,...]
            NDUMMY(8)=INDX
            NDUMMY(20)=NXXCH(J)
            DO 635 I=1,NXXCH(J)
               NDUMMY(8+I)=IXXCH(J,I)
 635        CONTINUE
            DO 640 I=1,MAXXCHAR
               IXXCH(J,NXXCH(J)+I)=XCHARLINE(I,INDX)
 640        CONTINUE
            NXXCH(J)=NXXCH(J)+MAXXCHAR
 650     CONTINUE
*        ELIMINATE MULTIPLE SPECIFICATION OF LINES
         DO 690 K=1,NIELE
            DO 680 I=1,NXXCH(K)
               ICOMP=IXXCH(K,I)
               DO 670 J=1,NXXCH(K)
                  IF ((ICOMP.EQ.IXXCH(K,J)).AND.(I.NE.J).AND.
     &                (IXXCH(K,J).NE.0)) IXXCH(K,J)=0
 670           CONTINUE
 680        CONTINUE
 690     CONTINUE
*        SORT IXXCH
         DO 730 K=1,NIELE
            NCNT=0
            DO 720 I=1,MAXXCHAR
               DO 710 J=1,NXXCH(K)
                  IF (IXXCH(K,J).EQ.I) THEN
                     NCNT=NCNT+1
                     IXXCH(0,NCNT)=IXXCH(K,J)
                  ENDIF
 710           CONTINUE
 720        CONTINUE
            NXXCH(K)=NCNT
            DO 725 J=1,NXXCH(K)
               IXXCH(K,J)=IXXCH(0,J)
 725        CONTINUE
            DO 726 J=NXXCH(K)+1,3*MAXXCHAR
               IXXCH(K,J)=0
 726        CONTINUE
 730     CONTINUE
*        UPDATE XCHARLINE
         DO 770 I=1,NELEM
            DO 760 J=1,NIELE
               IF (IELE(J).EQ.ATNUM(I)) THEN
                  DO 750 K=1,NXXCH(J)
                     XCHARLINE(K,I)=IXXCH(J,K)
 750              CONTINUE
               ENDIF
 760        CONTINUE
 770     CONTINUE
      ENDIF

 900  RETURN
      END

**************************************************************************
*last modification: 09-Dec-1996

      SUBROUTINE UPDXCH (IFLAG)
 
****  UPDXCH - UPDATE LINE TABLE
*
**    ARGUMENTS
*
*     IFLAG - DUMMY INTEGER
*
**    AUTHOR: C. Schiebl 13-Jan-1995
*             CEC-Vienna
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'
      INCLUDE 'DMA.INC'

**    START OF EXECUTABLE CODE
*     ************************

      NENTRY=NDMAXCONT+NDMAXCHAR-NDMAPTOB

*     INITIALIZE LINE TABLE ARRAYS
      PROPION=0
      MXINDX=NDMAREG*NDMAXCHAR
      IF (MXINDX.GT.0)
     &   CALL CHKBND(25,MXINDX,'UPDXCH    ','XCHAROFREG')
      MXINDX=(NDMASCRN+NDMAPS)*NDMAXCHAR
      IF (MXINDX.GT.0)
     &   CALL CHKBND(21,MXINDX,'UPDXCH    ','SCROFXCG/XCD')
      MXINDX=NDMAPM*NDMAXCHAR
      IF (MXINDX.GT.0)
     &   CALL CHKBND(22,MXINDX,'UPDXCH    ','PSMINOFXCG/XCD')
      MXINDX=NDMAHIST*NDMAXCHAR
      IF (MXINDX.GT.0)
     &   CALL CHKBND(23,MXINDX,'UPDXCH    ','XCHIS/XDETXCHIS')
      MXINDX=NDMAXDET*NDMAXCHAR
      IF (MXINDX.GT.0)
     &   CALL CHKBND(20,MXINDX,'UPDXCH    ','XDETOFXCHAR/....')
      DO 20 I=1,NDMAREG
         NXCHAROFREG(I)=0
         DO 10 J=1,NDMAXCHAR
            IXINDX=(J-1)*NDMAREG+I
CS            XCHAROFREG(I,J)=0
            XCHAROFREG(IXINDX)=0
 10      CONTINUE
 20   CONTINUE
      DO 30 I=1,NDMAXCHAR
         ELEMOFXCHAR(I)=0
         XCHAROFXCHAR(I)=0
         ENEROFXCHAR(I)=0
         SHELLOFXCHAR(I)=0
         XGENOFXCHAR(I)=0
         IXCHAROFXCONT(I)=0
         NSCROFXCG(I)=0
         NSCROFXCD(I)=0
         DO 21 J=1,NDMASCRN+NDMAPS
            IXINDX=(I-1)*(NDMASCRN+NDMAPS)+J
CS            SCROFXCG(I,J)=0
CS            SCROFXCD(I,J)=0
            SCROFXCG(IXINDX)=0
            SCROFXCD(IXINDX)=0
 21      CONTINUE
         NPSMINOFXCG(I)=0
         NPSMINOFXCD(I)=0
         DO 22 J=1,NDMAPM
            IXINDX=(I-1)*NDMAPM+J
CS            PSMINOFXCG(I,J)=0
CS            PSMINOFXCD(I,J)=0
            PSMINOFXCG(IXINDX)=0
            PSMINOFXCD(IXINDX)=0
 22      CONTINUE
CS         DO 23 K=0,MAXXDET
         DO 23 K=1,NDMAXDET
            IXINDX=(I-1)*NDMAXDET+K
CS            XDETOFXCHAR(K,I)=0
CS            XPEAKTOBACK(K,I)=0
CS            XFIORIFACT(K,I)=0
            XDETOFXCHAR(IXINDX)=0
            XPEAKTOBACK(IXINDX)=0
            XFIORIFACT(IXINDX)=0
 23      CONTINUE
         FYIELDOFXCHAR(I)=1
         FRELINTOFXCHAR(I)=1
         NXCHIS(I)=0
         DO 25 J=1,NDMAHIST
            IXINDX=(I-1)*NDMAHIST+J
CS            XCHIS(I,J)=0
CS            XDETXCHIS(I,J)=0
            XCHIS(IXINDX)=0
            XDETXCHIS(IXINDX)=0
            XCHISWINFRAC(IXINDX)=1
 25      CONTINUE
 30   CONTINUE
      MXINDX=NDMAXDET*NENTRY
      IF (MXINDX.GT.0) THEN
         CALL CHKBND(13,MXINDX,'UPDXCH    ','FWHMOFXCHAR')
         CALL CHKBND(13,MXINDX,'UPDXCH    ','XDETEFFOFXCHAR')
      ENDIF
      DO 40 I=1,NENTRY
         DO 35 K=0,NDMAXDET-1
            INDX=NDMAXDET*(I-1)+(K+1)
            FWHMOFXCHAR(INDX)=0
            XDETEFFOFXCHAR(INDX)=1
 35      CONTINUE
 40   CONTINUE
      DO 45 I=1,MAXSCR+MAXPST
         ILOGXCGTOSCR(I)=0
         ILOGXCDTOSCR(I)=0
 45   CONTINUE

*     UPDATE LINE TABLE ARRAYS
      NXCHAR=0
      DO 100 IREG=1,NREG
         NXCHAROFREG(IREG)=0
         DO 90 IEL1=1,NELEM
            DO 80 IEL2=1,NELE(IREG)
               IF (IEL1.EQ.ELEM(IREG,IEL2)) THEN
                  DO 70 I=1,MAXXCHAR
                     IF (XCHARLINE(I,IEL1).NE.0) THEN
                        IENTRY=0
                        DO 60 J=1,NXCHAR
C                           IF ((ELEMOFXCHAR(J).EQ.ATNUM(IEL1)).AND.
                           IF ((ELEMOFXCHAR(J).EQ.IEL1).AND.
     &                         (XCHAROFXCHAR(J).EQ.XCHARLINE(I,IEL1))
     &                         .AND.(XCHAROFXCHAR(J).NE.0)) THEN
                              IENTRY=J
                           ENDIF
 60                     CONTINUE
                        NXCHAROFREG(IREG)=NXCHAROFREG(IREG)+1
                        IF (IENTRY.EQ.0) THEN
*                          NEW ENTRY
                           IF (NXCHAR.LT.NDMAXCHAR) THEN
                              NXCHAR=NXCHAR+1
C                              ELEMOFXCHAR(NXCHAR)=ATNUM(IEL1)
                              ELEMOFXCHAR(NXCHAR)=IEL1
                              XCHAROFXCHAR(NXCHAR)=XCHARLINE(I,IEL1)
                              IXINDX=(NXCHAROFREG(IREG)-1)*NDMAREG+
     &                               IREG
                              CALL CHKBND(25,IXINDX,'UPDXCH    ',
     &                                    'XCHAROFREG')
CS                              XCHAROFREG(IREG,NXCHAROFREG(IREG))=NXCHAR
                              XCHAROFREG(IXINDX)=NXCHAR
                           ENDIF
                        ELSE
*                          OLD ENTRY
                           IXINDX=(NXCHAROFREG(IREG)-1)*NDMAREG+
     &                            IREG
                           CALL CHKBND(25,IXINDX,'UPDXCH    ',
     &                                 'XCHAROFREG')
CS                           XCHAROFREG(IREG,NXCHAROFREG(IREG))=IENTRY
                           XCHAROFREG(IXINDX)=IENTRY
                        ENDIF
                     ENDIF
 70               CONTINUE
               ENDIF
 80         CONTINUE
 90      CONTINUE
 100  CONTINUE

      DO 150 I=1,NXCHAR
         NXCHIS(I)=XCIODIS41(ELEMOFXCHAR(I),XCHAROFXCHAR(I))
         DO 130 J=1,NXCHIS(I)
           IXINDX=(I-1)*NDMAHIST+J
           CALL CHKBND(23,IXINDX,'UPDXCH    ','XCHIS/XDETXCHIS')
CS           XCHIS(I,J)=XCIODIS4(ELEMOFXCHAR(I),XCHAROFXCHAR(I),J,1)
CS           XDETXCHIS(I,J)=XCIODIS4(ELEMOFXCHAR(I),XCHAROFXCHAR(I),J,2)
           XCHIS(IXINDX)=XCIODIS4(ELEMOFXCHAR(I),
     &                            XCHAROFXCHAR(I),J,1)
           XDETXCHIS(IXINDX)=XCIODIS4(ELEMOFXCHAR(I),
     &                                XCHAROFXCHAR(I),J,2)
 130     CONTINUE
         NSCROFXCG(I)=XCIODIS21(ELEMOFXCHAR(I),XCHAROFXCHAR(I),2)
         NSCROFXCD(I)=XCIODIS21(ELEMOFXCHAR(I),XCHAROFXCHAR(I),3)
         DO 135 J=1,NDMASCRN+NDMAPS
           IXINDX=(I-1)*(NDMASCRN+NDMAPS)+J
           CALL CHKBND(21,IXINDX,'UPDXCH    ','SCROFXCG/XCD/...')
CS           SCROFXCG(I,J)=XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),2,J)
CS           SCROFXCD(I,J)=XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),3,J)
CS           XDETSCROFXCD(I,J)=
CS     &                   XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),4,J)
           SCROFXCG(IXINDX)=XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),2,J)
           SCROFXCD(IXINDX)=XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),3,J)
           XDETSCROFXCD(IXINDX)=
     &                   XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),4,J)
           IF (XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),6,J).GT.0) THEN
            ILOGXCGTOSCR(XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),2,J))=
     &                   XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),6,J)
           ENDIF
           IF (XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),7,J).GT.0) THEN
            ILOGXCDTOSCR(XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),3,J))=
     &                   XCIODIS2(ELEMOFXCHAR(I),XCHAROFXCHAR(I),7,J)
           ENDIF
 135     CONTINUE
         NPSMINOFXCG(I)=XCIODIS31(ELEMOFXCHAR(I),XCHAROFXCHAR(I),2)
         NPSMINOFXCD(I)=XCIODIS31(ELEMOFXCHAR(I),XCHAROFXCHAR(I),3)
         DO 140 J=1,NDMAPM
           IXINDX=(I-1)*NDMAPM+J
           CALL CHKBND(22,IXINDX,'UPDXCH    ','PSMINOFXCG/XCD')
CS           PSMINOFXCG(I,J)=
CS     &                   XCIODIS3(ELEMOFXCHAR(I),XCHAROFXCHAR(I),2,J)
CS           PSMINOFXCD(I,J)=
CS     &                   XCIODIS3(ELEMOFXCHAR(I),XCHAROFXCHAR(I),3,J)
CS           XDETPSMINOFXCD(I,J)=
CS     &                   XCIODIS3(ELEMOFXCHAR(I),XCHAROFXCHAR(I),4,J)
           PSMINOFXCG(IXINDX)=
     &                   XCIODIS3(ELEMOFXCHAR(I),XCHAROFXCHAR(I),2,J)
           PSMINOFXCD(IXINDX)=
     &                   XCIODIS3(ELEMOFXCHAR(I),XCHAROFXCHAR(I),3,J)
           XDETPSMINOFXCD(IXINDX)=
     &                   XCIODIS3(ELEMOFXCHAR(I),XCHAROFXCHAR(I),4,J)
 140     CONTINUE
 150  CONTINUE


      RETURN
      END

**************************************************************************

* last modification: CS 31-Jan-1995

      SUBROUTINE CHKAUG (IMODE,NDUMMY,CBUFF,ICLEN)

***   CHKAUG - CHECKS THE SPECIFICATION OF ELEMENT AND AUGER PEAK
*              AND UPDATES THE 'AUGERPEAK' ARRAY
*
*     SUPPORTED AUGER PEAKS:
*	       KLL=1, LMM=2, MNN=3
*
**    ARGUMENTS
*
*     IMODE  -  DUMMY FLAG
*     CBUFF  -  STRING CONTAINING ELEMENT AND AUGER PEAK INFORMATION
*     ICLEN  -  ACTUAL LENGTH OF CBUFF
*     
**    AUTHOR:  C. Schiebl  31-JAN-1995
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'

      DIMENSION NDUMMY(20)
      DIMENSION IPOS(5),NXAUG(MAXELE)
      DIMENSION IXAUG(0:MAXELE,3*MXAUGP),IELE(MAXELE)

      CHARACTER*70 CELEM
CS      CHARACTER*50 CBUFF
      CHARACTER*(*) CBUFF
      CHARACTER*40 CDUMMY
      CHARACTER*10 CBL
      CHARACTER*3  CD(14)
      CHARACTER*2  CEABR

      LOGICAL LALL

      PARAMETER (CBL='          ')

**    START OF EXECUTABLE CODE
*     ************************

CS      DO 1 I=ICLEN+1,50
CS         CBUFF(I:I)=' '
CS 1    CONTINUE
      CDUMMY=CBL//CBL//CBL//CBL
      CELEM=CBL//CBL//CBL//CBL//CBL//CBL//CBL
      DO 2 I=1,5
         IPOS(I)=0
 2    CONTINUE
      DO 7 I=1,MAXELE
         NXAUG(I)=0
         IELE(I)=0
         DO 5 J=1,3*MXAUGP
            IXAUG(I,J)=0
 5       CONTINUE
 7    CONTINUE
      DO 8 I=1,3*MXAUGP
         IXAUG(0,I)=0
 8    CONTINUE

*     DEFAULT: (FIRST ELEMENT, KLL PEAK)
CS    DO NOT TAKE ANY DEFAULTS
CS      NXAUG(1)=1
CS      IXAUG(1,1)=1
CS      NIELE=1
CS      IELE(1)=ATNUM(1)
      NIELE=0

      IE=IEND(CBUFF)
      CALL CCAP (CBUFF(1:IE),IE)

      IF (CBUFF(1:3).EQ.'DEF') THEN
*        DEFAULT SHOULD BE TAKEN, I.E. FIRST ELEMENT AND KLL PEAK
         GOTO 500
      ENDIF

      CALL FIND (CBUFF,',',1,IE,IP,IFND)

      IF (IFND.EQ.0) THEN
*        ONLY ELEMENT IS SPECIFIED
         NC=MIN(2,IE)
         CELEM(1:NC)=CBUFF(1:NC)
         CALL ELMENT(1,CELEM,IATNUM,CEABR,FRM,FDENS,IERR)
         IF (IERR.NE.0) THEN
*        SPECIFIED ELEMENT DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for Auger peak',
     &                     ' does not exist !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and KLL peak'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
         IEXIST=0
         DO 25 I=1,NELEM
            IF (IATNUM.EQ.ATNUM(I)) IEXIST=1
 25      CONTINUE
         IF (IEXIST.EQ.0) THEN
*        SPECIFIED ELEMENT IS NOT IN THE SAMPLE - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified element for Auger peak',
     &                     ' is not in the element table !'
CS            WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                     ' element in the table and KLL peak'
CS            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS            GOTO 500
            GOTO 900
         ENDIF
*        KLL PEAK SHOULD BE TAKEN INTO ACCOUNT
         NIELE=1
         IELE(1)=IATNUM
         NXAUG(1)=1
         IXAUG(1,J)=1
         GOTO 500
      ENDIF

      DO 90 I=1,5
         CD(I)='   '
 90   CONTINUE
      IPOS(1)=0
      I=2
      NPOS=2
      IPOS(I)=IP
 105  IST=IPOS(I)+1
      CALL FIND (CBUFF,',',IST,IE,IP,IFND)
      IF (IFND.EQ.1) THEN
         I=I+1
         IPOS(I)=IP
         NPOS=NPOS+1
         GOTO 105
      ENDIF
      NPOS=NPOS+1
      IPOS(NPOS)=IE+1
      NCOMMA=MIN(NPOS,3)
      DO 120 I=1,NCOMMA-1
         IF ((IPOS(I+1)-IPOS(I)).GT.1) THEN
            READ(CBUFF(IPOS(I)+1:IPOS(I+1)-1),'(A)',ERR=120) CD(I)
            CALL CCAP (CD(I),3)
         ENDIF
 120  CONTINUE
      CELEM(1:IEND(CD(1)))=CD(1)
      CALL ELMENT(1,CELEM,IATNUM,CEABR,FRM,FDENS,IERR)
      IF (IERR.NE.0) THEN
*     SPECIFIED ELEMENT DOES NOT EXIST - TAKE DEFAULT
         WRITE(6,'(A)')'       '
         WRITE(6,'(A)')' %%% WARNING %%% '
         WRITE(6,'(2A)')' Specified element for Auger peak',
     &                  ' does not exist !'
CS         WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                  ' element in the table and KLL peak)'
CS         WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS         GOTO 500
         GOTO 900
      ENDIF
      IEXIST=0
      DO 122 I=1,NELEM
         IF (IATNUM.EQ.ATNUM(I)) IEXIST=1
 122  CONTINUE
      IF (IEXIST.EQ.0) THEN
*     SPECIFIED ELEMENT IS NOT IN THE SAMPLE - TAKE DEFAULT
         WRITE(6,'(A)')'       '
         WRITE(6,'(A)')' %%% WARNING %%% '
         WRITE(6,'(2A)')' Specified element for Auger peak',
     &                  ' is not in the element table !'
CS         WRITE(6,'(2A)')' Default value will be taken (=first',
CS     &                  ' element in the table and KLL peak'
CS         WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
CS         GOTO 500
         GOTO 900
      ENDIF
      NIELE=1
      IELE(1)=IATNUM
*     WHAT PEAK SHOULD BE TAKEN INTO ACCOUNT?
      DO 127 I=1,NIELE
         NXAUG(I)=0
 127  CONTINUE
      DO 200 K=2,NCOMMA-1
         IF (CD(K)(1:1).EQ.'K') THEN
*           KLL PEAK
            DO 128 I=1,NIELE
               NXAUG(I)=NXAUG(I)+1
               IXAUG(I,NXAUG(I))=1
 128        CONTINUE
         ELSE IF (CD(K)(1:1).EQ.'L') THEN
*           LMM PEAK
            DO 133 I=1,NIELE
               NXAUG(I)=NXAUG(I)+1
               IXAUG(I,NXAUG(I))=2
 133        CONTINUE
         ELSE IF (CD(K)(1:1).EQ.'M') THEN
*           MNN PEAK
            DO 152 I=1,NIELE
               NXAUG(I)=NXAUG(I)+1
               IXAUG(I,NXAUG(I))=3
 152        CONTINUE
         ELSE
*           SPECIFIED PEAK DOES NOT EXIST - TAKE DEFAULT
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% WARNING %%% '
            WRITE(6,'(2A)')' Specified Auger peak does',
     &                     ' not exist !'
            WRITE(6,'(2A)')' Default value will be taken (=',
     &                     ' KLL peak)'
            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
            DO 180 I=1,NIELE
               NXAUG(I)=NXAUG(I)+1
               IXAUG(I,NXAUG(I))=1
 180        CONTINUE
         ENDIF
 200  CONTINUE

 500  CONTINUE
*     UPDATE AUGERPEAK
*     MERGE WITH ALREADY EXISTING AUGERPEAK SPECIFICATION
      DO 650 J=1,NIELE 
         DO 630 I=1,NELEM
            IF (IELE(J).EQ.ATNUM(I)) INDX=I
 630     CONTINUE
*           UPDATE NDUMMY(8,9,...) [ELEMENT,AUGER PEAK]
            NDUMMY(8)=INDX
            NDUMMY(20)=NXAUG(J)
            DO 635 I=1,NXAUG(J)
               NDUMMY(8+I)=IXAUG(J,I)
 635        CONTINUE
         DO 640 I=1,MXAUGP
            IXAUG(J,NXAUG(J)+I)=AUGERPEAK(I,INDX)
 640     CONTINUE
         NXAUG(J)=NXAUG(J)+MXAUGP
 650  CONTINUE
*     ELIMINATE MULTIPLE SPECIFICATION OF PEAKS
      DO 690 K=1,NIELE
         DO 680 I=1,NXAUG(K)
            ICOMP=IXAUG(K,I)
            DO 670 J=1,NXAUG(K)
               IF ((ICOMP.EQ.IXAUG(K,J)).AND.(I.NE.J).AND.
     &             (IXAUG(K,J).NE.0)) IXAUG(K,J)=0
 670        CONTINUE
 680     CONTINUE
 690  CONTINUE
*     SORT IXAUG
      DO 730 K=1,NIELE
         NCNT=0
         DO 720 I=1,MXAUGP
            DO 710 J=1,NXAUG(K)
               IF (IXAUG(K,J).EQ.I) THEN
                  NCNT=NCNT+1
                  IXAUG(0,NCNT)=IXAUG(K,J)
               ENDIF
 710        CONTINUE
 720     CONTINUE
         NXAUG(K)=NCNT
         DO 725 J=1,NXAUG(K)
            IXAUG(K,J)=IXAUG(0,J)
 725     CONTINUE
         DO 726 J=NXAUG(K)+1,3*MXAUGP
            IXAUG(K,J)=0
 726     CONTINUE
 730  CONTINUE
*     UPDATE AUGERPEAK
      DO 770 I=1,NELEM
         DO 760 J=1,NIELE
            IF (IELE(J).EQ.ATNUM(I)) THEN
               DO 750 K=1,NXAUG(J)
                  AUGERPEAK(K,I)=IXAUG(J,K)
 750           CONTINUE
            ENDIF
 760     CONTINUE
 770  CONTINUE

 900  RETURN
      END

**************************************************************************

*last modification: CS 31-Jan-1995

      SUBROUTINE UPDAUG (IFLAG)
 
****  UPDAUG - UPDATE AUGER TABLE
*
**    ARGUMENTS
*
*     IFLAG - DUMMY INTEGER
*
**    AUTHOR: C. Schiebl 31-Jan-1995
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'

**    START OF EXECUTABLE CODE
*     ************************

*     INITIALIZE LINE TABLE ARRAYS
      DO 20 I=1,MAXAUGER
         ELEMOFAUGER(I)=0
         LINEOFAUGER(I)=0
         NAETLOC(I)=0
         NAGDLOC(I)=0
         NADDLOC(I)=0
         DO 8 J=1,MAXSCR+MAXPST
            AETLOC(I,J)=0
            AGDLOC(I,J)=0
            ADDLOC(I,J)=0
 8       CONTINUE
         NAE_HIS(I)=0
         DO 12 J=1,MAXHIS
            AE_HIS(I,J)=0
 12      CONTINUE
 20   CONTINUE

      DO 40 I=1,MAXELE
         DO 30 J=1,MAXSHELL
            AUGERSHELL(I,J)=0
 30      CONTINUE
 40   CONTINUE

*     UPDATE AUGER TABLE ARRAYS
      NAUGER=0
      DO 100 IELE=1,NELEM
         DO 50 J=1,MXAUGP
            IF (AUGERPEAK(J,IELE).GT.0) THEN
               NAUGER=NAUGER+1
               ELEMOFAUGER(NAUGER)=IELE
               LINEOFAUGER(NAUGER)=AUGERPEAK(J,IELE)
            ENDIF
 50      CONTINUE
 100  CONTINUE

      DO 200 I=1,NAUGER
         IELMNT=ELEMOFAUGER(I)
         IAUGPK=LINEOFAUGER(I)
         NAE_HIS(I)=XCIODIS8(IELMNT,IAUGPK)
         DO 110 J=1,NAE_HIS(I)
            AE_HIS(I,J)=XCIODIS7(IELMNT,IAUGPK,J)
 110     CONTINUE
         NAETLOC(I)=XCIODIS6(IELMNT,IAUGPK,3)
         DO 120 J=1,NAETLOC(I)
            AETLOC(I,J)=XCIODIS5(IELMNT,IAUGPK,J,3)
 120     CONTINUE
         NAGDLOC(I)=XCIODIS6(IELMNT,IAUGPK,1)
         DO 130 J=1,NAGDLOC(I)
            AGDLOC(I,J)=XCIODIS5(IELMNT,IAUGPK,J,1)
 130     CONTINUE
         NADDLOC(I)=XCIODIS6(IELMNT,IAUGPK,2)
         DO 140 J=1,NADDLOC(I)
            ADDLOC(I,J)=XCIODIS5(IELMNT,IAUGPK,J,2)
            ADDLCED(I,J)=XCIODIS5(IELMNT,IAUGPK,J,4)
 140     CONTINUE
         IF (LINEOFAUGER(I).EQ.1) THEN
            AUGERSHELL(ELEMOFAUGER(I),1)=I
         ELSE IF (LINEOFAUGER(I).EQ.2) THEN
            AUGERSHELL(ELEMOFAUGER(I),2)=I
            AUGERSHELL(ELEMOFAUGER(I),3)=I
            AUGERSHELL(ELEMOFAUGER(I),4)=I
         ELSE IF (LINEOFAUGER(I).EQ.3) THEN
            AUGERSHELL(ELEMOFAUGER(I),5)=I
            AUGERSHELL(ELEMOFAUGER(I),6)=I
            AUGERSHELL(ELEMOFAUGER(I),7)=I
            AUGERSHELL(ELEMOFAUGER(I),8)=I
            AUGERSHELL(ELEMOFAUGER(I),9)=I
         ENDIF
 200  CONTINUE

      RETURN
      END

**************************************************************************
 
      SUBROUTINE CHKATO
 
****  CHKATO - PARSES ATOM RECORD IN &TARGET NAMELIST
*
**    ARGUMENTS
*
*     CBUFF - ATOM (&TARGET NAMELIST) CHARACTER STRING
*
**    AUTHOR: C. Schiebl 17-Jan-1995
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'

      CHARACTER*70 BUFFER
      CHARACTER*10 CBL
      CHARACTER*2  EABR
      DIMENSION IPOS(MAXELE+2)

      DATA CBL /'          '/

**    START OF EXECUTABLE CODE
*     ************************

      DO 10 I=1,MAXELE
         CATOM(I)='  '
 10   CONTINUE

      IE=IEND(ATOM)
      IPOS(1)=0
      I=1
      NPOS=1
 20   IST=IPOS(I)+1
      CALL FIND (ATOM,',',IST,IE,IP,IFND)
      IF (IFND.EQ.1) THEN
         I=I+1
         IPOS(I)=IP
         NPOS=NPOS+1
         GOTO 20
      ENDIF
      NPOS=NPOS+1
      IPOS(NPOS)=IE+1
      NCOMMA=MIN(MAXELE+2,NPOS)
      DO 50 I=1,NCOMMA-1
         IF ((IPOS(I+1)-IPOS(I)).GT.1) THEN
            READ(ATOM(IPOS(I)+1:IPOS(I+1)-1),'(A)',ERR=50) CATOM(I)
            CALL CCAP (CATOM(I),IEND(CATOM(I)))
         ENDIF
 50   CONTINUE

      NATOM=0
      DO 95 I=1,N2M
         BUFFER=CBL//CBL//CBL//CBL//CBL//CBL//CBL
         BUFFER(1:IEND(CATOM(I)))=CATOM(I)(1:IEND(CATOM(I)))
         IF (BUFFER(1:1).EQ.' ') THEN
            IATNUM=0
            GOTO 90
         ELSE
            CALL ELMENT(1,BUFFER,IATNUM,EABR,FRM,FDENS,IERR)
            IF (IERR.NE.0) THEN
               WRITE(6,'(A)')'       '
               WRITE(6,'(A)')' %%% ERROR %%% '
               WRITE(6,'(2A)')' Invalid Chemical Abbreviation for the',
     &                        ' Specification of the Element!'
               WRITE(6,'(A)')' In ATOM of the &TARGET namelist !'
               WRITE(6,'(2A)')' SESAME will terminate -',
     &                        ' check input deck !'
               WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
               GOTO 923
            ENDIF
            NATOM=NATOM+1
         ENDIF
 90      ANUM(I)=IATNUM
 95   CONTINUE

      RETURN

 923  STOP '&TARGET: NOT A VALID CHEMICAL ABBREVIATION'

      END
*************************************************************************
      SUBROUTINE CHKMAT(AFRAC)
 
****  CHKATO - PARSES ATMAT RECORD IN &TARGET NAMELIST
*
**    ARGUMENTS
*
*     CBUFF - ATOM (&TARGET NAMELIST) CHARACTER STRING
*
**    AUTHOR: H.WAGNER 11-JUL-95, 32deg celsius
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'

      DIMENSION AFRAC(N2M,NRM)
      CHARACTER*10 CBL
      CHARACTER*70 CBUFF,CMAT
      CHARACTER*2  CAT

      DATA CBL /'          '/
 
**    START OF EXECUTABLE CODE
*     ************************

      NATOM=0
      DO 10 I=1,MAXELE
         CATOM(I)='  '
 10   CONTINUE

      DO 100 IR=1,NR
       CMAT = ATMAT(IR)
       IE=IEND(CMAT)
       ICP = 1

20     CONTINUE

       CALL FIND(CMAT,',',ICP,IE,IP,IFND)
       IF (IFND.EQ.1) THEN
         IEE = IP-1
       ELSE
         IEE = IE
       ENDIF

       CAT(1:1)=CMAT(ICP:ICP)
       CAT(2:2)=' '
       ICP=ICP+1
       IF (ICP .LE. IEE) THEN
         IF (LGE(CMAT(ICP:ICP),'0') .AND.
     .       LLE(CMAT(ICP:ICP),'9'))THEN
c          one character element abbreviation
           CAT(2:2)=' '
         ELSE
           CAT(2:2)=CMAT(ICP:ICP)
           ICP=ICP+1
         ENDIF
       ENDIF

       CALL CCAP(CAT,2)
       CBUFF = CBL//CBL//CBL//CBL//CBL//CBL//CBL
       CBUFF(1:2) = CAT  
       CALL ELMENT(1,CBUFF,IATNUM,CAT,FRM,FDENS,IERR)

       IF (IERR.NE.0) THEN
               WRITE(6,'(A)')'       '
               WRITE(6,'(A)')' %%% ERROR %%% '
               WRITE(6,'(2A)')' Invalid Chemical Abbreviation for the',
     &                        ' Specification of the Element!'
               WRITE(6,'(A)')' In ATOM of the &TARGET namelist !'
               WRITE(6,'(2A)')' SESAME will terminate -',
     &                        ' check input deck !'
               WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
C      	       GOTO 923
       ENDIF

C   CHECK IF ELEMENT IS ALREADY TABULATED
       IATTAB = 0
       DO 30 I=1,NATOM
         IF (ANUM(I).EQ.IATNUM) IATTAB = I
30     CONTINUE

       IF (IATTAB.EQ.0) THEN
C         ADD TO TABLE
          NATOM=NATOM+1
          ANUM(NATOM)=IATNUM
          IATTAB = NATOM
       ENDIF
C      DETERMINE ATOMIC FRACTION

       IF (ICP .GT. IEE) THEN
         FRAC = 1.
       ELSE
         READ(CMAT(ICP:IEE),*) FRAC
       ENDIF

       AFRAC(IATTAB,IR) = FRAC

       IF (IEE+2 .LE. IE) THEN
         ICP = IEE+2
         GOTO 20
       ENDIF

100   CONTINUE

      RETURN

cs 923  STOP '&TARGET: NOT A VALID CHEMICAL ABBREVIATION'

      END

**************************************************************************

      SUBROUTINE AEINTV (EEMIN,EEMAX,IHELP)

***   AEINTV - ADDS ALL AUGER PEAKS WITH ENERGIES BETWEEN
*              EEMIN AND EEMAX TO AUGERPEAK()
*
**    ARGUMENTS
*
*     EEMIN - MINIMUM OF ENERGY INTERVALL
*     EEMAX - MAXIMUM OF ENERGY INTERVALL
*     
**    AUTHOR:  C. Schiebl  22-FEB-1995
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

      LOGICAL LALREA
      INTEGER IHELP(MAXELE,MXAUGP)

**    START OF EXECUTABLE CODE
*     ************************

      DO 50 I=1,NELEM
         DO 40 J=1,MXAUGP
            IHELP(I,J)=0
            IF ((AUGPKENER(J,I).GE.EEMIN).AND.
     &          (AUGPKENER(J,I).LE.EEMAX)) THEN
               IHELP(I,J)=1
               LALREA=.FALSE.
               IFLAG=0
               DO 30 K=1,MXAUGP
                  IF (AUGERPEAK(K,I).GT.0) IFLAG=K
                  IF (AUGERPEAK(K,I).EQ.J) THEN
*                    THIS AUGER PEAK IS ALREADY TAKEN INTO ACCOUNT
                     LALREA=.TRUE.
                  ENDIF
 30            CONTINUE
               IF (.NOT.LALREA) THEN
                  AUGERPEAK(IFLAG+1,I)=J
               ENDIF
            ENDIF
 40      CONTINUE
 50   CONTINUE

      RETURN
      END

******************************************************************************
* last modification: CS 5-Oct-1995

      SUBROUTINE TRNTAB ()

***   TRNTAB - TRANSFORMS TABLE (SHELL AND XCHAR) INFORMATION TO APPROPRIATE
*              OUTPUT INFORMATION NEEDED FOR OUTPUT OF LOCATION DEPENDENT
*              DISTRIBUTIONS (IOD, XGD, XDD)
*
**    ARGUMENTS
*
*     NONE
*
**    AUTHOR:  C. Schiebl  5-OCT-1995
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'COMMON.INC'
CS      INCLUDE 'HIST.INC'
CS      INCLUDE 'OUTPUT.INC'
      INCLUDE 'DMA.INC'


***   START OF EXECUTABLE CODE:
*
*     FOR SCREEN OUTPUT:

*     FOR IOD:
      DO 102 I=1,MAXSCR+MAXPST
         NIONTOSCR(I)=0
 102  CONTINUE
      MXIONSCR=0
      DO 110 ISH=1,NSHELL
         INS=NSCROFION(ISH)
         DO 105 ISC=1,INS
            INDX=(ISH-1)*(NDMASCRN+NDMAPS)+ISC
            CALL CHKBND(16,INDX,'TRNTAB    ','SCROFION')
CS            IS=SCROFION(ISH,ISC)
            IS=SCROFION(INDX)
            IF (IS.GT.MXIONSCR) MXIONSCR=IS
            IALRDY=0
            DO 103 I=1,NIONTOSCR(IS)
               INDX=(I-1)*(NDMASCRN+NDMAPS)+IS
               CALL CHKBND(16,INDX,'TRNTAB    ','IONTOSCR')
CS               IF (IONTOSCR(IS,I).EQ.ISH) IALRDY=1
               IF (IONTOSCR(INDX).EQ.ISH) IALRDY=1
 103        CONTINUE
            IF (IALRDY.EQ.0) THEN
               NIONTOSCR(IS)=NIONTOSCR(IS)+1
               INDX=(NIONTOSCR(IS)-1)+IS
               CALL CHKBND(16,INDX,'TRNTAB    ','IONTOSCR')
CS               IONTOSCR(IS,NIONTOSCR(IS))=ISH
               IONTOSCR(INDX)=ISH
            ENDIF
 105     CONTINUE
 110  CONTINUE

*     FOR XGD:
      DO 202 I=1,NDMASCRN+NDMAPS
         NXCGTOSCR(I)=0
 202  CONTINUE
      MXXCGSCR=0
      DO 210 IXC=1,NXCHAR
         INX=NSCROFXCG(IXC)
         DO 205 ISC=1,INX
            IALRDY=0
            IXINDX=(IXC-1)*(NDMASCRN+NDMAPS)+ISC
            CALL CHKBND(21,IXINDX,'TRNTAB    ','SCROFXCG')
CS            IS=SCROFXCG(IXC,ISC)
            IS=SCROFXCG(IXINDX)
            IF (IS.GT.MXXCGSCR) MXXCGSCR=IS
            DO 203 I=1,NXCGTOSCR(IS)
               IXINDX=(I-1)*(NDMASCRN+NDMAPS)+IS
               CALL CHKBND(21,IXINDX,'TRNTAB    ','XCGTOSCR')
CS               IF (XCGTOSCR(IS,I).EQ.IXC) IALRDY=1
               IF (XCGTOSCR(IXINDX).EQ.IXC) IALRDY=1
 203        CONTINUE
            IF (IALRDY.EQ.0) THEN
               NXCGTOSCR(IS)=NXCGTOSCR(IS)+1
               IXINDX=(NXCGTOSCR(IS)-1)*(NDMASCRN+NDMAPS)+IS
               CALL CHKBND(21,IXINDX,'TRNTAB    ','XCGTOSCR')
CS               XCGTOSCR(IS,NXCGTOSCR(IS))=IXC
               XCGTOSCR(IXINDX)=IXC
            ENDIF
 205     CONTINUE
 210  CONTINUE

*     FOR XDD:
      DO 222 I=1,NDMASCRN+NDMAPS
         NXCDTOSCR(I)=0
 222  CONTINUE
      MXXCDSCR=0
      DO 230 IXC=1,NXCHAR
         INX=NSCROFXCD(IXC)
         DO 225 ISC=1,INX
            IALRDY=0
            IXINDX=(IXC-1)*(NDMASCRN+NDMAPS)+ISC
            CALL CHKBND(21,IXINDX,'TRNTAB    ','SCROFXCD')
CS            IS=SCROFXCD(IXC,ISC)
            IS=SCROFXCD(IXINDX)
            IF (IS.GT.MXXCDSCR) MXXCDSCR=IS
            DO 223 I=1,NXCDTOSCR(IS)
               IXINDX=(I-1)*(NDMASCRN+NDMAPS)+IS
               CALL CHKBND(21,IXINDX,'TRNTAB    ','XCDTOSCR')
CS               IF (XCDTOSCR(IS,I).EQ.IXC) IALRDY=1
               IF (XCDTOSCR(IXINDX).EQ.IXC) IALRDY=1
 223        CONTINUE
            IF (IALRDY.EQ.0) THEN
               NXCDTOSCR(IS)=NXCDTOSCR(IS)+1
               IXINDX1=(NXCDTOSCR(IS)-1)*(NDMASCRN+NDMAPS)+IS
               CALL CHKBND(21,IXINDX1,'TRNTAB    ','XCDTOSCR/XDETSCR')
               IXINDX2=(IXC-1)*(NDMASCRN+NDMAPS)+ISC
               CALL CHKBND(21,IXINDX2,'TRNTAB    ','XDETSCROFXCE')
CS               XCDTOSCR(IS,NXCDTOSCR(IS))=IXC
CS               XDETSCR(IS,NXCDTOSCR(IS))=XDETSCROFXCD(IXC,ISC)
               XCDTOSCR(IXINDX1)=IXC
               XDETSCR(IXINDX1)=XDETSCROFXCD(IXINDX2)
            ENDIF
 225     CONTINUE
 230  CONTINUE


*     FOR POSTMINI OUTPUT:

*     FOR IOD:
      DO 302 I=1,MAXSAV
         NIONTOPM(I)=0
 302  CONTINUE
      MXIONPM=0
      DO 310 ISH=1,NSHELL
         INS=NPSMINOFION(ISH)
         DO 305 ISC=1,INS
            INDX=(ISH-1)*NDMAPM+ISC
            CALL CHKBND(17,INDX,'TRNTAB    ','PSMINOFION')
CS            IS=PSMINOFION(ISH,ISC)
            IS=PSMINOFION(INDX)
            IF (IS.GT.MXIONPM) MXIONPM=IS
            IALRDY=0
            DO 303 I=1,NIONTOPM(IS)
               INDX=(I-1)*NDMAPM+IS
               CALL CHKBND(17,INDX,'TRNTAB    ','IONTOPM')
CS               IF (IONTOPM(IS,I).EQ.ISH) IALRDY=1
               IF (IONTOPM(INDX).EQ.ISH) IALRDY=1
 303        CONTINUE
            IF (IALRDY.EQ.0) THEN
               NIONTOPM(IS)=NIONTOPM(IS)+1
               INDX=(NIONTOPM(IS)-1)*NDMAPM+IS
               CALL CHKBND(17,INDX,'TRNTAB    ','IONTOPM')
CS               IONTOPM(IS,NIONTOPM(IS))=ISH
               IONTOPM(INDX)=ISH
            ENDIF
 305     CONTINUE
 310  CONTINUE

*     FOR XGD:
      DO 402 I=1,NDMAPM
         NXCGTOPM(I)=0
 402  CONTINUE
      MXXCGPM=0
      DO 410 IXC=1,NXCHAR
         INX=NPSMINOFXCG(IXC)
         DO 405 ISC=1,INX
            IALRDY=0
            IXINDX=(IXC-1)*NDMAPM+ISC
            CALL CHKBND(22,IXINDX,'TRNTAB    ','PSMINOFXCG')
CS            IS=PSMINOFXCG(IXC,ISC)
            IS=PSMINOFXCG(IXINDX)
            IF (IS.GT.MXXCGPM) MXXCGPM=IS
            DO 403 I=1,NXCGTOPM(IS)
               IXINDX=(I-1)*NDMAPM+IS
               CALL CHKBND(22,IXINDX,'TRNTAB    ','XCGTOPM')
CS               IF (XCGTOPM(IS,I).EQ.IXC) IALRDY=1
               IF (XCGTOPM(IXINDX).EQ.IXC) IALRDY=1
 403        CONTINUE
            IF (IALRDY.EQ.0) THEN
               NXCGTOPM(IS)=NXCGTOPM(IS)+1
               IXINDX=(NXCGTOPM(IS)-1)*NDMAPM+IS
               CALL CHKBND(22,IXINDX,'TRNTAB    ','XCGTOPM')
CS               XCGTOPM(IS,NXCGTOPM(IS))=IXC
               XCGTOPM(IXINDX)=IXC
            ENDIF
 405     CONTINUE
 410  CONTINUE

*     FOR XDD:
      DO 422 I=1,NDMAPM
         NXCDTOPM(I)=0
 422  CONTINUE
      MXXCDPM=0
      DO 430 IXC=1,NXCHAR
         INX=NPSMINOFXCD(IXC)
         DO 425 ISC=1,INX
            IALRDY=0
            IXINDX=(IXC-1)*NDMAPM+ISC
            CALL CHKBND(22,IXINDX,'TRNTAB    ','PSMINOFXCD')
CS            IS=PSMINOFXCD(IXC,ISC)
            IS=PSMINOFXCD(IXINDX)
            IF (IS.GT.MXXCDPM) MXXCDPM=IS
            DO 423 I=1,NXCDTOPM(IS)
               IXINDX=(I-1)*NDMAPM+IS
               CALL CHKBND(22,IXINDX,'TRNTAB    ','XCDTOPM')
CS               IF (XCDTOPM(IS,I).EQ.IXC) IALRDY=1
               IF (XCDTOPM(IXINDX).EQ.IXC) IALRDY=1
 423        CONTINUE
            IF (IALRDY.EQ.0) THEN
               NXCDTOPM(IS)=NXCDTOPM(IS)+1
               IXINDX1=(NXCDTOPM(IS)-1)*NDMAPM+IS
               CALL CHKBND(22,IXINDX1,'TRNTAB    ','XCDTOPM/XDETPM')
               IXINDX2=(IXC-1)*NDMAPM+ISC
               CALL CHKBND(22,IXINDX2,'TRNTAB    ','XDETPSMINOFXCD')
CS               XCDTOPM(IS,NXCDTOPM(IS))=IXC
CS               XDETPM(IS,NXCDTOPM(IS))=XDETPSMINOFXCD(IXC,ISC)
               XCDTOPM(IXINDX1)=IXC
               XDETPM(IXINDX1)=XDETPSMINOFXCD(IXINDX2)
            ENDIF
 425     CONTINUE
 430  CONTINUE

      IF (LTEST) THEN
         IF (MXIONSCR.GT.0) THEN
         WRITE(6,'(A)')'   '
         WRITE(6,'(A)')' SCREEN TABLE FOR IOD:'
         WRITE(6,'(A)')' SCREEN#    ENTRY#      SHELLENTRY#'
         DO 590 I=1,MXIONSCR
            INDX=I
            CALL CHKBND(16,INDX,'TRNTAB    ','IONTOSCR')
CS            WRITE(6,'(I5,I10,I15)')I,NIONTOSCR(I),IONTOSCR(I,1)
            WRITE(6,'(I5,I10,I15)')I,NIONTOSCR(I),IONTOSCR(INDX)
            DO 585 J=2,NIONTOSCR(I)
               INDX=(J-1)*(NDMASCRN+NDMAPS)+I
               CALL CHKBND(16,INDX,'TRNTAB    ','IONTOSCR')
CS               WRITE(6,'(T16,I15)')IONTOSCR(I,J)
               WRITE(6,'(T16,I15)')IONTOSCR(INDX)
 585        CONTINUE
 590     CONTINUE
         ENDIF
         IF (MXXCGSCR.GT.0) THEN
         WRITE(6,'(A)')'   '
         WRITE(6,'(A)')' SCREEN TABLE FOR XGD:'
         WRITE(6,'(A)')' SCREEN#    ENTRY#      XCHARENTRY#'
         DO 610 I=1,MXXCGSCR
            IXINDX=I
CS            WRITE(6,'(I5,I10,I15)')I,NXCGTOSCR(I),XCGTOSCR(I,1)
            WRITE(6,'(I5,I10,I15)')I,NXCGTOSCR(I),XCGTOSCR(IXINDX)
            DO 605 J=2,NXCGTOSCR(I)
               IXINDX=(J-1)*(NDMASCRN+NDMAPS)+I
               CALL CHKBND(21,IXINDX,'TRNTAB    ','XCGTOSCR')
CS               WRITE(6,'(T16,I15)')XCGTOSCR(I,J)
               WRITE(6,'(T16,I15)')XCGTOSCR(IXINDX)
 605        CONTINUE
 610     CONTINUE
         ENDIF
         IF (MXXCDSCR.GT.0) THEN
         WRITE(6,'(A)')'   '
         WRITE(6,'(A)')' SCREEN TABLE FOR XDD:'
         WRITE(6,'(A)')' SCREEN#    ENTRY#      XCHARENTRY#   XDET#'
         DO 620 I=1,MXXCDSCR
            IXINDX=I
CS            WRITE(6,'(I5,I10,I15,I12)')
CS     &               I,NXCDTOSCR(I),XCDTOSCR(I,1),XDETSCR(I,1)
            WRITE(6,'(I5,I10,I15,I12)')
     &               I,NXCDTOSCR(I),XCDTOSCR(IXINDX),XDETSCR(IXINDX)
            DO 615 J=2,NXCDTOSCR(I)
               IXINDX=(J-1)*(NDMASCRN+NDMAPS)+I
               CALL CHKBND(21,IXINDX,'TRNTAB    ','XCDTOSCR/XDETSCR')
CS               WRITE(6,'(T16,I15,I12)')XCDTOSCR(I,J),XDETSCR(I,J)
               WRITE(6,'(T16,I15,I12)')XCDTOSCR(IXINDX),
     &                                 XDETSCR(IXINDX)
 615        CONTINUE
 620     CONTINUE
         ENDIF

         IF (MXIONPM.GT.0) THEN
         WRITE(6,'(A)')'   '
         WRITE(6,'(A)')' POSTMINI TABLE FOR IOD:'
         WRITE(6,'(A)')'   PM#      ENTRY#      SHELLENTRY#'
         DO 690 I=1,MXIONPM
            INDX=I
            CALL CHKBND(17,INDX,'TRNTAB    ','IONTOPM')
CS            WRITE(6,'(I5,I10,I15)')I,NIONTOPM(I),IONTOPM(I,1)
            WRITE(6,'(I5,I10,I15)')I,NIONTOPM(I),IONTOPM(INDX)
            DO 685 J=2,NIONTOPM(I)
               INDX=(J-1)*NDMAPM+I
               CALL CHKBND(17,INDX,'TRNTAB    ','IONTOPM')
CS               WRITE(6,'(T16,I15)')IONTOPM(I,J)
               WRITE(6,'(T16,I15)')IONTOPM(INDX)
 685        CONTINUE
 690     CONTINUE
         ENDIF
         IF (MXXCGPM.GT.0) THEN
         WRITE(6,'(A)')'   '
         WRITE(6,'(A)')' POSTMINI TABLE FOR XGD:'
         WRITE(6,'(A)')'   PM#      ENTRY#      XCHARENTRY#'
         DO 710 I=1,MXXCGPM
            IXINDX=I
CS            WRITE(6,'(I5,I10,I15)')I,NXCGTOPM(I),XCGTOPM(I,1)
            WRITE(6,'(I5,I10,I15)')I,NXCGTOPM(I),XCGTOPM(IXINDX)
            DO 705 J=2,NXCGTOPM(I)
               IXINDX=(J-1)*NDMAPM+I
               CALL CHKBND(22,IXINDX,'TRNTAB    ','XCGTOPM')
CS               WRITE(6,'(T16,I15)')XCGTOPM(I,J)
               WRITE(6,'(T16,I15)')XCGTOPM(IXINDX)
 705        CONTINUE
 710     CONTINUE
         ENDIF
         IF (MXXCDPM.GT.0) THEN
         WRITE(6,'(A)')'   '
         WRITE(6,'(A)')' POSTMINI TABLE FOR XDD:'
         WRITE(6,'(A)')'   PM#      ENTRY#      XCHARENTRY#   XDET#'
         DO 720 I=1,MXXCDPM
            IXINDX=I
CS            WRITE(6,'(I5,I10,I15,I12)')
CS     &               I,NXCDTOPM(I),XCDTOPM(I,1),XDETPM(I,1)
            WRITE(6,'(I5,I10,I15,I12)')
     &               I,NXCDTOPM(I),XCDTOPM(IXINDX),XDETPM(IXINDX)
            DO 715 J=2,NXCDTOPM(I)
               IXINDX=(J-1)*NDMAPM+I
               CALL CHKBND(22,IXINDX,'TRNTAB    ','XCDTOPM/XDETPM')
CS               WRITE(6,'(T16,I15,I12)')XCDTOPM(I,J),XDETPM(I,J)
               WRITE(6,'(T16,I15,I12)')XCDTOPM(IXINDX),
     &                                 XDETPM(IXINDX)
 715        CONTINUE
 720     CONTINUE
         ENDIF
      ENDIF

      RETURN
      END

******************************************************************************

      SUBROUTINE CHKXDET (IXD,IERR)
 
****  CHKXDET - PARSES XLAYMAT RECORD IN &DETECT NAMELIST
*
**    ARGUMENTS
*             IXD  - X-RAY DETECTOR NUMBER
*             IERR - ERROR FLAG
*
**    AUTHOR: C.SCHIEBL 22-JAN-96, -5deg celsius
*             H.WAGNER  11-JUL-95, 32deg celsius
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'

      CHARACTER*10 CBL
      CHARACTER*70 CBUFF,CMAT
      CHARACTER*2  CAT

      DATA CBL /'          '/
 
**    START OF EXECUTABLE CODE
*     ************************

      IERR=0

      DO 100 IL=1,NDLAY(IXD)
         WGTFR=0
         IELE=0
         CMAT = XLAYMAT(IXD,IL)
         IE=IEND(CMAT)
         ICP = 1

 20      CONTINUE

         CALL FIND(CMAT,',',ICP,IE,IP,IFND)
         IF (IFND.EQ.1) THEN
            IEE = IP-1
         ELSE
            IEE = IE
         ENDIF

         CAT(1:1)=CMAT(ICP:ICP)
         CAT(2:2)=' '
         ICP=ICP+1
         IF (ICP .LE. IEE) THEN
            IF (LGE(CMAT(ICP:ICP),'0') .AND.
     .         LLE(CMAT(ICP:ICP),'9'))THEN
c              one character element abbreviation
               CAT(2:2)=' '
            ELSE
               CAT(2:2)=CMAT(ICP:ICP)
               ICP=ICP+1
            ENDIF
         ENDIF

         CALL CCAP(CAT,2)
         CBUFF = CBL//CBL//CBL//CBL//CBL//CBL//CBL
         CBUFF(1:2) = CAT  
         CALL ELMENT(1,CBUFF,IATNUM,CAT,FRM,FDENS,IERR)

         IF (IERR.NE.0) THEN
            WRITE(6,'(A)')'       '
            WRITE(6,'(A)')' %%% ERROR %%% '
            WRITE(6,'(2A)')' Invalid Chemical Abbreviation for the',
     &                     ' Specification of the Element!'
            WRITE(6,'(A)')' In ATOM of the &TARGET namelist !'
            WRITE(6,'(2A)')' SESAME will terminate -',
     &                     ' check input deck !'
            WRITE(6,'(A)')' Protests are useless - ;-) SESAME'
C           GOTO 923
         ELSE
            IELE=IELE+1
            IF (IELE.GT.10) THEN
               WRITE(6,'(A)')'       '
               WRITE(6,'(A)')' %%% ERROR %%% '
               WRITE(6,'(2A,I3,2A,I3)')
     &                                 ' Too many elements specified',
     &                                 ' for layer#',IL,' of x-ray',
     &                                 ' detector#',IXD
               IELE=IELE-1
               GOTO 100
            ENDIF
            NXLAYMAT(IXD,IL)=IELE
            LAYXEL(IXD,IL,IELE)=IATNUM
         ENDIF

C        DETERMINE ATOMIC FRACTION

         IF (ICP .GT. IEE) THEN
            FRAC = 1.
         ELSE
            READ(CMAT(ICP:IEE),*) FRAC
         ENDIF

         XLAYAF(IXD,IL,IELE) = FRAC
         WGTFR=WGTFR+XLAYAF(IXD,IL,IELE)

         IF (IEE+2 .LE. IE) THEN
            ICP = IEE+2
            GOTO 20
         ENDIF

         DO 90 IEL=1,IELE
            XLAYAF(IXD,IL,IEL) = XLAYAF(IXD,IL,IEL)/WGTFR
 90      CONTINUE

100   CONTINUE

      RETURN
      END

******************************************************************************
*last modification: 19-Nov-1996

      SUBROUTINE CHKENER (E0,EDLIMT,ECLIMT,EDLIM,ECLIM)

****  CHKENER - ADJUSTS LIMITS FOR POSSIBLE IONISATIONS (EDGE ENERGY SMALLER
*               THAN E0)
*
**    ARGUMENTS
*             E0     - ACCELERATION ENERGY
*             EDLIMT - ORIG. ATOMIC NUMBER LIMITS FOR CRITICAL ENERGIES
*             ECLIMT - ORIG. ATOMIC NUMBER LIMITS FOR CHARACTERISTIC ENERGIES
*             EDLIM  - MOD. ATOMIC NUMBER LIMITS FOR CRITICAL ENERGIES
*             ECLIM  - MOD. ATOMIC NUMBER LIMITS FOR CHARACTERISTIC ENERGIES
*
*     AUTHOR: CHRISTIAN SCHIEBL
*             CONSULTANT (FORMER: CEC-VIENNA)
*             KREMS, NOVEMBER 1996
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'

      INTEGER EDLIMT(MAXSHELL,2),ECLIMT(MAXXCHAR,2)
      INTEGER EDLIM(MAXSHELL,2),ECLIM(MAXXCHAR,2)

**    START OF EXECUTABLE CODE
*     ************************

      DO 10 I=1,MAXSHELL
         EDLIM(I,1)=EDLIMT(I,1)
         EDLIM(I,2)=EDLIMT(I,2)
 10   CONTINUE
      DO 20 I=1,MAXXCHAR
         ECLIM(I,1)=ECLIMT(I,1)
         ECLIM(I,2)=ECLIMT(I,2)
 20   CONTINUE

      DO 50 ISHELL=1,MAXSHELL
         IELE=EDLIM(ISHELL,2)+1
 30      IELE=IELE-1
         IF (IELE.GT.EDLIM(ISHELL,1)) THEN
            CALL GETED(2,ISHELL,IELE,ECRIT,IERR)
            IF (IERR.EQ.0) THEN
               IF (ECRIT.GE.E0) THEN
                  EDLIM(ISHELL,2)=IELE-1
                  GOTO 30
               ENDIF
            ENDIF
         ENDIF
 50   CONTINUE

      ECLIM(1,2)=MIN(ECLIM(1,2),EDLIM(1,2))
      ECLIM(2,2)=MIN(ECLIM(2,2),EDLIM(1,2))
      ECLIM(3,2)=MIN(ECLIM(3,2),EDLIM(1,2))
      ECLIM(4,2)=MIN(ECLIM(4,2),EDLIM(4,2))
      ECLIM(5,2)=MIN(ECLIM(5,2),EDLIM(4,2))
      ECLIM(6,2)=MIN(ECLIM(6,2),EDLIM(3,2))
      ECLIM(7,2)=MIN(ECLIM(7,2),EDLIM(4,2))
      ECLIM(8,2)=MIN(ECLIM(8,2),EDLIM(2,2))
      ECLIM(9,2)=MIN(ECLIM(9,2),EDLIM(9,2))
      ECLIM(10,2)=MIN(ECLIM(10,2),EDLIM(8,2))

*     PUT THE UPDATED VALUES TO THE EDLIM AND ECLIM COMMON BLOCK ARRAYS
      CALL PUTLIM (EDLIM,ECLIM)

      RETURN
      END

***************************************************************************
*last modification: 19-Nov-1996

      SUBROUTINE PUTLIM (EDLIMT,ECLIMT)

****  CHKENER - ADJUSTS LIMITS FOR POSSIBLE IONISATIONS (EDGE ENERGY SMALLER
*               THAN E0)
*
**    ARGUMENTS
*             EDLIMT - ORIG. ATOMIC NUMBER LIMITS FOR CRITICAL ENERGIES
*             ECLIMT - ORIG. ATOMIC NUMBER LIMITS FOR CHARACTERISTIC ENERGIES
*
*     AUTHOR: CHRISTIAN SCHIEBL
*             CONSULTANT (FORMER: CEC-VIENNA)
*             KREMS, NOVEMBER 1996
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'

      INTEGER EDLIMT(MAXSHELL,2),ECLIMT(MAXXCHAR,2)

      DO 10 I=1,MAXSHELL
         EDLIM(I,1)=EDLIMT(I,1)
         EDLIM(I,2)=EDLIMT(I,2)
 10   CONTINUE
      DO 20 I=1,MAXXCHAR
         ECLIM(I,1)=ECLIMT(I,1)
         ECLIM(I,2)=ECLIMT(I,2)
 20   CONTINUE
       
      RETURN
      END

****************************************************************************
