c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
*      DMATH  : MATHEMATICAL UTILITIES (DSIMPSON,DINVERSE,DINTERPOL) 
*
*      AUTHOR : H.WAGNER MAR-JUL 1994
*
C      DOUBLE PRECISION FUNCTION DSIMPSON(X,Y,IFLAG)
C
C	Numerische Integration nach der Trapezregel.
C	(X,Y) Wertepaar zum integrieren
C	IFLAG = -1   : integration zuruecksetzen
C	IFLAG =  0   : normal integrieren
C	IFLAG = rest : nur aktuellen Wert zurueckliefern 
C
C      DOUBLE PRECISION FUNCTION DINVERSE(Y,A,N)
C
C
C	Berechnet fuer gegebenes Y ein linear interpoliertes X
C	Wobei Y(X) gegeben ist durch das Array A(N)
C
C
C      DOUBLE PRECISION FUNCTION DINTERPOL(YTAB,N,XMIN,XMAX,X,IER)
C
C
C	LINEAR INTERPOLATION OF Y(X) USING ARRAY YTAB(N)
C	WITH YTAB(1) = Y(XMIN) AND YTAB(N) = YTAB(XMAX)
C	IER = 1 IF X .LT. XMIN .OR X .GT. XMAX
C
C
C      SUBROUTINE DUMP(
C     I		      ARRAY,N,FILENAME)
C  
C       Dump contents of array ARRAY(N) to file FILENAME
C   
C
C     INTEGER FUNCTION IEND (STRING)
C
C     IEND - DETERMINE ACTUAL LENGTH OF STRING
C
C     M.THURNER 1991
C
C      SUBROUTINE MINMAX(A,J,N,IMIN,MIN,IMAX,MAX)
C
C
C    AUTHOR: H.WAGNER 28-DEC-94   
C
C
C     FINDS FIRST ABSOLUTE MAXIMUM AND MINIMUM (MAX,MIN) OF ARRAY A(J..J+N-1) 
C     AND THEIR POSITION (IMAX,IMIN)
C
C    START OF EXECUTABLE CODE
*****************************************************************************


      DOUBLE PRECISION FUNCTION DSIMPSON(X,Y,IFLAG)

      IMPLICIT NONE

      DOUBLE PRECISION X,Y
      INTEGER IFLAG 
      INTEGER ISTAT
      DOUBLE PRECISION QUADRATUR,XOLD,YOLD
      SAVE QUADRATUR,XOLD,YOLD,ISTAT
      DATA QUADRATUR /0./
      DATA ISTAT /-1/
      IF (IFLAG .EQ. 0) THEN
C     NORMAL INTEGRIEREN
   	IF (ISTAT .EQ. -1) THEN
  	   XOLD = X
      	   YOLD = Y
       	   ISTAT = 1
        ELSE 
           QUADRATUR = QUADRATUR + (X-XOLD)*(Y+YOLD)*0.5
      	   XOLD = X
      	   YOLD = Y
        ENDIF
      ELSE IF (IFLAG .EQ. -1) THEN
C     INTEGRATION RUECKSETZEN
        ISTAT = -1
        QUADRATUR = 0
      ENDIF
C     SONST NUR ERGEBNIS LIEFERN OHNE INTEGRATION
      DSIMPSON = QUADRATUR
      END

*****************************************************************************

      DOUBLE PRECISION FUNCTION DINVERSE(Y,A,N)

      IMPLICIT NONE

C
C	Berechnet fuer gegebenes Y ein linear interpoliertes X
C	Wobei Y(X) gegeben ist durch das Array A(N)


      DOUBLE PRECISION Y
      INTEGER N
      DOUBLE PRECISION A(N)

**    START OF EXECUTABLE CODE

      INTEGER I
      I=1
100   CONTINUE
       IF (Y.LT.A(I)) GOTO 200
       I=I+1
       IF (I.GT.N) THEN
      	DINVERSE = REAL(N) 
      	GOTO 300
       ELSE
        GOTO 100
       ENDIF 
200   CONTINUE
      IF (I.EQ.1) THEN 
      	DINVERSE = 0.
        GOTO 300
      ELSE
        DINVERSE = REAL(I) + (Y-A(I-1))/(A(I)-A(I-1))
      ENDIF
300   CONTINUE
      END

*****************************************************************************

      DOUBLE PRECISION FUNCTION DINTERPOL(YTAB,N,XMIN,XMAX,X,IER)

      IMPLICIT NONE

C
C	LINEAR INTERPOLATION OF Y(X) USING ARRAY YTAB(N)
C	WITH YTAB(1) = Y(XMIN) AND YTAB(N) = YTAB(XMAX)
C	IER = 1 IF X .LT. XMIN .OR X .GT. XMAX


      DOUBLE PRECISION Y,X,XMAX,XMIN
      INTEGER N,IER
      DOUBLE PRECISION YTAB(*)

      DOUBLE PRECISION DELTAX,XSTEP
      INTEGER IX

**    START OF EXECUTABLE CODE

      IF (X .LT. XMIN .OR. X .GT. XMAX) THEN
      	 IER = 1
      ELSE
      	 IER = 0
      	 XSTEP = (XMAX-XMIN)/(N-1)
         IX = AINT((X-XMIN)/XSTEP)+1
         IF (IX.EQ.N) THEN
            DINTERPOL = YTAB(N)
 	 ELSE
            DELTAX = (X-XMIN-(IX-1)*XSTEP)/XSTEP
            DINTERPOL = YTAB(IX)*(1-DELTAX) + YTAB(IX+1)*DELTAX
      	 ENDIF      	 
      ENDIF
      END

*****************************************************************************
      SUBROUTINE DUMP(
     I		      ARRAY,N,FILENAME)

      IMPLICIT NONE

      INTEGER N
      DOUBLE PRECISION ARRAY(N)
      CHARACTER*(*) FILENAME

      INTEGER I

      OPEN(12,FILE=FILENAME,STATUS='UNKNOWN',CARRIAGECONTROL='LIST')
      WRITE(12,'(F12.5)')(ARRAY(I),I=1,N)  
      CLOSE(12)
      END

*****************************************************************************

      INTEGER FUNCTION IEND (STRING)

*     IEND - DETERMINE ACTUAL LENGTH OF STRING
*
*     M.THURNER 1991
*

      CHARACTER*(*) STRING

      IL=LEN(STRING)
      I=IL+1
 1    I=I-1
      IF (STRING(I:I).EQ.' '.AND.I.GT.1) GOTO 1

      IEND=I

      RETURN
      END

*****************************************************************************

      SUBROUTINE MINMAX(A,J,N,MIN,IMIN,MAX,IMAX)

      IMPLICIT NONE

      INTEGER J,N,IMIN,IMAX
      DOUBLE PRECISION A(*)
      DOUBLE PRECISION MIN,MAX

**    AUTHOR: H.WAGNER 28-DEC-94   

*
*     FINDS FIRST ABSOLUTE MAXIMUM AND MINIMUM (MAX,MIN) OF ARRAY A(J..J+N-1) 
*     AND THEIR POSITION (IMAX,IMIN)

**    START OF EXECUTABLE CODE

      INTEGER I

      MIN = A(J)
      IMIN = J
      MAX = A(J)
      IMAX = J

      DO 100 I=J,J+N-1 
         IF (A(I) .LT. MIN) THEN
            IMIN = I
            MIN = A(I)
         ENDIF
         IF (A(I) .GT. MAX) THEN
            IMAX = I
            MAX = A(I)
         ENDIF

100   CONTINUE

      END       
