c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
***   ELASTIC.F - ROUTINES FOR ELASTIC SCATTERING BASING ON THE MOTT 
*                 CROSS SECTION
*
*     DESCRIPTION OF ROUTINES :
*
*
***   E_INIT   - INITIALIZES TABLES COSTHETA(),COSTHINTV() AND NCOSTHINTV()
*		 FOR USAGE WITH ELASTCOSL
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 21-JUN-1994
*
*     CALLS: DSIMPSON,DINVERSE,MCS
*
*
***   E_SIGMA - RETURNS VALUE FOR TOTAL ELASTIC CROSS SECTION OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     SIGMA	- CROSS SECTION
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
*
***   E_LAMBDA - RETURNS VALUE FOR TOTAL ELASTIC LAMBDA OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     LAMBDA    - ELASTIC MEAN FREE PATH
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
*
***   E_THETA - MONTE CARLO GENERATOR FOR ELASTIC SCATTERING USING
*		    LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     COSTH	- THETA
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 16-JUN-1994
*
**    START OF EXECUTABLE CODE
*     ************************

      SUBROUTINE E_INIT(
     O			  IER)

      IMPLICIT NONE

      INTEGER IER

***   E_INIT - INITIALIZES TABLES COSTHETA(),COSTHINTV() AND NCOSTHINTV()
*		 FOR USAGE WITH E_THETA()
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 21-JUN-1994
*
*     CALLS: DSIMPSON,DINVERSE,MCS

      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

       
**    START OF EXECUTABLE CODE
*     ************************
      DOUBLE PRECISION DINVERSE
      DOUBLE PRECISION DSIMPSON,MCS,TMFP
      EXTERNAL DSIMPSON,DINVERSE,MCS,STATXT
      
      INTEGER NTHETA
      PARAMETER (NTHETA = 90)
      INTEGER ITHETAMIN
      PARAMETER (ITHETAMIN = 10)      
      CHARACTER*5 CDUMM

      INTEGER IREG,IELE,IENER,ITHETA,IRAN,IATNUM

      DOUBLE PRECISION MCSTOTAL(0:NTHETA)
      DOUBLE PRECISION MCSDATA(90)
      DOUBLE PRECISION CMCSTOTAL(0:NTHETA)
      DOUBLE PRECISION X,Y,Z
      DOUBLE PRECISION E,THETA
      DOUBLE PRECISION A_MEAN

C     INITIALIZE ENERGY INTERVALS TO CONSTANT VALUES

      DO 50,IREG=1,MAXREG
      	 NCOSTHINTV(IREG) = 54
         COSTHINTV(IREG,1) = 10.      
         COSTHINTV(IREG,2) = 12.      
         COSTHINTV(IREG,3) = 15.      
      	 COSTHINTV(IREG,4) = 20.
      	 COSTHINTV(IREG,5) = 25.
      	 COSTHINTV(IREG,6) = 30.
      	 COSTHINTV(IREG,7) = 40.
      	 COSTHINTV(IREG,8) = 50.
      	 COSTHINTV(IREG,9) = 60.
      	 COSTHINTV(IREG,10) = 70.
      	 COSTHINTV(IREG,11) = 80.
      	 COSTHINTV(IREG,12) = 90.
         COSTHINTV(IREG,13) = 100.      
         COSTHINTV(IREG,14) = 120.      
         COSTHINTV(IREG,15) = 150.      
      	 COSTHINTV(IREG,16) = 200.
      	 COSTHINTV(IREG,17) = 250.
      	 COSTHINTV(IREG,18) = 300.
      	 COSTHINTV(IREG,19) = 400.
      	 COSTHINTV(IREG,20) = 500.
      	 COSTHINTV(IREG,21) = 600.
      	 COSTHINTV(IREG,22) = 700.
      	 COSTHINTV(IREG,23) = 800.
      	 COSTHINTV(IREG,24) = 900.
         COSTHINTV(IREG,25) = 1000.      
         COSTHINTV(IREG,26) = 1200.      
         COSTHINTV(IREG,27) = 1500.      
      	 COSTHINTV(IREG,28) = 2000.
      	 COSTHINTV(IREG,29) = 2500.
      	 COSTHINTV(IREG,30) = 3000.
      	 COSTHINTV(IREG,31) = 4000.
      	 COSTHINTV(IREG,32) = 5000.
      	 COSTHINTV(IREG,33) = 6000.
      	 COSTHINTV(IREG,34) = 7000.
      	 COSTHINTV(IREG,35) = 8000.
      	 COSTHINTV(IREG,36) = 9000.
         COSTHINTV(IREG,37) = 10000.      
         COSTHINTV(IREG,38) = 12000.      
         COSTHINTV(IREG,39) = 15000.      
      	 COSTHINTV(IREG,40) = 20000.
      	 COSTHINTV(IREG,41) = 25000.
      	 COSTHINTV(IREG,42) = 30000.
      	 COSTHINTV(IREG,43) = 40000.
      	 COSTHINTV(IREG,44) = 50000.
      	 COSTHINTV(IREG,45) = 60000.
      	 COSTHINTV(IREG,46) = 70000.
      	 COSTHINTV(IREG,47) = 80000.
      	 COSTHINTV(IREG,48) = 90000.
         COSTHINTV(IREG,49) = 100000.      
         COSTHINTV(IREG,50) = 120000.      
         COSTHINTV(IREG,51) = 150000.      
      	 COSTHINTV(IREG,52) = 200000.
      	 COSTHINTV(IREG,53) = 250000.
      	 COSTHINTV(IREG,54) = 300000.

50    CONTINUE

C     FOR ALL REGIONS

      DO 500,IREG=1,NREG
        WRITE(CDUMM,'(I3)') IREG
        CALL STATXT(' INITIALIZING ELASTIC SCATTERING FOR REGION '//
     .		     CDUMM ) 

C        FOR ALL ENERGIES

         DO 400,IENER=1,NCOSTHINTV(IREG)

            E = COSTHINTV(IREG,IENER)

C     	    CLEAR TOTAL MCS 

            DO 200,ITHETA=0,NTHETA
               MCSTOTAL(ITHETA) = 0.
      	    A_MEAN = 0.

200         CONTINUE

C           TAKE ALL ELEMENTS IN REGION IREG IN ACCOUNT

            DO 300,IELE=1,NELE(IREG)

              IATNUM = ATNUM(ELEM(IREG,IELE))
      	      A_MEAN = A_MEAN + ATFRAC(IREG,IELE)*
     .			        ATMASS(ELEM(IREG,IELE))
              CALL MCS(IATNUM,IENER,MCSDATA)		
      	      DO 250,ITHETA=1,NTHETA
      		 MCSTOTAL(ITHETA-1) = MCSTOTAL(ITHETA-1) +
     . 			ATFRAC(IREG,IELE) * MCSDATA(ITHETA)
250	      CONTINUE
300         CONTINUE
      		
     
C	    COMPUTE CUMULATIVE CROSS SECTION

            X = DSIMPSON(0,0,-1)
            DO 350,ITHETA=0,NTHETA
      	       THETA = PI/NTHETA*(ITHETA)
               CMCSTOTAL(ITHETA) = 
     .		    DSIMPSON(THETA,2*PI*MCSTOTAL(ITHETA)*SIN(THETA),0)
350 	    CONTINUE


C	    TABULATE CUMULATIVE NORMALIZED CROSS SECTION

            SIGMA_E(IENER,IREG) = DSIMPSON(0,0,1)
      	    LAMBDA_E(IENER,IREG) = 1/SIGMA_E(IENER,IREG)*
     .		A_MEAN/N_Avo/density(ireg)
      	    DO 380,ITHETA = 0,NTHETA
	        X = CMCSTOTAL(ITHETA)/SIGMA_E(IENER,IREG)
      		COSTHETA(IREG,IENER,ITHETA) = X
380 	    CONTINUE

C	    COMPUTE TRANSPORT MEAN FREE PATH

            X = DSIMPSON(0,0,-1)
            DO 385,ITHETA=0,NTHETA
      	       THETA = PI/NTHETA*(ITHETA)
               TMFP = DSIMPSON(THETA,2*PI*MCSTOTAL(ITHETA)*SIN(THETA)*
     .      		       (1-COS(THETA)),0)
385 	    CONTINUE

      	    LAMBDA_TR(IENER,IREG) = 1/TMFP*
     .		A_MEAN/N_Avo/density(ireg)

400      CONTINUE
      
500   CONTINUE

      IER = 0
cs      GOTO 1000

cs900   IER = -1

1000  END      
    
**************************************************************************

      SUBROUTINE E_SIGMA(
     I      		   IREG,E,
     O  		   SIGMA,IER)
      
      IMPLICIT NONE

      DOUBLE PRECISION E,SIGMA
      INTEGER IREG,IER

***   E_SIGMA - RETURNS VALUE FOR TOTAL ELASTIC CROSS SECTION OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     SIGMA	- CROSS SECTION
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      DOUBLE PRECISION X1,X2,Y1,Y2
      INTEGER IENER

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL
      
      IF (E .LT. COSTHINTV(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,NCOSTHINTV(IREG)
      	IF (E.LT.COSTHINTV(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   CONTINUE

C     LINEAR INTERPOLATION
      Y2 = SIGMA_E(IENER,IREG)
      Y1 = SIGMA_E(IENER-1,IREG)
      X1 = COSTHINTV(IENER-1,IREG)
      X2 = COSTHINTV(IENER,IREG)
      SIGMA  = Y1 + (Y2-Y1)/(X2-X1)*(E-X1)
      IER = 0

1000  END      
    
********************************************************************

      SUBROUTINE E_LAMBDA(
     I      		   IREG,E,
     O  		   LAMBDA,IER)
      
      IMPLICIT NONE

      DOUBLE PRECISION E,LAMBDA
      INTEGER IREG,IER

***   E_LAMBDA - RETURNS VALUE FOR TOTAL ELASTIC LAMBDA OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     LAMBDA    - ELASTIC MEAN FREE PATH
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      DOUBLE PRECISION X1,X2,Y1,Y2
      INTEGER IENER

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL

      IF (E .LT. COSTHINTV(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,NCOSTHINTV(IREG)
      	IF (E.LT.COSTHINTV(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   CONTINUE

C     USE MOTT CROSS SECTION
      
        Y2 = LAMBDA_E(IENER,IREG)
        Y1 = LAMBDA_E(IENER-1,IREG)
        X1 = COSTHINTV(IREG,IENER-1)
        X2 = COSTHINTV(IREG,IENER)
        LAMBDA  = Y1 + (Y2-Y1)/(X2-X1)*(E-X1)

      IER = 0

1000  END      

***********************************************************************

      SUBROUTINE TR_LAMBDA(
     I      		   IREG,E,
     O  		   LAMBDA,IER)
      
      IMPLICIT NONE

      DOUBLE PRECISION E,LAMBDA
      INTEGER IREG,IER

***   TR_LAMBDA - RETURNS VALUE FOR TRANSPORT MEAN FREE PATH OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     LAMBDA    - TRANSPORT MEAN FREE PATH
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 7-JUN-1995
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      DOUBLE PRECISION X1,X2,Y1,Y2
      INTEGER IENER

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL

      IF (E .LT. COSTHINTV(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,NCOSTHINTV(IREG)
      	IF (E.LT.COSTHINTV(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   CONTINUE

      Y2 = LAMBDA_TR(IENER,IREG)
      Y1 = LAMBDA_TR(IENER-1,IREG)
      X1 = COSTHINTV(IREG,IENER-1)
      X2 = COSTHINTV(IREG,IENER)
      LAMBDA  = Y1 + (Y2-Y1)/(X2-X1)*(E-X1)

      IER = 0

1000  END      

*********************************************************************
    
      SUBROUTINE E_THETA(
     I      		   IREG,E,R1,R3,
     O  		   THETA,IER)

      IMPLICIT NONE

      DOUBLE PRECISION E,THETA,R1,R3
      INTEGER IREG,IER

***   E_THETA     - MONTE CARLO GENERATOR FOR ELASTIC SCATTERING USING
*		    LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     R1,R3     - TWO UNIFORMLY DIST. RANDOM NUMBERS
*     COSTH	- COS(THETA)
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 16-JUN-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      
      INTEGER IR,IENER,ITHETA
      DOUBLE PRECISION X1,X2,Y1,Y2
      DOUBLE PRECISION R2
      INTEGER NTHETA
      PARAMETER (NTHETA = 90)
      DOUBLE PRECISION THETA1,THETA2

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL
C      PRINT *, 'ELASTCOSL :',NCOSTHINTV(IREG),E
      
      IF (E .LT. COSTHINTV(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,NCOSTHINTV(IREG)
      	IF (E.LT.COSTHINTV(IREG,IENER)) GOTO 150
100   CONTINUE

      IER = -1
      GOTO 1000

CHW   Select Energy Table to be used

150   CONTINUE
      X1 = COSTHINTV(IREG,IENER-1)
      X2 = COSTHINTV(IREG,IENER)

      IF (R3 .GT. (E-X1)/(X2-X1)) THEN
         IENER = IENER-1
      ENDIF

CHW   Find Angle

      DO 200,ITHETA = 0,NTHETA
      IF (COSTHETA(IREG,IENER,ITHETA) .GE. R1) GOTO 210	 
200   CONTINUE

      WRITE(*,*) 'ERROR E_THETA  IREG,IENER,R1:',IREG,IENER,R1

210   CONTINUE

CHW   LINEAR INTERPOLATION

      THETA1 = COSTHETA(IREG,IENER,ITHETA-1)
      THETA2 = COSTHETA(IREG,IENER,ITHETA)

      THETA = ((R1-THETA1)/(THETA2-THETA1)+ITHETA)*2.

      IER = 0

1000  END      
