c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
***   ELASTIC.F - ROUTINES FOR ELASTIC SCATTERING BASING ON THE MOTT 
*                 CROSS SECTION
*
*     DESCRIPTION OF ROUTINES :
*
*
***   E_INIT   - INITIALIZES TABLES COSTHETA(),COSTHINTV() AND NCOSTHINTV()
*		 FOR USAGE WITH ELASTCOSL
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 21-JUN-1994
*
*     CALLS: DSIMPSON,DINVERSE,MCS
*
*
***   E_SIGMA - RETURNS VALUE FOR TOTAL ELASTIC CROSS SECTION OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     SIGMA	- CROSS SECTION
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
*
***   E_LAMBDA - RETURNS VALUE FOR TOTAL ELASTIC LAMBDA OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     LAMBDA    - ELASTIC MEAN FREE PATH
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
*
***   E_THETA - MONTE CARLO GENERATOR FOR ELASTIC SCATTERING USING
*		    LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     COSTH	- THETA
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 16-JUN-1994
*
**    START OF EXECUTABLE CODE
*     ************************

      SUBROUTINE E_INIT(
     O			  IER)

      IMPLICIT NONE

      INTEGER IER

***   E_INIT - INITIALIZES TABLES COSTHETA(),COSTHINTV() AND NCOSTHINTV()
*		 FOR USAGE WITH E_THETA()
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 21-JUN-1994
*
*     CALLS: DSIMPSON,DINVERSE,MCS

      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

       
**    START OF EXECUTABLE CODE
*     ************************
      DOUBLE PRECISION DINVERSE
      DOUBLE PRECISION DSIMPSON,MCS
      EXTERNAL DSIMPSON,DINVERSE,MCS
      
      INTEGER NTHETA
      PARAMETER (NTHETA = 90)
      INTEGER ITHETAMIN
      PARAMETER (ITHETAMIN = 0)      

      INTEGER IREG,IELE,IENER,ITHETA,IRAN,IATNUM

      DOUBLE PRECISION MCSTOTAL(NTHETA)
      DOUBLE PRECISION CMCSTOTAL(NTHETA)
      DOUBLE PRECISION X,Y,Z
      DOUBLE PRECISION E,THETA
      DOUBLE PRECISION A_MEAN

C     INITIALIZE ENERGY INTERVALS TO CONSTANT VALUES

      DO 50,IREG=1,MAXREG
      	 NCOSTHINTV(IREG) = 12
         COSTHINTV(IREG,1) = 10.      
         COSTHINTV(IREG,2) = 20.      
         COSTHINTV(IREG,3) = 50.      
      	 COSTHINTV(IREG,4) = 100.
      	 COSTHINTV(IREG,5) = 200.
      	 COSTHINTV(IREG,6) = 500.
      	 COSTHINTV(IREG,7) = 1000.
      	 COSTHINTV(IREG,8) = 2000.
      	 COSTHINTV(IREG,9) = 5000.
      	 COSTHINTV(IREG,10) = 10000.
      	 COSTHINTV(IREG,11) = 20000.
      	 COSTHINTV(IREG,12) = 50000.
50    CONTINUE

C     INITIALIZE TRANSFORMATION MATRIX	

      DO 100,IRAN=0,NTRANS
         Z = (REAL(IRAN))/REAL(NTRANS)
         ELASTTRANS(IRAN,2)=((1.01**.2-0.01**.2)*Z+0.01**.2)**5.-0.01  

100   CONTINUE
      DO 110,IRAN=0,MAXRAN_E
         Z = (REAL(IRAN))/REAL(MAXRAN_E)
      	 ELASTTRANS(IRAN,1)=((Z +0.01)**.2-0.01**.2)/(1.01**.2-0.01**.2)

110   CONTINUE


C     FOR ALL REGIONS

      DO 500,IREG=1,NREG
         WRITE (*,*) 'INITIALIZING ELASTIC SCATTERING FOR REGION',IREG

C        FOR ALL ENERGIES

         DO 400,IENER=1,NCOSTHINTV(IREG)

            E = COSTHINTV(IREG,IENER)

C     	    CLEAR TOTAL MCS 

            DO 200,ITHETA=1,NTHETA
               MCSTOTAL(ITHETA) = 0.
      	    A_MEAN = 0.

200         CONTINUE

C           TAKE ALL ELEMENTS IN REGION IREG IN ACCOUNT

            DO 300,IELE=1,NELE(IREG)

              IATNUM = ATNUM(ELEM(IREG,IELE))
      	      A_MEAN = A_MEAN + ATFRAC(IREG,IELE)*
     .			        ATMASS(ELEM(IREG,IELE))		
      	      DO 250,ITHETA=1,NTHETA
      		 MCSTOTAL(ITHETA) = MCSTOTAL(ITHETA) +
     . 			ATFRAC(IREG,IELE) * MCS(IATNUM,ITHETA,IENER)
250	      CONTINUE
300         CONTINUE
      		
     
C	    COMPUTE CUMULATIVE CROSS SECTION

            X = DSIMPSON(0,0,-1)
            DO 350,ITHETA=1,NTHETA
      	       THETA = PI/NTHETA*(ITHETA-1)
      	       IF (ITHETA.LT.ITHETAMIN) THEN
      		  CMCSTOTAL(ITHETA) = 0.
      	       ELSE
                  CMCSTOTAL(ITHETA) = 
     .		    DSIMPSON(THETA,2*PI*MCSTOTAL(ITHETA)*SIN(THETA),0)
      	       ENDIF
350 	    CONTINUE


C	    COMPUTE INVERSE NORM. CUMUL. CROSS SECTION

            SIGMA_E_TOT(IREG,IENER) = DSIMPSON(0,0,1)
      	    LAMBDA_E(IREG,IENER) = 1/SIGMA_E_TOT(IREG,IENER)*
     .		A_MEAN/N_Avo/density(ireg)
      	    DO 380,IRAN = 1,MAXRAN_E
      		Y = ELASTTRANS(IRAN,1)*SIGMA_E_TOT(IREG,IENER)
      		X = (DINVERSE(Y,CMCSTOTAL,NTHETA)-1)*PI/
     .				REAL(NTHETA)
      		COSTHETA(IREG,IENER,IRAN) = X
380 	    CONTINUE

400      CONTINUE
      
500   CONTINUE

      IER = 0
      GOTO 1000

900   IER = -1

1000  END      
    
**************************************************************************

      SUBROUTINE E_SIGMA(
     I      		   IREG,E,
     O  		   SIGMA,IER)
      
      IMPLICIT NONE

      DOUBLE PRECISION E,SIGMA
      INTEGER IREG,IER

***   E_SIGMA - RETURNS VALUE FOR TOTAL ELASTIC CROSS SECTION OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     SIGMA	- CROSS SECTION
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      DOUBLE PRECISION X1,X2,Y1,Y2
      INTEGER IENER

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL
      
      IF (E .LT. COSTHINTV(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,NCOSTHINTV(IREG)
      	IF (E.LT.COSTHINTV(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   CONTINUE

C     LINEAR INTERPOLATION
      Y2 = SIGMA_E_TOT(IREG,IENER)
      Y1 = SIGMA_E_TOT(IREG,IENER-1)
      X1 = COSTHINTV(IREG,IENER-1)
      X2 = COSTHINTV(IREG,IENER)
      SIGMA  = Y1 + (Y2-Y1)/(X2-X1)*(E-X1)
      IER = 0

1000  END      
    
********************************************************************

      SUBROUTINE E_LAMBDA(
     I      		   IREG,E,
     O  		   LAMBDA,IER)
      
      IMPLICIT NONE

      DOUBLE PRECISION E,LAMBDA
      INTEGER IREG,IER

***   ELASTLAMBDA - RETURNS VALUE FOR TOTAL ELASTIC LAMBDA OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     LAMBDA    - ELASTIC MEAN FREE PATH
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      DOUBLE PRECISION X1,X2,Y1,Y2
      INTEGER IENER

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL
      
      IF (E .LT. COSTHINTV(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,NCOSTHINTV(IREG)
      	IF (E.LT.COSTHINTV(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   CONTINUE

C     LINEAR INTERPOLATION
      Y2 = LAMBDA_E(IREG,IENER)
      Y1 = LAMBDA_E(IREG,IENER-1)
      X1 = COSTHINTV(IREG,IENER-1)
      X2 = COSTHINTV(IREG,IENER)
      LAMBDA  = Y1 + (Y2-Y1)/(X2-X1)*(E-X1)
      IER = 0

1000  END      

*********************************************************************
    
      SUBROUTINE E_THETA(
     I      		   IREG,E,R1,R3,
     O  		   THETA,IER)

      IMPLICIT NONE

      DOUBLE PRECISION E,THETA,R1,R3
      INTEGER IREG,IER

***   E_THETA     - MONTE CARLO GENERATOR FOR ELASTIC SCATTERING USING
*		    LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     R1,R3     - TWO UNIFORMLY DIST. RANDOM NUMBERS
*     COSTH	- COS(THETA)
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 16-JUN-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      
      INTEGER IR,IENER
      DOUBLE PRECISION X1,X2,Y1,Y2
      DOUBLE PRECISION R2

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL
C      PRINT *, 'ELASTCOSL :',NCOSTHINTV(IREG),E
      
      IF (E .LT. COSTHINTV(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,NCOSTHINTV(IREG)
      	IF (E.LT.COSTHINTV(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   IR = NINT(R1*REAL(NTRANS))
      R2 = ELASTTRANS(IR,2)*REAL(MAXRAN_E)
      IR = AINT(R2)
      R2 = R2-REAL(IR)
C      PRINT *,IENER

C      PRINT *,R,E,IR,IENER,ISEED
C     LINEAR INTERPOLATION
c      Y2 = COSTHETA(IREG,IENER,IR)

      X1 = COSTHINTV(IREG,IENER-1)
      X2 = COSTHINTV(IREG,IENER)
      IF (R3 .GT. (E-X1)/(X2-X1)) THEN
         THETA = COSTHETA(IREG,IENER-1,IR)*(1-R2)
     .        + R2 * COSTHETA(IREG,IENER-1,IR+1)
      ELSE
       	 THETA = COSTHETA(IREG,IENER,IR)*(1-R2)
     .        + R2 * COSTHETA(IREG,IENER,IR+1)
      ENDIF
c      COSTH = Y1 + (Y2-Y1)/(X2-X1)*(E-X1)
c      Y1 = COSTHETA(IREG,IENER-1,IR)*(1-R2)
c     .     + R2 * COSTHETA(IREG,IENER-1,IR+1)
c      Y2 = COSTHETA(IREG,IENER,IR)*(1-R2)
c     .     + R2 * COSTHETA(IREG,IENER,IR+1)
CHW      COSTH = COS(COSTH)
      THETA = 180./PI*THETA
      IER = 0

1000  END      
