c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
      SUBROUTINE ELMENT(
     I                  NMODE,
     V                  ELE,IEL,
     O                  EABR,RM,DENS,IER)
 
***   ELMENT - GET INFORMATION ABOUT AN ELEMENT
*
**    ARGUMENT LIST:
*
*     NMODE  - MODE OF OPERATION, INDICATES WHICH VARIABLE IS INPUT:
*               0 ... ELEMENT IDENTIFICATION NUMBER IEL
*               1 ... ELEMENT NAME ELE
*               2 ... ELEMENT IDENTIFICATION NUMBER IEL
*               3 ... ELEMENT NAME ELE
*              ALSO INDICATES THE MEANING OF IEL AND ELE (SEE BELOW)
*     ELE    - ELEMENT NAME
*              IF IEL<93, NMODE=2,3: NAME OF A SUBSTRATE ATOM
*     IEL    - ELEMENT IDENTIFICATION NUMBER
*              IF IEL<93, NMODE=0,1: ATOMIC NUMBER OF A DOPAND
*     RM     - RELATIVE ATOMIC MASS
*     DENS   - TARGET DENSITY (CM-3)
*     IER    - ERROR INDICATOR
*              -1 ... ELEMENT NOT FOUND
*               0 ... NO ERROR
*               1 ... NO VALUE OF ION MASS IN THE TABLE
*               2 ... NO VALUE OF TARGET ATOM MASS IN THE TABLE
*               3 ... NO VALUE OF TARGET DENSITY IN THE TABLE
*               4 ... NO VALUE OF TARGET ATOM MASS AND NO VALUE OF
*                     TARGET DENSITY IN THE TABLE
*
**    SIDE EFFECTS: NO SIDE EFFECTS
**    I/O         : NO I/O IN THIS ROUTINE
*
**    AUTHORS     :
*
*     G. HOBLER       1985-1990   TU VIENNA
*     K. WIMMER       1988-       TU VIENNA
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT INTEGER (I-N)
 
*     GLOBAL VARIABLES

      CHARACTER*70 ELE

*     LOCAL VARIABLES

**    START OF EXECUTABLE CODE

      CHARACTER*2 ELET(100),EABR
      CHARACTER*12 ELEMT(2,100)
 
      DIMENSION RM1T(100),RM2T(100),DENST(100)
 
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=1,10) /
     1     'H ',  1.0078, 1.008,  4.271,'HYDROGEN    ','HYDROGEN    ',
     2     'HE',  4.0026, 4.0026, 1.894,'HELIUM      ','HELIUM      ',
     3     'LI',  7.016,  6.939,  4.597,'LITHIUM     ','LITHIUM     ',
     4     'BE',  9.012,  9.0122,12.046,'BERYLLIUM   ','BERYLLIUM   ',
     5     'B ', 11.009, 10.811, 13.093,'BORON       ','BORON       ',
     6     'C ', 12.   , 12.011, 11.364,'CARBON      ','CARBON      ',
     7     'N ', 14.003, 14.007,  3.481,'NITROGEN    ','NITROGEN    ',
     8     'O ', 15.995, 15.999,  4.302,'OXYGEN      ','OXYGEN      ',
     9     'F ', 18.998, 18.998,  3.522,'FLUORINE    ','FLUORINE    ',
     A     'NE', 19.992, 20.183,  3.585,'NEON        ','NEON        ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=11,20) /
     1     'NA', 22.99 , 22.989,  2.541,'SODIUM      ','NATRIUM     ',
     2     'MG', 23.985, 24.312,  4.302,'MAGNESIUM   ','MAGNESIUM   ',
     3     'AL', 26.982, 26.981,  6.023,'ALUMINIUM   ','ALUMINUM    ', 
     4     'SI', 27.977, 28.086,  4.977,'SILICON     ','SILICON     ',
     5     'P ', 30.994, 30.973,  3.542,'PHOSPHORUS  ','PHOSPHORUS  ',
     6     'S ', 31.97 , 32.064,  3.885,'SULFUR      ','SULFUR      ',
     7     'CL', 34.969, 35.453,  3.22 ,'CHLORINE    ','CHLORINE    ',
     8     'AR', 39.98 , 39.948,  2.488,'ARGON       ','ARGON       ',
     9     'K ', 38.96 , 39.102,  1.329,'POTASSIUM   ','KALIUM      ',
     A     'CA', 39.96 , 40.08 ,  2.014,'CALCIUM     ','CALCIUM     ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=21,30) /
     1     'SC', 44.956, 44.956,  4.015,'SCANDIUM    ','SCANDIUM    ',
     2     'TI', 47.95 , 47.9  ,  5.682,'TITANIUM    ','TITANIUM    ',
     3     'V ', 50.94 , 50.942,  7.213,'VANADIUM    ','VANADIUM    ',
     4     'CR', 51.94 , 51.996,  8.33 ,'CHROMIUM    ','CHROMIUM    ',
     5     'MN', 54.94 , 54.938,  8.15 ,'MANGANESE   ','MANGANESE   ',
     6     'FE', 55.94 , 55.847,  8.483,'IRON        ','IRON        ',
     7     'CO', 58.93 , 58.933,  8.989,'COBALT      ','COBALT      ',
     8     'NI', 57.94 , 58.71 ,  9.125,'NICKEL      ','NICKEL      ',
     9     'CU', 62.93 , 63.54 ,  8.483,'COPPER      ','COPPER      ',
     A     'ZN', 63.93 , 65.37 ,  6.546,'ZINC        ','ZINC        ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=31,40) /
     1     'GA', 68.93 , 69.72 ,  5.104,'GALLIUM     ','GALLIUM     ',
     2     'GE', 73.92 , 72.59 ,  4.428,'GERMANIUM   ','GERMANIUM   ',
     3     'AS', 74.92 , 74.922,  4.597,'ARSENIC     ','ARSENIC     ',
     4     'SE', 79.92 , 78.96 ,  3.65 ,'SELENIUM    ','SELENIUM    ',
     5     'BR', 78.92 , 79.909,  2.562,'BROMINE     ','BROMINE     ',
     6     'KR', 83.92 , 83.8  ,  1.87 ,'KRYPTON     ','KRYPTON     ',
     7     'RB', 84.91 , 85.47 ,  1.077,'RUBIDIUM    ','RUBIDIUM    ',
     8     'SR', 87.91 , 87.62 ,  1.787,'STRONTIUM   ','STRONTIUM   ',
     9     'Y ', 89.91 , 88.905,  3.041,'YTTRIUM     ','YTTRIUM     ',
     A     'ZR', 89.9  , 91.22 ,  4.271,'ZIRCONIUM   ','ZIRCONIUM   ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=41,50) /
     1     'NB', 92.91 , 92.906,  5.576,'NIOBIUM     ','NIOBIUM     ',
     2     'MO', 95.5  , 95.94 ,  6.407,'MOLYBDENUM  ','MOLYBDENUM  ',
     3     'TC',  0.   ,  99.  ,  0.   ,'TECHNETIUM  ','TECHNETIUM  ',
     4     'RU',101.9  ,101.07 ,  7.256,'RUTHENIUM   ','RUTHENIUM   ',
     5     'RH',102.9  ,102.91 ,  7.256,'RHODIUM     ','RHODIUM     ',
     6     'PD',105.9  ,106.4  ,  6.767,'PALLADIUM   ','PALLADIUM   ',
     7     'AG',106.9  ,107.87 ,  5.847,'SILVER      ','SILVER      ',
     8     'CD',113.9  ,112.4  ,  4.597,'CADMIUM     ','CADMIUM     ',
     9     'IN',114.9  ,114.82 ,  3.836,'INDIUM      ','INDIUM      ',
     A     'SN',119.9  ,118.69 ,  3.695,'TIN         ','TIN         ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=51,60) /
     1     'SB',120.9  ,121.75 ,  3.273,'ANTIMONY    ','STIBIUM     ',
     2     'TE',130.   ,127.6  ,  2.938,'TELLURIUM   ','TELLURIUM   ',
     3     'J ',126.9  ,126.9  ,  2.343,'IODINE      ','IODINE      ',
     4     'XE',129.5  ,131.3  ,  1.403,'XENON       ','XENON       ',
     5     'CS',134.   ,132.91 ,  0.86 ,'CAESIUM     ','CAESIUM     ',
     6     'BA',138.   ,137.34 ,  1.544,'BARIUM      ','BARIUM      ',
     7     'LA',139.   ,138.91 ,  2.676,'LANTHANUM   ','LANTHANUM   ',
     8     'CE',140.   ,140.12 ,  2.868,'CERIUM      ','CERIUM      ',
     9     'PR',141.   ,140.91 ,  2.895,'PRASEODYMIUM','PRASEODYMIUM',
     A     'ND',142.   ,144.24 ,  2.932,'NEODYMIUM   ','NEODYMIUM   ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=61,70) /
     1     'PM',  0.   ,147.   ,  0.   ,'PROMETHIUM  ','PROMETHIUM  ',
     2     'SM',152.   ,150.35 ,  3.026,'SAMARIUM    ','SAMARIUM    ',
     3     'EU',153.   ,151.96 ,  2.084,'EUROPIUM    ','EUROPIUM    ',
     4     'GD',157.9  ,157.25 ,  3.026,'GADOLINIUM  ','GADOLINIUM  ',
     5     'TB',159.9  ,158.92 ,  3.136,'TERBIUM     ','TERBIUM     ',
     6     'DY',164.   ,162.5  ,  3.17 ,'DYSPROSIUM  ','DYSPROSIUM  ',
     7     'HO',165.   ,164.93 ,  3.22 ,'HOLMIUM     ','HOLMIUM     ',
     8     'ER',166.   ,167.26 ,  3.273,'ERBIUM      ','ERBIUM      ',
     9     'TM',169.   ,168.93 ,  3.327,'THULIUM     ','THULIUM     ',
     A     'YB',174.   ,173.04 ,  2.428,'YTTERBIUM   ','YTTERBIUM   ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=71,80) /
     1     'LU',175.   ,174.97 ,  3.383,'LUTETIUM    ','LUTETIUM    ',
     2     'HF',180.   ,178.49 ,  4.428,'HAFNIUM     ','HAFNIUM     ',
     3     'TA',181.   ,180.95 ,  5.525,'TANTALUM    ','TANTALUM    ',
     4     'W ',184.   ,183.85 ,  6.32 ,'TUNGSTEN    ','WOLFRAM     ',
     5     'RE',187.   ,186.2  ,  6.805,'RHENIUM     ','RHENIUM     ',
     6     'OS',192.   ,190.2  ,  7.144,'OSMIUM      ','OSMIUM      ',
     7     'IR',193.   ,192.2  ,  7.052,'IRIDIUM     ','IRIDIUM     ',
     8     'PT',195.   ,195.09 ,  6.618,'PLATINUM    ','PLATINUM    ',
     9     'AU',197.   ,196.97 ,  5.904,'GOLD        ','GOLD        ',
     A     'HG',202.   ,200.59 ,  4.069,'MERCURY     ','MERCURY     ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=81,90) /
     1     'TL',205.   ,204.37 ,  3.501,'THALLIUM    ','THALLIUM    ',
     2     'PB',208.   ,207.19 ,  3.291,'LEAD        ','LEAD        ',
     3     'BI',209.   ,208.98 ,  2.827,'BISMUTH     ','BISMUTH     ',
     4     'PO',210.   ,210.   ,  2.653,'POLONIUM    ','POLONIUM    ',
     5     'AT',210.   ,210.   ,  0.   ,'ASTATINE    ','ASTATINE    ',
     6     'RN',222.   ,222.   ,  0.   ,'RADON       ','RADON       ',
     7     'FR',223.   ,223.   ,  0.   ,'FRANCIUM    ','FRANCIUM    ',
     8     'RA',226.   ,226.   ,  1.338,'RADIUM      ','RADIUM      ',
     9     'AC',227.   ,227.   ,  0.   ,'ACTINIUM    ','ACTINIUM    ',
     A     'TH',232.   ,232.   ,  3.026,'THORIUM     ','THORIUM     ' /
      DATA (ELET(I),RM1T(I),RM2T(I),DENST(I),ELEMT(1,I),ELEMT(2,I),
     +      I=91,100) /
     1     'PA',231.   ,231.   ,  4.015,'PROTACTINIUM','PROTACTINIUM',
     2     'U ',238.04 ,238.04 ,  4.818,'URANIUM     ','URANIUM     ',
     3     '93',  0.   ,  0.   ,  0.   ,'93          ','93          ',
     4     '94',  0.   ,  0.   ,  0.   ,'ASCLUSTER   ','ASCLUSTER   ',
     5     '95',  0.   ,  0.   ,  0.   ,'INTERSTITIAL','INTERSTITIAL',
     6     '96',  0.   ,  0.   ,  0.   ,'VACANCY     ','VACANCY     ',
     7     '97',  0.   ,  0.   ,  0.   ,'ELECTRON    ','ELECTRON    ',
     8     '98',  0.   ,  0.   ,  0.   ,'HOLE        ','HOLE        ',
     9     '99',  0.   ,  0.   ,  0.   ,'POTENTIAL   ','POTENTIAL   ',
     A     '00',  0.   ,  0.   ,  0.   ,'ARBITRARY   ','ARBITRARY   ' /
 
 
**    START OF EXECUTABLE CODE
*     ************************
 
 
      IER=-1
 
*     SEARCH ELEMENT IN LIST OR GET ELEMENT NAME

      IF (NMODE.EQ.1 .OR. NMODE.EQ.3) THEN
         IEL=0
 10      IEL=IEL+1
         IF (ELE.EQ.ELET(IEL) .OR. ELE.EQ.ELEMT(1,IEL) .OR.
     1       ELE.EQ.ELEMT(2,IEL)) GOTO 11 
         IF (IEL.LT.100) GOTO 10
         RETURN
 11      ELE=ELEMT(1,IEL)
      ELSE
         IF (IEL.LT.1 .OR. IEL.GT.100) RETURN
         ELE=ELEMT(1,IEL)
      ENDIF

      IER=0
      IF (IEL.GT.92) RETURN
 
*     GET MASS AND DENSITY

      EABR=ELET(IEL)
      IF (NMODE.EQ.0 .OR. NMODE.EQ.1) THEN
         RM=RM1T(IEL)
         DENS=0.
         IF (RM.LE.0.) IER=1
      ELSE
         RM=RM2T(IEL)
         DENS=DENST(IEL)*1.E22
         IF (RM.LE.0.) IER=2
         IF (DENS.LE.0.) IER=3
         IF (RM.LE.0. .AND. DENS.LE.0.) IER=4
      ENDIF
 
      END
