c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
*****************************************************************************
* last modification: CS 26-Feb-1997

      SUBROUTINE GETESC(ESCFAC,EMIN1,EPC1,CHBEG,NUMCH,COUNTS)

***   GETESC-- CORRECTS FOR ESCAPE IN THE SI DETECTOR
*
**    AUTHOR   VINOD AMBROSE
*              TCAD, DIGITAL SEMICONDUCTOR, HUDSON, MA
*
**    INPUT:
*     EMIN1  - MINIMUM ENERGY VALUE FOR THE SPEC. (VALMIN)
*     EPC1   - ENERGY PER CHAN. (VDELT)
*     CHBEG  - STARTING CHAN OF HIST. (IVBEG)
*     NUMCH  - NUMBER OF CHAN IN HIST (NVAL)
*     COUNTS - ARRAY CONTAINING COUNTS IN EACH CHAN. (VALHIS)
*
**    OUTPUT:
*     COUNTS - ARRAY CORRECTED FOR ESCAPE.
*
**    ESCAPE CORRECTION IS MADE USING THE PHOTO-ELECTRIC
*     X-SEC OF E.STORM AND H.ISRAEL. THE ACTUAL AMOUNT OF
*     X-RAYS ESCAPING DEPENDS ON DETECTOR COLLIMATION, AND
*     IS MODELLED BY AN ESCAPE FACTOR (ESCFAC IN PROGRAM).
*     ONCE THIS FACTOR IS SET BY COMPARING WITH MEASURED
*     DATA, IT SHOULDN'T CHANGE UNLESS DETECTOR COLLIMATION
*     IS CHANGED.

**    DECLARATIONS
      DOUBLE PRECISION EMIN1, EPC1, EN, iPHOELEC, NUMION, ESCADDEN
      DOUBLE PRECISION NUMESC, ESCFAC, EXPMN, EMIN, EPC
      INTEGER CHBEG, NUMCH, INDEX, CH, ESCADDCH

C     THE DIMENSION OF COUNTS DEPENDS ON NUMCH
      DOUBLE PRECISION BENCHKEV(1:14), PHOELEC(1:14), COUNTS(NUMCH)

      PARAMETER (EXPMN = 600)

**    CONSTANTS
      NUMBENCH = 14
C     THICKNESS OF SI DETECTOR IN G/CM^2
      DETTHICK = 0.466
C     SI K EDGE IN KEV
      SIEDGE = 1.839
C     SI K ALPHA ENERGY
      SIKEN = 1.74
C     FLUOR. YIELD OF SI K SHELL
      SIWK = 0.047
C     NO CORR MADE BEYOND THIS ENERGY (NEED TO ADD X-SEC VALS)
      MAXEN = 40.0

C     ENERGIES OF TABULATION (KEV)
      DATA BENCHKEV/1.0, 1.5, 1.839, 2.0, 3.0, 4.0, 5.0,
     &              6.0, 8.0, 10.0, 15.0, 20.0, 30.0,
     &              40.0/
 
C     PHOTO-ELECTRIC X-SEC (CM^2/G)
      DATA PHOELEC/1604.808, 544.608, 319.176, 3593.520,
     &             2901.600, 1022.256, 468.720, 249.984,
     &             149.321, 65.174, 33.926, 10.044, 4.174,
     &             1.183/

**    START OF EXECUTABLE CODE
*     ************************

      EMIN = EMIN1/1000
      EPC  = EPC1/1000
      ESCCHANSUB = INT(SIKEN/EPC)

C     LOOP FOR EACH CHAN
      DO 10 CH = 1, NUMCH
C        CALC ENERGY OF CHAN
         EN = (CH-1) * EPC + EMIN
C        DO ONLY IF GREATER THAN SI EDGE
         IF (EN .GE. SIEDGE .AND. EN .LE. MAXEN) THEN
C           FIND TABULATED ENERGY
            DO 20 INDEX = 1, NUMBENCH
               IF (BENCHKEV(INDEX) .GE. EN) GOTO 30
 20         CONTINUE
C           DO INTERPOLATION
 30         iPHOELEC = PHOELEC(INDEX - 1) + (PHOELEC(INDEX) - 
     &                 PHOELEC(INDEX - 1)) * 
     &                 (EN - BENCHKEV(INDEX - 1)) / 
     &                 (BENCHKEV(INDEX) - BENCHKEV(INDEX - 1))
C           FIND NUMBER OF K IONIZATIONS
            FACT = 1
            IF ((iPHOELEC*DETTHICK).LT.EXPMN)
     &           FACT = 1 - EXP(-iPHOELEC * DETTHICK)
            NUMION = COUNTS(CH) * FACT
C           SUBTRACT FROM COUNTS
            NUMESC = ESCFAC * NUMION * SIWK
            COUNTS(CH) = COUNTS(CH) - NUMESC
C           FIND THE LOWER CHAN AND ADD ESCAPE COUNTS
            ESCADDCH = CH - ESCCHANSUB
            COUNTS(ESCADDCH) = COUNTS(ESCADDCH) + NUMESC
         ENDIF
 10   CONTINUE

      RETURN
      END

********************************************************************



