c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
**    GETDAT: 
*     GETED --READS EDGE ENERGY FROM A FILE 
*     GETEC --READS CHARACTERISTIC ENERGY FROM A FILE
*     GETMY --READS OR CALCULATES MAC'S
*     GETXDE--READS (OR CALCULATES) X-RAY DETECTOR EFFICIENCIES
*     GETEDE--READS ELECTRON DETECTOR EFFICIENCIES
*     GETOME--READS FLUORSCENCE YIELDS
*     GETPRO--READS TRANSITION PROBABILITIES
*
*     USED ROUTINES:
*     FPREAD--READS DATA FROM FILES
*     CCLEAR--CLEARS STINGS AND BUFFERS
*     IGETPO--DETERMINES THE POSITION OF THE FIRST TOKEN CHARACTER 
*     ABSKAN--CALCULATES ABSORPTION EDGES (FIT)
*     MYRHO --CALCULATES MAC'S (FIT)
*
*  
****************************************************************************
* LAST MODIFICATION DEC.94

      SUBROUTINE GETED(IMODE,ISHELL,IATNUM,EC,IERR)

***   GETED-- READS EDGE ENERGY FROM A FILE
*
**    AUTHOR A. PFEIFFER DEC 1994
*
**    INPUT:
*     IMODE  - 1: NOT FOUND -> INPUT BY HAND
*              2: NOT FOUNE -> LEAVE WITHOUT VALUE
*     ISHELL - SHELL FOR IONISATION
*     IATNUM - ATOMIC NUMBER
*
**    OUTPUT:
*     EC     -  EEDGE ENERGY [eV!]
*     IERR   -  ERROR FLAG
*
**    CALLS: FPREAD 
      INCLUDE 'TYPE.INC' 
      INCLUDE 'PARAM.INC'
*      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'

      CHARACTER*4 CSHELL(MAXSHELL)
      CHARACTER*1 CYN  
      CHARACTER*80 BUFFER 
      CHARACTER*80 FILENAME
      REAL*4 ELVL(24)      

      DATA CSHELL / 'K',
     &              'L1','L2','L3',
     &              'M1','M2','M3','M4','M5',
     &              'N1','N2','N3','N4','N5','N6','N7'/

      INCLUDE 'PATH.INC'

**    START OF EXECUTABLE CODE:
*     *************************

      IERR = 0
      EC = 0.D0

      IF (IATNUM .LT. 1 .OR. IATNUM .GT. 100 
     &    .OR. ISHELL .LT. 1 .OR. ISHELL .GT. MAXSHELL) GOTO 100

      FILENAME=CFUND(1:IEND(CFUND))//'ATMLVL.TBL'

      OPEN(UNIT=65,NAME=FILENAME,ACCESS='DIRECT',FORM='UNFORMATTED',
     &	   READONLY,SHARED,TYPE='OLD',RECORDSIZE=24)
      
      READ(65'IATNUM) (ELVL(I),I=1,24)

      CLOSE(UNIT=65)

      EC = ELVL(ISHELL)*1.D0

100   CONTINUE      

      IF (EC.EQ.0) THEN
         IF (IMODE.EQ.2) THEN
            IERR=1
            GOTO 900
         ENDIF
         WRITE(6,'(A)')'     '
         WRITE(6,'(A)')' ERROR DURING EDGE ENERGY DETERMINATION !'
         WRITE(6,'(A,I4)')' ATOMIC NUMBER:',IATNUM
         WRITE(6,'(2A)')' SHELL: ',CSHELL(ISHELL)
         WRITE(6,'(A)')' MAY YOU INSERT VALUE BY HAND ? (Y/N)' 
         READ(*,'(A)')CYN
         CALL CCAP(CYN,1)
         IF (CYN.EQ.'Y') THEN 
 10         WRITE(6,'(A)')' INSERT EDGE ENERGY BY HAND [eV]'
            CALL CCLEAR(BUFFER)
            READ(*,'(A)')BUFFER(1:34)
            IB = IGETPO (BUFFER,' ')
            IPOINT = IGETPO (BUFFER,'.')
            IF (IPOINT.GT.33) THEN
               BUFFER(IB:IB)='.'
               IB=IB+1
            ENDIF
            READ(BUFFER(1:IB-1),'(F)',IOSTAT=IER) EC
               IF (IER.NE.0) THEN
                  WRITE(6,'(A)')' *'
                  WRITE(6,'(A)')' * DATA TYPE ERROR [MUFILM]'
                  WRITE(6,'(A)')' *'
                  GOTO 10
               ENDIF
          ELSE  
            CYN=' '
            WRITE(6,'(2A)')' CONTINUE SIMULATION ANYWAY ? (Y/N)'
            READ(*,'(A)')CYN
            CALL CCAP(CYN,1)
            IF (CYN.EQ.'N') STOP
            EC = 1000000.
         ENDIF

      ENDIF
 900  IF (EC.LE.0.) EC = 1000000.         
     
      RETURN
      END 

****************************************************************************

      SUBROUTINE GETAUG(IMODE,ILINE,IATNUM,EAUG,IERR)

***   GETAUG -- CALCULATES AUGERLINE ENERGY
*
**    AUTHOR H.WAGNER MAR 95
*
**    INPUT:  IMODE - 2 = SILENT MODE, 1 = INPUT BY HAND POSSIBLE
*      	      ILINE - 1=KLL,2=LMM,3=MNN
*             IATNUM - ATOMIC NUMBER
**    OUTPUT: EAUG  - APPROX. AUGER ENERGY, 0 IF ERROR OCCURED
*             IERR  - ERROR FLAG = 1 IF ERROR OCCURED
*
      INCLUDE 'TYPE.INC' 
      CHARACTER*80 FILENAME
      CHARACTER*50 BUFFER
      INCLUDE 'PATH.INC'

**    START OF EXECUTABLE CODE:
*     *************************

      EAUG = 0.      
      FILENAME=CFUND(1:IEND(CFUND))//'AUGLINE.DAT'
      OPEN (UNIT=33,IOSTAT=IERR,FILE=FILENAME,STATUS='OLD')
      IF (IERR.NE.0) THEN
          WRITE(6,'(A)')'*'
          WRITE(6,'(2A)')
     &    '* ERROR: COULD NOT OPEN FILE: AUGLINE.DAT'
          WRITE(6,'(A)')'*'
          GOTO 100
      ENDIF

 10   READ(33,'(A)',END=100)BUFFER
      READ(BUFFER,*)IZ,IAUG,E
      IF (IZ.EQ.IATNUM) THEN
          IF (ILINE.EQ.IAUG) THEN
      	     EAUG = E
      	     GOTO 100
          ENDIF
      ENDIF
      GOTO 10

100   CONTINUE
      CLOSE(33)
      IF (EAUG .EQ. 0) IERR = 1

      RETURN
      END


*      INTEGER AUGERXYZ(3,3)
*      DATA AUGERXYZ /1,4,9,3,7,10,4,8,10/

*      IERR = 0
 
*      CALL GETED(IMODE,AUGERXYZ(ILINE,1),IATNUM,EX,IER)
*      IF (IER .NE. 0) IERR = -1
*     CALL GETED(IMODE,AUGERXYZ(ILINE,2),IATNUM,EY,IER)
*      IF (IER .NE. 0) IERR = -1
*      CALL GETED(IMODE,AUGERXYZ(ILINE,2),IATNUM+1,EYP1,IER)
*      IF (IER .NE. 0) IERR = -1
*      CALL GETED(IMODE,AUGERXYZ(ILINE,3),IATNUM,EZ,IER)
*      IF (IER .NE. 0) IERR = -1
*      CALL GETED(IMODE,AUGERXYZ(ILINE,3),IATNUM+1,EZP1,IER)
*      IF (IER .NE. 0) IERR = -1
      
*      IF (IERR .EQ. 0) THEN
*         EAUG = EX-0.5*(EY+EYP1+EZ+EZP1)
*      ELSE
*         EAUG = 0.
*      ENDIF

*      END

****************************************************************************
* LAST MODIFICATION DEC.94

      SUBROUTINE GETEC(IXCHAR,IATNUM,ECC,IERR)

***   GETEC-- READS CHARACTERISTIC X-RAY ENERGY FROM A FILE
*
**    AUTHOR A. PFEIFFER DEC 1994
*
**    INPUT:
*     IXCHAR  - CHAR. X-RAY LINE
*     IATNUM   - ATOMIC NUMBER
**    OUTPUT:
*     ECC     - CHAR. X-RAY ENERGY
*     EXCHAR  - CHARACTARISTIC ENERGY [eV!] WRITEN INTO GLOBAL.INC
* 
      INCLUDE 'TYPE.INC' 
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

      CHARACTER*30 CHAREN(10)
      CHARACTER*4 CLINE(10)
      CHARACTER*1 CYN  
      CHARACTER*80 BUFFER  
      CHARACTER*70 FILENAME

      DATA CHAREN / 'LIN_KA1.DAT',
     &              'LIN_KA2.DAT',
     &              'LIN_KB1.DAT',
     &              'LIN_LA1.DAT',
     &              'LIN_LA2.DAT',
     &              'LIN_LB1.DAT',
     &              'LIN_LB2.DAT',
     &              'LIN_LB3.DAT',
     &              'LIN_MA.DAT', 
     &              'LIN_MB.DAT' /

      DATA CLINE  / 'KA1',
     &              'KA2',
     &              'KB1',
     &              'LA1',
     &              'LA2',
     &              'LB1',
     &              'LB2',
     &              'LB3',
     &              'MA',
     &              'MB' /

      INCLUDE 'PATH.INC'

**    START OF EXECUTABLE CODE:
*     *************************

      FILENAME=CFUND(1:IEND(CFUND))//CHAREN(IXCHAR)
      CALL FPREAD(FILENAME,IATNUM,ECC,.TRUE.,IERR)
      ECC=ECC*1000.

      IF (IERR.NE.0) THEN
         WRITE(6,'(2A)')' ERROR DURING CHARACTERISTIC ',
     &                  ' ENERGY DETERMINATION !'
         WRITE(6,'(A,I4)')' ATOMIC NUMBER:',IATNUM
         WRITE(6,'(A,I4)')' X-RAY LINE NUMBER:',IXCRAY
         WRITE(6,'(A)')' MAY YOU INSERT VALUE BY HAND ? (Y/N)' 
         READ(*,'(A)')CYN
         CALL CCAP(CYN,1)
         IF (CYN.EQ.'Y') THEN 
 10         WRITE(6,'(A)')' INSERT X-RAY ENERGY BY HAND [eV]'
            CALL CCLEAR(BUFFER)
            READ(*,'(A)')BUFFER(1:34)
            IB = IGETPO (BUFFER,' ')
            IPOINT = IGETPO (BUFFER,'.')
            IF (IPOINT.GT.33) THEN
               BUFFER(IB:IB)='.'
               IB=IB+1
            ENDIF
            READ(BUFFER(1:IB-1),'(F)',IOSTAT=IER) ECC
               IF (IER.NE.0) THEN
                  WRITE(6,'(A)')' *'
                  WRITE(6,'(A)')' * DATA TYPE ERROR '
                  WRITE(6,'(A)')' *'
                  GOTO 10
               ENDIF
          ELSE
            CYN=' '
            WRITE(6,'(2A)')' CONTINUE SIMULATION ANYWAY ? (Y/N)'
            READ(*,'(A)')CYN
            CALL CCAP(CYN,1) 
            IF (CYN.EQ.'N') STOP
            ECC = 1000000.
         ENDIF
      ENDIF
      IF (ECC.LE.0.) ECC = 1000000.           

      RETURN
      END 

****************************************************************************
* LAST MODIFICATION OCT 1995

      SUBROUTINE GETOME(ISHELL,IATNUM,OMEGA,IERR)

***   GETOME - READS FLUORESCENCE YIELD FROM A FILE
*
**    AUTHOR: SCHIEBL CHRITIAN
*             OCT 1995
*
**    INPUT:
*        ISHELL  -  SHELL
*        IATNUM  -  ATOMIC NUMBER
*
**    OUTPUT:
*        OMEGA   -  FLUORESCENCE YIELD
*        IERR    -  ERROR FLAG
* 
      INCLUDE 'TYPE.INC' 
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

      CHARACTER*30 COMEGA(10)
      CHARACTER*4  CSHELL(10)
      CHARACTER*1  CYN  
      CHARACTER*80 BUFFER  
      CHARACTER*70 FILENAME
      CHARACTER*10 CBL

CS    FOR THE TIME BEING WE TAKE THE M-FLUORESCENCE YIELDS (UNFORTUNATELY
CS    NOT THAT FOR THE SUB-SHELL - HAVE NO DATA PRESENTLY) FOR ALL M-SUBSHELLS
CS    THIS IS, OF COURSE, ONLY AN APPROXIMATION (GOD KNOWS HOW GOOD!)
      DATA COMEGA / 'OMEGA_K.DAT',
     &              'OMEGA_L1.DAT',
     &              'OMEGA_L2.DAT',
     &              'OMEGA_L3.DAT',
     &              'OMEGA_M5.DAT',
     &              'OMEGA_M5.DAT',
     &              'OMEGA_M5.DAT',
     &              'OMEGA_M5.DAT',
     &              'OMEGA_M5.DAT',
     &              ' ' / 

      DATA CSHELL / 'K',
     &              'L1',
     &              'L2',
     &              'L3',
     &              'M1',
     &              'M2',
     &              'M3',
     &              'M4',
     &              'M5',
     &              ' ' /

      INCLUDE 'PATH.INC'

**    START OF EXECUTABLE CODE:
*     *************************

      OMEGA=1
      IF (ISHELL.GT.9) THEN
         IERR=-1
         RETURN
      ENDIF

      CBL='          '
      FILENAME=CBL//CBL//CBL//CBL//CBL//CBL//CBL

      FILENAME=CFUND(1:IEND(CFUND))//COMEGA(ISHELL)
      CALL FPREAD(FILENAME,IATNUM,OMG,.TRUE.,IERR)

      IF (IERR.NE.0) THEN
         IERR=1
         WRITE(6,'(2A)')' ERROR DURING FLUORESCENCE',
     &                  ' YIELD DETERMINATION !'
         WRITE(6,'(A,I4)')' ATOMIC NUMBER:',IATNUM
         WRITE(6,'(A,I4,3A)')' SHELL NUMBER: ',ISHELL,' (',
     &                    CSHELL(ISHELL),')'
         WRITE(6,'(A)')' MAY YOU INSERT VALUE BY HAND ? (Y/N)' 
         READ(*,'(A)')CYN
         CALL CCAP(CYN,1)
         IF (CYN.EQ.'Y') THEN 
 10         WRITE(6,'(A)')' INSERT FLUORESCENCE YIELD BY HAND'
            CALL CCLEAR(BUFFER)
            READ(*,'(A)')BUFFER(1:34)
            IB = IGETPO (BUFFER,' ')
            IPOINT = IGETPO (BUFFER,'.')
            IF (IPOINT.GT.33) THEN
               BUFFER(IB:IB)='.'
               IB=IB+1
            ENDIF
            READ(BUFFER(1:IB-1),'(F)',IOSTAT=IER) OMG
            IF (IER.NE.0) THEN
               WRITE(6,'(A)')' *'
               WRITE(6,'(A)')' * DATA TYPE ERROR '
               WRITE(6,'(A)')' *'
               GOTO 10
            ENDIF
            IERR=2
         ELSE
            CYN=' '
            WRITE(6,'(2A)')' CONTINUE SIMULATION ANYWAY ? (Y/N)'
            READ(*,'(A)')CYN
            CALL CCAP(CYN,1) 
            IF (CYN.EQ.'N') STOP
            OMG = 1
            IERR=3
         ENDIF
      ENDIF
      OMEGA=OMG

      RETURN
      END 

****************************************************************************
*last modification: 27-Nov-1996

      SUBROUTINE GETPRO(IXLINE,ISHELL,IATNUM,PROB,IERR)

***   GETPRO - READS TRANSITION PROBABILITIES FROM A FILE
*              DEFINITION OF TRANSITION PROBABILITIES USED HERE:
*              THE TRANSITION PROBABILITY USED HERE IS DEFINED AS THE
*              PROBABILITY THAT A CERTAIN X-RAY QUANTUM (CERTAIN LINE)
*              WILL BE GENERATED IF A VACANCY OF THE ACCORDING SHELL
*              ALREADY EXISTS AND IF IT IS ALREADY DECIDED THAT AN X-RAY
*              AND NOT AN AUGER ELECTRON WILL BE GENERATED FILLING UP
*              THIS SHELL AGAIN. CONFUSED?
*              EXAMPLE: PROB FOR L-ALPHA1 IS (RELATIVE INTENSITY OF L-ALPHA1
*              LINE) DEVIDED BY (SUM OVER ALL RELATIVE INTENSITIES OF ALL
*              LINES GENERATED BY A TRANSITION TO THE SAME SHELL (L3))  
*
**    AUTHOR: SCHIEBL CHRITIAN
*             OCT 1995
*
**    INPUT:
*        ISHELL  -  SHELL
*        IXLINE  -  CHARACTERISTIC X-RAY LINE
*        IATNUM  -  ATOMIC NUMBER
*
**    OUTPUT:
*        PROB   -  FLUORESCENCE YIELD
*        IERR    -  ERROR FLAG
* 
      INCLUDE 'TYPE.INC' 
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

      CHARACTER*30 CPROB(10)
      CHARACTER*4  CSHELL(10),CLINE(10)
      CHARACTER*1  CYN  
      CHARACTER*80 BUFFER  
      CHARACTER*70 FILENAME
      CHARACTER*10 CBL

      DATA CPROB  / 'PROB_KA1.DAT',
     &              'PROB_KA2.DAT',
     &              'PROB_KB1.DAT',
     &              'PROB_LA1.DAT',
     &              'PROB_LA2.DAT',
     &              'PROB_LB1.DAT',
     &              'PROB_LB2.DAT',
     &              'PROB_LB3.DAT',
     &              'PROB_MA.DAT',
     &              ' ' / 

      DATA CSHELL / 'K',
     &              'L1',
     &              'L2',
     &              'L3',
     &              'M1',
     &              'M2',
     &              'M3',
     &              'M4',
     &              'M5',
     &              ' ' /

      DATA CLINE  / 'KA1',
     &              'KA2',
     &              'KB1',
     &              'LA1',
     &              'LA2',
     &              'LB1',
     &              'LB2',
     &              'LB3',
     &              'MA',
     &              ' ' /

      INCLUDE 'PATH.INC'

**    START OF EXECUTABLE CODE:
*     *************************

      IERR=0
      PROB=1

      IF (IXLINE.GT.3) THEN
         IF (IXLINE.EQ.4) PROB=0.342
         IF (IXLINE.EQ.5) PROB=0.038
         IF (IXLINE.EQ.6) PROB=0.532
         IF (IXLINE.EQ.7) PROB=0.076
         IF (IXLINE.EQ.8) PROB=0.012
         IF (IXLINE.EQ.9) PROB=0.625
         IF (IXLINE.EQ.10) PROB=0.375
         RETURN
      ENDIF

      CBL='          '
      FILENAME=CBL//CBL//CBL//CBL//CBL//CBL//CBL
      FILENAME=CFUND(1:IEND(CFUND))//CPROB(IXLINE)
      CALL FPREAD(FILENAME,IATNUM,PRB,.TRUE.,IERR)

      IF (IERR.NE.0) THEN
         IERR=1
         WRITE(6,'(2A)')' ERROR DURING TRANSITION',
     &                  ' PROBABILITY DETERMINATION !'
         WRITE(6,'(A,I4)')' ATOMIC NUMBER:',IATNUM
         WRITE(6,'(A,I4,3A)')' LINE NUMBER:  ',IXLINE,' (',
     &                    CLINE(IXLINE),')'
         WRITE(6,'(A)')' MAY YOU INSERT VALUE BY HAND ? (Y/N)' 
         READ(*,'(A)')CYN
         CALL CCAP(CYN,1)
         IF (CYN.EQ.'Y') THEN 
 10         WRITE(6,'(A)')' INSERT TRANSITION PROBABILITY BY HAND'
            CALL CCLEAR(BUFFER)
            READ(*,'(A)')BUFFER(1:34)
            IB = IGETPO (BUFFER,' ')
            IPOINT = IGETPO (BUFFER,'.')
            IF (IPOINT.GT.33) THEN
               BUFFER(IB:IB)='.'
               IB=IB+1
            ENDIF
            READ(BUFFER(1:IB-1),'(F)',IOSTAT=IER) PRB
            IF (IER.NE.0) THEN
               WRITE(6,'(A)')' *'
               WRITE(6,'(A)')' * DATA TYPE ERROR '
               WRITE(6,'(A)')' *'
               GOTO 10
            ENDIF
            IERR=2
         ELSE
            CYN=' '
            WRITE(6,'(2A)')' CONTINUE SIMULATION ANYWAY ? (Y/N)'
            READ(*,'(A)')CYN
            CALL CCAP(CYN,1) 
            IF (CYN.EQ.'N') STOP
            PRB = 1
            IERR=3
         ENDIF
      ENDIF
      PROB=PRB

      RETURN
      END 

****************************************************************************
*last modification: 28-Nov-1996

      SUBROUTINE GETMY (
     I                  IABS,IRAD,IXLINE,EC,
     O                  RHOMY              )

C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C      IMPLICIT INTEGER (I-N) 
      INCLUDE 'TYPE.INC'

***  GETMY - GETS THE MASS ABSORPTION COEFFICIENT BACK TO CALLING ROUTINE 
*
**    CHRISTIAN SCHIEBL
*     CEC-VIENNA
*     JANUARY 1993
*
**    INPUT PARAMETERS:
*
*         IABS.......ATOMIC NUMBER OF THE ABSORBING MATERIAL (ELEMENT)
*         IRAD.......ATOMIC NUMBER OF FOR RADIATION EMITTER MATERIAL
*         IXLINE.....LINE (K,L,M) OF EMITTED RADIATION
*         EC.........CHARACTERISTIC ENERGY OF EMITTED RADIATION [KEV!]
*
**    OUTPUT PARAMETERS:
*
*         RHOMY......MASS ABSORPTION COEFFICIENT
*
*     INTERNAL SUCCESS INDICATOR
*
*         IFOUND.....1 FOUND FROM FILE          
*                    2 CALCULATED FROM FIT
*                   -1 NO MASS ABSORPTION COEFFICIENT FOUND
*
 
      CHARACTER*80 BUFFER
      CHARACTER*70 FILENAME

***** START OF EXECUTABLE CODE *****
*     ************************
*

      INCLUDE 'PATH.INC'

**    ATTENTION: THE LINE NUMBERS IN THE FILE 'MYRHO.DAT' DO NOT
*                CORRESPOND TO THE NUMBER SUBMITTED TO THIS ROUTINE (ILINE)
*                IN 'MYRHO.DAT' 0 STANDS FOR A K-ALPHA LINE, 1 FOR A L-ALPHA
*                LINE AND 2 FOR A M-ALPHA LINE
*                THEREFORE SOME TRANSFORMATION IS NEEDED
      ILINE=IXLINE
      IF ((ILINE.EQ.1).OR.(ILINE.EQ.2)) THEN
         ILINE=0
      ELSE IF ((ILINE.EQ.4).OR.(ILINE.EQ.5)) THEN
         ILINE=1
      ELSE IF ((ILINE.EQ.9).OR.(ILINE.EQ.10)) THEN
         ILINE=2
      ELSE
         ILINE=-1
      ENDIF

      IFOUND=-1
      FILENAME=CFUND(1:IEND(CFUND))//'MYRHO.DAT'
*      OPEN (UNIT=33,IOSTAT=IERR,FILE='EPMA$FUNDPAR:MYRHO.DAT',
      OPEN (UNIT=33,IOSTAT=IERR,FILE=FILENAME,STATUS='OLD')
      IF (IERR.NE.0) THEN
          WRITE(6,'(A)')'*'
          WRITE(6,'(2A)')
     &    '* ERROR: COULD NOT OPEN FILE: MYRHO.DAT'
          WRITE(6,'(A)')'*'
          GOTO 100
      ENDIF

      READ(33,'(A)')BUFFER
 10   READ(33,'(A)',END=100)BUFFER
      READ(BUFFER,*)IE,IL,IA,RM
      IF (IE.EQ.IRAD) THEN
          IF (IA.EQ.IABS) THEN
              IF (IL.EQ.ILINE) THEN
                  RHOMY=RM
                  IFOUND=1
                  GOTO 200
              ENDIF
          ENDIF
      ENDIF
      GOTO 10

 100  CONTINUE
cap
C      WRITE(6,'(A)')' INPUT MYRHO'
C      WRITE(6,*) IABS,IRAD,ILINE,EC
C      READ(*,*)RM 

      CALL MYRHO(IABS,EC,RM)
      RHOMY=RM
      IFOUND=2

 200  CLOSE(33)

cs-test
cs      write(6,'(f15.5,f18.5)')ec,rhomy
cs      read(*,*)

      RETURN
      END

****************************************************************************
*last modification: 29-Apr-1997

      SUBROUTINE GETXDE ()

***   GETXDE - READS FROM FILE OR CALCULATES X-RAY DETECTOR EFFICIENCIES 
*
**    AUTHOR: CHRISTIAN SCHIEBL  JUN 1995
*
**    INPUT: NONE
*
**    OUTPUT: NONE
*
*     REMARK: ALL ENERGIES IN DETECTOR EFFICIENCY FILE IN KEV
*             PROGRAM INTERN IN EV

      INCLUDE 'TYPE.INC' 
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'DMA.INC'

      CHARACTER*80 BUFFER

      INCLUDE 'PATH.INC'

***   START OF EXECUTABLE CODE
*     ========================

      IF (NXCONT.GT.0) CALL CHKBND (7,NXCONT,'GETXDE','IXCONTOFXCHAR')
      DO 100 IXC=1,NXCHAR+NXCONT-NXPTOB
         IF (IXC.GT.NXCHAR) THEN
            NN=0
            IDIFF=IXC-NXCHAR
            DO 10 K1K=1,NXCONT
               IF (IXCONTOFXCHAR(K1K).GT.NXCHAR) THEN
                  NN=NN+1
                  IF (NN.EQ.IDIFF) THEN
                     INCXC=IXCONTOFXCHAR(K1K)
                     INXCONTABLE=K1K
                  ENDIF
               ENDIF
 10         CONTINUE
         ELSE
             INCXC=IXC
         ENDIF
         IF ((NXCHAR.GT.0).AND.(IXC.LE.NXCHAR))
     &      CALL CHKBND (6,INCXC,'GETXDE','ENEROFXCHAR')
         IF ((NXCONT.GT.0).AND.(IXC.GT.NXCHAR))
     &      CALL CHKBND (7,INXCONTABLE,'GETXDE','ENEROFXCONT')
         MXINDX=NDMAXDET*(INCXC-1)+(NXDET+1)
         CALL CHKBND (13,MXINDX,'GETXDE','XDETEFFOFXCHAR')
         DO 50 IXDET=0,NXDET
            IXEFFTP=0
            NFLOW=0
            INDX=NDMAXDET*(INCXC-1)+(IXDET+1)
            XDETEFFOFXCHAR(INDX)=1
            IF (XDEFFTYPE(IXDET).EQ.0) THEN
*              IDEAL DETECTOR - DETECTOR EFFICIENCY IS ONE
CS               XDETEFFOFXCHAR(IXDET,INCXC)=1
               XDETEFFOFXCHAR(INDX)=1
            ELSE IF (XDEFFTYPE(IXDET).GT.0) THEN
*             READ FROM FILE OR CALCULATE (OR BOTH)
 14           IF (XDEFFTYPE(IXDET).EQ.12) THEN
                IF (IXEFFTP.EQ.0) THEN
                   IXEFFTP=2
                ELSE
                   IXEFFTP=10
                ENDIF
              ELSE
                IXEFFTP=XDEFFTYPE(IXDET)
              ENDIF
              IF (IXEFFTP.EQ.2) THEN
*               CALCULATE DETECTOR EFFICIENCY FROM XLAYTH, XLAYDN, ETC.
                IF (IXC.LE.NXCHAR) THEN
                   ECCHAR=ENEROFXCHAR(INCXC)/1000
                ELSE
                   ECCHAR=ENEROFXCONT(INXCONTABLE)/1000
                ENDIF
                EPSILON=1
                DO 20 ILAY=1,NDLAY(IXDET)
                  FMUERO=0
                  DO 15 IEL=1,NXLAYMAT(IXDET,ILAY)
                     CALL MYRHO(LAYXEL(IXDET,ILAY,IEL),ECCHAR,FMRHO)
                     FMUERO=FMUERO+XLAYAF(IXDET,ILAY,IEL)*FMRHO
 15               CONTINUE
                  IF (ILAY.LT.NDLAY(IXDET)) THEN
                     EPSILON=EPSILON*
     &                       DEXP(-FMRHO*XLAYDN(IXDET,ILAY)*
     &                       XLAYTH(IXDET,ILAY)*1.0D-4)
                  ELSE IF (ILAY.EQ.NDLAY(IXDET)) THEN
                     EPSILON=EPSILON*
     &                       (1.-DEXP(-FMRHO*XLAYDN(IXDET,ILAY)*
     &                       XLAYTH(IXDET,ILAY)*1.0D-4))
                  ENDIF
 20             CONTINUE
CS                XDETEFFOFXCHAR(IXDET,INCXC)=EPSILON
                XDETEFFOFXCHAR(INDX)=EPSILON
CS                IF ((XDETEFFOFXCHAR(IXDET,INCXC).LT.0).OR.
CS     &              (XDETEFFOFXCHAR(IXDET,INCXC).GT.1)) THEN
                IF ((XDETEFFOFXCHAR(INDX).LT.0).OR.
     &              (XDETEFFOFXCHAR(INDX).GT.1)) THEN
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(2A)')
     &                        ' CALCULATED DETECTOR EFFICIENCY OUT O',
     &                        'F RANGE !'
                  IF (IXC.LE.NXCHAR) THEN
                     WRITE(6,'(A,I2,A,I3,A)')
     &                  ' DETECTOR EFFICIENCY FOR LINE ',
     &                  XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                  ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                  ELSE
                     WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' keV IS SET TO ONE!'
                  ENDIF
                  WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=1
                  XDETEFFOFXCHAR(INDX)=1
                ENDIF
              ELSE IF (IXEFFTP.EQ.10) THEN
*               READ DETECTOR EFFICIENCY FROM FILE
                OPEN (43,FILE=CFUND(1:IEND(CFUND))//DXFDEFF(IXDET),
     &                FORM='FORMATTED',STATUS='UNKNOWN',
     &                READONLY,SHARED,IOSTAT=IOS)
                IF (IOS.NE.0) THEN
                   OPEN (43,FILE=DXFDEFF(IXDET),FORM='FORMATTED',
     &                   STATUS='UNKNOWN',READONLY,SHARED,IOSTAT=IOS)
                ENDIF
                IF (IOS.NE.0) THEN
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(3A)')' COULD NOT OPEN X-RAY DETECTOR',
     &                           ' EFFICIENCY FILE: ',
     &                           DXFDEFF(IXDET)(1:IEND(DXFDEFF(IXDET)))
                  IF (IXC.LE.NXCHAR) THEN
                     WRITE(6,'(A,I2,A,I3,A)')
     &                 ' DETECTOR EFFICIENCY FOR LINE ',
     &                 XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                 ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                  ELSE
                     WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' keV IS SET TO ONE!'
                  ENDIF
                  WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=1
CS                  XDETEFFOFXCHAR(INDX)=1
                  GOTO 50
                ENDIF
  30            READ(43,'(A)',IOSTAT=IOS) BUFFER
                IF (IOS.LT.0) THEN
*                 END OF FILE
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(3A)')' END OF FILE DETECTED BEFORE',
     &                           ' READING DETECTOR EFFICIENCY: ',
     &                           DXFDEFF(IXDET)(1:IEND(DXFDEFF(IXDET)))
                  IF (IXC.LE.NXCHAR) THEN
                     WRITE(6,'(A,I2,A,I3,A)')
     &                        ' DETECTOR EFFICIENCY FOR LINE ',
     &                        XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                        ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                  ELSE
                     WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' keV IS SET TO ONE!'
                  ENDIF
                  WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=1
CS                  XDETEFFOFXCHAR(INDX)=1
                  GOTO 48
                ENDIF
                IF (IOS.GT.0) THEN
*                 ERROR DURING READ
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(3A)')' ERROR OCCURD DURING READING',
     &                           ' DETECTOR EFFICIENCY: ',
     &                           DXFDEFF(IXDET)(1:IEND(DXFDEFF(IXDET)))
                  IF (IXC.LE.NXCHAR) THEN
                     WRITE(6,'(A,I2,A,I3,A)')
     &                     ' DETECTOR EFFICIENCY FOR LINE ',
     &                     XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                     ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                  ELSE
                     WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' keV IS SET TO ONE!'
                  ENDIF
                  WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=1
CS                  XDETEFFOFXCHAR(INDX)=1
                  GOTO 48
                ENDIF
                IF (BUFFER(1:1).EQ.'#') GOTO 30
                READ(BUFFER,*,IOSTAT=IOS)ENER1,DEFF1
                IF (IOS.NE.0) THEN
*                 DATA CONVERSION ERROR
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(3A)')' DATA CONVERSION ERROR DURING',
     &                           ' READING DETECT. EFFICIENCY: ',
     &                           DXFDEFF(IXDET)(1:IEND(DXFDEFF(IXDET)))
                  WRITE(6,'(A)')' READING NEXT RECORD'
C                  WRITE(6,'(A,I2,A,I3,A)')
C     &                   ' DETECTOR EFFICIENCY FOR LINE ',
C     &                   XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
C     &                   ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
C                  WRITE(6,'(A)')
C     &                           ' Protests are useless - SESAME ;-)'
CSC                  XDETEFFOFXCHAR(IXDET,INCXC)=1
C                  XDETEFFOFXCHAR(INDX)=1
C                  GOTO 50
                   GOTO 30
                ENDIF
                ENER1=ENER1*1000
                IF (IXC.GT.NXCHAR) THEN
                  ENERLINE=ENEROFXCONT(INXCONTABLE)
                ELSE
                  ENERLINE=ENEROFXCHAR(IXC)
                ENDIF
                IF (ENER1.LT.ENERLINE) THEN
                  ENL=ENER1
                  DEFL=DEFF1
                  NFLOW=1
                  ENER1=0
                  DEFF1=-1
                  GOTO 30
                ENDIF
                IF (ENER1.EQ.ENERLINE) THEN
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=DEFF1
                  XDETEFFOFXCHAR(INDX)=XDETEFFOFXCHAR(INDX)*DEFF1
                  GOTO 45
                ENDIF
                IF (NFLOW.EQ.0) THEN
*                 NO LOWER VALUE FOR INTERPOLATION
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(2A)')' NO DETECTOR EFFICIENCY VALUE',
     &                 ' FOUND AT LOW ENERGY SIDE FOR INTERPOLATION '
                  WRITE(6,'(2A)')' DETECTOR EFFICIENCY FILE: ',
     &                 DXFDEFF(IXDET)(1:IEND(DXFDEFF(IXDET)))
                  IF (IXC.LE.NXCHAR) THEN
                     WRITE(6,'(A,I2,A,I3,A)')
     &                 ' DETECTOR EFFICIENCY FOR LINE ',
     &                 XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                 ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                  ELSE
                     WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' eV IS SET TO ONE!'
                  ENDIF
                  WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=1
CS                  XDETEFFOFXCHAR(INDX)=1
                  GOTO 48
                ENDIF
                IF (DEFF1.LT.0) THEN
*                 NO UPPER VALUE FOR INTERPOLATION
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(2A)')' NO DETECTOR EFFICIENCY VALUE ',
     &                 'FOUND AT HIGH ENERGY SIDE FOR INTERPOLATION '
                  WRITE(6,'(2A)')' DETECTOR EFFICIENCY FILE: ',
     &                 DXFDEFF(IXDET)(1:IEND(DXFDEFF(IXDET)))
                  IF (IXC.LE.NXCHAR) THEN
                     WRITE(6,'(A,I2,A,I3,A)')
     &                 ' DETECTOR EFFICIENCY FOR LINE ',
     &                 XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                 ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                  ELSE
                     WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' keV IS SET TO ONE!'
                  ENDIF
                  WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=1
CS                XDETEFFOFXCHAR(INDX)=1
                  GOTO 48
                ENDIF
*               INTERPOLATION
CS                XDETEFFOFXCHAR(IXDET,INCXC)=
                XDETEFFOFXCHAR(INDX)=XDETEFFOFXCHAR(INDX)*
     &            (DEFL+(DEFF1-DEFL)*(ENERLINE-ENL)/
     &            (ENER1-ENL))
CS 45             IF ((XDETEFFOFXCHAR(IXDET,INCXC).GT.1).OR.
CS     &             (XDETEFFOFXCHAR(IXDET,INCXC).LT.0)) THEN
 45             IF ((XDETEFFOFXCHAR(INDX).GT.1).OR.
     &              (XDETEFFOFXCHAR(INDX).LT.0)) THEN
*                 DETECTOR EFFICIENCY OUT OF RANGE
                  WRITE(6,'(A)')'    '
                  WRITE(6,'(A)')' %%% WARNING %%%'
                  WRITE(6,'(4A)')' THE DETECTOR EFFICIENCY VALUE',
     &                 ' FOUND IN FILE ',
     &                 DXFDEFF(IXDET)(1:IEND(DXFDEFF(IXDET))),
     &                 'IS OUT OF RANGE'
                  IF (IXC.LE.NXCHAR) THEN
                     WRITE(6,'(A,I2,A,I3,A)')
     &                 ' DETECTOR EFFICIENCY FOR LINE ',
     &                 XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                 ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                  ELSE
                     WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' keV IS SET TO ONE!'
                  ENDIF
                  WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                  XDETEFFOFXCHAR(IXDET,INCXC)=1
                  XDETEFFOFXCHAR(INDX)=1
                ENDIF
  48            CLOSE(43)
              ELSE
                WRITE(6,'(A)')'    '
                WRITE(6,'(A)')' %%% WARNING %%%'
                WRITE(6,'(2A)')' Incorrect specification of x-ray',
     &               ' detector efficiency '
                IF (IXC.LE.NXCHAR) THEN
                  WRITE(6,'(A,I2,A,I3,A)')
     &                     ' DETECTOR EFFICIENCY FOR LINE ',
     &                     XCHAROFXCHAR(IXC),' OF ELEMENT Z=',
     &                     ATNUM(ELEMOFXCHAR(IXC)),' IS SET TO ONE!'
                ELSE
                  WRITE(6,'(2A,F14.6,A)')
     &                  ' DETECTOR EFFICIENCY FOR BACKGROUND',
     &                  ' AT ENERGY =',ENEROFXCONT(INXCONTABLE)/1000,
     &                  ' keV IS SET TO ONE!'
                ENDIF
                WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
CS                XDETEFFOFXCHAR(IXDET,INCXC)=1
                XDETEFFOFXCHAR(INDX)=1
              ENDIF
              IF ((XDEFFTYPE(IXDET).EQ.12).AND.(IXEFFTP.EQ.2)) GOTO 14
            ENDIF
  50     CONTINUE
 100  CONTINUE

      IF(LTEST) THEN
        WRITE(6,'(A)')'         '
        WRITE(6,'(A)')'   I    KK   INDX0  DET0  INDX1   DET1  ENERGY'
        DO 910 I=1,NDMAXCONT+NDMAXCHAR-NDMAPTOB
          INDX=NDMAXDET*(I-1)+1
          IF (I.LE.NDMAXCHAR) THEN
            ENER=ENEROFXCHAR(I)
          ELSE
            KK=0
            DO 905 II=1,NDMAXCONT
               IF (IXCONTOFXCHAR(II).EQ.I) KK=II
 905        CONTINUE
            ENER=ENEROFXCONT(KK)
          ENDIF
          ENER=ENER/1000
          WRITE(6,'(3I5,F9.4,I5,2F9.4)')I,KK,INDX,XDETEFFOFXCHAR(INDX),
     &                         INDX+1,XDETEFFOFXCHAR(INDX+1),ENER
 910    CONTINUE
      ENDIF

 999  RETURN
      END

****************************************************************************
*last modification: 26-Feb-1997

      SUBROUTINE GETFUDGE ()

***   GETFUDGE - GET FUDGE FACTORS FOR CHAR. AND CONT. X-RAYS
*
**    AUTHOR: CHRISTIAN O. SCHIEBL
*             CONSULTENT
*             KREMS/HUDSON, FEB. 1997
*
**    INPUT: NONE
*
**    OUTPUT: NONE
*

      INCLUDE 'TYPE.INC' 
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'DMA.INC'

      DOUBLE PRECISION FUDGE(MAXXCHAR)

cs      DATA FUDGE / 0.65,0.65,0.8,2.5,2.5,2.0,2.0,2.0,22,22 /
      DATA FUDGE / 1,1,1,1,1,1,1,1,22,22 /

***   START OF EXECUTABLE CODE
*     ========================

      IF (LWIEDKIRK) THEN
         BREMSFUDGE=1.0
cs         BREMSFUDGE=0.45
      ELSE IF (LKQP) THEN
         BREMSFUDGE=1.0
cs         BREMSFUDGE=0.66
      ELSE
         BREMSFUDGE=1
      ENDIF

      DO 10 I=1,NXCHAR
         FFUDGEOFXCHAR(I)=FUDGE(XCHAROFXCHAR(I))
 10   CONTINUE

      RETURN
      END

****************************************************************************

        SUBROUTINE MYRHO (Z,ECA,MUERO)
        COMMON/EDGE/EK,EL1,EL2,EL3,EM1,EM2,EM3,EM4,EM5,EN1,EN2,EN3,
     A              EN4,EN5

***     CHRISTIAN SCHIEBL
*       TECHNICAL UNIVERSITY OF VIENNA
*       APRIL 1986
*
*       THE CALCULATION OF THE MASS ABSORPTION COEFFICIENTS FOR ENERGIES
*       BETWEEN 1 KEV AND 50 KEV AND FOR Z BETWEEN 11 AND 83 WERE
*       PUPLISHED BY WERNISCH ET AL. (1984)
*       DATA FOR THE FITS BELOW Z=11 AND ENERGIES ABOUT 1.3 KEV ARE
*       GIVEN BY MC MASTER ET AL. (1969)
*       DATA FOR THE FITS BELOW Z=11 AND ENERGIES BELOW 1.3 KEV ARE
*       GIVEN BY HENKE ET AL. (1974)
*       DATA FOR THE FITS FOR Z BETWEEN 11 AND 83 AND ENERGIES BELOW
*       1 KEV ARE GIVEN BY HENKE ET AL. (1974)
*       BECAUSE OF THE DIFFERENCE OF HENKE'S AND MC MASTER'S VALUES
*       THERE IS AN ALGORITHM TO MAKE THE FITS CONTINOUS AT 1.3 KEV
*       RESPECTIVELY AT 1 KEV.
*
**      INPUT:
*       Z     ... ATOMIC NUMBER
*       ECA   ... CHARACTERISTIC ENERGY [KEV]
*
**      OUTPUT:
*       MUERO ... MASS ABSORPTION COEFFICIENT FOR X-RAYS OF ENERGY
*                 ECA IN MATERIAL WITH ATOMIC NUMBER Z

        INTEGER           XY,XYZ,Z
        DOUBLE PRECISION  D,K1,K2,K3,H,ECA0,MUERO0
        DOUBLE PRECISION  AMYRO,BMYRO,MULT,ECA,MUERO
        DOUBLE PRECISION  EK,EL1,EL2,EL3,EM1,EM2,EM3,EM4,EM5
        DOUBLE PRECISION  EN1,EN2,EN3,EN4,EN5

        INCLUDE 'PATH.INC'

*       READ ALL EDGE ENERGIES FOR ELEMENT WITH ATOMIC NUMBER Z FROM 
*       DATA FILES
CS      CALCULATE THE EDGE ENERGIES (TO KEEP THE VALUES FOR THE N-SHELL)
        CALL ABSKAN (Z)
        
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_K.DAT',
CHW     &                                Z,EK,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_L1.DAT',
CHW     &                               Z,EL1,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_L2.DAT',
CHW     &                               Z,EL2,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_L3.DAT',
CHW     &                               Z,EL3,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_M1.DAT',
CHW     &                               Z,EM1,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_M2.DAT',
CHW     &                               Z,EM2,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_M3.DAT',
CHW     &                               Z,EM3,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_M4.DAT',
CHW     &                               Z,EM4,.FALSE.,IERR)
CHW        CALL FPREAD(CFUND(1:IEND(CFUND))//'CRIT_M5.DAT',
CHW     &                               Z,EM5,.FALSE.,IERR)

CHW     USE GETED WITH BINARY FILE INSTEAD 

        CALL GETED(2,1,Z,EK ,IERR)
        CALL GETED(2,2,Z,EL1,IERR)
        CALL GETED(2,3,Z,EL2,IERR)
        CALL GETED(2,4,Z,EL3,IERR)
        CALL GETED(2,5,Z,EM1,IERR)
        CALL GETED(2,6,Z,EM2,IERR)
        CALL GETED(2,7,Z,EM3,IERR)
        CALL GETED(2,8,Z,EM4,IERR)
        CALL GETED(2,9,Z,EM5,IERR)

*       TRANSFORM FROM eV INTO keV
        EK=EK/1000
        EL1=EL1/1000
        EL2=EL2/1000
        EL3=EL3/1000
        EM1=EM1/1000
        EM2=EM2/1000
        EM3=EM3/1000
        EM4=EM4/1000
        EM5=EM5/1000

        MULT=1.
        IF (Z.LE.10) GOTO 600
        IF (ECA.GE.0.9.AND.ECA.LE.1.) THEN
                XY=0
        ELSE
                XY=1
        END IF
        IF (ECA.GE.1.) GOTO 300

C       --------------------------------------------------------------
C       --------------------------------------------------------------
C       FROM Z=11 TO Z=83 ; FROM ECA=0.15 KEV TO ECA=1 KEV
C       --------------------------------------------------------------
        IF (Z.GE.11.AND.Z.LE.13) GOTO 9
        IF (ECA.GT.EL3.AND.Z.GE.14)  GOTO 9
        IF (EM5.EQ.0..AND.Z.GE.16.AND.Z.LE.36) GOTO 30
        IF (ECA.GT.EM5.AND.Z.GE.16.AND.Z.LE.36) GOTO 30
        IF (ECA.GT.EM5.AND.Z.GE.37)  GOTO 30
        IF (ECA.GE.0.15.AND.Z.GE.39) GOTO 150
C       --------------------------------------------------------------
C       REGION B  (FROM THE L3-EDGE TO 1 KEV)

    9   XYZ=0
   10   XYZ=XYZ+1
        D=-3.1054+1.5342*Z-0.081*Z**2+0.002055*Z**3-1.978E-5*Z**4
        K3=0
        IF (Z.GE.11.AND.Z.LE.15) THEN
          K1=-3.0769-0.0225*Z+0.002614*Z**2
          K2=-1.9428+0.4792*Z-0.0432*Z**2+0.0012*Z**3
        END IF
        IF (Z.GE.16.AND.Z.LE.26) THEN
          K1=-36.4751+5.9997*Z-0.4003*Z**2+0.0119*Z**3-1.329E-4*Z**4
        END IF
        IF (Z.EQ.19) THEN
          K1=-2.5078
          K2=-0.2095
        END IF
        IF (Z.EQ.16) K2=-0.5008
        IF (Z.EQ.17) K2=-0.3216
        IF (Z.EQ.18) K2=-0.3465
        IF (Z.EQ.19) K2=-0.2095
        IF (Z.EQ.24) K2=-0.2431
        IF (Z.EQ.25) K2=-0.1653
        IF (Z.EQ.26) K2=-0.1965
        IF (Z.GE.20.AND.Z.LE.23) THEN
          K2=-32.571+4.4098*Z-0.2004*Z**2+0.003034*Z**3
        END IF
        IF (Z.GE.27.AND.Z.LE.29) THEN
          K1=-2.4638
          K2=0.3509
        END IF
        IF (Z.EQ.15.OR.Z.EQ.16.OR.Z.EQ.28.OR.Z.EQ.29) H=0.88
        IF (Z.GE.17.AND.Z.LE.19) H=0.64
        IF (Z.GE.20.AND.Z.LE.23) H=0.76
        IF (Z.GE.24.AND.Z.LE.27) H=0.79
        IF (ECA.GE.EL1.AND.ECA.LE.1.) H=1.
        IF (ECA.GE.EL3.AND.ECA.LE.EL2) H=0.611
        GOTO 200
C       --------------------------------------------------------------
C       REGION C  (FROM THE M5-EDGE TO THE L3-EDGE)

   30   XYZ=0
   31   XYZ=XYZ+1
        IF (Z.EQ.17) THEN
                D=  5.7278
                K1=-2.2982
                K2=-0.2316
        END IF
        IF (Z.EQ.16) THEN
                K1=-3.2011
                K2=-0.4222
        END IF
        IF (Z.EQ.18) THEN
                K1=-3.6567
                K2=-0.603
        END IF
        IF (Z.EQ.16.OR.Z.EQ.18.OR.Z.EQ.19) THEN
                D=131.286-15.085*Z+0.4473*Z**2
        END IF
        IF (Z.EQ.20) THEN
                D=  5.5063
                K1=-3.3499
        END IF
        IF (Z.EQ.19) THEN
                K1=-2.4522
                K2=-0.3365
        END IF
        IF (Z.GE.21.AND.Z.LE.61) THEN
          D=-3.442+0.8921*Z-0.0309*Z**2+5.519E-4*Z**3-3.82E-6*Z**4
        END IF
        IF (Z.GE.21.AND.Z.LE.24) THEN
          K1=-17.2813+1.1959*Z-0.0242*Z**2
        END IF
        IF (Z.EQ.25) K1=-2.4988
        IF (Z.EQ.26) K1=-2.4868
        IF (Z.GE.27.AND.Z.LE.44) THEN
          K1=3.6571-0.6266*Z+0.0227*Z**2-3.436E-4*Z**3+1.895E-6*Z**4
        END IF
        IF (Z.EQ.29) K1=-2.464
        IF (Z.GE.45.AND.Z.LE.47) THEN
          K1=-26.681+1.0429*Z-0.0111*Z**2
        END IF
        IF (Z.GE.48.AND.Z.LE.61) THEN
          K1=3.8737-0.2702*Z+0.003*Z**2
        END IF
        IF (Z.GE.20.AND.Z.LE.26) THEN
          K2=-308.2929+51.9196*Z-3.2838*Z**2+0.0923*Z**3-9.724E-4*Z**4
        END IF
        IF (Z.EQ.24) K2=-0.3109
        IF (Z.EQ.27) K2=-0.326
        IF (Z.EQ.28) K2=-0.3244
        IF (Z.GE.29.AND.Z.LE.33) THEN
          K2=33.6624-3.3683*Z+0.1161*Z**2-1.536E-3*Z**3+4.93E-6*Z**4
        END IF
        IF (Z.EQ.34) K2=-0.319
        IF (Z.EQ.35) K2=-0.341
        IF (Z.EQ.40) K2=-0.3188
        IF (Z.GE.36.AND.Z.LE.39) THEN
          K2=3.2816-0.1747*Z+0.0021*Z**2
        END IF
        IF (Z.GE.41.AND.Z.LE.44) THEN
          K2=0.0365-0.0151*Z+1.667E-4*Z**2
        END IF
        IF (Z.EQ.42) K2=-0.3225
        IF (Z.GE.45.AND.Z.LE.61) THEN
          K2=-232.3419+10.1344*Z-0.0295*Z**2-0.003549*Z**3+3.88E-5*Z**4
        END IF
        IF (Z.EQ.49) K2=-0.4677
        K3=0

        IF (ECA.GE.EM1.AND.ECA.LE.1.) H=1.
        IF (ECA.GE.EM2.AND.ECA.LE.EM1) GOTO 100
        IF (ECA.GE.EM3.AND.ECA.LE.EM2) GOTO 110
        IF (ECA.GE.EM4.AND.ECA.LE.EM3) GOTO 120
        IF (ECA.GE.EM5.AND.ECA.LE.EM4) H=0.443
        GOTO 200

  100   IF (Z.EQ.31.OR.Z.EQ.32) H=0.918
        IF (Z.EQ.33) H=0.882
        IF (Z.EQ.34) H=0.949
        IF (Z.EQ.35) H=0.78
        IF (Z.GE.36.AND.Z.LE.44) H=0.964
        IF (Z.EQ.45) H=0.867
        IF (Z.GE.46.AND.Z.LE.50) H=0.914
        IF (Z.GE.51.AND.Z.LE.54) H=0.963
        GOTO 200

  110   IF (Z.GE.34.AND.Z.LE.50) H=0.844
        IF (Z.EQ.51) H=0.784
        IF (Z.EQ.52) H=0.96
        IF (Z.GE.53.AND.Z.LE.55) H=1.07
        GOTO 200

  120   IF (Z.EQ.34) H=0.782
        IF (Z.EQ.35) H=-0.0156+3.5845*ECA
        IF (Z.EQ.36) H=-0.1361+4.3978*ECA
        IF (Z.EQ.37) H=-0.4873+5.7511*ECA
        IF (Z.EQ.38) H=-0.9702+7.5829*ECA
        IF (Z.EQ.39) H=2.2594-19.2919*ECA+50.1692*ECA**2
        IF (Z.EQ.40) H=0.0882+2.1933*ECA
        IF (Z.EQ.41.OR.Z.EQ.42) H=0.683
        IF (Z.EQ.43) H=0.093+1.6568*ECA
        IF (Z.GE.44.AND.Z.LE.48) H=0.723
        IF (Z.EQ.49.OR.Z.EQ.50) H=0.806
        IF (Z.EQ.51) H=0.872
        IF (Z.EQ.52) H=16.8582-43.0813*ECA+28.8545*ECA**2
        IF (Z.EQ.53) H=3.2133-5.5432*ECA+3.2619*ECA**2
        IF (Z.EQ.54) H=13.7625-29.6636*ECA+17.0547*ECA**2
        IF (Z.EQ.55) H=6.8035-12.7341*ECA+6.9733*ECA**2
        IF (Z.EQ.56) H=1.076
        IF (Z.EQ.57) H=1.21
        IF (Z.EQ.58.OR.Z.EQ.59) H=1.5
        GOTO 200

C       --------------------------------------------------------------
C       REGION D  (FROM 0.15 KEV TO THE M5-EDGE)

  150   XYZ=0
  151   XYZ=XYZ+1
        IF (Z.EQ.39) THEN
                D=  7.0645
                K1=-0.652
                K2=-0.0284
                K3=0.
        END IF
        IF (Z.EQ.40) THEN
                D = 7.458
                K1=-0.5362
                K2=-0.0384
                K3=0.
        END IF
        IF (Z.EQ.41) THEN
                D=  6.948
                K1=-0.8431
                K2=-0.0047
                K3= 0
        END IF
        IF (Z.EQ.42.OR.Z.EQ.43) THEN
                D = 6.3443
                K1=-1.9597
                K2=-0.4087
                K3= 0
        END IF
        IF (Z.GE.44.AND.Z.LE.47) THEN
                D = 67.6303-3.89356*Z+0.0897147*Z**2-7.7478E-4*Z**3
                K1=110.417-6.82633*Z+0.154764*Z**2-1.36298E-3*Z**3
                K2= 57.3889-3.40331*Z+7.63608E-2*Z**2-6.85696E-4*Z**3
                K3=  8.12811-0.461668*Z+1.02792E-2*Z**2-9.58133E-5*Z**3
        END IF
        IF (Z.EQ.48) THEN
                D = 7.02988
                K1=-1.32149
                K2= 0.367552
                K3= 0.314483
        END IF
        IF (Z.EQ.49) THEN
                D = 7.04621
                K1=-1.49466
                K2= 0.226478
                K3= 0.289793
        END IF
        IF (Z.EQ.50) THEN
                D = 6.87809
                K1=-2.0796
                K2=-0.224172
                K3= 0.180342
        END IF
        IF (Z.EQ.51) THEN
                D = 7.05079
                K1=-1.80117
                K2=-1.79748E-2
                K3= 0.250139
        END IF
        IF (Z.EQ.52) THEN
                D = 7.15391
                K1=-1.66014
                K2= 0.0709383
                K3= 0.277404
        END IF
        IF (Z.EQ.53) THEN
                D = 7.06452
                K1=-2.26311
                K2=-0.536917
                K3= 0.101905
        END IF
        IF (Z.GE.54.AND.Z.LE.83) THEN
        D=1.90893+0.09933*Z+8.5672E-4*Z**2-2.2988E-5*Z**3+1.2187E-7*Z**4
        K3=0
        END IF
        IF (Z.GE.54.AND.Z.LE.64) THEN
                K1=-27.5016+0.65019*Z-9.2977E-4*Z**2-4.6549E-5*Z**3
        END IF
        IF (Z.GE.65.AND.Z.LE.83) K1=-1.9179
        IF (Z.GE.54.AND.Z.LE.63) THEN
                K2=-15.4787+0.40583*Z-1.6563E-3*Z**2-1.5548E-5*Z**3
        END IF
        IF (Z.GE.64.AND.Z.LE.78) K2=-0.376
        IF (Z.EQ.72.OR.Z.EQ.76.OR.Z.EQ.77) K2=-0.4232
        IF (Z.EQ.74.OR.Z.EQ.75) K2=-0.3267
        IF (Z.EQ.79) K2=-0.376
        IF (Z.EQ.80) K2=-0.2294
        IF (Z.EQ.81) K2=-0.467
        IF (Z.EQ.82) K2=-0.1877
        IF (Z.EQ.83) K2= 0.0019

        IF (ECA.GE.EN1.AND.ECA.LE.1) THEN
                H=1.
                GOTO 200
        END IF
        IF (ECA.GE.EN2.AND.ECA.LE.EN1) GOTO 155
        IF (ECA.GE.EN3.AND.ECA.LE.EN2) GOTO 165
        IF (ECA.GE.EN4.AND.ECA.LE.EN3) GOTO 175
        IF (ECA.GE.EN5.AND.ECA.LE.EN4) GOTO 185
        IF (ECA.LT.EN5) GOTO 195

  155   IF (Z.EQ.51) H=0.796
        IF (Z.EQ.52.OR.Z.EQ.53) H=0.817
        IF (Z.EQ.54) H=5.3066-51.0807*ECA+144.6515*ECA**2
        IF (Z.EQ.55) H=0.47+1.5658*ECA
        IF (Z.EQ.56) H=0.72
        IF (Z.EQ.57) H=0.683
        IF (Z.EQ.58) H=0.892
        IF (Z.GE.59.AND.Z.LE.83) H=0.935
        GOTO 200

  165   IF (Z.GE.52.AND.Z.LE.83) H=0.856
        GOTO 200

  175   IF (Z.GE.55.AND.Z.LE.57) H=0.386
        IF (Z.EQ.58) H=1.3505-3.8527*ECA
        IF (Z.EQ.59) H=1.6418-4.91*ECA
        IF (Z.EQ.60) H=1.7819-5.3899*ECA
        IF (Z.EQ.61) H=1.8238-5.3223*ECA
        IF (Z.EQ.62) H=1.7644-4.7159*ECA
        IF (Z.EQ.63) H=1.5636-3.8925*ECA
        IF (Z.EQ.64) H=1.0845-1.9358*ECA
        IF (Z.EQ.65) H=1.5556-6.0597*ECA+11.6195*ECA**2
        IF (Z.GE.66.AND.Z.LE.70) H=0.874
        IF (Z.GE.71.AND.Z.LE.81) H=0.768
        IF (Z.EQ.80) H=0.2531+0.9484*ECA
        IF (Z.EQ.82) H=0.2027+0.8926*ECA
        IF (Z.EQ.83) H=0.1751+0.8488*ECA
        GOTO 200

  185   IF (Z.GE.66.AND.Z.LE.70) H=0.891
        IF (Z.GE.71.AND.Z.LE.81) H=0.67
        IF (Z.EQ.82.OR.Z.EQ.83)  H=0.49
        GOTO 200

  195   IF (Z.GE.66.AND.Z.LE.70) H=0.801
        IF (Z.EQ.71) H=0.4668+1.412*ECA
        IF (Z.EQ.72) H=0.565+1.1633*ECA
        IF (Z.EQ.73) H=0.2185+2.1803*ECA
        IF (Z.EQ.74) H=-0.005+2.6808*ECA
        IF (Z.EQ.75) H=-0.1094+2.9167*ECA
        IF (Z.EQ.76) H=-0.0916+3.1098*ECA
        IF (Z.EQ.77) H=-0.1038+2.7193*ECA
        IF (Z.EQ.78) H=-0.2719+2.9476*ECA
        IF (Z.EQ.79) H=-0.336+2.9654*ECA
        IF (Z.EQ.80) H=0.1006-1.8851*ECA+9.9837*ECA**2
        IF (Z.EQ.81) H=0.1831-2.6151*ECA+12.8147*ECA**2
        IF (Z.EQ.82) H=-0.0563-0.0582*ECA+3.6491*ECA**2
        IF (Z.EQ.83) H=0.0821-1.2986*ECA+5.2779*ECA**2
        GOTO 200

  200   MUERO=H*DEXP(D+K1*DLOG(ECA)+K2*DLOG(ECA)**2+K3*DLOG(ECA)**3)

        IF (XY.EQ.0.AND.XYZ.EQ.1) THEN
                ECA0=ECA
                ECA =1.
                MUERO0=MUERO
                IF (Z.GE.11.AND.Z.LE.29) THEN
                        GOTO 10
                ELSE IF (Z.GE.30.AND.Z.LE.60) THEN
                        GOTO 31
                ELSE IF (Z.GE.61.AND.Z.LE.83) THEN
                        GOTO 151
                END IF
        ELSE IF (XY.EQ.0.AND.XYZ.GE.2) THEN
                BMYRO=MUERO
                GOTO 300
        ELSE
                GOTO 500
        END IF

C       --------------------------------------------------------------
C       --------------------------------------------------------------
C       FROM Z=11 TO Z=83 ; FROM ECHAR=1 KEV TO ECHAR=50 KEV
C       --------------------------------------------------------------
  300   IF (ECA.GE.EK.AND.Z.LE.63)    GOTO 305
        IF (ECA.LT.EK.AND.Z.LE.29)    GOTO 315
        IF (ECA.GE.EL3.AND.Z.GE.30)   GOTO 315
        IF (ECA.LE.EL3.AND.Z.LE.60)   GOTO 325
        IF (ECA.GE.EM5.AND.Z.GE.61)   GOTO 325
        IF (ECA.LE.EM5.AND.Z.GE.61)   GOTO 335
C       --------------------------------------------------------------
C       REGION A  (FROM THE K-EDGE TO 50 KEV)

  305   D=5.955+3.917E-1*Z-1.054E-2*Z**2+1.52E-4*Z**3-8.508E-7*Z**4
        K1=-2.685
        H=1.
        GOTO 350

C       --------------------------------------------------------------
C       REGION B  (FROM THE EL3-EDGE TO THE K-EDGE)

  315   D=3.257+3.936E-1*Z-8.483E-3*Z**2+9.491E-5*Z**3-4.058E-7*Z**4
        K1=-2.669
        H=1.
        IF (ECA.GE.EL3.AND.ECA.LT.EL2) H=0.611
        IF (ECA.GE.EL2.AND.ECA.LT.EL1) H=0.862
        GOTO 350

C       --------------------------------------------------------------
C       REGION C  (FROM THE M5-EDGE TO THE EL3-EDGE)

  325   D=2.382+0.2212*Z-2.028E-3*Z**2+6.891E-6*Z**3
        K1=-2.514
        H=1.
        IF (ECA.GE.EM5.AND.ECA.LT.EM4) H=0.443
        IF (ECA.GE.EM4.AND.ECA.LT.EM3) H=0.638
        IF (ECA.GE.EM3.AND.ECA.LT.EM2) H=0.842
        IF (ECA.GE.EM2.AND.ECA.LT.EM1) H=0.935
        GOTO 350

C       --------------------------------------------------------------
C       REGION D  (FROM 1 KEV TO THE M5-EDGE)

  335   D=4.838+4.911E-2*Z
        K1=-2.451
        H=1.
        GOTO 350

  350   MUERO=H*DEXP(D+K1*DLOG(ECA))

  500   IF (XY.EQ.0) THEN
                ECA=ECA0
                AMYRO=MUERO
                MUERO=MUERO0
        ELSE IF (XY.EQ.1) THEN
                ECA=ECA
                MUERO=MUERO
        END IF

        IF (ECA.LE.0.9.OR.ECA.GE.1.) GOTO 1000
        IF (Z.EQ.27.AND.ECA.LE.0.93) GOTO 1000
        IF (Z.EQ.29.AND.ECA.LE.0.96) GOTO 1000
        IF (Z.EQ.53.AND.ECA.LE.0.94) GOTO 1000
        IF (Z.EQ.54.AND.ECA.LE.0.94) GOTO 1000
        IF (Z.EQ.59.AND.ECA.LE.0.95) GOTO 1000
        IF (Z.EQ.60.AND.ECA.LE.0.99) GOTO 1000
        IF (Z.EQ.83.AND.ECA.LE.0.94) GOTO 1000

        MULT=((AMYRO-BMYRO)*ECA+BMYRO-0.9*AMYRO)/(0.1*BMYRO)
        IF (XY.EQ.1) MULT=1.
        MUERO=MUERO*MULT
        GOTO 1000

C       --------------------------------------------------------------
C       --------------------------------------------------------------
  600   IF (ECA.GE.1.2.AND.ECA.LT.1.3) THEN
                XY=0
        ELSE
                XY=1
        END IF
        IF (ECA.GE.1.3) GOTO 700
C       --------------------------------------------------------------
C       --------------------------------------------------------------
C       FROM Z=4 TO Z=10 ; FROM ECHAR=0.15 KEV TO ECHAR=1.3
C       --------------------------------------------------------------
        IF (ECA.GE.EK.AND.ECA.LE.1.3)  GOTO 605
        IF (ECA.GE.0.15.AND.ECA.LT.EK) GOTO 615

C       --------------------------------------------------------------
C       REGION A  (FROM THE K-EDGE TO 1.3 KEV)

  605   D=4.9031-0.3216*Z+0.3226*Z**2-0.0422*Z**3+0.0017*Z**4
        K1=-2.5776
        H=1.
        IF (Z.EQ.8.OR.Z.EQ.9) H=1.1
        IF (Z.EQ.10) H=1.22
        K2=0.
        IF (Z.EQ.4) THEN
                D=6.4699
                K1=-2.6605
        END IF
        GOTO 650

C       --------------------------------------------------------------
C       REGION B  (FROM 0.15 KEV TO THE K-EDGE)

  615   D=-14.5984+8.7406*Z-1.4978*Z**2+0.1169*Z**3-0.0034*Z**4
        K1=-2.3887
        K2=0.
        IF (Z.EQ.4) THEN
                D=6.4699
                K1=-2.6605
        END IF
        H=1.
        IF (Z.EQ.5.OR.Z.EQ.6) H=0.92
        IF (Z.EQ.8) H=1.26
        IF (Z.EQ.9) H=1.31
        IF (Z.EQ.10) H=1.5
        GOTO 650

  650   MUERO=H*DEXP(D+K1*DLOG(ECA)+K2*DLOG(ECA)**2)

        D=4.9031-0.3216*Z+0.3226*Z**2-0.0422*Z**3+0.0017*Z**4
        K1=-2.5776
        H=1.
        IF (Z.EQ.8.OR.Z.EQ.9) H=1.1
        IF (Z.EQ.10) H=1.22
        K2=0.
        IF (Z.EQ.4) THEN
                D=6.4699
                K1=-2.6605
        END IF
        BMYRO=H*DEXP(D+K1*DLOG(dble(1.3)))

        IF (XY.EQ.0) THEN
                ECA0=ECA
                ECA=1.3
                MUERO0=MUERO
                GOTO 700
        ELSE
                GOTO 950
        END IF


C       --------------------------------------------------------------
C       --------------------------------------------------------------
C       FROM Z=4 TO Z=10 ; FROM ECHAR=1.3 KEV TO ECHAR=50 KEV
C       --------------------------------------------------------------

  700   IF (Z.EQ.4.AND.ECA.LT.10.1) THEN
                D = 6.5181
                K1=-3.3524
                K2= 0.1175
        END IF
        IF (Z.EQ.4.AND.ECA.GT.10.1) THEN
                D = 6.9356
                K1=-4.7076
                K2= 0.6311
        END IF
        IF (Z.EQ.5) THEN
                D = 7.4483
                K1=-3.992
                K2= 0.3977
        END IF
        IF (Z.EQ.6) THEN
                D = 8.0458
                K1=-3.8002
                K2= 0.3135
        END IF
        IF (Z.EQ.7) THEN
                D = 8.4381
                K1=-3.5862
                K2= 0.2354
        END IF
        IF (Z.EQ.8) THEN
                D = 8.7006
                K1=-3.3711
                K2= 0.1686
        END IF
        IF (Z.EQ.9) THEN
                D = 8.8806
                K1=-3.1472
                K2= 0.1031
        END IF
        IF (Z.EQ.10) THEN
                D = 9.1363
                K1=-3.0556
                K2= 0.0752
        END IF
        H=1.

  900   MUERO=H*DEXP(D+K1*DLOG(ECA)+K2*DLOG(ECA)**2)

  950   IF (XY.EQ.0) THEN
                ECA=ECA0
                AMYRO=MUERO
                MUERO=MUERO0
        ELSE IF (XY.EQ.1) THEN
                ECAJ=ECA
                MUERO=MUERO
        END IF

        IF (ECA.LE.1.2.OR.ECA.GE.1.3) GOTO 1000
        MULT=((AMYRO-BMYRO)*ECA-1.2*AMYRO+1.3*BMYRO)/(0.1*BMYRO)
        IF (XY.EQ.1) MULT=1
        MUERO=MUERO*MULT

 1000   RETURN
        END

****************************************************************************

        SUBROUTINE ABSKAN (Z)
        COMMON/EDGE/EK,EL1,EL2,EL3,EM1,EM2,EM3,EM4,EM5,EN1,EN2,EN3,EN4,
     A              EN5

C       DATA FOR THE FITS OF EK, EL1, EL2, EL3, EM1, EM2, EM3, EM4 AND
C       EM5 FOR ENERGIES ABOUT 1 KEV WERE TAKEN FROM THE TABLES GIVEN
C       BY MCMASTER ET AL (1969).
C       DATA FOR THE FIT OF EK FOR ENERGIES BELOW 1.3 KEV WERE TAKEN
C       FROM THE TABLES GIVEN BY HENKE ET AL (1974).
C       DATA FOR THE FITS OF EL1, EL2 AND EL3 FOR ENERGIES BELOW 1 KEV
C       WERE TAKEN FROM THE EDAX PEAK IDENTIFACATION CHART-EPIC.
C       DATA FOR THE FITS OF EM1, EM2, EM3, EM4, EM5, EN1, EN2, EN3,
C       EN4 AND EN5 FOR ENERGIES BELOW 1 KEV WERE TAKEN FROM
C       AMERICAN INSTITUTE OF PHYSICS HANDBOOK (1972).

        INTEGER Z
        REAL    EK,EL1,EL2,EL3,EM1,EM2,EM3,EM4,EM5,EN1,EN2,EN3,EN4,EN5

        IF (Z.LE.4.OR.Z.GE.64) THEN
                EK=0.
        ELSE IF (Z.GE.5.AND.Z.LE.10) THEN
                EK=-0.234+0.0978*Z-0.0058*Z**2+0.0007*Z**3
        ELSE IF (Z.GE.11.AND.Z.LE.63) THEN
                EK=-0.1304-2.633E-3*Z+9.781E-3*Z**2+4.144E-5*Z**3
        END IF

        IF (Z.LE.13.OR.Z.GE.84) THEN
                EL1=0.
        ELSE IF (Z.GE.14.AND.Z.LE.29) THEN
                EL1=0.4422-0.0851*Z+5.095E-3*Z**2-4.734E-5*Z**3
        ELSE IF (Z.GE.30.AND.Z.LE.83) THEN
                EL1=-0.4506+1.566E-2*Z+7.599E-4*Z**2+1.792E-5*Z**3
        END IF

        IF (Z.LE.13.OR.Z.GE.84) THEN
                EL2=0.
        ELSE IF (Z.GE.14.AND.Z.LE.29) THEN
                EL2=-0.0797-0.006574*Z+1.27E-3*Z**2+0.645E-5*Z**3
        ELSE IF (Z.GE.30.AND.Z.LE.83) THEN
                EL2=-0.6018+1.964E-2*Z+5.935E-4*Z**2+1.843E-5*Z**3
        END IF

        IF (Z.LE.13.OR.Z.GE.84) THEN
                EL3=0.
        ELSE IF (Z.GE.14.AND.Z.LE.29) THEN
                EL3=-0.1039-3.3956E-3*Z+1.15802E-3*Z**2+6.70795E-6*Z**3
        ELSE IF (Z.GE.30.AND.Z.LE.83) THEN
                EL3=0.339-4.931E-2*Z+2.336E-3*Z**2+1.836E-6*Z**3
        END IF

        IF (Z.LE.26.OR.Z.GE.84) THEN
                EM1=0.
        ELSE IF (Z.GE.27.AND.Z.LE.51) THEN
                EM1=0.2995-0.0284*Z+0.7296E-3*Z**2+0.146E-5*Z**3
        ELSE IF (Z.GE.52.AND.Z.LE.83) THEN
                EM1=-8.645+0.3977*Z-5.963E-3*Z**2+3.624E-5*Z**3
        END IF

        IF (Z.LE.30.OR.Z.GE.84) THEN
                EM2=0.
        ELSE IF (Z.GE.31.AND.Z.LE.55) THEN
                EM2=-1.1145+0.0768*Z-1.904E-3*Z**2+2.233E-5*Z**3
        ELSE IF (Z.GE.56.AND.Z.LE.83) THEN
                EM2=-7.499+0.3459*Z-5.25E-3*Z**2+3.263E-5*Z**3
        END IF

        IF (Z.LE.31.OR.Z.GE.84) THEN
                EM3=0.
        ELSE IF (Z.GE.32.AND.Z.LE.55) THEN
                EM3=-1.0658+0.0716*Z-1.72E-3*Z**2+2E-5*Z**3
        ELSE IF (Z.GE.56.AND.Z.LE.83) THEN
                EM3=-6.28+0.2831*Z-4.117E-3*Z**2+2.505E-5*Z**3
        END IF

        IF (Z.LE.36.OR.Z.GE.84) THEN
                EM4=0.
        ELSE IF (Z.GE.37.AND.Z.LE.61) THEN
                EM4=1.9369-0.1256*Z+2.448E-3*Z**2-1.023E-5*Z**3
        ELSE IF (Z.GE.62.AND.Z.LE.83) THEN
                EM4=-4.778+0.2184*Z-3.303E-3*Z**2+2.115E-5*Z**3
        END IF

        IF (Z.LE.36.OR.Z.GE.84) THEN
                EM5=0.
        ELSE IF (Z.GE.37.AND.Z.LE.61) THEN
                EM5=2.0128-0.1309*Z+2.579E-3*Z**2-1.137E-5*Z**3
        ELSE IF (Z.GE.62.AND.Z.LE.83) THEN
                EM5=-2.421+0.1172*Z-1.845E-3*Z**2+1.397E-5*Z**3
        END IF

        IF (Z.LE.47.OR.Z.GE.84) THEN
                EN1=0.
        ELSE
                EN1=-5.0115+0.2367*Z-3.712E-3*Z**2+2.077E-5*Z**3
        END IF

        IF (Z.LE.51.OR.Z.GE.84) THEN
                EN2=0.
        ELSE
                EN2=-6.2126+0.2886*Z-4.473E-3*Z**2+2.429E-5*Z**3
        END IF

        IF (Z.LE.52.OR.Z.GE.84) THEN
                EN3=0.
        ELSE
                EN3=-5.6695+0.2627*Z-4.04E-3*Z**2+2.164E-5*Z**3
        END IF

        IF (Z.LE.57.OR.Z.GE.84) THEN
                EN4=0.
        ELSE
                EN4=-0.9151+0.0598*Z-1.203E-3*Z**2+0.823E-5*Z**3
        END IF

        IF (Z.LE.57.OR.Z.GE.84) THEN
                EN5=0.
        ELSE
                EN5=-2.1659+0.1139*Z-1.969E-3*Z**2+1.175E-5*Z**3
        END IF

        IF (Z.GE.37.AND.Z.LE.41) EM4=EM4*1.015

        RETURN
        END

**************************************************************************

        SUBROUTINE ECHAR (Z,LINE,ECA)

*       AUTHOR: SCHEIBL CHRISTIAN
*               TECHNICAL UNIVERSITY OF VIENNA
*               1987

C       DATA FOR THE FITS WERE TAKEN FROM BERTIN (1975) FOR K AND
C       L RADIATION.
C       DATA FOR THE FIT WERE TAKEN FROM JOHNSON AND WHITE (1970) FOR
C       M RADIATION.

        INTEGER Z,LINE
        REAL    ECA

        IF (LINE.EQ.1) THEN
                GOTO 10
        ELSE IF (LINE.EQ.2) THEN
                GOTO 20
        ELSE IF (LINE.EQ.3) THEN
                GOTO 30
        END IF

   10   IF (Z.LE.10.OR.Z.GE.51) THEN
                ECA=0.
        ELSE
        ECA=-.06654-8.609E-3*Z+9.63E-3*Z**2+7.268E-6*Z**3+1.131E-7*Z**4
        END IF
        GOTO 40
   20   IF (Z.LE.26.OR.Z.GE.84) THEN
                ECA=0.
        ELSE IF (Z.GE.27.AND.Z.LE.29) THEN
                ECA=-0.160697-4.4502E-3*Z+1.45E-3*Z**2
        ELSE IF (Z.GE.30.AND.Z.LE.83) THEN
        ECA=.04776-1.706E-2*Z+1.523E-3*Z**2+4.414E-6*Z**3-1.739E-8*Z**4
        END IF
        GOTO 40
   30   IF (Z.LE.56.OR.Z.GE.84) THEN
                ECA=0.
        ELSE IF (Z.GE.57.AND.Z.LE.83) THEN
                ECA=0.37471-2.797E-2*Z+6.3386E-4*Z**2
        END IF

   40   RETURN
        END

****************************************************************************
* LAST MODIFICATION JUNE 92

      SUBROUTINE FPREAD (FNAME,IZ,VALUE,LDISP,IERROR)

*     FPREAD - READS THE VALUE OF A FUNDAMENTAL PARAMETER (ATOMIC WEIGHT,
*              ENERGIES, FLUORESCENCE YIELDS, ETC.) FOR AN ELEMENT WITH
*              ATOMIC NUMBER IZ
*
*     CHRISTIAN SCHIEBL
*     CEC-VIENNA
*     JUNE 1992
*
*     INPUT:
*     FNAME  ... NAME OF DATA FILE
*     IZ     ... ATOMIC NUMBER
*     LDISP  ... DISPLAY OF "END OF FILE ..." ERROR MESSAGE IF TRUE
*     
*     OUTPUT:
*     VALUE  ... VALUE OF FUNDAMENTAL PARAMETER
*     IERROR ... 0: NO ERROR
*                1: FILE OPENING ERROR
*                2: READ ERROR (IZ OUT OF RANGE)
*                3: END OF FILE DETECTED BEFORE DATA FOUND

CAP      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
CAP      IMPLICIT INTEGER (I-N)
      
      INCLUDE 'TYPE.INC'
 
      CHARACTER*(*)  FNAME
      CHARACTER*80  BUFFER

      LOGICAL LDISP

      PARAMETER (NF=66)

**    START OF EXECUTABLE CODE
*     ************************
 
      IERROR = 0
      OPEN (UNIT=NF,IOSTAT=IERR,FILE=FNAME,STATUS='OLD')
      IF (IERR.NE.0) THEN
          WRITE(6,'(A)')'*'
          WRITE(6,'(2A)')'* ERROR: COULD NOT OPEN FILE: ',FNAME
          WRITE(6,'(A)')'*'
          IERROR = 1
          RETURN
      ENDIF

 10   CALL CCLEAR(BUFFER)
      READ(NF,'(A)',END=20)BUFFER
      READ(BUFFER,'(I3,F8.5)',IOSTAT=IERR)I,V 
      IF (IERR.NE.0) THEN
          WRITE(6,'(A)')'*'
          WRITE(6,'(A)')'* DATA TYPE ERROR [FPREAD]'
          WRITE(6,'(A)')'*'
          IERROR = 2
          RETURN
      ENDIF

      IF (I.NE.IZ) GOTO 10
      VALUE = V
      GOTO 30

 20   IF (LDISP) THEN
          WRITE(6,'(A)')'*'
          WRITE(6,'(2A)')'* END OF FILE DETECTED BEFORE DATA ',
     &                   'FOUND [FPREAD]'
          WRITE(6,'(2A)')'* FILE NAME: ',FNAME
          WRITE(6,'(A,I3)')'* ATOMIC NUMBER:',IZ
          WRITE(6,'(A)')'*'
      ENDIF
      IERROR = 3

 30   CLOSE(NF)
      RETURN
      END

*****************************************************************************
* LAST MODIFICATION 1991

      SUBROUTINE CCLEAR (STRING)

***   CCLEAR - CLEAR STRING
*
**    M.THURNER 1991
*
**    INPUT 
*     STRING - 

      CHARACTER*(*) STRING

**    START OF EXECUTABLE CODE
*     ************************
 
      DO 1 I=1,LEN(STRING)
 1    WRITE(STRING(I:I),'(A)') ' '

      RETURN
      END

****************************************************************************
* LAST MODIFICATION 1993

      FUNCTION IGETPO (STRING,TOKEN)

*     DETERMINES THE POSITION OF THE FIRST TOKEN CHARACTER

*     M.KOKESCH   JAN 1993

      CHARACTER*(*)  STRING
      CHARACTER*1 TOKEN
      INTEGER I,IL

***   START OF EXECUTABLE CODE
*     ========================

      IL=LEN(STRING)
      DO 10 I=1, IL
         IF (STRING(I:I).EQ.TOKEN) THEN
         IGETPO = I
         GOTO 20
      ENDIF
10    CONTINUE
C     TOKEN NOT FOUND
      IGETPO = IL
20    CONTINUE

      RETURN
      END

*****************************************************************************
*last modification: 29-Apr-1997

      SUBROUTINE GETEDE ()

***   GETEDE - READS FROM FILE ELECTRON DETECTOR EFFICIENCIES 
*
**    AUTHOR: HORST WAGNER  AUG 1995
*             CHRISTIAN SCHIEBL OCT 1995 (MODIFICATIONS)
*
**    INPUT: NONE
*
**    OUTPUT: NONE
*

      INCLUDE 'TYPE.INC' 
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'

      CHARACTER*80 ERRSTR,BUFFER

      INCLUDE 'PATH.INC'

***   START OF EXECUTABLE CODE
*     ========================

      DO 90 IEDT=0,NEDET
         NEDTRN(IEDT) = 0
         IF (FEDTRN(IEDT)(1:5).EQ.'     ') GOTO 100
         OPEN (43,FILE=CFUND(1:IEND(CFUND))//FEDTRN(IEDT),
     &         FORM='FORMATTED',STATUS='UNKNOWN',
     &         READONLY,SHARED,IOSTAT=IOS)
         IF (IOS.NE.0) THEN
            OPEN (43,FILE=FEDTRN(IEDT),FORM='FORMATTED',
     &            STATUS='UNKNOWN',READONLY,SHARED,IOSTAT=IOS)
         ENDIF
         IF (IOS.NE.0) THEN
           WRITE(6,'(A)')'    '
           WRITE(6,'(A)')' %%% WARNING %%%'
           WRITE(6,'(2A,I3)')' ELECTRON DETECTOR TRANSMISSION ',
     &                   'FILE NUMBER: ',IEDT
           WRITE(6,'(3A)')' COULD NOT OPEN ELECTRON DETECTOR',
     &                    ' TRANSMISSION FILE: ',
     &                      FEDTRN(IEDT)(1:IEND(FEDTRN(IEDT)))
           WRITE(6,'(A)')
     &                 ' DETECTOR TRANSMISSION IS SET TO UNITY '
           WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
           GOTO 50
         ENDIF
  30     READ(43,'(A)',IOSTAT=IOS) BUFFER
         IF (IOS.LT.0) THEN
*          END OF FILE
           IF (NEDTRN(IEDT) .EQ. 0) THEN
             ERRSTR=' END OF FILE DETECTED BEFORE'//
     &            ' READING DETECTOR EFFICIENCY'
             GOTO 48
           ENDIF
           GOTO 50
         ENDIF
         IF (IOS.GT.0) THEN
*          ERROR DURING READ
           ERRSTR = ' ERROR OCCURD DURING READING DETECTOR EFFICIENCY'
           GOTO 48
         ENDIF
         IF (BUFFER(1:1).EQ.'#') GOTO 30
         NEDTRN(IEDT) = NEDTRN(IEDT)+1
         IF (NEDTRN(IEDT) .GT. MAXEDTRN) THEN
            WRITE(6,'(A)')'    '
            WRITE(6,'(A)')' %%% WARNING %%%'
            WRITE(6,'(3A)')'NUMBER OF DATAPOINTS IN ELECTRON',
     &                     ' DETECTOR TRANSMIISION FILE ',
     &                       FEDTRN(IEDT)(1:IEND(FEDTRN(IEDT)))
            WRITE(6,'(A,I3,A)')'EXCEEDS MAXIMUM OF ',MAXEDTRN,
     &                         ' VALUES '
            WRITE(6,'(A)')' FURTHER VALUES IGNORED'
            NEDTRN(IEDT) = NEDTRN(IEDT) -1
            GOTO 50
         ENDIF  
         READ(BUFFER,*,IOSTAT=IOS)
     &      EDTRN(IEDT,1,NEDTRN(IEDT)),EDTRN(IEDT,2,NEDTRN(IEDT))
         IF (IOS.NE.0) THEN
*           DATA CONVERSION ERROR
            WRITE(6,'(A)')'    '
            WRITE(6,'(A)')' %%% WARNING %%%'
            WRITE(6,'(3A)')' DATA CONVERSION ERROR DURING',
     &                     ' READING DETECT. EFFICIENCY: ',
     &                       FEDTRN(IEDT)(1:IEND(FEDTRN(IEDT)))
            WRITE(6,'(A)')' IGNORED, READING NEXT RECORD'
            NEDTRN(IEDT) = NEDTRN(IEDT) -1
            GOTO 30
         ENDIF
         IF (NEDTRN(IEDT) .GT. 1) THEN
            IF (EDTRN(IEDT,1,NEDTRN(IEDT)).LE.
     &          EDTRN(IEDT,1,NEDTRN(IEDT)-1)) THEN
               ERRSTR = 'ELECTRON TRANSMISSION DATA POINTS NOT IN '//
     &                  ' ASCENDING ORDER' 
               GOTO 48
            ENDIF
         ENDIF
         GOTO 30

  48     CONTINUE
C ERROR ROUTINE   
         WRITE(6,'(A)')'    '
         WRITE(6,'(A)')' %%% WARNING %%%'
         WRITE(6,'(A)')  ERRSTR
         WRITE(6,'(2A)')' ELECTRON DETECTOR TRANSMISSION FILE:',
     &                    FEDTRN(IEDT)(1:IEND(FEDTRN(IEDT)))
         WRITE(6,'(A)')' DETECTOR TRANSMISSION IS SET TO UNITY '
         WRITE(6,'(A)')' Protests are useless - SESAME ;-)'
         NEDTRN(IEDT) = 0
         GOTO 50
 50      CLOSE(43)
 90   CONTINUE

 100  END

***************************************************************************
