c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
* last modification: CS 05-Nov-1996

      SUBROUTINE HISTOG(
     I                  NDIM,J,VAL,X,Y,Z)

***   HIST - UPDATES HISTOGRAMS
*
**    ARGUMENTS
*
*     NDIM   - DIMENSION
*     J      - HISTOGRAM INDEX
*     VAL    - MAGNIFICATION OF INCREMENTATION AT X/Y/Z
*     X      - FIRST AXIS
*     Y      - SECOND AXIS
*     Z      - THIRD AXIS
*
**    AUTHOR:  C. Schiebl  19-OCT-1994
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'HIST.INC'

      DIMENSION A(3),IND(3)

**    START OF EXECUTABLE CODE
*     ************************

      A(1)=X
      A(2)=Y
      A(3)=Z
      IND(2)=1
      IND(3)=1
      DO 10 I=1,NDIM
         IF (((A(I)-VALMIN(J,I)).LE.SMALL).OR.
     &        (A(I).GT.VALMAX(J,I))) RETURN
 10   CONTINUE
      DO 20 I=1,NDIM
         RANGE=VALMAX(J,I)-VALMIN(J,I)
         DIND = NVAL(J,I)*(A(I)-VALMIN(J,I))/RANGE
         IF (DINT(DIND).EQ.DIND) THEN
            IND(I) = INT(DIND)
         ELSE
            IND(I) = INT(DIND+1.D0)
         ENDIF
 20   CONTINUE
      INDREL=IND(1)+(IND(2)-1)*NVAL(J,1)+(IND(3)-1)*NVAL(J,1)*NVAL(J,2)
      INDABS=IVBEG(J)+INDREL-1
      CALL CHKBND(8,INDABS,'HISTOG','VALHIS')
      VALHIS(INDABS)=VALHIS(INDABS)+VAL

      RETURN
      END

*************************************************************************

      SUBROUTINE HISGET(
     I                  NDIM,J,VAL,X,Y,Z,IER)

***   HISGET - GETS HISTOGRAM VALUE
*
**    ARGUMENTS
*
*     NDIM   - DIMENSION
*     J      - INDEX IN VALHIS() ARRAY
*     VAL    - VALUE OF HISTOGRAM AT X/Y/Z
*     X      - FIRST AXIS
*     Y      - SECOND AXIS
*     Z      - THIRD AXIS
*     IER    - ERROR FLAG , .NE. 0 IF OUT OF HISTOGRAM BOUNDS
*
**    AUTHOR:  C. Schiebl  19-OCT-1994
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'HIST.INC'

      DIMENSION A(3),IND(3)

**    START OF EXECUTABLE CODE
*     ************************

      A(1)=X
      A(2)=Y
      A(3)=Z
      IND(2)=1
      IND(3)=1
      DO 10 I=1,NDIM
         IF (((A(I)-VALMIN(J,I)).LE.SMALL).OR.
     &        (A(I).GT.VALMAX(J,I))) THEN
           IER = -1
           RETURN
         ENDIF
 10   CONTINUE
      DO 20 I=1,NDIM
         RANGE=VALMAX(J,I)-VALMIN(J,I)
         IND(I)=INT(NVAL(J,I)*(A(I)-VALMIN(J,I))/RANGE+HBND)
 20   CONTINUE
      INDREL=IND(1)+(IND(2)-1)*NVAL(J,1)+(IND(3)-1)*NVAL(J,1)*NVAL(J,2)
      INDABS=IVBEG(J)+INDREL-1
      CALL CHKBND(8,INDABS,'INIHIS','VALHIS')
      VAL = VALHIS(INDABS)

      IER = 0
      RETURN
      END

**************************************************************************
*last modification: 05-Nov-1996

      SUBROUTINE HISEVA(
     I                  J,TOTSIGN,PORTION,
     O                  IMX,VMX,XMX,YMX,ZMX,
     O                  BPORL,BPORR,SUMREL,IPFLAG)

***   HISEVA - EVALUATES HISTOGRAMS
*
**    ARGUMENTS
*
*     J       - HISTOGRAM INDEX
*     TOTSIGN - TOTAL SIGNAL (IS A REFERENCE VALUE, E.G. TOTAL NUMBER OF
*               TRANSMITTED ELECTRONS)
*     PORTION - A RELATIVE (TO TOTSIGN) NUMBER BETWEEN 0 AND 1 
*     IMX     - (RELATIVE) INDEX NUMBER OF ENTRY CONTAINING THE MAXIMUM
*               VALUE
*     VMX     - MAXIMUM VALUE (AT ENTRY IMX)
*     XMX     - X-VALUE OF MAXIMUM
*     YMX     - Y-VALUE OF MAXIMUM
*     ZMX     - Z-VALUE OF MAXIMUM
*     BPORL   - LEFT BOUNDARY CONTAINING PORTION OF TOTSIGN (ONLY 1D)
*     BPORR   - RIGHT BOUNDARY CONTAINING PORTION OF TOTSIGN (ONLY 1D)
*     IPFLAG  - 0: PORTION*TOTSIGN WITHIN THE HISTOGRAM BOUNDARIES
*               1: PORTION*TOTSIGN NOT WITHIN THE HISTOGRAM BAOUNDARIES
*
**    AUTHOR:  C. Schiebl  21-MAR-1996
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'HIST.INC'

**    START OF EXECUTABLE CODE
*     ************************

*     INITIALIZATION
      IMX=1
      VMX=0
      XMX=0
      YMX=0
      ZMX=0
      IPFLAG=0
      SUM=0

      NDIM=NVDIM(J)
      IFENTRY=IVBEG(J)
      NENTRIES=NVAL(J,1)*NVAL(J,2)*NVAL(J,3)
      ILENTRY=IFENTRY+NENTRIES-1

*     DETERMINE MAXIMUM
      NVH=IFENTRY+NENTRIES-1
      CALL CHKBND(8,NVH,'HISEVA','VALHIS')
      VMX=VALHIS(IFENTRY)
      DO 10 I=IFENTRY+1,IFENTRY+NENTRIES-1
         SUM=SUM+VALHIS(I)
         IF (VALHIS(I).GT.VMX) THEN
            VMX=VALHIS(I)
            IMX=I
         ENDIF
 10   CONTINUE
      SUML=0
      DO 12 I=IFENTRY+1,IMX-1
         SUML=SUML+VALHIS(I)
 12   CONTINUE
      SUMR=0
      DO 14 I=IMX+1,IFENTRY+NENTRIES-1
         SUMR=SUMR+VALHIS(I)
 14   CONTINUE

      IREL=IMX-IFENTRY+1
      SUMREL=SUM/TOTSIGN
      IF (NDIM.EQ.1) THEN
         XMX=VALMIN(J,1)+(DBLE(IREL)-0.5)*VDELT(J,1)
      ELSE IF (NDIM.EQ.2) THEN
         I1=MOD(IREL,NVAL(J,1))
         IF (I1.EQ.0) I1=NVAL(J,1)
         I2=IREL/NVAL(J,1)+1
         XMX=VALMIN(J,1)+(DBLE(I1)-0.5)*VDELT(J,1)
         YMX=VALMIN(J,2)+(DBLE(I2)-0.5)*VDELT(J,2)
      ELSE IF (NDIM.EQ.3) THEN
         II=NVAL(J,1)*NVAL(J,2)
         I3=IREL/II+1
         IJ=MOD(IREL,II)
         IF (IJ.EQ.0) IJ=II
         I1=MOD(IJ,NVAL(J,1))
         IF (I1.EQ.0) I1=NVAL(J,1)
         I2=IJ/NVAL(J,1)+1
         XMX=VALMIN(J,1)+(DBLE(I1)-0.5)*VDELT(J,1)
         YMX=VALMIN(J,2)+(DBLE(I2)-0.5)*VDELT(J,2)
         ZMX=VALMIN(J,3)+(DBLE(I3)-0.5)*VDELT(J,3)
      ENDIF

*     DETERMINE BOUNDARIES FOR REGION CONTAINING PORTION*100% OF THE SIGNAL
      IF (NDIM.NE.1) GOTO 999
      IF (SUM.LT.(PORTION*TOTSIGN)) THEN
*        HISTOGRAM REGION OBVIOUSLY TOO SMALL
         BPORL=VALMIN(J,1)-0.1*(VALMAX(J,1)-VALMIN(J,1))
         BPORR=VALMAX(J,1)+0.1*(VALMAX(J,1)-VALMIN(J,1))
         IPFLAG=1
         GOTO 999
      ENDIF
      SUM1=VALHIS(IMX)
      ILASTL=IMX
      ILASTR=IMX
      IF (SUM1.GE.(PORTION*TOTSIGN)) GOTO 900
 110  IF (((ILASTR+1).GT.ILENTRY).AND.((ILASTL-1).LT.IFENTRY)) THEN
         BPORL=VALMIN(J,1)
         BPORR=VALMAX(J,1)
         IPFLAG=1
         GOTO 999
      ELSE
         IF ((ILASTR+1).GT.ILENTRY) THEN
            ILASTL=ILASTL-1
            SUM1=SUM1+VALHIS(ILASTL)
            SUML=SUML-VALHIS(ILASTL)
         ELSE IF ((ILASTL-1).LT.IFENTRY) THEN
            ILASTR=ILASTR+1
            SUM1=SUM1+VALHIS(ILASTR)
            SUMR=SUMR-VALHIS(ILASTR)
         ELSE
            IF (((ILASTL-3).GE.IFENTRY).AND.
     &          ((ILASTR+3).LE.ILENTRY)) THEN
               VL=VALHIS(ILASTL-1)+VALHIS(ILASTL-2)+
     &            VALHIS(ILASTL-3)
               VR=VALHIS(ILASTR+1)+VALHIS(ILASTR+2)+
     &            VALHIS(ILASTR+3)
               IF (VALHIS(ILASTR+1).GT.VALHIS(ILASTL-1)) THEN
                  IF (SUML.GT.(1.3*SUMR)) THEN
                     ILASTL=ILASTL-1
                     SUM1=SUM1+VALHIS(ILASTL)
                     SUML=SUML-VALHIS(ILASTL)
                  ELSE IF (VL.GT.(1.25*VR)) THEN
                     ILASTL=ILASTL-1
                     SUM1=SUM1+VALHIS(ILASTL)
                     SUML=SUML-VALHIS(ILASTL)
                  ELSE
                     ILASTR=ILASTR+1
                     SUM1=SUM1+VALHIS(ILASTR)
                     SUMR=SUMR-VALHIS(ILASTR)
                  ENDIF
               ELSE
                  IF (SUMR.GT.(1.3*SUML)) THEN
                     ILASTR=ILASTR+1
                     SUM1=SUM1+VALHIS(ILASTR)
                     SUMR=SUMR-VALHIS(ILASTR)
                  ELSE IF (VR.GT.(1.25*VL)) THEN
                     ILASTR=ILASTR+1
                     SUM1=SUM1+VALHIS(ILASTR)
                     SUMR=SUMR-VALHIS(ILASTR)
                  ELSE
                     ILASTL=ILASTL-1
                     SUM1=SUM1+VALHIS(ILASTL)
                     SUML=SUML-VALHIS(ILASTL)
                  ENDIF
               ENDIF
            ELSE
               IF (VALHIS(ILASTR+1).GT.VALHIS(ILASTL-1)) THEN
                  ILASTR=ILASTR+1
                  SUM1=SUM1+VALHIS(ILASTR)
                  SUMR=SUMR-VALHIS(ILASTR)
               ELSE
                  ILASTL=ILASTL-1
                  SUM1=SUM1+VALHIS(ILASTL)
                  SUML=SUML-VALHIS(ILASTL)
               ENDIF
            ENDIF
         ENDIF
      ENDIF
CS      write(6,'(i4,f8.1,i4,5f8.1)')ilastl-1,valhis(ilastl-1),
CS     &      ilastr+1,
CS     &      valhis(ilastr+1),vl,vr,suml,sumr
      IF (SUM1.LT.(PORTION*TOTSIGN)) GOTO 110
 900  IRLASTL=ILASTL-IFENTRY+1
      IRLASTR=ILASTR-IFENTRY+1
      BPORL=VALMIN(J,1)+(IRLASTL)*VDELT(J,1)
      BPORR=VALMIN(J,1)+(IRLASTR-1)*VDELT(J,1)
      SUMDIFF=PORTION*TOTSIGN-SUM1+VALHIS(ILASTL)+VALHIS(ILASTR)
      BPORL=BPORL-VDELT(J,1)*SUMDIFF/(VALHIS(ILASTL)+VALHIS(ILASTR))
      BPORR=BPORR+VDELT(J,1)*SUMDIFF/(VALHIS(ILASTL)+VALHIS(ILASTR))

 999  RETURN
      END
