/*****************************************************************************/
/*     Copyright (c) 2000, Compaq Computer Corporation                       */
/*                                                                           */
/*  See the file "license.terms" for information on usage and redistribution */
/*  of this file, and for a DISCLAIMER OF ALL WARRANTIES.                    */
/*****************************************************************************/
/*
                              S E S A M E
                          --------------------
                              imsurfgeo.h


  header file for imsurfgeo.c

  AUTHORS:   HW  Horst Wagner, CEC Vienna, Digital Equipment Corporation

  who when        what
  ---------------------------------------------------------------------------
  HW  1994-10-14  new from scratch
  ===========================================================================*/

/*** i m S u r f M a k e

      initializes and allocates structures for use of imsurf routines

***/ 	


int imsurfMake (
		   int        numAmbients,	/* # of lines for the hull */
		   int        ambientBorder[],	/* lines of simulation hull */
		   double     pcoord[],		/* coordinates of the points */
      		   double     minX,double maxX, /* Maximum coordinates of- */
                   double     minZ,double maxZ); /* Simulation Area */

/*** i m s u r f g e t d e p t h 

calculates for given point the depth from the surface and lateral distance 
to the left end of the sample

iError : 0 - OK
	 1 - Point lies in area without solution 

***/

#if !( defined(vms) || defined(__vms) )
void imsurfgetdepth_(
#else
void imsurfgetdepth(
#endif
		  double *x,		/* x coordinate of point */
		  double *z,		/* z coordinate of point */
		  double *depth,	/* depth */
	          double *length,	/* lateral distance */
		  int *iError		/* Error Flag */
		  );

/*** i m s u r f s e t d e p t h 

calculates for given depth from the surface and lateral distance 
to the left end of the sample the coordinates of the appropriate point
(inverse function to imsurfgetdepth)

iError : 0 - OK
	-1 - lateral distance < 0
	 1 - lateral distance exceeds sample dimensions 

***/

#if !( defined(vms) || defined(__vms) )
void imsurfsetdepth_(
#else
void imsurfsetdepth(
#endif
		  double *x,		/* x coordinate of point */
		  double *z,		/* z coordinate of point */
		  double *depth,	/* depth */
	          double *length,	/* lateral distance */
		  int *iError		/* Error Flag */
		  );

#if !( defined(vms) || defined(__vms) )
void imsurfgetcoord_(
#else
void imsurfgetcoord(
#endif
		  double *xcoord,	/* x coordinate of point */
		  double *zcoord,	/* z coordinate of point */
		  int    *numcoord	/* Number of coordinates */
		  );
