c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
** file contains the following routines:
*
*     INIHIS - histogram initialization
*     IPOLHI - interpolation of XDETOFXCONT onto histogram
*     ADDVAL - adds array values to a histogram
*     CHKHIS - checks for XRY histograms and makes connection to x-ray
*              x-cont tables
*
******************************************************************************
* last modification: CS 06-Jan-1998

      SUBROUTINE INIHIS (
     I			 NDIM,
     I			 NX1,X1MIN,X1MAX,
     I			 NX2,X2MIN,X2MAX,
     I			 NX3,X3MIN,X3MAX,
     I			 NOUT,CV,CE,CF,
     O			 IER		 )

*     INIHIS - INITIALIZE HISTOGRAM
*
*     ARGUMENTS:
*
*     NDIM   - DIMENSION OF HISTOGRAM
*     NX1    - NUMBER OF INTERVALS (STUETZSTELLEN) OF AXIS 1
*     X1MIN  - MINIMUM VALUE OF AXIS 1
*     X1MAX  - MAXIMUM VALUE OF AXIS 1
*     NX2    - NUMBER OF INTERVALS (STUETZSTELLEN) OF AXIS 2
*     X2MIN  - MINIMUM VALUE OF AXIS 2
*     X2MAX  - MAXIMUM VALUE OF AXIS 2
*     NX3    - NUMBER OF INTERVALS (STUETZSTELLEN) OF AXIS 3
*     X3MIN  - MINIMUM VALUE OF AXIS 3
*     X3MAX  - MAXIMUM VALUE OF AXIS 3
*     NOUT   - OUTPUT OF HISTOGRAM TO (SETTING OF LOUTTO - SEE HIST.INC)
*     NDTCT  - DETECTOR NUMBER
*     CV     - DEFINES STORED INFORMATION
*     CE     - SPECIFICATION OF OUTPUT FILE (EXTENSION)
*     CF     - OUTPUT FILE NAME
*     IER    - ERROR FLAG (0=O.K.)
*
*     REMARKS
*
*     1) NO REMARKS PRESENTLY
*
*     AUTHOR:  C.SCHIEBL   NOVEMBER 1994

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'HIST.INC'

      CHARACTER*70 CF
      CHARACTER*10 CBL
      CHARACTER*3  CE,CV

**    START OF EXECUTABLE CODE
*     ************************

      IER=0

      IF ((NDIM.LT.1).OR.(NDIM.GT.MAXDIM)) THEN
	 IER=1
	 WRITE(6,'(A)')'     '
	 WRITE(6,'(A)')' WARNING: not a valid dimension for histogram'
	 WRITE(6,'(A,I4)')' Histogram number: ',nhist+1
	 WRITE(6,'(2A)')' CVALUE: ',CV
	 WRITE(6,'(A)')' No new histogram generated!'
	 RETURN
      ENDIF

      NH=NHIST+1
      IF (NH.GT.MXHIST) THEN
	 IER=2
	 WRITE(6,'(A)')'     '
	 WRITE(6,'(A)')' WARNING: maximum number of histograms exceeded'
	 WRITE(6,'(A,I4)')' Histogram number: ',nh
	 WRITE(6,'(2A)')' CVALUE: ',CV
	 WRITE(6,'(A)')' No new histogram generated!'
	 RETURN
      ENDIF

      IF (NDIM.EQ.1) THEN
	 IF (NX1.LT.1) THEN
	    IER=3
	    WRITE(6,'(A)')'     '
	    WRITE(6,'(2A)')' WARNING: interval number of axis 1',
     &			   ' too small'
	    WRITE(6,'(A,I4)')' Histogram number: ',nh
	    WRITE(6,'(2A)')' CVALUE: ',CV
	    WRITE(6,'(A)')' No new histogram generated!'
	    RETURN
	 ENDIF
	 NX2=1
	 NX3=1
	 X1DLT=(X1MAX-X1MIN)/NX1
	 X2DLT=0
	 X3DLT=0
      ELSE IF (NDIM.EQ.2) THEN
	 IF (NX1.LT.1) THEN
	    IER=3
	    WRITE(6,'(A)')'     '
	    WRITE(6,'(2A)')' WARNING: interval number of axis 1',
     &			   ' too small'
	    WRITE(6,'(A,I4)')' Histogram number: ',nh
	    WRITE(6,'(2A)')' CVALUE: ',CV
	    WRITE(6,'(A)')' No new histogram generated!'
	    RETURN
	 ENDIF
	 IF (NX2.LT.1) THEN
	    IER=3
	    WRITE(6,'(A)')'     '
	    WRITE(6,'(2A)')' WARNING: interval number of axis 2',
     &			   ' too small'
	    WRITE(6,'(A,I4)')' Histogram number: ',nh
	    WRITE(6,'(2A)')' CVALUE: ',CV
	    WRITE(6,'(A)')' No new histogram generated!'
	    RETURN
	 ENDIF
	 NX3=1
	 X1DLT=(X1MAX-X1MIN)/NX1
	 X2DLT=(X2MAX-X2MIN)/NX2
	 X3DLT=0
      ELSE IF (NDIM.EQ.3) THEN
	 IF (NX1.LT.1) THEN
	    IER=3
	    WRITE(6,'(A)')'     '
	    WRITE(6,'(2A)')' WARNING: interval number of axis 1',
     &			   ' too small'
	    WRITE(6,'(A,I4)')' Histogram number: ',nh
	    WRITE(6,'(2A)')' CVALUE: ',CV
	    WRITE(6,'(A)')' No new histogram generated!'
	    RETURN
	 ENDIF
	 IF (NX2.LT.1) THEN
	    IER=3
	    WRITE(6,'(A)')'     '
	    WRITE(6,'(2A)')' WARNING: interval number of axis 2',
     &			   ' too small'
	    WRITE(6,'(A,I4)')' Histogram number: ',nh
	    WRITE(6,'(2A)')' CVALUE: ',CV
	    WRITE(6,'(A)')' No new histogram generated!'
	    RETURN
	 ENDIF
	 IF (NX3.LT.1) THEN
	    IER=3
	    WRITE(6,'(A)')'     '
	    WRITE(6,'(2A)')' WARNING: interval number of axis 3',
     &			   ' too small'
	    WRITE(6,'(A,I4)')' Histogram number: ',nh
	    WRITE(6,'(2A)')' CVALUE: ',CV
	    WRITE(6,'(A)')' No new histogram generated!'
	    RETURN
	 ENDIF
	 X1DLT=(X1MAX-X1MIN)/NX1
	 X2DLT=(X2MAX-X2MIN)/NX2
	 X3DLT=(X3MAX-X3MIN)/NX3
      ENDIF
      NX=NX1*NX2*NX3
      IF ((NVTOT+NX).GT.MAXVAL) THEN
	 IER=4
	 WRITE(6,'(A)')'     '
	 WRITE(6,'(2A)')' WARNING: maximum number of histogram values',
     &			' exceeded'
	 WRITE(6,'(A,I4)')' Histogram number: ',nh
	 WRITE(6,'(2A)')' CVALUE: ',CV
	 WRITE(6,'(A)')' No new histogram generated!'
	 RETURN
      ENDIF

      NHIST=NH
      NVDIM(NH)=NDIM
      IVBEG(NH)=NVTOT+1
      NVAL(NH,1)=NX1
      NVAL(NH,2)=NX2
      NVAL(NH,3)=NX3
      NVTOT=NVTOT+NX
      HISEDT(NH)=0
      HISENWIN(NH,1)=0
      HISENWIN(NH,2)=0
      VALMIN(NH,1)=X1MIN
      VALMAX(NH,1)=X1MAX
      VALMIN(NH,2)=X2MIN
      VALMAX(NH,2)=X2MAX
      VALMIN(NH,3)=X3MIN
      VALMAX(NH,3)=X3MAX
      VDELT(NH,1)=X1DLT
      VDELT(NH,2)=X2DLT
      VDELT(NH,3)=X3DLT
      CVALUE(NH)=CV
      COUT(NH)=CE
      IF (CF(1:1).EQ.' ') THEN
         CHFILE(NH)=JOB(1:IEND(JOB))//'.HXD'
      ELSE IF (CF(1:1).EQ.'.') THEN
         IE=IEND(JOB)
         CHFILE(NH)(IE+1:IE+4)=CF
         CHFILE(NH)(1:IE)=JOB(1:IE)
      ELSE
         CHFILE(NH)=CF
      ENDIF
      CHISTITLE(NH)=CBL//CBL//CBL//CBL
      IF ((NOUT.LT.0).OR.(NOUT.GT.3)) NOUT=0
      LOUTTO(NH,NOUT)=.TRUE.
      LHIS=.TRUE.

      RETURN
      END

****************************************************************************
*last modification 25-Nov-1996

      SUBROUTINE IPOLHI(IHIS,IXDET,E0)

**    IPOLHI - INTERPOLATION OF XDETOFXCONT ONTO HISTOGRAM IHIS (FOR
*              1-DIMENSIONAL HISTOGRAMS ONLY)
*
**    ARGUMENTS:
*     IHIS  - HISTOGRAM NUMBER
*     IXDET - X-RAY DETECTOR NUMBER
*     E0    - ACCELERATION ENERGY
*
*     AUTHOR: CHRISTIAN SCHIEBL
*             CONSULTANT (FORMER: CEC-VIENNA)
*             KREMS, OCTOBER 1996

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'DMA.INC'

***   START OF EXECUTABLE CODE
******************************

cs-test-start
cs      write(6,'(a)')'  I    ENEROFXCONT    XDETOFXCONT '
cs      do 10 i=1,nxcont
cs       indx=ndmaxdet*(i-1)+(ixdet+1)
cscs       write(6,'(i4,f10.3,d18.9)')i,enerofxcont(i),xdetofxcont(ixdet,i)
cs       write(6,'(i4,f10.3,d18.9)')i,enerofxcont(i),xdetofxcont(indx)
cs 10   continue
cs      write(6,'(a)')'  I    VALHIS '
cs-test-end

      ISTART=IVBEG(IHIS)-1
      NVH=ISTART+NVAL(IHIS,1)
      CALL CHKBND(8,NVH,'INIHIS','VALHIS')
      VALPOS=VALMIN(IHIS,1)-VDELT(IHIS,1)
      VLEFTPOS=VALMIN(IHIS,1)-VDELT(IHIS,1)
      IF (VLEFTPOS.LT.0) VLEFTPOS=0
      VRIGHTPOS=VALMAX(IHIS,1)+VDELT(IHIS,1)
      IF (VRIGHTPOS.GT.E0) VRIGHTPOS=E0
      MAXINDX=0
      CALL CHKBND (7,NXCONT,'INIHIS','ENEROFXCONT')
      DO 20 I=1,NVAL(IHIS,1)
         VALPOS=VALPOS+VDELT(IHIS,1)
         J=0
 15      J=J+1
         IF (J.LT.NXCONT) THEN
            IF (ENEROFXCONT(J).LE.VALPOS) GOTO 15
         ELSE IF (J.EQ.NXCONT) THEN
            IF (ENEROFXCONT(J).LE.VALPOS) J=J+1
         ENDIF
         IF (J.LT.1) J=1
         IF (J.EQ.1) THEN
            INDX1=NDMAXDET*(J-1)+(IXDET+1)
            IF (INDX1.GT.MAXINDX) MAXINDX=INDX1
            INDX2=NDMAXDET*(J)+(IXDET+1)
            IF (INDX2.GT.MAXINDX) MAXINDX=INDX2
            V1POS=VLEFTPOS
            V2POS=ENEROFXCONT(J)
CS            V1=XDETOFXCONT(IXDET,1)
CS            V2=XDETOFXCONT(IXDET,J)
CS            V2=XDETOFXCONT(1)
CS            GRADK=(XDETOFXCONT(INDX2)-XDETOFXCONT(INDX1))/
CS     &            (ENEROFXCONT(INDX2)-ENEROFXCONT(INDX1))
CS            DELTA=XDETOFXCONT(INDX1)-GRADK*ENEROFXCONT(INDX1)
CS            V1=GRADK*V1POS+DELTA
CS            V1=XDETOFXCONT(INDX1)
CS            V2=V1
              V1=0
              V2=0
         ELSE IF (J.GT.NXCONT) THEN
            INDX1=NDMAXDET*(J-2)+(IXDET+1)
            IF (INDX1.GT.MAXINDX) MAXINDX=INDX1
CS            INDX2=NDMAXDET*(NXCONT-1)+(IXDET+1)
CS            IF (INDX2.GT.MAXINDX) MAXINDX=INDX2
            V1POS=ENEROFXCONT(NXCONT)
            V2POS=VRIGHTPOS
CS            V1=XDETOFXCONT(IXDET,J)
CS            V2=XDETOFXCONT(IXDET,NXCONT)
            V1=XDETOFXCONT(INDX1)
            IF (VALPOS.GT.E0) V1=0
CS            V2=XDETOFXCONT(INDX2)
            V2=0
         ELSE
            INDX1=NDMAXDET*(J-2)+(IXDET+1)
            IF (INDX1.GT.MAXINDX) MAXINDX=INDX1
            INDX2=NDMAXDET*(J-1)+(IXDET+1)
            IF (INDX2.GT.MAXINDX) MAXINDX=INDX2
            V1POS=ENEROFXCONT(J-1)
            V2POS=ENEROFXCONT(J)
CS            V1=XDETOFXCONT(IXDET,J-1)
CS            V2=XDETOFXCONT(IXDET,J)
            V1=XDETOFXCONT(INDX1)
            V2=XDETOFXCONT(INDX2)
         ENDIF
         GRADK=(V2-V1)/(V2POS-V1POS)
         DELTA=V1-GRADK*V1POS
         VALHIS(ISTART+I)=GRADK*VALPOS+DELTA
         VALHIS(ISTART+I)=VALHIS(ISTART+I)*VDELT(IHIS,1)
cs         write(6,'(i4,d18.9)')istart+i,valhis(istart+i)
 20   CONTINUE

      CALL CHKBND (13,MAXINDX,'IPOLHI','XDETOFXCONT')

      RETURN
      END

*****************************************************************************
*last modification 19-Feb-1997

      SUBROUTINE ADDVAL(IHIS,NHELP,RHELP)

**    ADDVAL - ADDS ARRAY VALUES TO A HISTOGRAM
*
**    ARGUMENTS:
*     IHIS  - HISTOGRAM NUMBER
*     NHELP - DIMENSION OF ARRAY 
*     RHELP - ARRAY
*
*     AUTHOR: CHRISTIAN SCHIEBL
*             CONSULTANT (FORMER: CEC-VIENNA)
*             KREMS, OCTOBER 1996

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'HIST.INC'

      DIMENSION RHELP(NHELP)

***   START OF EXECUTABLE CODE
******************************

      ISTART=IVBEG(IHIS)-1
      NVH=ISTART+NHELP
      CALL CHKBND(8,NVH,'INIHIS','VALHIS')
      VMAX=VALHIS(ISTART+1)
      DO 10 I=2,NHELP
         IF (VALHIS(ISTART+I).GT.VMAX) VMAX=VALHIS(ISTART+I)
 10   CONTINUE
      VMAX=VMAX*2
      DO 20 I=1,NHELP
         VALHIS(ISTART+I)=VALHIS(ISTART+I)+RHELP(I)
CS       AN ARTIFICIAL MULTIPLICATION FACTOR!
CS       VALHIS(ISTART+I)=MIN(VMAX,VALHIS(ISTART+I)+RHELP(I)/100)
 20   CONTINUE

      RETURN
      END

*****************************************************************************
*last modification 13-Feb-1997

      SUBROUTINE CHKHIS()

**    CHKHIS - CHECKS HISTOGRAMS FOR 'XRY' AND MAKES NECESSARY CONNECTIONS
*              TO XCHAR AND XCONT TABLES
*
**    ARGUMENTS:
*             NONE
*
*     AUTHOR: CHRISTIAN SCHIEBL
*             CONSULTANT (FORMER: CEC-VIENNA)
*             KREMS/HUDSON, FEBRUARY 1997

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'DMA.INC'


***   START OF EXECUTABLE CODE
******************************

      IF (NXCONT.LE.0) THEN
         WRITE(6,'(A)')'        '
         WRITE(6,'(A)')' *** ERROR ***'
         WRITE(6,'(A)')' CHKHIS: No Bremsstrahlung will be calculated'
         WRITE(6,'(2A)')' Since needed for the specified XRY ',
     &                  'histogram SeSAME stops now'
         STOP
      ENDIF

      FACT=3/2.355

      DO 10 I=1,NXCONT
         NXKHIS(I)=0
 10   CONTINUE

      DO 100 IHIS=1,NHIST
         IF ((CVALUE(IHIS)(1:3).EQ.'XCY').OR.
     &       (CVALUE(IHIS)(1:3).EQ.'XKY').OR.
     &       (CVALUE(IHIS)(1:3).EQ.'XRY')) THEN
            IF (CVALUE(IHIS)(1:3).EQ.'XCY') THEN
              IF ((HISENWIN(IHIS,1).LT.0).OR.
     &            (HISENWIN(IHIS,2).LE.HISENWIN(IHIS,1))) GOTO 100
            ELSE
              IF ((HISENWIN(IHIS,1).GE.HISENWIN(IHIS,2)).OR.
     &            (HISENWIN(IHIS,1).LT.0).OR.
     &            (HISENWIN(IHIS,2).LE.0)) THEN
*                ENERGY WINDOW HAS BEEN NOT OR IMPROPERLY DEFINED
*                SET THE ENERGY WINDOW AND LET USER KNOW
                 WRITE(6,'(A)')'           '
                 WRITE(6,'(A)')' *** WARNING ***'
                 WRITE(6,'(2A,I1,2A)')' CHKHIS: the energy window ',
     &                'for the x-ray detector #',HISEDT(IHIS),' not',
     &                ' properly specified'
                 WRITE(6,'(A)')' no action so far!'
              ENDIF
            ENDIF
            IXDET=HISEDT(IHIS)
            IF (CVALUE(IHIS)(1:3).EQ.'XKY') GOTO 55
            DO 50 IX=1,NXCHAR
               INDX2=(IX-1)*NDMAXDET+IXDET+1
               IF ((DABS(ENEROFXCHAR(IX)-HISENWIN(IHIS,1)).LT.
     &              FACT*FWHMOFXCHAR(INDX2)).OR.
     &             (DABS(ENEROFXCHAR(IX)-HISENWIN(IHIS,2)).LT.
     &              FACT*FWHMOFXCHAR(INDX2))) THEN
*                 TAKE THIS LINE INTO ACCOUNT AND ALSO CALCULATE 
*                 FRACTION SEEN WITHIN THE WINDOW
                  ISSPECIFIED=0
                  DO 45 J=1,NXCHIS(IX)
                     INDX=(IX-1)*NDMAHIST+J
                     IF (XCHIS(INDX).EQ.IHIS) THEN
                        ISSPECIFIED=J
CS                        WRITE(6,'(A)')'      '
CS                        WRITE(6,'(2A)')' CHKHIS: Attention, current ',
CS     &                       'line arlready directed to current HIS!'
CS                        WRITE(6,'(A)')' press return to continue'
CS                        READ(*,*)
                     ENDIF
 45               CONTINUE
                  IF (ISSPECIFIED.EQ.0) THEN
*                    ADD THIS LINE TO HISTOGRAMM IHIS
                     NXCHIS(IX)=NXCHIS(IX)+1
                     INDX=(IX-1)*NDMAHIST+NXCHIS(IX)
                     XCHIS(INDX)=IHIS
                     XDETXCHIS(INDX)=HISEDT(IHIS)
                  ELSE
                     INDX=(IX-1)*NDMAHIST+ISSPECIFIED
                  ENDIF
                  IF (IXDET.NE.XDETXCHIS(INDX)) THEN
                     WRITE(6,'(A)')'           '
                     WRITE(6,'(A)')' *** WARNING ***'
                     WRITE(6,'(2A)')' CHKHIS: something wrong with',
     &                              ' detector number!'
                     WRITE(6,'(A)')' no action so far!'
                     WRITE(6,'(A)')' Press <RETURN> to continue!'
                     read(*,*)
                  ENDIF
*                 CALCULATE FRACTION WITHIN THE WINDOW
                  INDX2=(IX-1)*NDMAXDET+IXDET+1
                  XCHISWINFRAC(INDX)=GETGAUSSFRACT(ENEROFXCHAR(IX),
     &                               HISENWIN(IHIS,1),HISENWIN(IHIS,2),
     &                               FWHMOFXCHAR(INDX2))
                  IF (LTEST) THEN
                     WRITE(6,'(A)')'           '
                     WRITE(6,'(2A)')' FRACTIONS OF PEAKS WITHIN',
     &                              ' SPECIFIED ENERGY WINDOW:'
                     WRITE(6,'(A,I5)')' ENTRY IN XCHAR-TABLE:',IX
                     WRITE(6,'(A,F9.3)')' X-RAY ENERGY IN EV:  ',
     &                                  ENEROFXCHAR(IX)
                     WRITE(6,'(A,I5)')' HISTOGRAM NUMBER:    ',IHIS
                     WRITE(6,'(A,I5)')' NXCHIS:              ',
     &                                  NXCHIS(IX)
                     WRITE(6,'(A,I5)')' INDEX IN XCHIS:      ',INDX
                     WRITE(6,'(A,I5)')' XCHIS:               ',
     &                                  XCHIS(INDX)
                     WRITE(6,'(A,I5)')' DETECTOR NUMBER:     ',
     &                                  XDETXCHIS(INDX)
                     WRITE(6,'(A,F10.4)')' FRACTION IN E-WINDOW:',
     &                                  XCHISWINFRAC(INDX)
                  ENDIF
               ENDIF
 50         CONTINUE
 55         CONTINUE
            IF (CVALUE(IHIS)(1:3).EQ.'XCY') GOTO 100
            INSIDE=0
            VCENTER=0
            ITAKETHIS=0
            DO 60 IX=1,NXCONT
               IF ((ENEROFXCONT(IX).GE.HISENWIN(IHIS,1)).AND.
     &             (ENEROFXCONT(IX).LE.HISENWIN(IHIS,2))) THEN
                  INSIDE=1
*                 TAKE THE MOST CENTERED ONE
                  DL=ENEROFXCONT(IX)-HISENWIN(IHIS,1)
                  DR=HISENWIN(IHIS,2)-ENEROFXCONT(IX)
                  DLR=DL*DR
                  IF (DLR.GT.VCENTER) THEN
                     VCENTER=DLR
                     ITAKETHIS=IX
                  ENDIF
               ENDIF
 60         CONTINUE
            IF (ITAKETHIS.LE.0) THEN
               WRITE(6,'(A)')'           '
               WRITE(6,'(A)')' *** WARNING ***'
               WRITE(6,'(3A)')' CHKHIS: there is no Bremsstrahlung',
     &                        ' reference energy within the energy',
     &                        ' window!'
               WRITE(6,'(2A)')' SeSAME will take the closest to',
     &                        ' the specified energy window.'
               WRITE(6,'(2A)')' Be aware that results may be very',
     &                        ' inaccurate!'
               ITAKETHIS=1
               DELTE=MIN(DABS(ENEROFXCONT(1)-HISENWIN(IHIS,1)),
     &                   DABS(ENEROFXCONT(1)-HISENWIN(IHIS,2)))
               DO 70 IXK=2,NXCONT
                  D=MIN(DABS(ENEROFXCONT(IXK)-HISENWIN(IHIS,1)),
     &                  DABS(ENEROFXCONT(IXK)-HISENWIN(IHIS,2)))
                  IF (D.LT.DELTE) THEN
                     DELTE=D
                     ITAKETHIS=IXK
                  ENDIF
 70            CONTINUE
               WRITE(6,'(A,F10.2,A,F10.2)')' Energy Window [eV] from:',
     &                         HISENWIN(IHIS,1),' to',HISENWIN(IHIS,2)
               WRITE(6,'(2A,F10.2,A)')' Bremsstrahlung calculation',
     &               ' at energy:',ENEROFXCONT(ITAKETHIS),' eV'
            ENDIF
            INDX=(ITAKETHIS-1)*NDMAHIST+NXKHIS(ITAKETHIS)+1
            IF ((NXKHIS(ITAKETHIS).LT.NHIST).AND.
     &          (XKHIS(INDX).EQ.0)) THEN
               NXKHIS(ITAKETHIS)=NXKHIS(ITAKETHIS)+1
               XKHIS(INDX)=IHIS
               XDETXKHIS(INDX)=HISEDT(IHIS)
            ELSE
               WRITE(6,'(A)')'      '
               WRITE(6,'(A)')' CHKHIS: Warning'
               WRITE(6,'(A)')' NXKHIS larger than NHIST!'
               WRITE(6,'(A)')' press return to continue!'
               STOP
            ENDIF
            IF (LTEST) THEN
               WRITE(6,'(A)')'           '
               WRITE(6,'(2A)')' HISTOGRAMS FOR LINE SCANS TAKING',
     &                        ' BREMSSTRAHLUNG INTO ACCOUNT:'
               WRITE(6,'(A,I5)')' ENTRY IN XCONT-TABLE:',ITAKETHIS
               WRITE(6,'(A,I5)')' HISTOGRAM NUMBER:    ',IHIS
               WRITE(6,'(A,I5)')' NXKHIS:              ',
     &                            NXKHIS(ITAKETHIS)
               WRITE(6,'(A,I5)')' INDEX IN XKHIS:      ',INDX
               WRITE(6,'(A,I5)')' XKHIS:               ',
     &                            XKHIS(INDX)
               WRITE(6,'(A,I5)')' DETECTOR NUMBER:     ',
     &                            XDETXKHIS(INDX)
            ENDIF
         ENDIF

 100  CONTINUE

      IF (LTEST) THEN
         WRITE(6,'(A)')'           '
         WRITE(6,'(A)')' XCHAR TABLES WITH DETAILED XCHIS-INFO:'
         WRITE(6,'(3A)')' Line#  Energy  H# ',
     &                  'D#  Fraction  Omega ',
     &                  '  rel.I.  Deff   L  Z    FWHM     E-Win'
         DO 210 I=1,NXCHAR
           INDX=(I-1)*NDMAHIST+1
           INDX2=(I-1)*NDMAXDET+XDETXCHIS(INDX)+1
           IF (XCHIS(INDX).GT.0) THEN
             WRITE
     &       (6,'(I4,F11.2,I3,I3,F10.6,2F8.4,F7.3,I3,I3,F10.2,F8.1)')
     &             I,ENEROFXCHAR(I),
     &             XCHIS(INDX),XDETXCHIS(INDX),
     &             XCHISWINFRAC(INDX),
     &             FYIELDOFXCHAR(I),FRELINTOFXCHAR(I),
     &             XDETEFFOFXCHAR(INDX2),
     &             XCHAROFXCHAR(I),ATNUM(ELEMOFXCHAR(I)),
     &             FWHMOFXCHAR(INDX2),
     &             HISENWIN(XCHIS(INDX),2)-HISENWIN(XCHIS(INDX),1)
           ELSE
             WRITE
     &       (6,'(I4,F11.2,I3,I3,F10.6,2F8.4,F7.3,I3,I3,F10.2)')
     &             I,ENEROFXCHAR(I),
     &             XCHIS(INDX),XDETXCHIS(INDX),
     &             XCHISWINFRAC(INDX),
     &             FYIELDOFXCHAR(I),FRELINTOFXCHAR(I),
     &             XDETEFFOFXCHAR(INDX2),
     &             XCHAROFXCHAR(I),ATNUM(ELEMOFXCHAR(I)),
     &             FWHMOFXCHAR(INDX2)
           ENDIF
           DO 200 J=2,NXCHIS(I) 
             INDX=(I-1)*NDMAHIST+J
             INDX2=(I-1)*NDMAXDET+XDETXCHIS(INDX)+1
             IF (XCHIS(INDX).GT.0) THEN
               WRITE(6,'(A,I3,I3,F10.6,A,F7.3,A,F10.2,F8.1)')
     &            '               ',
     &            XCHIS(INDX),XDETXCHIS(INDX),
     &            XCHISWINFRAC(INDX),
     &            '                ',
     &            XDETEFFOFXCHAR(INDX2),'      ',FWHMOFXCHAR(INDX2),
     &            HISENWIN(XCHIS(INDX),2)-HISENWIN(XCHIS(INDX),1)
             ELSE
               WRITE(6,'(A,I3,I3,F10.6,A,F7.3,A,F10.2)')
     &            '               ',
     &            XCHIS(INDX),XDETXCHIS(INDX),
     &            XCHISWINFRAC(INDX),
     &            '                ',
     &            XDETEFFOFXCHAR(INDX2),'      ',FWHMOFXCHAR(INDX2)
             ENDIF
 200       CONTINUE
 210     CONTINUE
         WRITE(6,'(A)')'           '
         WRITE(6,'(A)')' XCONT TABLES WITH DETAILED XKHIS-INFO:'
         WRITE(6,'(3A)')'   #    Energy   Hist# ',
     &                  'Detector#   Energy-window   '
         IF (NDMAHIST.GT.0) THEN
           DO 310 I=1,NXCONT
             INDX=(I-1)*NDMAHIST+1
             IF (XKHIS(INDX).GT.0) THEN
               K=XKHIS(INDX)
               WRITE(6,'(I4,F11.2,I5,I7,2F12.2)')
     &               I,ENEROFXCONT(I),
     &               XKHIS(INDX),XDETXKHIS(INDX),
     &               HISENWIN(K,1),HISENWIN(K,2)
             ELSE
               WRITE(6,'(I4,F11.2,I5,I7)')
     &               I,ENEROFXCONT(I),
     &               XKHIS(INDX),XDETXKHIS(INDX)
             ENDIF
             DO 300 J=2,NXKHIS(I)
               INDX=(I-1)*NDMAHIST+J
               IF (XKHIS(INDX).GT.0) THEN
                 K=XKHIS(INDX)
                 WRITE(6,'(A,I5,I7,2F12.2)')'               ',
     &                 XKHIS(INDX),XDETXKHIS(INDX),
     &                 HISENWIN(K,1),HISENWIN(K,2)
               ELSE
                 WRITE(6,'(A,I5,I7)')'               ',
     &                 XKHIS(INDX),XDETXKHIS(INDX)
               ENDIF
 300         CONTINUE
 310       CONTINUE
        ENDIF
      ENDIF

      RETURN
      END

*****************************************************************************
*last modification: 07-Feb-1997

      DOUBLE PRECISION FUNCTION GETGAUSSFRACT (E,E1,E2,FWHM)

***   GETGAUSSFRACT - GETS THE FRACTION OF A GAUSSION SHAPED PEAK WHICH
*                     WITHIN A DEFINED ENERGY WINDOW
*
*     E    - ENERGY AT PEAK POSITION
*     E1   - ENERGY AT LEFT ENERGY WINDOW BOUNDARY
*     E2   - ENERGY AT RIGHT ENERGY WINDOW BOUNDARY
*     FWHM - FULL WIDTH AT HALF MAXIMUM OF GAUSSION SHAPED PEAK
*
**    AUTHOR: CHRISTIAN SCHIEBL
*             CONSULTANT (FORMER: CEC-VIENNA)
*             KREMS/HUDSON, FEBRUARY 1997

      INCLUDE 'TYPE.INC'

      DOUBLE PRECISION F(0:30)

      DATA F / 0.5,0.5398,0.5793,0.6179,0.6554,
     &         0.6915,0.7257,0.758,0.7881,0.8159,
     &         0.8413,0.8643,0.8849,0.9032,0.9192,
     &         0.9332,0.9452,0.9554,0.9641,0.9713,
     &         0.9772,0.9821,0.9861,0.9893,0.9918,
     &         0.9938,0.9953,0.9965,0.9974,0.9981,0.9987 /

***   START OF EXECUTABLE CODE
******************************

      GETGAUSSFRACT = 1

      SIGMA = FWHM/2.355

      IF (E.LE.E1) THEN
         IF ((E+3*SIGMA).LE.E1) THEN
            GETGAUSSFRACT = 0
         ELSE
            INDX=INT(10*(E1-E)/SIGMA+0.5)
            GETGAUSSFRACT = 1-F(INDX)
            IF ((E+3*SIGMA).GT.E2) THEN
               INDX=INT(10*(E2-E)/SIGMA+0.5)
               GETGAUSSFRACT = GETGAUSSFRACT+F(INDX)-1
            ENDIF
         ENDIF
      ELSE IF ((E.GT.E1).AND.(E.LT.E2)) THEN
         IF ((E-3*SIGMA).LT.E1) THEN
            INDX=INT(10*(E-E1)/SIGMA+0.5)
            GETGAUSSFRACT = F(INDX)
         ENDIF
         IF ((E+3*SIGMA).GT.E2) THEN
            INDX=INT(10*(E2-E)/SIGMA+0.5)
            GETGAUSSFRACT = GETGAUSSFRACT+F(INDX)-1
         ENDIF
      ELSE IF (E.GE.E2) THEN
         IF ((E-3*SIGMA).GE.E2) THEN
            GETGAUSSFRACT = 0
         ELSE
            INDX=INT(10*(E-E2)/SIGMA+0.5)
            GETGAUSSFRACT = 1-F(INDX)
            IF ((E-3*SIGMA).LT.E1) THEN
               INDX=INT(10*(E-E1)/SIGMA+0.5)
               GETGAUSSFRACT = GETGAUSSFRACT+F(INDX)-1
            ENDIF
         ENDIF
      ENDIF

      RETURN
      END

****************************************************************************
