c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
*last modification: 07-Mar-1997

      SUBROUTINE INIT_MAC (     
     D                     NPTS,NFCS,XCABS,
     O                     IER) 

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'DMA.INC'

*** INIT_MAC - INITIALIZES ARRAY FOR SPACIAL ABSORPTION FACTORS FOR
*              1D AND 2D DEPTH DISTRIBUTIONS (CHARACTERISTIC X-RAYS)
*
**  ARGUMENTS 
*
*   INPUT:
*        NPTS  - NUMBER OF POINTS OF SIMULATION GEOMETRY
*        NFCS  - NUMBER OF FACES OF SIMULATION GEOMETRY
*
*   OUTPUT:
*        IER   - ERROR FLAG    
*
**  AUTHOR: A.PFEIFFER OCT-1994
*           C.SCHIEBL  JUN-1994, JAN-1996 (BUG FIXES, CLEANED UP)
*

      PARAMETER(DELTA = 1D-8) 
*     SMALLEST NEGATIV EXPONENT TO AVOID FLOATING UNDERFLOW
      PARAMETER(EXPMN = 600.)

*     ABSORPTION FACTORS (FOR CHARACTERISTIC AND BACKGROUND RADIATION)
      DOUBLE PRECISION XCABS(0:NDMAXDET-1,NDMAXCONT+NDMAXCHAR-NDMAPTOB,
     &                       NDMAXGRID,NDMAZGRID,NDMAYGRID)

      LOGICAL LSIDE(0:MAXXDET)
      DOUBLE PRECISION MUERHO 
      DOUBLE PRECISION XDIR(0:MAXXDET),ZDIR(0:MAXXDET)
      DOUBLE PRECISION RMM(1)
      POINTER (P2RMM,RMM)
      DOUBLE PRECISION HELPXC(1)
      POINTER (P2HELPXC,HELPXC)

CS      DIMENSION RMM(MAX(MAXXCHARS,MAXXCONT),MAXREG)
CS      DOUBLE PRECISION HELPXC(NXM,NZM)

CS      EQUIVALENCE (RMM,HELPXC)

**    START OF EXECUTABLE CODE
*     ************************

      CALL STATXT(' INITIALIZING X-RAY ABSORPTION FACTOR ARRAY')
      IER =  0

*     ALLOCATE MEMORY FOR RMM
CS      NENTRY=MAX(NDMAXCHAR,NDMAXCONT)
      NENTRY=NDMAXCONT+NDMAXCHAR-NDMAPTOB
      NALLOC1=NENTRY*NDMAREG
      IF (NALLOC1.GT.0) THEN
         P2RMM=MALLOC(SIZEOF(RMM(1))*NALLOC1)
         IF (P2RMM.EQ.0) THEN
            WRITE(6,'(A)')'     '
            WRITE(6,'(A)')' *** ERROR ***'
            WRITE(6,'(2A)')' INIT_MAC: Could not allocate memory',
     &                     ' for RMM !'
            STOP
         ENDIF
      ELSE
         WRITE(6,'(A)')'     '
         WRITE(6,'(A)')' *** ERROR ***'
         WRITE(6,'(2A)')' INIT_MAC: N<1 => did not allocate memory',
     &                  ' for RMM !'
         IER=1
         RETURN
      ENDIF

*     ALLOCATE MEMORY FOR HELPXC
      NALLOC2=NDMAXGRID*NDMAZGRID*NDMAYGRID
      IF (NALLOC2.GT.0) THEN
         P2HELPXC=MALLOC(SIZEOF(HELPXC(1))*NALLOC2)
         IF (P2HELPXC.EQ.0) THEN
            WRITE(6,'(A)')'     '
            WRITE(6,'(A)')' *** ERROR ***'
            WRITE(6,'(2A)')' INIT_MAC: Could not allocate memory',
     &                     ' for HELPXC !'
            STOP
         ENDIF
      ELSE
         WRITE(6,'(A)')'     '
         WRITE(6,'(A)')' *** ERROR ***'
         WRITE(6,'(2A)')' INIT_MAC: N<1 => did not allocate memory',
     &                  ' for HELPXC !'
         IER=1
         RETURN
      ENDIF

*     DETERMINE DIRECTIONS OT THE X-RAY DETECTORS
      DO 2 IXD=0,NDMAXDET-1
         XDIR(IXD) = DCOS(TAKOFR(IXD))*DCOS(DEROTR(IXD))
         ZDIR(IXD) = -DSIN(TAKOFR(IXD)) 

         IF (((DXDEROT(IXD).GT.89.9).AND.(DXDEROT(IXD).LT.90.1)).OR.
     &       ((DXDEROT(IXD).GT.269.9).AND.(DXDEROT(IXD).LT.270.1)))
     &      THEN
            XDIR(IXD)=0
         ENDIF
 2    CONTINUE

      DIRDOX=0.   
      DIRDOZ=1.

*     EXPANSION OF GEOMETRY AT EAST AND WEST BOUNDARIES (IF VERTICAL)  
      DO 3 I=0,NDMAXDET-1
         IF ((ABS(ZDIR(I)).EQ.1.)) THEN
            LSIDE(I)=.FALSE.
         ELSE
            LSIDE(I)=.TRUE.
         ENDIF
         IF ((SCORDZ(1).GT.Z_MAX).OR.
     &       (SCORDZ(NSCORD).GT.Z_MAX)) THEN
            LSIDE(I)=.FALSE.
         ELSE
            LSIDE(I)=.TRUE.
         ENDIF
 3    CONTINUE
       
*     INITIALIZE THE MASS ABSORPTION COEFFICIENTS FOR ALL REGIONS AND ALL LINES
      DO 15 I_XCHAR = 1,NDMAXCHAR-NDMAPTOB+NDMAXCONT
         DO 19 IX = 1,NDMAXGRID
            DO 18 IZ = 1,NDMAZGRID
               DO 17 IY = 1,NDMAYGRID
                  DO 16 IXD = 0,NDMAXDET-1
                     XCABS(IXD,I_XCHAR,IX,IZ,IY) = 1
16                CONTINUE
17             CONTINUE
18          CONTINUE
19       CONTINUE 
15    CONTINUE
      DO 5 I_XCHAR = 1,NDMAXCHAR
         I_ELE=ELEMOFXCHAR(I_XCHAR)
         EXC = ENEROFXCHAR(I_XCHAR)
         I_XCLINE=XCHAROFXCHAR(I_XCHAR)
         IRAD = ATNUM(I_ELE)
         IF (EXC.EQ.1000000.) GOTO 5
         IF (IRAD.EQ.0.) GOTO 5               
         EXC = EXC/1000.
         IF (LTEST) THEN
            WRITE(6,'(A)')'REGION Z-EMIT L-EMIT E-EMIT   MAC '
         ENDIF
         DO 7 IRH = 1,NDMAREG
            INDX=NENTRY*(IRH-1)+I_XCHAR
            IF (INDX.GT.NALLOC1) THEN
               WRITE(6,'(A)')'     '
               WRITE(6,'(A)')' *** WARNING ***'
               WRITE(6,'(A)')' INIT_MAC: INDEX > NALLOC1'
            ENDIF
            RMM(INDX)=0
CS            RMM(I_XCHAR,IRH)=0
            DO 8 I_ELEM = 1,NELE(IRH)
               C_WEIGHT=WEIGHTFRAC(IRH,I_ELEM)
               IATNUM=ATNUM(ELEM(IRH,I_ELEM))
               IF (IATNUM.EQ.0) GOTO 7  
               CALL GETMY(
     I                    IATNUM,IRAD,I_XCLINE,EXC,
     O                    RMUE)
CS               IF (LTEST) THEN
CS                  WRITE(6,'(3I6,F12.4,F16.4)')IRAD,I_XCLINE,IATNUM,
CS     &                      EXC,RMUE
CS               ENDIF
*              CALCULATION OF MEAN MAC IN REGION IRH
               RMM(INDX)=RMM(INDX)
     &                          +C_WEIGHT*RMUE*DENS(IRH)*1.D-8
CS               RMM(I_XCHAR,IRH)=RMM(I_XCHAR,IRH)
CS     &                          +C_WEIGHT*RMUE*DENS(IRH)*1.D-8
  8         CONTINUE
            IF (LTEST) THEN
               WRITE(6,'(I4,2I7,F10.3,E14.4)')IRH,IRAD,I_XCLINE,EXC,
     &                                        RMM(INDX)
            ENDIF
  7      CONTINUE
  5   CONTINUE

      I_XCLINE=-1
      IF (NDMAXCONT.GT.0)
     &   CALL CHKBND (7,NDMAXCONT,'INIT_MAC','IXCONTOFXCHAR')
CS      DO 350 I_XCONT = 1,NDMAXCONT+NDMAXCHAR-NDMAPTOB
      DO 350 I_XCONT = 1,NDMAXCONT
         IF (IXCONTOFXCHAR(I_XCONT).LE.NDMAXCHAR) GOTO 350
         I_X = IXCONTOFXCHAR(I_XCONT)
         EXC = ENEROFXCONT(I_XCONT)
         IRAD = -1
         IF (EXC.EQ.1000000.) GOTO 350
         IF (IRAD.EQ.0.) GOTO 350
         EXC = EXC/1000.
         IF (LTEST) THEN
            WRITE(6,'(A)')'REGION Z-EMIT L-EMIT E-EMIT   MAC '
         ENDIF
         DO 340 IRH = 1,NDMAREG
            INDX=NENTRY*(IRH-1)+I_X
            IF (INDX.GT.NALLOC1) THEN
               WRITE(6,'(A)')'     '
               WRITE(6,'(A)')' *** WARNING ***'
               WRITE(6,'(A)')' INIT_MAC: INDEX > NALLOC1'
            ENDIF
            RMM(INDX)=0
CS            RMM(I_X,IRH)=0
            DO 330 I_ELEM = 1,NELE(IRH)
               C_WEIGHT=WEIGHTFRAC(IRH,I_ELEM)
               IATNUM=ATNUM(ELEM(IRH,I_ELEM))
               IF (IATNUM.EQ.0) GOTO 340
               CALL GETMY(
     I                    IATNUM,IRAD,I_XCLINE,EXC,
     O                    RMUE)
CS               IF (LTEST) THEN
CS                  WRITE(6,'(3I6,F12.4,F16.4)')IRAD,I_XCLINE,IATNUM,
CS     &                      EXC,RMUE
CS               ENDIF
*              CALCULATION OF MEAN MAC IN REGION IRH
               RMM(INDX)=RMM(INDX)
     &                      +C_WEIGHT*RMUE*DENS(IRH)*1.D-8
CS               RMM(I_X,IRH)=RMM(I_X,IRH)
CS     &                      +C_WEIGHT*RMUE*DENS(IRH)*1.D-8
 330        CONTINUE
            IF (LTEST) THEN
               WRITE(6,'(I4,2I7,F10.3,E14.4)')IRH,IRAD,I_XCLINE,EXC,
     &                                        RMM(INDX)
            ENDIF
 340     CONTINUE
 350  CONTINUE

      IF (LTEST) THEN
         WRITE(6,'(A)')'       '
         WRITE(6,'(A)')'    INDX        RMM(INDX)'
         DO 351 I=1,NALLOC1
            WRITE(6,'(I7,E18.9)')I,RMM(I)
 351     CONTINUE
      ENDIF

*     DETERMINE NEED OF LATERAL GEOMETRY EXPANSION TO INFINITY
      DO 11 I=0,NDMAXDET-1
         IF (LSIDE(I)) THEN
            IF (XDIR(I).GT.0.) THEN
               SURX=SCORDX(NSCORD)-DELTA
               SURZ=SCORDZ(NSCORD)+DELTA    
            ELSE
               SURX=SCORDX(1)+DELTA
               SURZ=SCORDZ(1)+DELTA
            ENDIF
            CALL IMGREG(IUSOLL,
     I 	                SURX,SURZ,PCOORD,
     O	   	        IRNXT,IER)           
 130        IRH=IRNXT             
            CALL INTERSECT (
     D	                    NPTS,NFCS,
     I	                    PCOORD,ILINES,IENDIX,
     I                      IRH,SURX,SURZ,DIRDOX,DIRDOZ,
     O                      XNXT,ZNXT,IRNXT,RLDB,IER)
            SURX=XNXT
            SURZ=ZNXT+DELTA 

C           STOP INTERS=0!!
            IF (IRNXT.NE.0) THEN
               GOTO 130
            ELSE
               IF (ZNXT.LT.Z_MAX) THEN
C                 NO EXPANSION INTO INFINITY
                  LSIDE(I)=.FALSE.  
               ENDIF
            ENDIF
         ENDIF
 11   CONTINUE

*     LOOPS OVER ALL X-RAY DETECTORS AND ALL GRID POINTS
      DO 20 IXD = 0,NDMAXDET-1
         DIRX=XDIR(IXD)
         DIRZ=ZDIR(IXD)
         DO 10 IX = 1, NDMAXGRID
           DO 9 IZ = 1, NDMAZGRID
             DO 12 IY = 1, NDMAYGRID
*              CALCULATION OF POINT LOCATION (REGION), INTERSECTION IN
*              TAKE-OFF DIRECTION AND PATH LENGTH IN REGION
               XBOX=XGRIDL(IX)
               ZBOX=ZGRIDL(IZ)
	       IF (XBOX.EQ.X_MIN) XBOX=XBOX+DELTA
               IF (ZBOX.EQ.Z_MIN) ZBOX=ZBOX+DELTA             
               IF (XBOX.EQ.X_MAX) XBOX=XBOX-DELTA
               IF (ZBOX.EQ.Z_MAX) ZBOX=ZBOX-DELTA
*              GRIDPOINT MUST NOT BE EQUAL TO GEOMETRIE POINT (FOR GEO-CHECKS) 
               DO 220 I = 1,2*NPTS,2
                  IF ((XBOX.EQ.PCOORD(I))) THEN  
                     IF (XBOX.GE.X_MIN) THEN 
                        XBOX = XBOX+DELTA
                     ELSE
                        XBOX = XBOX-DELTA
                     ENDIF  
C                     XBOX = XBOX + DIRX*DELTA
                     GOTO 221
                  ENDIF
 220           CONTINUE
 221           CONTINUE      
               DO 222 I = 2,2*NPTS,2   
                  IF (ZBOX.EQ.PCOORD(I)) THEN
                     IF (ZBOX.GT.Z_MIN) THEN 
                        ZBOX = ZBOX+DELTA
                     ELSE
                        ZBOX = ZBOX-DELTA
                     ENDIF   
C                     ZBOX = ZBOX + DIRZ*DELTA
                     GOTO 24
                  ENDIF
 222           CONTINUE
                  
 24            IUSOLL=0      
*              DETERMINE IN WHICH REGION GRIDPOINT XBOX/ZBOX LIES
               CALL IMGREG(IUSOLL,
     I 	                   XBOX,ZBOX,PCOORD,
     O        	           IRNXT,IER) 
               IF (IRNXT.EQ.0.OR.IRNXT.EQ.-1) THEN
*                 THIS GRIDPOINT LIES OUTSIDE THE GEOMETRY
                  DO 26 I = 1,NDMAXCHAR
                     XCABS(IXD,I,IX,IZ,IY)=1
 26               CONTINUE
*                 TAKE NEXT GRIDPOINT
                  GOTO 12
               ENDIF
                        
               IRFIRST=IRNXT

*              IRFIRST ... REGION OF GRIDPOINT
*              IREG    ... REGION OF ABSORPTION
*              IRNXT   ... NEXT REGION OF ABSORPTION

30             IREG = IRNXT 

*              POINT IN AIR ?
               IF (IRNXT.LE.0) THEN
*                 POINT LIES OUTSIDE THE GEOMETRY
*                 CHECK WHETHER THERE IS AN INTERSECTION WITH THE GEOMETRY
*                 IN DIRECTION TO X-RAY DETECTOR
                  CALL IMENTR (
     I                         XBOX, ZBOX, DIRX, DIRZ, PCOORD,
     O                         INTERS, XNXT, ZNXT, IRNXT, IER)
*                 POINT LEFT SIMULATION AREA?
                  IF (INTERS.LE.0 .OR. 
     &               IRNXT .LE.0 ) THEN
*                    NO INTERSECTION WITH GEOMETRY - TAKE NEXT GRIDPOINT
                     GOTO 12
                  ELSE
*                    INTERSECTION - MOVE POINT XBOX/ZBOX INTO NEXT REGION
*                    THIS POINT LIES NOW IN REGION IREG
                     XBOX = XNXT+DIRX*DELTA
                     ZBOX = ZNXT+DIRZ*DELTA 
                     GOTO 30
                  END IF
               ELSE

*                 DETERMINE LENGTH TO BOUNDARY OF IREG IN DIRECTION TO THE
*                 X-RAY DETECTOR (=ABSORPTION LENGTH RLDB)           
                  CALL INTERSECT (
     D	                          NPTS,NFCS,
     I	                          PCOORD,ILINES,IENDIX,
     I                            IREG,XBOX,ZBOX,DIRX,DIRZ,
     O	                          XNXT,ZNXT,IRNXT,RLDB,IER)
*                 TROUBLES WITH GRID POINTS VERY CLOSE TO GEOMETRY LINES
                  IF (IER.NE.0) THEN
                     DO 28 I = 1,NDMAXCHAR
                        XCABS(IXD,I,IX,IZ,IY)=1
 28                  CONTINUE
*                    TAKE NEXT GRID POINT
                     GOTO 12
                  ENDIF

*                 SHIFT POINT XBOX/ZBOX INTO THE NEXT REGION
                  XBOX = XNXT+DIRX*DELTA
                  ZBOX = ZNXT+DIRZ*DELTA
*                 DETERMINE REGION OF NEW POINT XBOX/ZBOX
                  CALL IMGREG(IUSOLL,
     I 	                      XBOX,ZBOX,PCOORD,
     O	   	              IRBOX,IER)
                  IF ( (IRBOX.LE.0).AND.(IRNXT.LE.0) ) IRBOX = IRNXT
                  IF (IRBOX.NE.IRNXT) THEN
                     XBOX = XBOX+DIRX*DELTA    
                     ZBOX = ZBOX+DIRZ*DELTA
                     IRNXT = IRBOX
                     GOTO 30
                  ENDIF 

*                 THIS LOCATION IN THE ROUTINE IS ONLY REACHED IF THE
*                 GRID POINT LIES IN A REGION OF THE GEOMETRY
*                 LOOP OVER ALL X-RAY LINES

                  DO 32 IXCHAR = 1,NDMAXCHAR
                     INDX=NENTRY*(IREG-1)+IXCHAR
                     IF (INDX.GT.NALLOC1) THEN
                        WRITE(6,'(A)')'     '
                        WRITE(6,'(A)')' *** WARNING ***'
                        WRITE(6,'(A)')' INIT_MAC: INDEX > NALLOC1'
                     ENDIF
                     MUERHO = RMM(INDX)
CS                     MUERHO = RMM(IXCHAR,IREG) 
                     XCABS(IXD,IXCHAR,IX,IZ,IY)=
     &                             XCABS(IXD,IXCHAR,IX,IZ,IY)
     &                             *DEXP(-MIN(EXPMN,MUERHO*RLDB))
 32               CONTINUE
cs-start
cs                  write(6,'(2i5,4d14.5)')ix,iz,muerho,rldb,
cs     &                               DEXP(-MIN(EXPMN,MUERHO*RLDB)),
cs     &                               XCABS(IXD,1,IX,IZ,IY)


CS                  DO 33 IXC=1,NDMAXCONT+NDMAXCHAR-NDMAPTOB
                  DO 33 IXC=1,NDMAXCONT
                     IXCLOC=IXCONTOFXCHAR(IXC)
                     IF (IXCLOC.GT.NDMAXCHAR) THEN
                        INDX=NENTRY*(IREG-1)+IXCLOC
                        IF (INDX.GT.NALLOC1) THEN
                           WRITE(6,'(A)')'     '
                           WRITE(6,'(A)')' *** WARNING ***'
                           WRITE(6,'(A)')' INIT_MAC: INDEX > NALLOC1'
                        ENDIF
                        MUERHO = RMM(INDX)
CS                        MUERHO = RMM(IXCLOC,IREG) 
                        XCABS(IXD,IXCLOC,IX,IZ,IY)=
     &                             XCABS(IXD,IXCLOC,IX,IZ,IY)
     &                             *DEXP(-MIN(EXPMN,MUERHO*RLDB))
                     ENDIF
 33               CONTINUE

                  IF (LSIDE(IXD)) THEN 
*                    TAKE GEOMETRY EXPANSION INTO ACCOUNT
                     IF ((XBOX.LE.X_MIN).OR.
     &                   (XBOX.GE.X_MAX)) THEN
*                       POINT XBOX/ZBOX LIES EITHER LEFT OF LEFT BOUNDARY
*                       OF RIGHT OF RIGHT BOUNDARY
                        DIRHLZ=DIRZ/ABS(DIRZ)
                        DIRHLX=0.
 137                    XBOX = XNXT-(DIRX/ABS(DIRX))*DELTA
                        ZBOX = ZNXT+DIRHLZ*DELTA
                        CALL INTERSECT (
     D	                                NPTS,NFCS,
     I	                                PCOORD,ILINES,IENDIX,
     I                                  IREG,XBOX,ZBOX,DIRHLX,DIRHLZ,
     O                                  XNXT,ZNXT,IRNXT,RLDB,IER)          
                        RLDB=RLDB/DABS(DSIN(TAKOFR(IXD)))
                        DO 37 IXCHAR=1,NDMAXCHAR
                           IF (DSIN(TAKOFR(IXD)).LT.SMALL) THEN
                              XCABS(IXD,IXCHAR,IX,IZ,IY)=0
                              GOTO 37
                           ENDIF  
                           INDX=NENTRY*(IREG-1)+IXCHAR
                           IF (INDX.GT.NALLOC1) THEN
                              WRITE(6,'(A)')'     '
                              WRITE(6,'(A)')' *** WARNING ***'
                              WRITE(6,'(A)')
     &                             ' INIT_MAC: INDEX > NALLOC1'
                           ENDIF
                           MUERHO = RMM(INDX)
CS                           MUERHO = RMM(IXCHAR,IREG)
                           XCABS(IXD,IXCHAR,IX,IZ,IY)=
     &                           XCABS(IXD,IXCHAR,IX,IZ,IY)
     &                           *DEXP(-MIN(EXPMN,MUERHO*RLDB))
 37                     CONTINUE
cs-start
cs                  write(6,'(2i5,4d14.5)')ix,iz,rmm(indx),rldb,
cs     &                              DEXP(-MIN(EXPMN,MUERHO*RLDB)),
cs     &                              XCABS(IXD,1,IX,IZ,IY)

CS                        DO 38 IXC=1,NDMAXCONT+NDMAXCHAR-NDMAPTOB
                        DO 38 IXC=1,NDMAXCONT
                           IXCLOC=IXCONTOFXCHAR(IXC)
                           IF (IXCLOC.GT.NDMAXCHAR) THEN
                              IF (DSIN(TAKOFR(IXD)).LT.SMALL) THEN
                                 XCABS(IXD,IXCLOC,IX,IZ,IY)=0
                                 GOTO 38
                              ENDIF
                              INDX=NENTRY*(IREG-1)+IXCLOC
                              IF (INDX.GT.NALLOC1) THEN
                                 WRITE(6,'(A)')'     '
                                 WRITE(6,'(A)')' *** WARNING ***'
                                 WRITE(6,'(A)')
     &                                ' INIT_MAC: INDEX > NALLOC1'
                              ENDIF
                              MUERHO = RMM(INDX) 
CS                              MUERHO = RMM(IXCLOC,IREG) 
                              XCABS(IXD,IXCLOC,IX,IZ,IY)=
     &                           XCABS(IXD,IXCLOC,IX,IZ,IY)*
     &                           DEXP(-MIN(EXPMN,MUERHO*RLDB))
                           ENDIF
 38                     CONTINUE

                        IF (IRNXT.EQ.0) THEN
                           GOTO 12
                        ELSE
                           IREG = IRNXT
                           GOTO 137
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
               GOTO 30
 12          CONTINUE
  9        CONTINUE
 10      CONTINUE          
 20   CONTINUE   

*     MAKE SURE THAT A GRIDPOINT WHICH LIES IN AIR GETS THE SAME
*     ABSORPTION VALUE OF ITS NABOURPOINT IF THIS NABOUR LIES IN A REGION
      DO 199 IXD=0,NDMAXDET-1
         DO 200 I = 1,NDMAXCHAR-NDMAPTOB+NDMAXCONT
            DO 190 IX = 1,NDMAXGRID
              DO 185 IZ = 1,NDMAZGRID
                DO 180 IY = 1,NDMAYGRID
                  INDX=NDMAXGRID*NDMAZGRID*(IY-1)+NDMAXGRID*(IZ-1)+IX
                  IF (INDX.GT.NALLOC2) THEN
                     WRITE(6,'(A)')'     '
                     WRITE(6,'(A)')' *** WARNING ***'
                     WRITE(6,'(A)')' INIT_MAC: INDEX > NALLOC2'
                  ENDIF
                  HELPXC(INDX)=1
                  ABSNEW=1
                  IF (XCABS(IXD,I,IX,IZ,IY).EQ.1.) THEN 
                     DO 196 IXN = -1,1,1
                        IF (IX.EQ.1.AND.IXN.EQ.-1) GOTO 196
                        IF (IX.EQ.NDMAXGRID.AND.IXN.EQ.1) GOTO 196
                        DO 195 IZN = -1,1,1
                           IF (IXN.EQ.0.AND.IZN.EQ.0) GOTO 195
                           IF (IZ.EQ.1.AND.IZN.EQ.-1) GOTO 195
                           IF (IZ.EQ.NDMAZGRID.AND.IZN.EQ.1) GOTO 195
                           ABSIDE=XCABS(IXD,I,IX+IXN,IZ+IZN,IY) 
                           IF (ABSIDE.LT.1.0) THEN
                              IF (ABSIDE.LT.ABSNEW) ABSNEW=ABSIDE
                           ENDIF 
 195                    CONTINUE
 196                 CONTINUE
                     HELPXC(INDX) = ABSNEW
                  ELSE
                     HELPXC(INDX) = XCABS(IXD,I,IX,IZ,IY)
                  ENDIF
 180            CONTINUE
 185          CONTINUE
 190        CONTINUE 
         
            DO 2190 IX = 1,NDMAXGRID
               DO 2185 IZ = 1,NDMAZGRID
                  DO 2180 IY = 1,NDMAYGRID
                     INDX=NDMAXGRID*NDMAZGRID*(IY-1)+NDMAXGRID*(IZ-1)+IX
                     IF (INDX.GT.NALLOC2) THEN
                        WRITE(6,'(A)')'     '
                        WRITE(6,'(A)')' *** WARNING ***'
                        WRITE(6,'(A)')' INIT_MAC: INDEX > NALLOC2'
                     ENDIF
                     XCABS(IXD,I,IX,IZ,IY) = HELPXC(INDX)
2180              CONTINUE
2185           CONTINUE
2190        CONTINUE 

 200     CONTINUE
 199  CONTINUE 

c2200  CONTINUE

cs-test
cs      do 3010 l=1,nxchar+nxcont-nxptob
cs        write(6,'(a,i7)')' xcharline =',l
cs        do 3009 i=1,ngridx
cs          do 3007 j=1,ngridz
cs            do 3005 k=1,ngridy
cs              write(6,*)i,j,xcabs(1,l,i,j,k)
cs              write(6,*)l,xcabs(1,l,2,3,1)
cs3005        continue
cs3007      continue
cs3009    continue
3010  continue

      WRITE(6,'(A)')'        '
      DO 202 I=0,NDMAXDET-1
         IF (LSIDE(I)) THEN
            WRITE(6,'(2A,I2,A)')' GEOMETRY LATERALLY EXPANDED FOR',
     &              ' X-RAY ABSORPTION CALCULATION (DET#',I,')'
         ELSE
            WRITE(6,'(2A,I2,A)')' GEOMETRY NOT LATERALLY EXPANDED FOR',
     &              ' X-RAY ABSORPTION CALCULATION (DET#',I,')'
         ENDIF
 202  CONTINUE


      RETURN
      END 





