c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
* THIS FILE CONTAINS THE FOLLOWING ROUTINES:
* INIT_STOPP
* STBETH
* STLUOJ
* STGRYZ
* INIT_SIGMAION
* INIT_DELTAE
* GRYDSDW (FUNCTION)
**************************************************************************

      SUBROUTINE INIT_STOPP (
     O                       IER)

      INCLUDE 'TYPE.INC'      
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'DMA.INC'

      INTEGER IER

***   INIT_STOPP - INITIALIZES TABLE STOPPING() FOR USAGE WITH CONTINUOUS
*                  SLOWING DOWN FOR HYBRIDMODEL
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: A.PFEIFFER SEP-1994
*             C.SCHIEBL  JAN-1996
*
*     CALLS: STBETH,STGRYZ

**    START OF EXECUTABLE CODE
*     ************************


      ESTEP=(EMAX-EMIN)/FLOAT(MAXENER-1)

      DO 80 IREG=1,MAXREG
      ENER=EMIN

        DO 60 IENER=1,MAXENER
          DENSI=DENSITY(IREG)
          STPBTH=0
          STPGRY=0
C         FOR EACH ELEMENT (1-NELE(IREG))
          DO 40 IELE=1,NELE(IREG)
C           CALCULATE BETHE STOPPING POWER
            STPB=0
            IF (ELEM(IREG,IELE).EQ.0) GOTO 40
            IF (LBETHE) THEN
              CALL STBETH (ATNUM(ELEM(IREG,IELE)),
     &                     ATMASS(ELEM(IREG,IELE)),DENSI,ENER,STPB)
            ELSE
              CALL STLUOJ (ATNUM(ELEM(IREG,IELE)),
     &                     ATMASS(ELEM(IREG,IELE)),DENSI,ENER,STPB)
            ENDIF
            STPB=STPB*WEIGHTFRAC(IREG,IELE)
            STPBTH=STPBTH+STPB
40        CONTINUE

C         CALCULATE GRYZINSKI STOPPING POWER
C         FOR EACH SHELL (1-NSHELL)
          STPG=0

          IF(.NOT.LSING(IREG))THEN

	    DO 20 ISHELL=1,NSHELLOFREG(IREG)

              INDX = (ISHELL-1)*NDMAREG+IREG
              CALL CHKBND(15,INDX,'INIT_STOPP','SHELLOFREG')
CS              ISHELN = SHELLOFREG(IREG,ISHELL)
      	      ISHELN = SHELLOFREG(INDX)
              CALL CHKBND(5,ISHELN,'INIT_STOPP','ELEM/ENEROFSHELL')
              I_ELE=ELEMOFSHELL(ISHELN)
              I_SHELL=SHELLOFSHELL(ISHELN)
      	      DO 250 I_ELEM = 1,NELE(IREG)
      	        IF (ELEM(IREG,I_ELEM) .EQ. I_ELE) 
     &                             ATD = ATDENS(IREG,I_ELEM)    
250 	      CONTINUE

              IF (I_ELE.EQ.0) GOTO 20

              IF ((ENEROFSHELL(ISHELN).LT.ENER).AND.
     &            (ENEROFSHELL(ISHELN).GT.0))THEN
                STP=0
                CALL STGRYZ (E_SHELL(I_SHELL),ATD,
     1                      ENER,ENEROFSHELL(ISHELN),
     2                      STP)
                STPG=STPG+STP
              ENDIF
20          CONTINUE
            
          ELSE
            STPG=0.
          ENDIF

CHW          STPG=STPG*WEIGHTFRAC(IREG,IELE)
          STPGRY=STPGRY+STPG

cap40       CONTINUE
          STOPPING(IENER,IREG)=STPBTH-STPGRY

          ENER=ENER+ESTEP 

60      CONTINUE         

80    CONTINUE
     
      RETURN
      END

*******************************************************************************

      SUBROUTINE STBETH(
     I                  IATNUM,WEIGHT,DENS,ENER,STP)

***   STBETH - BETHE STOPPING POWER
*              FOR SMALL ENERGIES (E<6.338*J) THE EQUATION FROM
*              RAO SAHIB AND WITTRY (1974) IS TAKEN
*              FOR HIGH ENERGIES (E>60 KEV) A RELATIVISTIC BETHE
*              FORMULA IS TAKEN (EQUATION 3.116 IN REIMER BOOK)
*              BETWEEN 40 AND 60 KEV AN ENERGY DEPENDENT COMBINATION
*              OF THE BETHE AND THE RELATIVISTIC BETHE FORMALAE IS TAKEN
*
**    ARGUMENTS
*
*     IATNUM - ATOMIC NUMBER
*     WEIGHT - ATOMIC WEIGHT
*     ENER   - ELECTRON ENERGY (eV)
*     DENS   - DENSITY (g/Ang**3)
*     STP    - BETHE STOPPING POWER (eV/Ang)
*
**    AUTHOR:  C. SCHIEBL  16-JUN-1994
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

      DOUBLE PRECISION JPOT

**    START OF EXECUTABLE CODE
*     ************************

      STP=0
      FJP=JPOT(IATNUM)
      IF (ENER.GT.60000) THEN
C        RELATIVISTIC BETHE
         BETA2=1-1/(1+ENER/511000)**2
         EEPS=ENER/511000
         STP=DLOG(EEPS*EEPS*(EEPS+2)/2/(FJP/511000)**2)+
     1       (1+EEPS*EEPS/8-(2*EEPS+1)*0.693147181)/(EEPS+1)**2
         STP=STP*1.535D21*DENS*DBLE(IATNUM)/WEIGHT/BETA2
      ELSE IF (ENER.GT.40000) THEN
*        MIXTURE: BETHE AND RELATIVISTIC BETHE
         FACT=DBLE(IATNUM)*DENS*N_AVO/WEIGHT*2.*PI*E2*E2
         STP_A=FACT*DLOG(1.166*ENER/FJP)/ENER
         BETA2=1-1/(1+ENER/511000)**2
         EEPS=ENER/511000
         STP_B=DLOG(EEPS*EEPS*(EEPS+2)/2/(FJP/511000)**2)+
     1       (1+EEPS*EEPS/8-(2*EEPS+1)*0.693147181)/(EEPS+1)**2
         STP_B=STP_B*1.535D21*DENS*DBLE(IATNUM)/WEIGHT/BETA2
         FCTA=3-ENER/20000
         STP=FCTA*STP_A+(1-FCTA)*STP_B
      ELSE IF (ENER.GT.6.338*FJP) THEN
C        BETHE 
CAP      FACT=785*DBLE(IATNUM)*DENS/WEIGHT
         FACT=DBLE(IATNUM)*DENS*N_AVO/WEIGHT*2.*PI*E2*E2
         STP=FACT*DLOG(1.166*ENER/FJP)/ENER
      ELSE
C        RAO SAHIB AND WITTRY
CAP      FACT=785*DBLE(IATNUM)*DENS/WEIGHT
         FACT=DBLE(IATNUM)*DENS*N_AVO/WEIGHT*2.*PI*E2*E2
         STP=FACT/SQRT(ENER*FJP)/1.26
      ENDIF

      RETURN
      END

*******************************************************************************

      SUBROUTINE STLUOJ(
     I                  IATNUM,WEIGHT,DENS,ENER,STP)

***   STLUOJ - STOPPING POWER FROM LUO AND JOY 1989
*              FOR ENERGIES <40 keV: LUO AND JOY 1989
*              FOR ENERGIES >40 AND <60 keV: ENERGY DEPENDENT MIXTURE
*                           OF LUO AND JOY AND RELATIVISTIC BETHE
*              FOR ENERGIES >60 keV: RELATIVISTIC BETHE (EQUATION 3.116
*                           IN REIMER BOOK)
*
**    ARGUMENTS
*
*     IATNUM - ATOMIC NUMBER
*     WEIGHT - ATOMIC WEIGHT
*     ENER   - ELECTRON ENERGY (eV)
*     DENS   - DENSITY (g/**3)
*     STP    - STOPPING POWER FROM LUO AND JOY (eV/)
*
**    AUTHOR:  C. SCHIEBL  01-JAN-1996
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

      DOUBLE PRECISION JPOT

**    START OF EXECUTABLE CODE
*     ************************

      STP=0
      FJP=JPOT(IATNUM)
      IF (ENER.LT.40000) THEN
*        JOY AND LUO 1989
         FACT=785D24*DBLE(IATNUM)*DENS/WEIGHT
         STP=FACT*DLOG(1.166*(ENER+0.85*FJP)/FJP)/ENER
      ELSE IF (ENER.LT.60000) THEN
*        JOY AND LUO 1989
         FACT_A=785D24*DBLE(IATNUM)*DENS/WEIGHT
         STP_A=FACT*DLOG(1.166*(ENER+0.85*FJP)/FJP)/ENER
*        RELATIVISTIC BETHE, EQUATION 3.116 
         BETA2=1-1/(1+ENER/511000)**2
         EEPS=ENER/511000
         STP_B=DLOG(EEPS*EEPS*(EEPS+2)/2/(FJP/511000)**2)+
     1         (1+EEPS*EEPS/8-(2*EEPS+1)*0.693147181)/(EEPS+1)**2
         STP_B=STP_B*1.535D21*DENS*DBLE(IATNUM)/WEIGHT/BETA2
         FCTA=3-ENER/20000
         STP=FCTA*STP_A+(1-FCTA)*STP_B
      ELSE
*        RELATIVISTIC BETHE, EQUATION 3.116 
         BETA2=1-1/(1+ENER/511000)**2
         EEPS=ENER/511000
         STP=DLOG(EEPS*EEPS*(EEPS+2)/2/(FJP/511000)**2)+
     1       (1+EEPS*EEPS/8-(2*EEPS+1)*0.693147181)/(EEPS+1)**2
         STP=STP*1.535D21*DENS*DBLE(IATNUM)/WEIGHT/BETA2
      ENDIF

      RETURN
      END

*******************************************************************************

      SUBROUTINE STGRYZ(
     I                  ISHEEL,ATOMDENS,ENER,EDGE,STP)

***   STGRYZ - GRYZINSKI (INNER SHELL IONIZATION) STOPPING POWER
*
**    ARGUMENTS
*
*     ISHEEL - NUMBER OF ELECTRONS IN SHELL
CHW*     WEIGHT - WEIGHT FRACTION
CHW*     ATWEIGHT - ATOMIC MASS (g/mol)
CHW*     DENS   - DENSITY (g/ų)
*     ATOMDENS - ATOMIC DENSITY (At/ų)
*     ENER   - ELECTRON ENERGY (eV)
*     EDGE   - EDGE ENERGY
*     STP    - GRYZINSKI STOPPING POWER (eV/)
*
**    AUTHOR:  C. SCHIEBL  16-JUN-1994
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

**    START OF EXECUTABLE CODE
*     ************************

CHW      STP=0

      FACT=651.4*ISHEEL*ATOMDENS
      STP=FACT*(DLOG(ENER/EDGE)+4/3*DLOG(2.7+SQRT(ENER/EDGE-1)))*
     &    ((ENER-EDGE)/(ENER+EDGE))**(1.5)/ENER
CHW      FACT=392*ISHEEL*DENS*N_AVO/WEIGHT
CHW      STP=FACT*(DLOG(ENER/EDGE)+4/3*DLOG(2.7+SQRT(ENER/EDGE-1)))*
CHW     &    ((ENER-EDGE)/(ENER+EDGE))**(1.5)/ENER

      RETURN
      END

************************************************************************

      SUBROUTINE INIT_SIGMAION(
     I                         I_MODEL)  

***    SUBROUTINE INIT_ION_TOT INITIALIZES AND CALCULATES 
*     THE TABLES OF THE TOTAL IONAZIATION CROSS SECTION
*
**    INPUT:
*     I_MODEL...MODEL NUMBER (1=>GRYZINSKY(1965))
*
**    OUTPUT (IN COMMON BLOCK /SESAME/):
*     SIGMA_ION...CUMULATED IONAZIATION CROSS SECTION,
*                DEPENDS ON REGION AND ENERGY AND SHELL
*     
**    CALLS:  SIGMA_ION_U...CALCULATES G(U)
                        
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'DMA.INC'

**    START OF EXECUTABLE CODE:
*     *************************

* MODEL=1:
      IF (I_MODEL.EQ.1) THEN 
      
* LENGTH ENERGY INTERVAL
*      EMAX=19.D3
*      E0=EMAX 
*      EMIN=1.5D3
       
      ESTEP_IO = (EMAX-EMIN)/(FLOAT(MAXENER-1))

* LOOP OVER REGIONS
*
      DO 100 I_REG = 1,NREG                                                

* LOOP OVER SIMULATED SHELLS IN IREG

         DO 200 ISHELL = 1,NSHELLOFREG(I_REG)

*	 DETERMINE SHELL AND ELEMENT AND ATOMIC FRACTION
         INDX = (ISHELL-1)*NDMAREG+I_REG
         CALL CHKBND(15,INDX,'INIT_SIGMA','SHELLOFREG')
CS         ISHELN = SHELLOFREG(I_REG,ISHELL)
         ISHELN = SHELLOFREG(INDX)
         CALL CHKBND(5,ISHELN,'INIT_SIGMA','ELEM/ENEROFSHELL')
      	 I_ELEM = ELEMOFSHELL(ISHELN)
      	 I_SHELL = SHELLOFSHELL(ISHELN)
      	 DO 250 I_ELE = 1,NELE(I_REG)
      	    IF (ELEM(I_REG,I_ELE) .EQ. I_ELEM) C = ATFRAC(I_REG,I_ELE)    
250 	 CONTINUE
         E_CRIT = ENEROFSHELL(ISHELN)

* JUST FOR DEBUGGING
         IF (E_CRIT .GT. 0 .AND. EPRIM .GE. E_CRIT) THEN
      		U = EPRIM/E_CRIT
                CALL SIGMA_ION_U (
     I                         U,
     O                         G_U) 
         ELSE 
                G_U = 0.
      	 ENDIF
      
      	 IF (LTEST) WRITE(*,*) 'ION : ',I_REG,I_ELEM,I_SHELL,C,
     .                                  E_CRIT,G_U,tot_atdens(I_REG)
* END OF JUST FOR DEBUGGING

* LOOP OVER ENERGY
         E_ENER = EMIN
      	 DO 300 I_ENER  = 1,MAXENER        


             IF (E_CRIT.EQ.0.D0 .OR. E_ENER .LE. E_CRIT) THEN
      		SIGMA = 0.D0
      	     ELSE
             
            	U = E_ENER/E_CRIT
                CALL SIGMA_ION_U (
     I                         U,
     O                         G_U) 
      
                SIGMA = E_SHELL(I_SHELL)/E_CRIT**2.D0 * G_U * C
 	     ENDIF 	
      	      	
             IF (ISHELL .EQ. 1) THEN               
      	        SIGMA_ION(I_ENER,I_REG,ISHELL) = SIGMA
      	     ELSE	                   
      	        SIGMA_ION(I_ENER,I_REG,ISHELL) = SIGMA +
     +      	       		SIGMA_ION(I_ENER,I_REG,ISHELL-1)
             ENDIF			
             E_ENER = E_ENER+ESTEP_IO

 300        CONTINUE 

 200     CONTINUE

 100  CONTINUE

      ELSE 

             WRITE(*,'(2A)')' MODEL OF IONIZATION CROSS', 
     &                      ' NOT IMPLEMENTED' 
             WRITE(*,'(A)')' STOP IN SUBROUTINE INIT_ION_TOT'    
             STOP

      ENDIF 
      
      RETURN

      END  
************************************************************************
       
      SUBROUTINE INIT_DELTAE(
     O                       IER)

C      IMPLICIT NONE
      INCLUDE 'TYPE.INC'      
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'DMA.INC'

      INTEGER IER,IU,IW
      DOUBLE PRECISION R(MAXW,MAXU)
 
***   INIT_DELTAE - INITIALIZES TABLE ENER_DELTA() FOR ENERGY LOSS AFTER
*                   IONISATION
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: A.PFEIFFER SEP-1994
*
*     CALLS: GRYDSDW

**    START OF EXECUTABLE CODE
*     ************************

*  MINIMAL EDGE ENERGY
      EEDGE_HELP = 1000000.
      EEDGE_MIN = EEDGE_HELP
      DO 11 IREG=1,MAXREG
         DO 12 ISHELL = 1,NSHELLOFREG(IREG)

*	       GET GLOBAL SHELL NUMBER
               INDX = (ISHELL-1)*NDMAREG+IREG
               CALL CHKBND(15,INDX,'INIT_DELTA','SHELLOFREG')
CS               ISHELN = SHELLOFREG(IREG,ISHELL)
               ISHELN = SHELLOFREG(INDX)
               CALL CHKBND(5,ISHELN,'INIT_DELTA','ELEM/ENEROFSHELL')

               I_ELE=ELEMOFSHELL(ISHELN)
               I_SHELL=SHELLOFSHELL(ISHELN)
               IF (I_ELE.EQ.0) WRITE(*,*)'ERROR IN ELEOFSHELL ?'
               EEDGE_NEW = ENEROFSHELL(ISHELN)
                
               IF (EEDGE_NEW.LT.EEDGE_MIN) THEN
                   EEDGE_MIN = EEDGE_NEW
                 ELSE 
                   CONTINUE
               ENDIF
 12         CONTINUE
 11   CONTINUE
      IF(EEDGE_MIN.EQ.EEDGE_HELP .OR.
     &   EEDGE_MIN.GT.EMAX )          GOTO 1000

* TABLE OF ENERGY TRANSFER ACCORDING TO REIMER,STELTER (1986)

      UMIN = 1.1
      UMAX=EMAX/EEDGE_MIN
      USTEP = (UMAX-UMIN)/FLOAT(MAXU-1)
      U = UMIN
      DO 98 I =1,MAXU
         DO 99 J = 1,MAXW
               ENER_DELTA(J,I)=0.
 99      CONTINUE
 98   CONTINUE     
       
      DO 110 IU=1,MAXU
         W = 1.1/U    
         WSTEP = (1-1.1/U)/FLOAT(MAXW-1)
         R(1,IU)=0.            
         IF (W.GT.1) THEN 
               GU2 = 0.
            ELSE
               GU2=GRYDSDW(U,W) 
         ENDIF

         DO 111 IW = 2,MAXW
            W = W + WSTEP
            GU1 = GU2
            IF (W.GT.1) THEN 
                  GU2 = 0.
               ELSE
                  GU2=GRYDSDW(U,W) 
            ENDIF
            R(IW,IU)=R(IW-1,IU)+(GU2+GU1)*WSTEP/2. 
 111     CONTINUE

* NORMALIZATION 
         R(MAXW,1)=1.

         DO 112 IW = 2,100
            R(IW,IU)=R(IW,IU)/R(MAXW,IU)
 112     CONTINUE
         
         DO 113 IHELP = 1,MAXW
            RX=FLOAT(IHELP)/FLOAT(MAXW)

            DO 114 IW =1,MAXW
               IF (R(IW,IU).LT.RX) GOTO 114
               REL = (RX-R(IW-1,IU))/(R(IW,IU)-R(IW-1,IU))
               ENER_DELTA(IHELP,IU)=1.1/U+(FLOAT(IW)-2.)*WSTEP
     &                               +REL*WSTEP

cs               write(35,*)IHELP,IU,ENER_DELTA(IHELP,IU)

               GOTO 113
 114        CONTINUE
 113     CONTINUE
 
         U=U+USTEP
 110  CONTINUE

      IER = 0
cs      GOTO 1000   

cs 900  IER = -1 
 
 1000 END

*******************************************************************************
      DOUBLE PRECISION FUNCTION GRYDSDW(X,Y)

      INCLUDE 'TYPE.INC'
       
      GRYDSDW = 1./(Y**3.*X) *(X/(1.+X))**1.5 *(1.-Y)**(1./(1+X*Y))*
     &          (X*Y-Y+4./3.*DLOG(2.7+(X-X*Y)**.5))

      END  
*******************************************************************************

