c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
      SUBROUTINE INIT_ION_TOT (
     I                         I_MODEL)  

***    SUBROUTINE INIT_ION_TOT INITIALIZES AND CALCULATES 
*     THE TABLES OF THE TOTAL IONAZIATION CROSS SECTION
*
**    INPUT:
*     I_MODEL...MODEL NUMBER (1=>GRYZINSKY(1965))
*
**    OUTPUT (IN COMMON BLOCK /SESAME/):
*     SIGMA_IO...AREA OF TOTAL IONAZIATION CROSS SECTION,
*                DEPENDS ON REGION AND ENERGY
*     SIGMA_IO_CW...AREA OF IONAZIATION CROSS SECTION SUMMARIZED 
*                OVER ELEMENTS ATOMICFRACTION,
*                DEPENDS ON REGION, ELEMENT AND ENERGY
*     SIGMA_IO_NI...AREA OF IONAZIATION CROSS SECTION SUMMARIZED
*                OVER SHELLS,
*                DEPENDS ON ELEMENT, SHELLS AND ENERGY
*     LAMBDA_IO_TOT...MEAN FREE PATH FOR INELATIC SCATTERING
*     
**    CALLS:  SIGMA_ION_U...CALCULATES G(U)
                        
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'

**    START OF EXECUTABLE CODE:
*     *************************

* MODEL=1:
      IF (I_MODEL.EQ.1) THEN 
      
* LENGTH ENERGY INTERVAL
*      EMAX=19.D3
*      E0=EMAX 
*      EMIN=1.5D3
       
      ESTEP_IO = (EMAX-EMIN)/(FLOAT(MAXENER-1))

* LOOP OVER REGIONS
*
      DO 100 I_REG = 1,NREG                                                
         E_ENER = EMIN

* LOOP OVER ENRGY
        
           
         DO 200 I_ENER = 1,MAXENER
             SIGMA_IO_TOTHEL=0.D0

* LOOP OVER ELEMENTS

            DO 300 I_ELE = 1,NELEM

             SIGMA_IO_HELP=0.D0

* LOOP OVER SHELLS (MAXSHELL?, BETTER NSHELL(ELEM(I_REG,I_ELE)))

	       DO 400 I_SHELL = 1,MAXSHELL
                      E_CRIT = EEDGE(I_ELE,I_SHELL)
                      IF (E_CRIT.EQ.0.D0) GOTO 400
                      U = E_ENER/E_CRIT
                                                                  
                      IF (U.LE.1.D0) GOTO 400 

                            CALL SIGMA_ION_U (
     I                                        U,
     O                                        G_U) 
                            
                      
                  SIGMA_IO_HELP=SIGMA_IO_HELP+E_SHELL(I_SHELL)/
     &                                 E_CRIT**2.D0 * G_U  	 

                  SIGMA_IO_NI(I_ELE,I_SHELL,I_ENER)=
     &                                        SIGMA_IO_HELP    

 400           CONTINUE

               C_WEIGHT=WEIGHTFRAC(I_REG,I_ELE)
               AT_WEIGHT=ATMASS(I_ELE)
               SIGMA_IO_TOTHEL=
     &                  SIGMA_IO_TOTHEL + SIGMA_IO_HELP*C_WEIGHT/
     &                                    AT_WEIGHT   
               SIGMA_IO_CW(I_REG,I_ELE,I_ENER)=
     &                                 SIGMA_IO_TOTHEL 


 300        CONTINUE 
            SIGMA_IO(I_REG,I_ENER)=SIGMA_IO_TOTHEL
            DENSI=DENSITY(I_REG) 
            IF (SIGMA_IO_TOTHEL.LE.0D0) THEN
                LAMBDA_IO(I_REG,I_ENER)=0.D0
               ELSE 
            LAMBDA_IO(I_REG,I_ENER)=1.D0/DENSI/N_AVO/
     &                                  SIGMA_IO_TOTHEL
*           WRITE(67,*)E_ENER,SIGMA_IO_TOTHEL
            ENDIF 
            E_ENER = E_ENER+ESTEP_IO
 200     CONTINUE

 100  CONTINUE

          ELSE 

             WRITE(*,'(2A)')' MODEL OF IONIZATION CROSS', 
     &                      ' NOT IMPLEMENTED' 
             WRITE(*,'(A)')' STOP IN SUBROUTINE INIT_ION_TOT'    
             STOP

      ENDIF 
      
      RETURN

      END  
***************************************************************

      SUBROUTINE SIGMA_ION_U (
     I                        U,
     O                        G_U)

* 
**    CALCULATES OVERVOLTAGE DEPENDENT FACTOR OF IONISATION 
*     CROSS SECTION
* 
**    INPUT:
*     U...OVERVOLTAGE RATIO E0/E_CRIT
*
**    OUTPUT:
*     G_U...FATOR G(U) IN CROSS SECTION OF GRYZINSKI(1965)

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC' 

**    START OF EXECUTABLE CODE:
*     *************************   

*     E4=E^4 [A^2EV^2]
      E4=207.36D0     
      G_U=E4*PI*1.D0/U*( (U-1.D0)/(U+1.D0) )**1.5D0*( 1.D0+2.D0/3.D0*
     &    ( 1.-0.5D0/U )*DLOG( 2.7D0+( U-1.D0 )**.5D0 ) )

      RETURN
      END 
       
