c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
      SUBROUTINE INTERSECT (
     D                   NPTS, NFCS,
     I                   PCOORD, ILINES, IENDIX,
     I                   IR, XCOORD, ZCOORD, DIRX, DIRZ,
     O                   XINTER, ZINTER, IRNEW, RLBD, IER)

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'

***   INTERS - DETERMINES THE INTERSECTION POINT OF A LINE GIVEN BY A
*              POINT AND A MOVING DIRECTION WITH THE BORDER OF REGION
*              IR
*
**    ARGUMENTS
*
*        NPTS   - NUMBER OF POINTS
*        NFCS   - NUMBER OF FACES
*        PCOORD - COORDINATES OF THE POINTS
*        ILINES - INDICES OF THE FACES
*        IENDIX - ENDINDICES OF THE LINES
*        IR     - REGION, OF POINT (XCOORD/ZCOORD)
*        XCOORD - X-COORDINATE OF THE CURRENT LOCATION
*        ZCOORD - Z-COORDINATE OF THE CURRENT LOCATION
*        DIRX   - X-COMPONENT OF MOTION
*        DIRZ   - Z-COMPONENT OF MOTION
*        XINTER - X-COORDINATE OF THE INTERSECTION POINT
*        ZINTER - Z-COORDINATE OF THE INTERSECTION POINT
*        IRNEW  - NEW REGION INDEX
*        RLBD   - INTERSECTION DISTANCE IN (DIRX.DIRZ)
*        IER    - ERROR INDICATOR: 0 ... EVERYTHING OK, 1 ... ERROR
*
**    AUTHOR:  H.STIPPEL   4-MAI-1994
*
      DIMENSION PCOORD(2*NPTS), ILINES(3*NPTS-6), IENDIX(NFCS)
      LOGICAL   LFIRST

**    START OF EXECUTABLE CODE
*     ************************

      IER = 0
      LFIRST = .TRUE.

*     LOOP OVER ALL LINES OF THIS REGION
      IBEGIN = 1
      IF (IR.GT.1) IBEGIN = IBEGIN + IENDIX(IR-1)
      IFROM = ILINES(IENDIX(IR))
      DO 10 IPT = IBEGIN, IENDIX(IR)
         ITO = ILINES(IPT)
         VX = PCOORD(2*ITO-1) - PCOORD(2*IFROM-1)
         VZ = PCOORD(2*ITO)   - PCOORD(2*IFROM)
         DET = VZ * DIRX - VX * DIRZ
         IF (DET.NE.ZERO) THEN
*     LINES ARE NOT PARALLEL
            DX = XCOORD - PCOORD(2*IFROM-1)
            DZ = ZCOORD - PCOORD(2*IFROM)
            DLBD = (VX * DZ - VZ * DX) / DET
            IF (ZERO.LE.DLBD) THEN
*     INTERSECTION IS POSSIBLE
               DMUE = (DIRX * DZ - DIRZ * DX) / DET
               IF (ZERO.LE.DMUE .AND. DMUE.LE.ONE) THEN
*     INTERSECTION FOUND
                  IF (LFIRST) THEN
                     RLBD = DLBD
                     IIPF = IFROM
                     IIPT = ITO
                     LFIRST = .FALSE.
                  ELSE
                     IF (DLBD.LT.RLBD) THEN
                        RLBD = DLBD
                        IIPF = IFROM
                        IIPT = ITO
                     ENDIF
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
         IFROM = ITO
 10   CONTINUE

      IF (LFIRST) THEN
*     NO INTERSECTION FOUND
         IER = 1
         RETURN
      ENDIF

*     THE INTERSECTION POINT
      XINTER = XCOORD + RLBD * DIRX
      ZINTER = ZCOORD + RLBD * DIRZ

*     GET THE NEW REGION INDEX
      IRNEW = 0
      IBEGIN = 1
      DO 30 IFC = 1, NFCS
         IF (IFC.NE.IR) THEN
            IFROM = ILINES(IENDIX(IFC))
            DO 20 IPT = IBEGIN, IENDIX(IFC)
               ITO = ILINES(IPT)
               IF ((IFROM.EQ.IIPF .AND. ITO.EQ.IIPT) .OR.
     .             (IFROM.EQ.IIPT .AND. ITO.EQ.IIPF)) THEN
                  IRNEW = IFC
                  GOTO 40
               ENDIF
               IFROM = ITO
 20         CONTINUE
         ENDIF
         IBEGIN = IENDIX(IFC) + 1
 30   CONTINUE
 40   CONTINUE

      RETURN
      END
