c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
      SUBROUTINE NDIRA(
     V                 DIRX,DIRY,DIRZ,
     I                 COSPSI,SINPSI,COSFI,SINFI)
 
***   NDIRA - COMPUTES THE NEW DIRECTION AFTER A COLLISION IN AN 
*             AMORPHOUS TARGET
*
**    ARGUMENT LIST
*
*     DIRX   - X-COMPONENT OF THE DIRECTION OF MOTION
*     DIRY   - Y-COMPONENT OF THE DIRECTION OF MOTION
*     DIRZ   - Z-COMPONENT OF THE DIRECTION OF MOTION
*              (DIRX**2 + DIRY**2 + DIRZ**2 = 1)
*     COSPSI - COSINE OF THE DEFLECTION ANGLE
*     SINPSI - SINE OF THE DEFLECTION ANGLE
*     COSFI  - COSINE OF THE AZIMUTHAL DEFLECTION ANGLE
*     SINFI  - SINE OF THE AZIMUTHAL DEFLECTION ANGLE
*
**    REMARKS
*
*     1) TO MAKE SURE THAT DIRX**2 + DIRY**2 + DIRZ**2 = 1 IN SPITE OF
*        ROUNDING ERRORS, A CORRECTION IS APPLIED BEFORE THE ROUTINE
*        IS LEFT
*
**    AUTHOR:  G.HOBLER  9-AUG-1990  (ADAPTED FROM PROMIS)
*      
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'

      PARAMETER (AMIN=1.E-10,ONE5=THREE/TWO)
 
 
**    START OF EXECUTABLE CODE
*     ************************
 
 
C TRANSFORM DIRECTION VECTOR TO POLAR ANGLES
 
      COSALF=DIRZ
      SINALF=SQRT(DIRX**2 + DIRY**2)
      IF (SINALF.GT.AMIN) THEN
         COSPHI=DIRX/SINALF
         SINPHI=DIRY/SINALF
      ELSE
         COSPHI=ONE
         SINPHI=ZERO
      ENDIF
 
C CALCULATE NEW DIRECTION
 
      H0=SINPSI*COSFI
      H1=SINALF*COSPSI + H0*COSALF
      H2=SINPSI*SINFI
      DIRX=H1*COSPHI - H2*SINPHI
      DIRY=H1*SINPHI + H2*COSPHI
      DIRZ=COSALF*COSPSI - H0*SINALF
 
C TO GUARANTEE  DIRX**2 + DIRY**2 + DIRZ**2 = 1  :
 
      DELTA=ONE5 - HALF*(DIRX**2+DIRY**2+DIRZ**2)
      DIRX=DELTA*DIRX
      DIRY=DELTA*DIRY
      DIRZ=DELTA*DIRZ
 
      END
