c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
***  PLASMON.F
*
*    ROUTINES FOR PLASMON SCATTERING BASED ON THE DIELECTRIC THEORY.
*   
*    DESCRIPTION OF ROUTINES :
*
*
***   P_INIT       - INITIALIZES TABLES PLASMON_TAB(),PLASMON_INT()
*		     AND N_PLASMON_INT() FOR USAGE WITH PLASMONLOSS
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 21-JUN-1994
*
*     CALLS: DSIMPSON,DINVERSE
*
*
*
***   P_LAMBDA   - RETURNS VALUE FOR INELASTIC LAMBDA OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     LAMBDA    - PLASMON MEAN FREE PATH
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
***   P_DELTAE     - MONTE CARLO GENERATOR FOR INELASTIC SCATTERING USING
*		     LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     DELTAE	- SELECTED ENERGY LOSS
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 27-JUL-1994
*
****************************************************************************

      SUBROUTINE P_INIT(
     O			  IER)

      IMPLICIT NONE
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'

      INTEGER IER

***   P_INIT       - INITIALIZES TABLES PLASMON_TAB(),PLASMON_INT()
*		     AND N_PLASMON_INT() FOR USAGE WITH PLASMONLOSS
*
**    ARGUMENTS 
*
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 21-JUN-1994
*
*     CALLS: DSIMPSON,DINVERSE


**    START OF EXECUTABLE CODE
*     ************************
      DOUBLE PRECISION DINVERSE
      DOUBLE PRECISION DSIMPSON
      EXTERNAL DSIMPSON,DINVERSE,STATXT
      CHARACTER*5 CDUMM      

      INTEGER MAX_IDELTAE
      PARAMETER(MAX_IDELTAE = 1000)
      DOUBLE PRECISION MAX_DELTAE
      PARAMETER(MAX_DELTAE = 1000.)

      INTEGER IREG,IELE,IENER,IDELTAE,IRAN,IATNUM,I,J,NDATA
      DOUBLE PRECISION TEw(10000),TEPSIM(10000)
      DOUBLE PRECISION DIMFP(0:MAX_IDELTAE),CDIMFP(0:MAX_IDELTAE)
      DOUBLE PRECISION Q,Ew,DELTAE,X,Y,Ewmax,E,A,B,Ewp1,DQ
      DOUBLE PRECISION EPSIM,EPSRE,EPSIMo      

C     INITIALIZE ENERGY INTERVALS TO CONSTANT VALUES
      DO 20,IREG=1,MAXREG
      	 N_PLASMON_INT(IREG) = 12
      	 PLASMON_INT(IREG,1) = 10.
      	 PLASMON_INT(IREG,2) = 20.
      	 PLASMON_INT(IREG,3) = 50.
      	 PLASMON_INT(IREG,4) = 100.
      	 PLASMON_INT(IREG,5) = 200.
      	 PLASMON_INT(IREG,6) = 500.
      	 PLASMON_INT(IREG,7) = 1000.
      	 PLASMON_INT(IREG,8) = 2000.
      	 PLASMON_INT(IREG,9) = 5000.
      	 PLASMON_INT(IREG,10) = 10000.
      	 PLASMON_INT(IREG,11) = 20000.
      	 PLASMON_INT(IREG,12) = 50000.
      	 PLASMON_INT(IREG,13) = 100000.
      	 PLASMON_INT(IREG,14) = 200000.
      	 PLASMON_INT(IREG,15) = 500000.
20    CONTINUE


C     FOR ALL REGIONS

      DO 500,IREG=1,NREG
         IF (.NOT. LPLAS(IREG)) GOTO 500
         WRITE(CDUMM,'(I3)') IREG
         CALL STATXT(' INITIALIZING PLASMON LOSS TABLE FOR REGION '//
     .		      CDUMM)

C        READ OPTICAL DATA

         OPEN(12,FILE=FMAT(IREG),STATUS='OLD')
         I = 1
10       CONTINUE
           READ(12,FMT='(4F12.5)',END=50) TEw(I),EPSRE,X,TEPSIM(I)
           I=I+1
           GOTO 10
50       CONTINUE
         NDATA = I
         CLOSE(12)


C        FOR ALL ENERGIES
         DO 400,IENER=1,N_PLASMON_INT(IREG)

            E = PLASMON_INT(IREG,IENER)
                
C	    COMPUTE ARRAY WITH DIIMFP

            DO 300,IDELTAE = 0,MAX_IDELTAE
      	       
      	       DELTAE = MAX_DELTAE*IDELTAE/MAX_IDELTAE
 	
C	       CALCULATE DIMFP FOR GIVEN DELTAE AND E
      	
               IF (DELTAE .GT. E .OR. DELTAE .EQ. 0) THEN
          	DIMFP(IDELTAE) = 0.
          	GOTO 300
       	       ELSE
      		 J = 0
         	 Ewmax = 2*(DELTAE-E)+2*SQRT(E*(E-DELTAE))
      	       ENDIF
       	       Q = 0
100   	       CONTINUE
          	 J=J+1
      		 IF (J .GE. NDATA .OR. TEw(J) .GE. Ewmax) THEN
      			GOTO 250
      	         ENDIF

      		 A = (TEPSIM(J+1)-TEPSIM(J))/(TEw(J+1)-TEw(J))
      	         B = TEPSIM(J) - A*TEw(J)
      		 Ewp1=TEw(J+1)
      		 IF (Ewp1 .GT. Ewmax) Ewp1 = Ewmax

      		 DQ =  1./2.*A*TEw(J)**2. - (A*DELTAE+B)*
     .		      (-TEw(J) - DELTAE*LOG(DELTAE-TEw(J)))
     .		      -1./2.*A*Ewp1**2.   + (A*DELTAE+B)*
     .                (-Ewp1   - DELTAE*LOG(DELTAE-Ewp1  ))	

      		 Q = Q + DQ
             GOTO 100
250          DIMFP(IDELTAE) = Q/(2*PI*a0*E*DELTAE)
300    	     CONTINUE

C	     INTEGRATE DIIMFP      	
      	     X = DSIMPSON(0,0,-1)
      	     DO 350,IDELTAE = 1,MAX_IDELTAE
               CDIMFP(IDELTAE) = DSIMPSON(
     .           IDELTAE*MAX_DELTAE/MAX_IDELTAE,DIMFP(IDELTAE),0)
350          CONTINUE

C	     NORMALIZE DIIMFP
	     LAMBDA_I(IREG,IENER) = 1./CDIMFP(MAX_IDELTAE)
      	     DO 355,IDELTAE = 1,MAX_IDELTAE
               CDIMFP(IDELTAE) = CDIMFP(IDELTAE)/CDIMFP(MAX_IDELTAE)
355          CONTINUE


C	    COMPUTE INVERSE NORM. CUMUL. DIMFP

      	    DO 380,IRAN = 1,MAXRAN_I
      		Y = DBLE(IRAN)/DBLE(MAXRAN_I)*CDIMFP(MAX_IDELTAE)
      		X = (DINVERSE(Y,CDIMFP,MAX_IDELTAE)-1)
     .				*MAX_DELTAE/DBLE(MAX_IDELTAE)
      		PLASMON_TAB(IREG,IENER,IRAN) = X
380 	    CONTINUE

**       IF (E.GE.EMAX) IENER=N_PLASMON_INT(IREG)+1
         IF (E.GE.EMAX) GOTO 500
400      CONTINUE

500   CONTINUE

      IER = 0
cs      GOTO 1000

cs900   IER = -1

1000  END      

***************************************************************************
    
      SUBROUTINE P_LAMBDA(
     I      		   IREG,E,
     O  		   LAMBDA,IER)
      
      IMPLICIT NONE

      DOUBLE PRECISION E,LAMBDA
      INTEGER IREG,IER

***   P_LAMBDA   - RETURNS VALUE FOR INELASTIC LAMBDA OF REGION 
*		   IREG USING LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     LAMBDA    - PLASMON MEAN FREE PATH
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 26-JUN-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      DOUBLE PRECISION X1,X2,Y1,Y2
      INTEGER IENER

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL
      
      IF (E .LT. PLASMON_INT(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,N_PLASMON_INT(IREG)
      	IF (E.LT.PLASMON_INT(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   CONTINUE

C     LINEAR INTERPOLATION
      Y2 = LAMBDA_I(IREG,IENER)
      Y1 = LAMBDA_I(IREG,IENER-1)
      X1 = PLASMON_INT(IREG,IENER-1)
      X2 = PLASMON_INT(IREG,IENER)
      LAMBDA  = Y1 + (Y2-Y1)/(X2-X1)*(E-X1)
      IER = 0

1000  END      

***************************************************************************
    
      SUBROUTINE P_DELTAE(
     I      		   IREG,E,R1,R2,
     O  		   DELTAE,IER)

      IMPLICIT NONE

      DOUBLE PRECISION E,R1,R2,DELTAE
      INTEGER IREG,IER

***   P_DELTAE     - MONTE CARLO GENERATOR FOR INELASTIC SCATTERING USING
*		     LINEAR INTERPOLATION FOR ENERGY
*
**    ARGUMENTS 
*
*     E 	- ENERGY OF SCATTERED ELECTRON
*     IREG	- REGION NUMBER
*     R1,R2     - TWO UNIFORMLY DIST. RANDOM NUMBERS
*     DELTAE	- SELECTED ENERGY LOSS
*     IER	- ERROR FLAG (IN CASE ENERGY EXCEEDS TABULATED ENERGY RANGE)
*
**    AUTHOR: H.WAGNER 27-JUL-1994
*
      INCLUDE 'PARAM.INC'
      INCLUDE 'GLOBAL.INC'
      
      INTEGER IR,IENER
      DOUBLE PRECISION X1,X2,Y1,Y2

**    START OF EXECUTABLE CODE
*     ************************

C     DETERMINE TABULATED ENERGY INTERVAL
C      PRINT *, 'ELASTCOSL :',NCOSTHINTV(IREG),E
      
      IF (E .LT. PLASMON_INT(IREG,1)) THEN
      	IER = -1
      	GOTO 1000
      ENDIF

      DO 100, IENER=2,N_PLASMON_INT(IREG)
      	IF (E.LT.PLASMON_INT(IREG,IENER)) GOTO 200
100   CONTINUE

      IER = -1
      GOTO 1000

200   R2 = R2*REAL(MAXRAN_I)
      IR = AINT(R2)
      R2 = R2-REAL(IR)

C     STOCHASTIC LINEAR INTERPOLATION

      X1 = PLASMON_INT(IREG,IENER-1)
      X2 = PLASMON_INT(IREG,IENER)
      IF (R1 .GT. (E-X1)/(X2-X1)) THEN
         DELTAE = PLASMON_TAB(IREG,IENER-1,IR)*(1-R2)
     .        + R2 * PLASMON_TAB(IREG,IENER-1,IR+1)
      ELSE
       	 DELTAE = PLASMON_TAB(IREG,IENER,IR)*(1-R2)
     .        + R2 * PLASMON_TAB(IREG,IENER,IR+1)
      ENDIF
      IER = 0

1000  END      
