c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
*last modification: 04-Nov-1996

      SUBROUTINE RDGEO (
     I                  CGFILE,XSCALE,TILT,ROTAT,EPSMA,
     D                  NPTM,NFCM,NR,NXM,NZM,NYM,
     O                  NPTS,NFCS,NGRIDX,NGRIDZ,NGRIDY,
     O                  PCOORD,ILINES,IENDIX,ISGMNT,NSGMNT,
     O                  XGRIDL,ZGRIDL,YGRIDL,
     O                  XMIN,XMAX,ZMIN,ZMAX,
     O                  XW1,XW2,ZW,IER)

      INCLUDE 'TYPE.INC'
      INCLUDE 'DMA.INC'

***   RDGEO - READS THE GEOMETRY AND THE GRID FROM A GIVEN FILE.
*             IF NO GRID AT ALL OR NO GRIDLINES IN ONE DIRECTION IS/ARE
*             SPECIFIED (AT LEAST: (NGRIDX.EQ.0 .OR. NGRIDZ.EQ.0)) THEN
*             IN THE RELEVANT DIRECTION(S) AN EQUIDISTANT GRID IS
*             PRODUCED WITH THE MAXIMUM NUMBER OF GRID LINES IN THIS
*             DIRECTION. ADDITIONALLY THE IMPLANTATION WINDOW IS COMPUTED
*             FROM THE TILT AND ROTATION ANGLE.
*
**    ARGUMENTS
*
*        CGFILE - NAME OF THE GEOMETRY FILE
*        XSCALE - A QUARTER UNIT CELL EDGE OF SILICON DIAMOND LATTICE
*        TILT   - IMPLANTATION (TILT-) ANGLE
*        ROTAT  - ROTATION ANGLE
*        EPSMA  - EPSILON MACHINE (SEE PARAM.INC)
*        NPTM   - MAXIMUM NUMBER OF POINTS
*        NFCM   - MAXIMUM NUMBER OF FACES
*        NR     - NUMBER OF REGIONS
*        NXM    - MAXIMUM NUMBER OF GRIDLINES IN X-DIRECTION
*        NZM    - MAXIMUM NUMBER OF GRIDLINES IN Z-DIRECTION
*        NYM    - MAXIMUM NUMBER OF GRIDLINES IN Y-DIRECTION
*        NPTS   - ACTUAL NUMBER OF POINTS
*        NFCS   - ACTUAL NUMBER OF FACES
*        NGRIDX - ACTUAL NUMBER OF GRIDLINES IN X-DIRECTION
*        NGRIDZ - ACTUAL NUMBER OF GRIDLINES IN Z-DIRECTION
*        NGRIDY - ACTUEL NUMBER OF GRIDLINES IN Y-DIRECTION
*        PCOORD - COORDINATES OF THE POINTS
*        ILINES - INDICES OF THE FACES
*        IENDIX - ENDINDICES OF THE LINES
*        XGRIDL - GRIDLINES IN X-DIRECTION
*        ZGRIDL - GRIDLINES IN Z-DIRECTION
*        YGRIDL - GRIDLINES IN Y-DIRECTION
*        XMIN   - MINIMUM X-COORDINATE OF THE SIMULATION AREA
*        XMAX   - MAXIMUM X-COORDINATE OF THE SIMULATION AREA
*        ZMIN   - MINIMUM Z-COORDINATE OF THE SIMULATION AREA
*        ZMAX   - MAXIMUM Z-COORDINATE OF THE SIMULATION AREA
*        XW1    - MINIMUM X-COORDINATE OF IMPLANTATION WINDOW
*        XW2    - MAXIMUM X-COORDINATE OF IMPLANTATION WINDOW
*        ZW     - Z-COORDINATE OF IMPLANTATION WINDOW
*        IER    - ERROR INDICATOR: 0 ... EVERYTHING OK, 1 ... ERROR
*
**    REMARKS:
*
*        NOTE: THE MAXIMUM NUMBER OF LINES, WHICH CAN BE DEFINED OVER
*              NPTM POINTS IS 3*NPTM-6, IF THE LINES DO NOT INTERSECT
*              ONE ANOTHER! THEREFORE WE DO NOT NEED A SEPARATE LIMIT
*              FOR THE LINE INDEX ARRAY (ILINES).
*
**    SIDE EFFECTS: THE TILT ANGLE MAY BE ADJUSTED TO GUARANTEE
*                   NUMERICAL STABILITY
*
**    AUTHOR:  H.STIPPEL   1-FEB-1994
*

      DOUBLE PRECISION PI,ZERO,ONE
      CHARACTER*(*) CGFILE
      DIMENSION     PCOORD(2*NPTM), ILINES(3*NPTM-6), IENDIX(NFCM)
      DIMENSION     ISGMNT(NFCM)
      DIMENSION     XGRIDL(NDMAXGRID),ZGRIDL(NDMAZGRID)
      DIMENSION     YGRIDL(NDMAYGRID)
      CHARACTER*70  CDUMMY

      PARAMETER (PI=3.141592654,ZERO=0,ONE=1)

**    START OF EXECUTABLE CODE
*     ************************

      IER = 0
      OPEN (33,FILE=CGFILE,STATUS='OLD',FORM='FORMATTED',ERR=10)
      GOTO 20
 10   CONTINUE
      WRITE (*,*) ' '
      WRITE (*,*) ' '
      WRITE (*,*) '***** ERROR: COULD NOT OPEN GEOMETRY FILE: <',
     ,            CGFILE, '>! *****'
      WRITE (*,*) ' '
      WRITE (*,*) ' '
      IER = 1
      RETURN

 20   CONTINUE

*     ---READ THE GEOMETRY


*     READ THE NUMBER OF FACES AND SEGMENTS
      READ (33,'(A)') CDUMMY
      ISEG = 0
      READ (CDUMMY,*,ERR=21) NFCS,ISEG
21    CONTINUE
      IF (ISEG .EQ. 0) THEN
         NSGMNT = NFCS
      ELSE
         NSGMNT = ISEG
      ENDIF
      IF (NFCS.GT.NFCM) THEN
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         WRITE (*,*) '***** ERROR: TOO MANY FACES! *****'
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         CLOSE (33)
         IER = 1
         RETURN
      ENDIF
      IF (NFCS.NE.NR) THEN
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         WRITE (*,*) '***** ERROR: WRONG NUMBER OF REGIONS! *****'
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         CLOSE (33)
         IER = 1
         RETURN
      ENDIF

      NLNT = 0
      DO 40 IFC = 1, NFCS
*     READ THE NUMBER OF LINES FOR THIS FACE
         READ (33,'(A)') CDUMMY
         ISEG=0
         READ (CDUMMY,*,ERR=41) NLNS,ISEG
41       CONTINUE
         IF (ISEG .EQ. 0) THEN
            ISGMNT(IFC) = IFC
         ELSE
            ISGMNT(IFC)=ISEG
         ENDIF
         IF (ISGMNT(IFC) .GT. NSGMNT) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: ILLEGAL SEGMENT NUMBER *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            CLOSE (33)
            IER = 1
            RETURN
         ENDIF
*     STORE THE BEGIN OF THIS FACE
         NLNH = NLNT + 1
*     COMPUTE THE TOTAL NUMBER OF LINES
         NLNT = NLNT + NLNS
         IF (NLNT.GT.3*NPTM-6) THEN
*     THAT'S QUITE IMPOSSIBLE!!!
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: TOO MANY LINES! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            CLOSE (33)
            IER = 1
            RETURN
         ENDIF
*     SET THE ENDINDEX FOR THIS FACE
         IENDIX(IFC) = NLNT
*     READ THE FACE
         DO 30 ILN = NLNH, NLNT
            READ (33,*) ILINES(ILN)
 30      CONTINUE
 40   CONTINUE

*     READ THE NUMBER OF POINTS
      READ (33,*) NPTS
      IF (NPTS.GT.NPTM) THEN
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         WRITE (*,*) '***** ERROR: TOO MANY POINTS! *****'
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         CLOSE (33)
         IER = 1
         RETURN
      ENDIF

*     ACTUALLY READ THE POINTS
      DO 50 IPT = 1, NPTS
         READ (33,*) PCOORD(2*IPT-1), PCOORD(2*IPT)
 50   CONTINUE

*     SCALE THE GEOMETRY
      DO 60 IPT = 1, 2*NPTS
         PCOORD(IPT) = PCOORD(IPT) / XSCALE
 60   CONTINUE

*     DETERMINE THE MINIMUM AND MAXIMUM X/Z VALUES OF THE GEOMETRY
      XMIN = PCOORD(1)
      XMAX = XMIN
      ZMIN = PCOORD(2)
      ZMAX = ZMIN
      XT1Z = PCOORD(2)
      XB1Z = PCOORD(2)
      XT2Z = PCOORD(2)
      XB2Z = PCOORD(2)
      DO 70 IPT = 2, NPTS
         IF (PCOORD(2*IPT-1).LT.XMIN) THEN
            XMIN = PCOORD(2*IPT-1)
*     STORE THE Z-VALUE FOR THE IMPLANTATION WINDOW
            XT1Z = PCOORD(2*IPT)
            XB1Z = PCOORD(2*IPT)
         ELSEIF (PCOORD(2*IPT-1).LE.XMIN) THEN
            IF (PCOORD(2*IPT).LT.XT1Z) THEN
               XT1Z = PCOORD(2*IPT)
            ENDIF
            IF (PCOORD(2*IPT).GT.XB1Z) THEN
               XB1Z = PCOORD(2*IPT)
            ENDIF
         ENDIF
         IF (PCOORD(2*IPT-1).GT.XMAX) THEN
            XMAX = PCOORD(2*IPT-1)
*     STORE THE Z-VALUE FOR THE IMPLANTATION WINDOW
            XT2Z = PCOORD(2*IPT)
            XB2Z = PCOORD(2*IPT)
         ELSEIF (PCOORD(2*IPT-1).GE.XMAX) THEN
            IF (PCOORD(2*IPT).LT.XT2Z) THEN
               XT2Z = PCOORD(2*IPT)
            ENDIF
            IF (PCOORD(2*IPT).GT.XB2Z) THEN
               XB2Z = PCOORD(2*IPT)
            ENDIF
         ENDIF
         IF (PCOORD(2*IPT).LT.ZMIN) ZMIN = PCOORD(2*IPT)
         IF (PCOORD(2*IPT).GT.ZMAX) ZMAX = PCOORD(2*IPT)
 70   CONTINUE

*     CHECK GEOMETRY
      IF (XMAX-XMIN.GT.ZMAX-ZMIN) THEN
         DISTNC = EPSMA * 1.0e3 * (ZMAX-ZMIN)
      ELSE
         DISTNC = EPSMA * 1.0e3 * (XMAX-XMIN)
      ENDIF
      CALL TSTGEO (
     D             NPTS, NFCS,
     V             PCOORD, ILINES, IENDIX,
     I             DISTNC,
     O             IER)
      IF (IER.NE.0) THEN
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         WRITE (*,*) '***** ERROR: DURING TESTING THE GEOMETRY! *****'
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         CLOSE (33)
         IER = 1
         RETURN
      ENDIF

*     CHECK ANGLE (TILT) ON NUMERICAL PROBLEMS (TANGENS)
      IF (TILT.NE.ZERO) THEN
         NUMRD = TILT / 360.0
         TILT  = TILT - NUMRD * 360.0
      ENDIF
      OLDTLT = TILT
      IF (TILT.GT.89.0 .AND. TILT.LE.90.0) THEN
         TILT = 89.0
      ELSEIF (TILT.GT.90.0 .AND. TILT.LE.91.0) THEN
         TILT = 91.0
      ELSEIF (TILT.LT.-89.0 .AND. TILT.GE.-90) THEN
         TILT = -89.0
      ELSEIF (TILT.LT.-90.0 .AND. TILT.GE.-91.0) THEN
         TILT = -91.0
      ENDIF

      IF (OLDTLT.NE.TILT) THEN
         WRITE (*,*) ' ' 
         WRITE (*,*) ' ' 
         WRITE (*,*) '***** WARNING: CANNOT HANDLE THIS TILT-ANGLE (',
     ,                OLDTLT,' DEGREE)! *****'
         WRITE (*,*) 'TILT-ANGLE CHANGED TO ',TILT,' DEGREE' 
         WRITE (*,*) ' ' 
         WRITE (*,*) ' ' 
      ENDIF

*     COMPUTE THE IMPLANTATION WINDOW
      HEIGHT = ONE
      IF ((TILT.GT.90.0  .AND. TILT.LT.270.0) .OR.
     .    (TILT.LT.-90.0 .AND. TILT.GT.-270.0)) THEN
*     IMPLANTATION FROM THE BOTTOM
         ZW  = ZMAX + HEIGHT
         XW1 = XMIN + (ZW - XB1Z) * TAN(TILT*PI/180.0) *
     *                              COS(ROTAT*PI/180.0)
         XW2 = XMAX + (ZW - XB2Z) * TAN(TILT*PI/180.0) *
     *                              COS(ROTAT*PI/180.0)
      ELSE
*     IMPLANTATION FROM THE TOP
         ZW  = ZMIN - HEIGHT
         XW1 = XMIN - (XT1Z - ZW) * TAN(TILT*PI/180.0) *
     *                              COS(ROTAT*PI/180.0)
         XW2 = XMAX - (XT2Z - ZW) * TAN(TILT*PI/180.0) *
     *                              COS(ROTAT*PI/180.0)
      ENDIF

cs    no grid lines are read from the geometry file in case of SESAME
cs    is is a feature only need for IMSIL (documented out by cs)
cs*     READ THE NUMBER OF GRIDLINES IN X-DIRECTION
cs      READ (33,*) NGRIDX
cs      IF (NGRIDX.GT.NXM) THEN
cs         WRITE (*,*) ' '
cs         WRITE (*,*) ' '
cs         WRITE (*,*) '***** ERROR: TOO MANY GRIDLINES X! *****'
cs         WRITE (*,*) ' '
cs         WRITE (*,*) ' '
cs         CLOSE (33)
cs         IER = 1
cs         RETURN
cs      ENDIF

cs      IF (NGRIDX.GT.0) THEN
cs*     READ AND SCALE THE GRIDLINES IN X-DIRECTION
cs         DO 80 IX = 1, NGRIDX
cs            READ (33,*) XGRIDL(IX)
cs            XGRIDL(IX) = XGRIDL(IX) / XSCALE
cs 80      CONTINUE
cs      ELSE
cs*     GENERATE EQUIDISTANT GRID WITH MAXIMUM NUMBER OF LINES
cs         NGRIDX = NXM
cs         DX = (XMAX - XMIN) / (NXM - 1)
cs         XGRIDL(1) = XMIN
cs         DO 90 IX = 2, NXM
cs            XGRIDL(IX) = XMIN + (IX - 1) * DX
cs 90      CONTINUE
cs      ENDIF
csc      GOTO 111
cs*     READ THE NUMBER OF GRIDLINES IN Z-DIRECTION
cs      READ (33,*) NGRIDZ
cs      IF (NGRIDZ.GT.NZM) THEN
cs         WRITE (*,*) ' '
cs         WRITE (*,*) ' '
cs         WRITE (*,*) '***** ERROR: TOO MANY GRIDLINES Z! *****'
cs         WRITE (*,*) ' '
cs         WRITE (*,*) ' '
cs         CLOSE (33)
cs         IER = 1
cs         RETURN
cs      ENDIF
cs
cs      IF (NGRIDZ.GT.0) THEN
cs*     READ AND SCALE THE GRIDLINES IN X-DIRECTION
cs         DO 100 IZ = 1, NGRIDZ
cs            READ (33,*) ZGRIDL(IZ)
cs            ZGRIDL(IZ) = ZGRIDL(IZ) / XSCALE
cs 100     CONTINUE
cs      ELSE
cs*     GENERATE EQUIDISTANT GRID WITH MAXIMUM NUMBER OF LINES
cs         NGRIDZ = NZM
cs         DZ = (ZMAX - ZMIN) / (NZM - 1)
cs         ZGRIDL(1) = ZMIN
cs         DO 110 IZ = 2, NZM
cs            ZGRIDL(IZ) = ZMIN + (IZ - 1) * DZ
cs 110     CONTINUE
cs      ENDIF

 111  CLOSE (33)

      RETURN
      END


***************************************************************************
*last modification: 11-Dec-1996

      SUBROUTINE RDGRID(
     I                  NPTS,NFCS,
     O                  XGRMIN,XGRMAX,ZGR,IER)

      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'DMA.INC'

***  RDGRID - READS GRID FROM A GIVEN FILE FOR ION AND X-RAY SIMULATION. 
*             IF NO GRID AT ALL OR NO GRIDLINES IN ONE DIRECTION IS/ARE
*             SPECIFIED (AT LEAST: (NGRIDX.EQ.0 .OR. NGRIDZ.EQ.0)) THEN
*             IN THE RELEVANT DIRECTION(S) AN GRID IS
*             PRODUCED WITH THE MAXIMUM NUMBER OF GRID LINES IN THIS
*             DIRECTION (EQUIDISTANT IN Z-DIRECTION, SURFACE AND SCANNING MODE 
*             DEPENDENT IN X-DIRECTION) 
*             DEFAULT: NO GRIDFILE: AUTOMATIC GRID GENERATION
**   INPUT:
*           
*        CGRDFL - NAME OF THE GRID FILE         
*        NPTM   - MAXIMUM NUMBER OF POINTS
*        NFCM   - MAXIMUM NUMBER OF FACES
*        NR     - NUMBER OF REGIONS
*        NXM    - MAXIMUM NUMBER OF GRIDLINES X
*        NZM    - MAXIMUM NUMBER OF GRIDLINES Z
*        NPTS   - ACTUAL NUMBER OF POINTS
*        NFCS   - ACTUAL NUMBER OF FACES
*        X_MIN  - MINIMUM X-COORDINATE OF GEOMETRY
*
*    OUTPUT:
*
*        NGRIDX - ACTUAL NUMBER OF GRIDLINES X
*        NGRIDZ - ACTUAL NUMBER OF GRIDLINES Z
*        XGRIDL - GRIDLINES X
*        ZGRIDL - GRIDLINES Z 
*        XGRMIN - MINIMUM X-COORDINATE OF GRID
*        XGRMAX - MAXIMUM X-COORDINATE OF GRID
*        ZGR    - Z-COORDINATE OF GRID
*        IER    - ERROR INDICATOR: 0 ... EVERYTHING OK, 1 ... ERROR
*
*
**  AUTHO: A.PFEIFFER JAN-1995
*          C.SCHIEBL  JAN-1996
*
*

      PARAMETER(DELTA = 2d-1)
      CHARACTER*10  CBL 
      DIMENSION     XGRIDS(NXM),ZGRIDS(NXM),DELMGR(NXM)
      DIMENSION     NINTX(NXM),NINTZ(NZM)      
      LOGICAL       LGRDFL,LXEQGR,LZEQGR
  
      DATA CBL /'          '/

** START OF EXECUTABLE CODE
*  ************************

      LDBG =  .FALSE.
      LGRDFL = .FALSE.
      LXEQGR = .FALSE.
      LZEQGR = .FALSE.
c      lscan = .FALSE.
c      xespot = 50000.
c      xrange = 5000.
      IER = 0

C CALCULATEN OF ANALYTICAL MAXIMAL POSIBLE ELECTRONRANGE 
C (ACCORDING TO KANAYA-OKAYAMA 1972)       

      EE = E0/1000.
      RMAX = 0.
       
CS      IF (CGRDFL.EQ.CBL//CBL//CBL//CBL//CBL//CBL//CBL) LAUTOG=.TRUE.

      IF (LAUTOG) THEN
C AUTOMATIC GRID GENERATION
         WRITE(6,'(A)')' AUTOMATIC GRID GENERATING'
cs         NGRIDX=NXM
cs         NGRIDZ=NZM
         NGRIDX=NXGRID
         NGRIDZ=NZGRID
         NGRIDY=NYGRID
         DO 4 IR  = 1,NREG 
            AA=0.
            ZZ=0.
             DO 3 IE = 1,NELE(IR)
               AA = AA+WEIGHTFRAC(IR,IE)*ATMASS(ELEM(IR,IE))
               ZZ = ZZ+WEIGHTFRAC(IR,IE)*ATNUM(ELEM(IR,IE))
 3          CONTINUE    
            RKANAY= 0.0276D04*EE**1.67*AA/ZZ**0.89/DENS(IR)
            IF (RMAX.LT.RKANAY) THEN
               RMAX=RKANAY
            ENDIF
 4       CONTINUE
        
         IF (SCAN) THEN
            XLEFT = XESPOT-0.5*XRANGE-RMAX
            XRIGT = XESPOT+0.5*XRANGE+RMAX
         ELSE
CS          DIVERGENCE NOT TAKEN INTO ACCOUNT
            XLEFT = XESPOT-RMAX
            XRIGT = XESPOT+RMAX
         ENDIF
         IF (XLEFT.LT.X_MIN) XLEFT=X_MIN
         IF (XRIGT.GT.X_MAX) XRIGT=X_MAX

CS       CHECK WHETHER THE NUMBER OF GEOMETRY POINTS INBETWEEN XLEFT
CS       AND XRIGT IS LESS THAN NXGRID-2 OR NOT!
         IXBETW=0
         DO 5 I=1,2*NPTS,2
            IF ((PCOORD(I).GT.XLEFT).AND.(PCOORD(I).LT.XRIGT))
     &           IXBETW=IXBETW+1
 5       CONTINUE

         IF (IXBETW.GT.(NXGRID-2)) THEN
C           SIMPLE EQUIDISTANT GRID FORM XLEFT TO XRIGT
            WRITE(6,'(A)')' GENERATING A SIMPLE EQUIDISTANT GRID'
            IGRDPX = 2
            XGRIDS(1) = XLEFT
            NINTX(1)  = NGRIDX-2
            XGRIDS(2) = XRIGT 
            LXEQGR = .TRUE.
         ELSE
            XGRIDS(1)=XLEFT
            IGRDPX=2
            DO 11 I=1,2*NPTS,2
               IF ((PCOORD(I).GT.XLEFT).AND.(PCOORD(I).LT.XRIGT)) THEN
                  XGRIDS(IGRDPX)=PCOORD(I)
                  DO 9 J=1,IGRDPX-1
                    IF (XGRIDS(J).EQ.XGRIDS(IGRDPX)) GOTO 11
                    IF (XGRIDS(J).GT.XGRIDS(IGRDPX)) THEN
                       XGRID=XGRIDS(J)
                       XGRIDS(J)=XGRIDS(IGRDPX)
                       XGRIDS(IGRDPX)=XGRID
                    ENDIF
 9                CONTINUE
                  IGRDPX=IGRDPX+1
               ENDIF
 11         CONTINUE
            XGRIDS(IGRDPX)=XRIGT
         ENDIF                 

CS       DETERMINE ZMIN AND ZMAX
         IXFRST=0
         IXLAST=0
         IFLAG=0
         DO 12 I=1,NSCORD
            IF ((SCORDX(I).GT.XLEFT).AND.(SCORDX(I).LT.XRIGT)) THEN
               IF (IFLAG.EQ.0) THEN
                  IXFRST=I
                  IFLAG=1
               ENDIF
               IXLAST=I
            ENDIF
 12      CONTINUE
         IF (IXFRST.GT.IXLAST) THEN
            WRITE(6,'(A)')'     '
            WRITE(6,'(A)')' %%% WARNING %%%'
            WRITE(6,'(2A)')' Check Grid ',
     &                     '- Grid may be unproperly defined!'
            IDMY=IXLAST
            IXLAST=IXFRST
            IXFRST=IDMY
         ENDIF

         IF ((IXFRST.EQ.0).AND.(IXLAST.EQ.0)) THEN
            ILSCORD=0
            IRSCORD=0
            IFLAG=0
            DO 13 I=1,NSCORD
               IF (SCORDX(I).LE.XLEFT) ILSCORD=I
               IF (SCORDX(I).GE.XRIGT) THEN
                  IF (IFLAG.EQ.0) THEN
                     IFLAG=1
                     IRSCORD=I
                  ENDIF
               ENDIF
 13         CONTINUE
            IF ((ILSCORD.EQ.0).OR.(IRSCORD.EQ.0)) THEN
               IF (ILSCORD.EQ.0) THEN
                  I1=1
               ELSE
                  I1=ILSCORD
               ENDIF
               IF (IRSCORD.EQ.0) THEN
                  I2=NSCORD
               ELSE
                  I2=IRSCORD
               ENDIF
               ZMIN=SCORDZ(I1)
               ZMAX=ZMIN
               DO 14 I=I1+1,I2
                  IF (SCORDX(I).GT.ZMAX) THEN
                     ZMAX=SCORDZ(I)
                     IMAX=I
                  ENDIF
                  IF (SCORDX(I).LT.ZMIN) THEN
                     ZMIN=SCORDZ(I)
                     IMIN=I
                  ENDIF
 14            CONTINUE
            ELSE
               IF (SCORDZ(ILSCORD).GT.SCORDZ(IRSCORD)) THEN
                  ZMIN=SCORDZ(IRSCORD)
                  ZMAX=SCORDZ(ILSCORD)
                  IMIN=IRSCORD
                  IMAX=ILSCORD
               ELSE
                  ZMAX=SCORDZ(IRSCORD)
                  ZMIN=SCORDZ(ILSCORD)
                  IMIN=ILSCORD
                  IMAX=IRSCORD
               ENDIF
            ENDIF
            IXFRST=1
            IXLAST=NSCORD
         ELSE
            IF (IXFRST.EQ.0) IXFRST=1
            IF (IXLAST.EQ.0) IXLAST=NSCORD
            ZMIN=SCORDZ(IXFRST)
            ZMAX=ZMIN
            IMAX=IXFRST
            IMIN=IXFRST
            DO 15 I=IXFRST+1,IXLAST
               IF (SCORDZ(I).GT.ZMAX) THEN
                  ZMAX=SCORDZ(I)
                  IMAX=I
               ENDIF
               IF (SCORDZ(I).LT.ZMIN) THEN
                  ZMIN=SCORDZ(I)
                  IMIN=I
               ENDIF
 15         CONTINUE
            IF (IMIN.EQ.IMAX) IMIN=IMAX-1
         ENDIF

         IF (IMIN.EQ.IMAX) THEN
            WRITE(6,'(A)')'     '
            WRITE(6,'(A)')' %%% WARNING %%%'
            WRITE(6,'(2A)')' Check Grid ',
     &                     '- cannot generate propriate Grid!'
            ZMIN=SCORDZ(IMIN)
            IF (ZMIN.LT.Z_MIN) ZMIN=Z_MIN
            ZMAX=ZMIN+2*RMAX
            IF (ZMAX.GT.Z_MAX) ZMAX=Z_MAX
         ELSE
            IF (SCORDX(IXFRST).GT.XLEFT) THEN
               IF (IXFRST.GT.1) THEN
                  IF (SCORDZ(IXFRST-1).LT.ZMIN) THEN
                     ZMIN=SCORDZ(IXFRST-1)
                  ELSE IF (SCORDZ(IXFRST-1).GT.ZMAX) THEN
                     ZMAX=SCORDZ(IXFRST-1)
                  ENDIF
               ENDIF
            ENDIF
            IF (SCORDX(IXLAST).LT.XRIGT) THEN
               IF (IXLAST.LT.NSCORD) THEN
                  IF (SCORDZ(IXLAST+1).LT.ZMIN) THEN
                     ZMIN=SCORDZ(IXLAST+1)
                  ELSE IF (SCORDZ(IXLAST+1).GT.ZMAX) THEN
                     ZMAX=SCORDZ(IXLAST+1)
                  ENDIF
               ENDIF
            ENDIF
            ZMAX=ZMAX+RMAX
            IF (ZMAX.GT.Z_MAX) ZMAX=Z_MAX
         ENDIF

CS       CHECK WHETHER THE NUMBER OF GEOMETRY POINTS INBETWEEN XMIN
CS       AND ZMAX IS LESS THAN NZGRID-2 OR NOT!
         IZBETW=0
         DO 16 I=2,2*NPTS,2
            IF ((PCOORD(I).GT.ZMIN).AND.(PCOORD(I).LT.ZMAX))
     &           IZBETW=IZBETW+1
 16      CONTINUE

         IF (IZBETW.GT.(NZGRID-2)) THEN
C           SIMPLE EQUIDISTANT GRID FORM ZMIN TO ZMAX
            IGRDPZ = 2
            ZGRIDS(1) = ZMIN
            NINTZ(1)  = NGRIDZ-2
            ZGRIDS(2) = ZMAX
            LZEQGR = .TRUE. 
         ELSE
            ZGRIDS(1)=ZMIN
            IGRDPZ=2
            DO 44 I=2,2*NPTS,2
               IF ((PCOORD(I).GT.ZMIN).AND.(PCOORD(I).LT.ZMAX)) THEN
                  ZGRIDS(IGRDPZ)=PCOORD(I)                     
                  DO 45 J = 1,IGRDPZ-1
                     IF (ZGRIDS(J).EQ.ZGRIDS(IGRDPZ)) GOTO 44
                     IF (ZGRIDS(J).GT.ZGRIDS(IGRDPZ)) THEN
                        ZGRID=ZGRIDS(J)
                        ZGRIDS(J)=ZGRIDS(IGRDPZ)
                        ZGRIDS(IGRDPZ)=ZGRID
                     ENDIF  
 45               CONTINUE
                  IGRDPZ=IGRDPZ+1
               ENDIF 
 44         CONTINUE
            ZGRIDS(IGRDPZ)=ZMAX
         ENDIF

      ELSE
         WRITE(6,'(A)')' READING GRID FILE'
C         READ GRID FILE (NUMBER OF INTERVALPOINTS(>1), STRATING X-COORD., 
C         NUMBER OF GRIDLINES OF INTERVAL.,ANALOGUOUS FOR Z, 
C                 E.G.: 5 
C                        0.0 
C                        10 
C                        100. 
C                        3
C                        2000. 
C                        14                      
C                        5000. 
C                        25
C                        10000.
C                       2
C                        1000.
C                        50 
C                        20000 
         
         OPEN (33,FILE=CGRDFL,STATUS='OLD',FORM='FORMATTED',ERR=10)
         GOTO 20
 10      CONTINUE
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         WRITE (*,*) '***** ERROR: COULD NOT OPEN GRID  FILE: <',
     ,               CGRDFL, '>! *****'
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         IER = 1
         RETURN
c         WRITE(*,'(A)') ' SELF DEFINED GRID NOT IMPLEMENTED YET'
c         STOP 'STOPPED IN SUBROUTINE RDGRID'
         
 20      CONTINUE
*     ---READ THE GRID FILE 
         LGRDFL=.TRUE.
         NGRIDX=1                   
         READ(33,*)IGRDPX
         IF (IGRDPX.LT.2) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: TOO FEW GRIDLINES X! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            CLOSE (33)
            IER = 1
            RETURN
         ENDIF          

         DO  102 I=1,IGRDPX-1
             READ(33,*)XGRIDS(I)
             READ(33,*)NLINE
             NINTX(I)=NLINE
             NGRIDX=NGRIDX+NINTX(I)+1
 102     CONTINUE
         READ (33,*)XGRIDS(IGRDPX)
         IF (NGRIDX.GT.NXM) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: TOO MANY GRIDLINES X! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (6,'(A,I2,2A)')
     &                  ' AN EQUIDISTANT GRID WITH ',NXM,' GRIDLINES',
     &                  ' IN X-DIRECTION WILL BE GENERATED BETWEEN:'
            WRITE (*,'(A,F20.5)')' LEFT X-COORDINATE:  ',XGRIDS(1)
            WRITE (*,'(A,F20.5)')' RIGHT X-COORDINATE: ',XGRIDS(IGRDPX)
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            NGRIDX = NXM
            XGRIDS(2)=XGRIDS(IGRDPX)
            IGRDPX = 2  
            NINTX(IGRDPX-1)=NGRIDX-2 
c            CLOSE (33)
C            IER = 1
C            RETURN
         ENDIF
C         READ (33,*)XGRIDS(IGRDPX)
         
         NGRIDZ=1                   
         READ(33,*)IGRDPZ
         IF (IGRDPZ.LT.2) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: TOO FEW GRIDLINES Z! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            CLOSE (33)
            IER = 1
            RETURN
         ENDIF
         DO  104 I=1,IGRDPZ-1
             READ(33,*)ZGRIDS(I)
             READ(33,*)NLINE
             NINTZ(I)=NLINE
             NGRIDZ=NGRIDZ+NINTZ(I)+1
 104     CONTINUE
         READ (33,*)ZGRIDS(IGRDPZ) 
         IF (NGRIDZ.GT.NZM) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: TOO MANY GRIDLINES Z! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (6,'(A,I2,2A)')
     &                   ' AN EQUIDISTAND GRID WITH ',NZM,' GRIDLINES',
     &                   ' IN Z-DIRECTION WILL BE GENERATED BETWEEN:'
            WRITE (*,'(A,F20.5)')' LEFT Z-COORDINATE:  ',ZGRIDS(1)
            WRITE (*,'(A,F20.5)')' RIGHT Z-COORDINATE: ',ZGRIDS(IGRDPZ)
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            NGRIDZ = NZM
            ZGRIDS(2)=ZGRIDS(IGRDPZ)
            IGRDPZ = 2  
            NINTZ(IGRDPZ-1)=NGRIDZ-2 
         ENDIF
C         READ (33,*)ZGRIDS(IGRDPZ) 

         NGRIDY=1

         CLOSE (33)     
       
      ENDIF
        
C         IF (LDBG) THEN
C            DO 111 I = 1,IGRDPX
C               WRITE(*,*) I,XGRIDS(I)
C 111        CONTINUE   
C            DO 113 I = 1,IGRDPZ
C               WRITE(*,*) I,ZGRIDS(I)
C 113        CONTINUE
C         ENDIF
C         STOP 'ITS ALL RIGHT ?'

      IF ((.NOT.LGRDFL).AND.(.NOT.LXEQGR)) THEN
         DMAXX=XGRIDS(IGRDPX)-XGRIDS(1)
         DWIDTH=DMAXX/(NGRIDX-1)
         NXGTOT=0
         DELTMX=XGRIDS(2)-XGRIDS(1)
         DO 46 I=1,IGRDPX-1
            DELMGR(I)=XGRIDS(I+1)-XGRIDS(I)
CS          NUMBER OF GRID LINES BETWEEN MAIN GRID LINES (XGRIDS)
            NINTX(I)=INT(DELMGR(I)/DWIDTH+0.5)
            IF (NINTX(I).GT.0) NINTX(I)=NINTX(I)-1
            NXGTOT=NXGTOT+NINTX(I)
 46      CONTINUE
 47      IF ((NXGTOT+IGRDPX).GT.NGRIDX) THEN
CS          TOO MANY GRID LINES INSERTED, REMOVE SOME
            GRCRIT=0
            IGCRIT=0
            NAGRMX=0
            IAGRMX=0
            DO 48 I=1,IGRDPX-1
               AA=NINTX(I)**2/DELMGR(I)
               IF (AA.GT.GRCRIT) THEN
                  GRCRIT=AA
                  IGCRIT=I
               ENDIF
               IF (NINTX(I).GT.NAGRMX) THEN
                  NAGRMX=NINTX(I)
                  IAGRMX=I
               ENDIF
 48         CONTINUE
            IF (IGCRIT.LE.0) THEN
               NINTX(IAGRMX)=NINTX(IAGRMX)-1
            ELSE
               NINTX(IGCRIT)=NINTX(IGCRIT)-1
            ENDIF
            NXGTOT=NXGTOT-1
            GOTO 47
         ENDIF
 49      IF ((NXGTOT+IGRDPX).LT.NGRIDX) THEN
CS          INSERT GRID LINES
            GRCRIT=(NINTX(1)+1)**2/DELMGR(1)
            IGCRIT=0
            DO 50 I=1,IGRDPX-1
               AA=(NINTX(I)+1)**2/DELMGR(I)
               IF (AA.LE.GRCRIT) THEN
                  GRCRIT=AA
                  IGCRIT=I
               ENDIF
 50         CONTINUE
            NINTX(IGCRIT)=NINTX(IGCRIT)+1
            NXGTOT=NXGTOT+1
            GOTO 49
         ENDIF
      ENDIF

      IGLX = 1
      DO 51 IGX=1,IGRDPX-1
         NUMX=NINTX(IGX)
         IF (NUMX.EQ.0) THEN
            CALL CHKBND(2,IGLX,'RDGRID','XGRIDL')
            XGRIDL(IGLX)=XGRIDS(IGX)
            IGLX=IGLX+1 
            GOTO 61
         ENDIF 
         DELTX=(XGRIDS(IGX+1)-XGRIDS(IGX))/(NUMX+1)
         CALL CHKBND(2,IGLX,'RDGRID','XGRIDL')
         XGRIDL(IGLX)=XGRIDS(IGX)
         IINTX=IGLX+NUMX
CS         IF (IINTX.GE.NGRIDX) IINTX=NGRIDX-1
         CALL CHKBND(2,IINTX,'RDGRID','XGRIDL')
         DO 52 IPX=IGLX,IINTX-1    
            XGRIDL(IPX+1)=XGRIDL(IPX)+DELTX
 52      CONTINUE      
         IGLX=IGLX+NUMX+1
 61      IF (IGLX.GT.NGRIDX) THEN
            WRITE(6,'(A)')'     '
            WRITE(6,'(A)')' %%% WARNING %%%'
            WRITE(6,'(2A)')' Check Grid - too many auxiliary',
     &                     ' grid lines (x-direction)!'
         ENDIF
 51   CONTINUE
      NGRIDX=IGLX
      XGRIDL(NGRIDX)=XGRIDS(IGRDPX)


      IF ((.NOT.LGRDFL).AND.(.NOT.LZEQGR)) THEN
         DMAXZ=ZGRIDS(IGRDPZ)-ZGRIDS(1)
         DWIDTH=DMAXZ/(NGRIDZ-1)
         NZGTOT=0
         DELTMX=ZGRIDS(2)-ZGRIDS(1)
         DO 53 I=1,IGRDPZ-1
            DELMGR(I)=ZGRIDS(I+1)-ZGRIDS(I)
CS          NUMBER OF GRID LINES BETWEEN MAIN GRID LINES (ZGRIDS)
            NINTZ(I)=INT(DELMGR(I)/DWIDTH+0.5)
            IF (NINTZ(I).GT.0) NINTZ(I)=NINTZ(I)-1
            NZGTOT=NZGTOT+NINTZ(I)
 53      CONTINUE
 54      IF ((NZGTOT+IGRDPZ).GT.NGRIDZ) THEN
CS          TOO MANY GRID LINES INSERTED, REMOVE SOME
            GRCRIT=0
            IGCRIT=0
            NAGRMX=0
            IAGRMX=0
            DO 55 I=1,IGRDPZ-1
               AA=NINTZ(I)**2/DELMGR(I)
               IF (AA.GT.GRCRIT) THEN
                  GRCRIT=AA
                  IGCRIT=I
               ENDIF
               IF (NINTZ(I).GT.NAGRMX) THEN
                  NAGRMX=NINTZ(I)
                  IAGRMX=I
               ENDIF
 55         CONTINUE
            IF (IGCRIT.LE.0) THEN
               NINTZ(IAGRMX)=NINTZ(IAGRMX)-1
            ELSE
               NINTZ(IGCRIT)=NINTZ(IGCRIT)-1
            ENDIF
            NZGTOT=NZGTOT-1
            GOTO 54
         ENDIF
 56      IF ((NZGTOT+IGRDPZ).LT.NGRIDZ) THEN
CS          INSERT GRID LINES
            GRCRIT=(NINTZ(1)+1)**2/DELMGR(1)
            IGCRIT=0
            DO 57 I=1,IGRDPZ-1
               AA=(NINTZ(I)+1)**2/DELMGR(I)
               IF (AA.LE.GRCRIT) THEN
                  GRCRIT=AA
                  IGCRIT=I
               ENDIF
 57         CONTINUE
            NINTZ(IGCRIT)=NINTZ(IGCRIT)+1
            NZGTOT=NZGTOT+1
            GOTO 56
         ENDIF
      ENDIF

      IGLZ = 1
      DO 59 IGZ=1,IGRDPZ-1
         NUMZ=NINTZ(IGZ)
         IF (NUMZ.EQ.0) THEN
            CALL CHKBND(3,IGLZ,'RDGRID','ZGRIDL')
            ZGRIDL(IGLZ)=ZGRIDS(IGZ)
            IGLZ=IGLZ+1 
            GOTO 69
         ENDIF 
         DELTZ=(ZGRIDS(IGZ+1)-ZGRIDS(IGZ))/(NUMZ+1)
         CALL CHKBND(3,IGLZ,'RDGRID','ZGRIDL')
         ZGRIDL(IGLZ)=ZGRIDS(IGZ)
         IINTZ=IGLZ+NUMZ
CS         IF (IINTZ.GE.NGRIDZ) IINTZ=NGRIDZ-1
         CALL CHKBND(3,IINTZ,'RDGRID','ZGRIDL')
         DO 58 IPZ=IGLZ,IINTZ-1    
            ZGRIDL(IPZ+1)=ZGRIDL(IPZ)+DELTZ
 58      CONTINUE      
         IGLZ=IGLZ+NUMZ+1
 69      IF (IGLZ.GT.NGRIDZ) THEN
            WRITE(6,'(A)')'     '
            WRITE(6,'(A)')' %%% WARNING %%%'
            WRITE(6,'(2A)')' Check Grid - too many auxiliary',
     &                     ' grid lines (z-direction)!'
         ENDIF
 59   CONTINUE
      NGRIDZ=IGLZ
      ZGRIDL(NGRIDZ)=ZGRIDS(IGRDPZ)


c     WEIGHTINGS FOR GRIDPOINTS
      MXINDX=NDMAXGRID*NDMAZGRID*(NGRIDY-1)+
     &       NDMAXGRID*(NGRIDZ-1)+NGRIDX
      CALL CHKBND(10,MXINDX,'RDGRID','WGTGRD')
      DO 211 IX = 1,NGRIDX
         DO 213 IZ = 1,NGRIDZ
            IF (IX.EQ.1) THEN
               XWEIGHT= (XGRIDL(IX+1)-XGRIDL(IX))/2.
            ELSEIF (IX.EQ.NGRIDX) THEN
               XWEIGHT= (XGRIDL(IX)-XGRIDL(IX-1))/2.
            ELSE
               XWEIGHT=(XGRIDL(IX+1)-XGRIDL(IX-1))/2.
            ENDIF
            IF (IZ.EQ.1) THEN
               ZWEIGHT= (ZGRIDL(IZ+1)-ZGRIDL(IZ))/2.
            ELSEIF (IZ.EQ.NGRIDZ) THEN
               ZWEIGHT= (ZGRIDL(IZ)-ZGRIDL(IZ-1))/2.
            ELSE
                ZWEIGHT= (ZGRIDL(IZ+1)-ZGRIDL(IZ-1))/2.
            ENDIF
                    
CS          FOR THE TIME BEING:
CS          ONLY ONE GRIDLINE IN Y-DIRECTION (IY=1)
            IY=1
            INDX=NDMAXGRID*NDMAZGRID*(IY-1)+NDMAXGRID*(IZ-1)+IX
            WGTGRD(INDX)=XWEIGHT*ZWEIGHT 

 213     CONTINUE
 211  CONTINUE
      

      RETURN 
      END 


