c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
* last modification: CS 10-NOV-1997
      
      SUBROUTINE STATCS(
     I                  MODE,X,Y,Z,XSP,YSP,ZSP,XAIR,YAIR,DFRAC,
     I                  DIRX,DIRY,DIRZ,E,VLUE,IETYP,IESUBTYP)

***   STATIS - STATISTICAL EVALUATION
*
**    ARGUMENTS
*
*     MODE   - CONTROLS OPERATION
*              MODE=10 ... INITIALIZATION
*              MODE=21 ... EVALUATION, START OF TRAJECTORY
*              MODE=22 ... EVALUATION, ALONG THE TRAJECTORY
*              MODE=23 ... EVALUATION, END OF TRAJECTORY
*              MODE=24 ... EVALUATION, ELECTRON IS BACKSCATTERED
*              MODE=25 ... EVALUATION, ELECTRON IS TRANSMITTED
*              MODE=30 ... OUTPUT
*     X      - X-COORDINATE OF THE ELECTRON
*     Y      - Y-COORDINATE OF THE ELECTRON
*     Z      - Z-COORDINATE OF THE ELECTRON
*     XSP    - X-COORDINATE OF THE STARTING POSITION OF THE ELECTRON
*     YSP    - Y-COORDINATE OF THE STARTING POSITION OF THE ELECTRON
*     ZSP    - Z-COORDINATE OF THE STARTING POSITION OF THE ELECTRON
*     XAIR   - X-COMPONENT OF THE PATH IN THE AIR
*     YAIR   - Y-COMPONENT OF THE PATH IN THE AIR
*     DFRAC  - DETECTED FRACTION OF ELECTRON
*     DIRX   - X-COORDINATE OF ELECTRON VELOCITY VECTOR
*     DIRY   - Y-COORDINATE OF ELECTRON VELOCITY VECTOR
*     DIRZ   - Z-COORDINATE OF ELECTRON VELOCITY VECTOR
*     E      - ENERGY OF THE ELECTRON
*     VLUE   - VALUE OF UPDATE
*     IETYP  - TYPE OF ELECTRON : 
*		IETYP = 1 : PRIMARY      
*		IETYP = 5 : AUGER    
*		IETYP = 4 : SECONDARY
*		IETYP = 6 : XPS      
*     IESUBTYP - AUGERLINE IF IETYP = 1
*
**    REMARKS
*
*     1) DENSUM AND DEESUM ARE PASSED VIA COMMON BLOCK
*
**    AUTHOR:  C.SCHIEBL  14-OCT-1994
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'
      INCLUDE 'COMMON.INC'
      INCLUDE 'GLOBAL.INC'
      INCLUDE 'FILE.INC'
      INCLUDE 'HIST.INC'
      INCLUDE 'OUTPUT.INC'

*     ---DECLARATIONS FOR ENERGY STATISTICS OF BACKSCATTERED AND 
*     ---TRANSMITTED ELECTRONS
      PARAMETER (NEMAX=10)
      DIMENSION EMAXB(NEMAX),EMAXT(NEMAX)

      CHARACTER*4  CEXT
      CHARACTER*70 SYSTXT

      DOUBLE PRECISION DFRAC(0:MAXEDET)
      INCLUDE 'PATH.INC'

**    START OF EXECUTABLE CODE
*     ************************

      LBACK =.FALSE.
      LTRAN =.FALSE.

      MODE1=MODE/10
      GOTO (100,200,300) MODE1
      STOP 'SUSPECT MODE IN STATIS'

C INITIALIZATION
C --------------

100   CONTINUE

*     ---ELECTRONS INSIDE, BACKSCATTERED, TRANSMITTED
      NIN=0
      NBACK=0
      NTRAN=0
      NOUTS=0
      NOUTL=0
      NOUTR=0
      NOUTLR=0
*     ---INITIALIZE ENERGY LOSS SUMS
      SUMDEE=ZERO
      SUMDEN=ZERO
*     ---INITIALIZE THE HISTOGRAM(S)
*     THIS IS DONE IN THE INIT ROUTINE

*     ---OPEN THE TRAJECTORY FILE
      IF (LPOI.OR.LTRA.OR.LINT) THEN
         OPEN (NTRAJ,FILE=JOB(1:IEND(JOB))//'.TRJ',FORM='FORMATTED',
     &   STATUS='UNKNOWN',ERR=980)
         IF (LINT) THEN
            WRITE (NTRAJ,*) 3
         ELSEIF (LTRA) THEN
            WRITE (NTRAJ,*) 2
         ELSEIF (LPOI) THEN
            WRITE (NTRAJ,*) 1
         ENDIF
      ENDIF

      RETURN

C EVALUATION
C ----------

200   CONTINUE

      MODE2=MOD(MODE,10)
      GOTO (210,220,230,230,230) MODE2
      STOP 'SUSPECT MODE IN STATIS'

C START OF TRAJECTORY

210   CONTINUE

      DENSUM=ZERO
      DEESUM=ZERO
*     ---WRITE STARTING POINT TO THE TRAJECTORY FILE
      IF (LTRA.OR.LINT) THEN
         IF (LINT) THEN
            WRITE (NTRAJ,*) X,Y,Z,DENSUM,DEESUM,
     1                   ZERO,ZERO,ZERO,ZERO,ZERO,ZERO,.FALSE.
         ELSEIF (LTRA) THEN
            WRITE (NTRAJ,*) X,Y,Z,.FALSE.
         ENDIF
      ENDIF

      RETURN

C ALONG THE TRAJECTORY

220   CONTINUE

*     ---WRITE THE POINT TO THE TRAJECTORY FILE
      IF (LTRA.OR.LINT) THEN
         IF (LINT) THEN
            WRITE (NTRAJ,*) X,Y,Z,DENSUM,DEESUM,
     1                   XL,YL,ZL,
     2                   XR,YR,ZR,.FALSE.
         ELSEIF (LTRA) THEN
            WRITE (NTRAJ,*) X,Y,Z,.FALSE.
         ENDIF
      ENDIF

      RETURN

C END OF TRAJECTORY

230   CONTINUE

*     ---ADD ENERGY LOSS SUMS
      SUMDEE=SUMDEE+DEESUM
      SUMDEN=SUMDEN+DENSUM
*     ---COUNT ELECTRON
      IF (MODE2.EQ.3) THEN
         NIN=NIN+1
         GOTO 251
      ELSEIF (MODE2.EQ.4) THEN
234      IF (IETYP .EQ. 1) NBACK=NBACK+1
         LBACK=.TRUE.
         IF (NLOCOUT(2).GT.0) THEN
            DO 235 I=1,NLOCOUT(2)
               ILOC = LOCOUT(2,I)
               IF (Z.LT.ZMNSCD) GOTO 235
               IF (Z.LT.SCMNMX(3,ILOC)) GOTO 235
               Z1=MAX(ZMNSCD,SCMNMX(3,ILOC))
               FKX=DIRZ/DIRX
               FKY=DIRZ/DIRY
               X1=X+(Z1-Z)/FKX
               Y1=Y+(Z1-Z)/FKY
               CALL F_OUTPUTTO(NXLOC(ILOC),NS)
               CALL TRNXYZ(ILOC,X,Y,Z,XX,ZZ)
               CALL F_MOVE(XX,ZZ,NS)
               CALL TRNXYZ(ILOC,X1,Y1,Z1,XX,ZZ)
               CALL F_FLINETO(XX,ZZ,NS)
235         CONTINUE
         ENDIF
         IF (LBE.AND.IETYP.EQ.1) THEN
*        IF A BE HISTOGRAM SHOULD BE UPDATED AND ELECTRON IS
*        PRIMARY ELECTRON THEN DO:
            IF (LAANG) AANG=ANGLE(DIRX,DIRY)
            IF (LPANG) PANG=180/PI*DACOS(-DIRZ)
            DO 240 I=1,NBEHIS
               FRAC = 1.
               NDIM=1
*	          ELECTRON DETECTOR (DE? or AE?)
CS      	       IF (CVALUE(BEHIS(I))(1:1).EQ.'D' .OR. 
CS     .		   CVALUE(BEHIS(I))(1:1).EQ.'A') THEN
CS      		   FRAC = DFRAC
CS      	       ENDIF
               IF (CVALUE(BEHIS(I))(3:3).EQ.'E' .OR.
     .		   CVALUE(BEHIS(I))(3:3).EQ.'B' .OR.
     .		   CVALUE(BEHIS(I))(3:3).EQ.'S') THEN
*                 ENERGY DISTRIBUTION OF BACKSCATTERED ELECTRONS
                  VAL1=E
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'A') THEN
*                 AZIMUTHAL ANGLE DISTRIBUTION OF BACKSCATTERED ELECTRONS
                  VAL1=AANG
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'P') THEN
*                 POLAR ANGLE DISTRIBUTION OF BACKSCATTERED ELECTRONS
                  VAL1=PANG
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'Y') THEN
*                 BACKSCATTERED ELECTRON YIELD DISTRIBUTION
                  VAL1=XPREF
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'L') THEN
*                 SPACIAL RESOLUTION OF BACKSCATTERED ELECTRON IN X-DIRECTION
                  VAL1=X
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'H') THEN
*                 SPACIAL RESOLUTION OF BACKSCATTERED ELECTRON IN Y-DIRECTION
                  VAL1=Y
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'W') THEN
*                 ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION OF
*                 BACKSCATTERED ELECTRONS
                  VAL1=AANG
                  VAL2=PANG
                  NDIM=2
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'R') THEN
*                 SPACIAL RESOLUTION OF BACKSCATTERED ELECTRONS
                  VAL1=X
                  VAL2=Y
                  NDIM=2
               ELSE IF (CVALUE(BEHIS(I))(3:3).EQ.'D') THEN
*                 ENERGY AND ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION
*                 OF BACKSCATTERED ELECTRONS
                  VAL1=E
                  VAL2=AANG
                  VAL3=PANG
                  NDIM=3
               ENDIF
               FRAC=DFRAC(HISEDT(BEHIS(I)))
               CALL HISTOG(NDIM,BEHIS(I),VLUE*FRAC,VAL1,VAL2,VAL3)
 240        CONTINUE
         ELSE IF (LSE.AND.IETYP.EQ.4) THEN
*        IF A SE HISTOGRAM SHOULD BE UPDATED AND CURRENT ELECTRON IS A
*        SECONDARY ELECTRON THEN DO
*        (LBACK=TRUE AND LSEC=TRUE MEANS THAT SECONDARY ELECTRON LEFT
*        SURFACE OF SAMPLE)
            IF (LAANG) AANG=ANGLE(DIRX,DIRY)
            IF (LPANG) PANG=180/PI*DACOS(-DIRZ)
            DO 241 I=1,NSEHIS
               NDIM=1
 	       FRAC = 1.
*	          ELECTRON DETECTOR (DE? or AE?)
CS      	       IF (CVALUE(SEHIS(I))(1:1).EQ.'D' .OR. 
CS     .		   CVALUE(SEHIS(I))(1:1).EQ.'A') THEN
CS      		   FRAC = DFRAC
CS      	       ENDIF
               IF (CVALUE(SEHIS(I))(3:3).EQ.'E' .OR.
     .		   CVALUE(SEHIS(I))(3:3).EQ.'B' .OR.
     .		   CVALUE(SEHIS(I))(3:3).EQ.'S') THEN
*                 ENERGY DISTRIBUTION OF SECONDARY ELECTRONS
                  VAL1=E
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'S') THEN
*                 AUGER ENERGY DISTRIBUTION
*                 (ALSO SECONDARY ELECTRONS INCLUDED IN AUGER ELECTRON SPECTRUM
                  VAL1=E
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'A') THEN
*                 AZIMUTHAL ANGLE DISTRIBUTION OF SECONDARY ELECTRONS
                  VAL1=AANG
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'P') THEN
*                 POLAR ANGLE DISTRIBUTION OF SECONDARY ELECTRONS
                  VAL1=PANG
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'Y') THEN
*                 SECONDARY ELECTRON YIELD DISTRIBUTION
                  VAL1=XPREF
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'W') THEN
*                 ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION OF
*                 SECONDARY ELECTRONS
                  VAL1=AANG
                  VAL2=PANG
                  NDIM=2
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'D') THEN
*                 ENERGY AND ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION
*                 OF SECONDARY ELECTRONS
                  VAL1=E
                  VAL2=AANG
                  VAL3=PANG
                  NDIM=3
               ENDIF
               FRAC=DFRAC(HISEDT(SEHIS(I)))
               CALL HISTOG(NDIM,SEHIS(I),FRAC*VLUE,VAL1,VAL2,VAL3)
 241        CONTINUE
         ELSE IF (LAE.AND.IETYP.EQ.5) THEN
*        IF A AE HISTOGRAM SHOULD BE UPDATED AND CURRENT ELECTRON IS AN
*        AUGER ELECTRON THEN DO:
*        (LBACK=TRUE AND LAUGER=TRUE MEANS THAT AUGER ELECTRON LEFT
*        SURFACE OF SAMPLE)
            IF (LAANG) AANG=ANGLE(DIRX,DIRY)
            IF (LPANG) PANG=180/PI*DACOS(-DIRZ)
            DO 242 I=1,NAE_HIS(IESUBTYP)
      	       FRAC = 1.
               NDIM=1
*	          ELECTRON DETECTOR (DE? or AE?)
CS      	       IF (CVALUE(AE_HIS(IESUBTYP,I))(1:1).EQ.'D' .OR. 
CS     .		   CVALUE(AE_HIS(IESUBTYP,I))(1:1).EQ.'A') THEN
CS      		   FRAC = DFRAC
CS      	       ENDIF
               IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'E' .OR.
     .		   CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'B' .OR.
     .		   CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'S') THEN
*                 AUGER ELECTRON SPECTRUM/ENERGY (SECONDARIES INCLUDED)
                  VAL1=E
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'A') THEN
*                 AZIMUTHAL ANGLE DISTRIBUTION OF BACKSCATTERED ELECTRONS
                  VAL1=AANG
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'P') THEN
*                 POLAR ANGLE DISTRIBUTION OF BACKSCATTERED ELECTRONS
                  VAL1=PANG
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'Y') THEN
*                 AUGER ELECTRON YIELD DISTRIBUTION
                  VAL1=XPREF
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'W') THEN
*                 ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION OF
                  VAL1=AANG
                  VAL2=PANG
                  NDIM=2
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'D') THEN
*                 ENERGY AND ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION
*                 OF BACKSCATTERED ELECTRONS
                  VAL1=E
                  VAL2=AANG
                  VAL3=PANG
                  NDIM=3
               ENDIF
               FRAC=DFRAC(HISEDT(AE_HIS(IESUBTYP,I)))
               CALL HISTOG(NDIM,AE_HIS(IESUBTYP,I),FRAC*VLUE,
     .			   VAL1,VAL2,VAL3)
 242        CONTINUE
         ENDIF
         GOTO 251
      ELSEIF (MODE2.EQ.5) THEN
247      IF (IETYP.EQ.1) NTRAN=NTRAN+1
         LTRAN=.TRUE.
         IF (NLOCOUT(3).GT.0) THEN
            DO 252 I=1,NLOCOUT(3)
               ILOC = LOCOUT(3,I)
               IF (Z.GT.ZMXSCD) GOTO 252
               IF (Z.GT.SCMNMX(4,ILOC)) GOTO 252
               Z1=MIN(ZMXSCD,SCMNMX(4,ILOC))
               FKX=DIRZ/DIRX
               FKY=DIRZ/DIRY
               X1=X+(Z1-Z)/FKX
               Y1=Y+(Z1-Z)/FKY
               CALL F_OUTPUTTO(NXLOC(ILOC),NS)
               CALL TRNXYZ(ILOC,X,Y,Z,XX,ZZ)
               CALL F_MOVE(XX,ZZ,NS)
               CALL TRNXYZ(ILOC,X1,Y1,Z1,XX,ZZ)
               CALL F_FLINETO(XX,ZZ,NS)
252         CONTINUE
         ENDIF

         IF (LTE.AND.IETYP.EQ.1) THEN
*        IF A TE HISTOGRAM SHOULD BE UPDATED AND ELECTRON IS NEITHER
*        A SECONDARY NOR A AUGER ELECTRON THEN DO:
            IF (LAANG) AANG=ANGLE(DIRX,DIRY)
            IF (LPANG) PANG=180/PI*DACOS(-DIRZ)
            DO 248 I=1,NTEHIS
               NDIM=1
      	       FRAC = 1.
*	          ELECTRON DETECTOR (DE? or AE?)
CS      	       IF (CVALUE(TEHIS(I))(1:1).EQ.'D' .OR. 
CS     .		   CVALUE(TEHIS(I))(1:1).EQ.'A') THEN
CS      		  FRAC = DFRAC
CS      	       ENDIF
               IF (CVALUE(TEHIS(I))(1:3).EQ.'TEB') THEN
*                 TRANSMITTED ELECTRON BEAM BROADENING
                  XXREL=X-XESPOT
                  DISTANC=DSQRT(XXREL*XXREL+Y*Y)
                  IF (X.LT.XESPOT) THEN
                     VAL1=XESPOT-DISTANC
                  ELSE
                     VAL1=XESPOT+DISTANC
                  ENDIF
               ELSE IF (CVALUE(TEHIS(I))(3:3).EQ.'E' .OR.
     .		        CVALUE(TEHIS(I))(3:3).EQ.'B' .OR.
     .		        CVALUE(TEHIS(I))(3:3).EQ.'S') THEN
*                 ENERGY DISTRIBUTION OF TRANSMITTED ELECTRONS
                  VAL1=E
               ELSE IF (CVALUE(TEHIS(I))(3:3).EQ.'A') THEN
*                 AZIMUTHAL ANGLE DISTRIBUTION OF TRANSMITTED ELECTRONS
                  VAL1=AANG
               ELSE IF (CVALUE(TEHIS(I))(3:3).EQ.'P') THEN
*                 POLAR ANGLE DISTRIBUTION OF TRANSMITTED ELECTRONS
                  VAL1=PANG
               ELSE IF (CVALUE(TEHIS(I))(3:3).EQ.'Y') THEN
*                 TRANSMITTED ELECTRON YIELD DISTRIBUTION
                  VAL1=XPREF
               ELSE IF (CVALUE(TEHIS(I))(3:3).EQ.'W') THEN
*                 ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION OF
*                 TRANSMITTED ELECTRONS
                  VAL1=AANG
                  VAL2=PANG
                  NDIM=2
               ELSE IF (CVALUE(TEHIS(I))(3:3).EQ.'D') THEN
*                 ENERGY AND ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION
*                 OF TRANSMITTED ELECTRONS
                  VAL1=E
                  VAL2=AANG
                  VAL3=PANG
                  NDIM=3
               ENDIF
               FRAC=DFRAC(HISEDT(TEHIS(I)))
               CALL HISTOG(NDIM,TEHIS(I),VLUE*FRAC,VAL1,VAL2,VAL3)
 248        CONTINUE
         ELSE IF (LSE.AND.IETYP.EQ.4) THEN
*        IF A SE HISTOGRAM SHOULD BE UPDATED AND CURRENT ELECTRON IS A
*        SECONDARY ELECTRON THEN DO
*        (LTRAN=TRUE AND LSEC=TRUE MEANS THAT SECONDARY ELECTRON LEFT
*        BACK OF SAMPLE)
            IF (LAANG) AANG=ANGLE(DIRX,DIRY)
            IF (LPANG) PANG=180/PI*DACOS(-DIRZ)
            DO 249 I=1,NSEHIS
               NDIM=1
      	       FRAC = 1.
*	          ELECTRON DETECTOR (DE? or AE?)
CS      	       IF (CVALUE(SEHIS(I))(1:1).EQ.'D' .OR. 
CS     .		   CVALUE(SEHIS(I))(1:1).EQ.'A') THEN
CS      		  FRAC = DFRAC
CS      	       ENDIF
               IF (CVALUE(SEHIS(I))(3:3).EQ.'E' .OR.
     .		   CVALUE(SEHIS(I))(3:3).EQ.'B' .OR.
     .		   CVALUE(SEHIS(I))(3:3).EQ.'S') THEN
*                 ENERGY DISTRIBUTION OF SECONDARY ELECTRONS
                  VAL1=E
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'S') THEN
*                 AUGER ENERGY DISTRIBUTION
*                 (ALSO SECONDARY ELECTRONS INCLUDED IN AUGER ELECTRON SPECTRUM
                  VAL1=E
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'A') THEN
*                 AZIMUTHAL ANGLE DISTRIBUTION OF SECONDARY ELECTRONS
                  VAL1=AANG
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'P') THEN
*                 POLAR ANGLE DISTRIBUTION OF SECONDARY ELECTRONS
                  VAL1=PANG
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'W') THEN
*                 ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION OF
*                 SECONDARY ELECTRONS
                  VAL1=AANG
                  VAL2=PANG
                  NDIM=2
               ELSE IF (CVALUE(SEHIS(I))(3:3).EQ.'D') THEN
*                 ENERGY AND ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION
*                 OF SECONDARY ELECTRONS
                  VAL1=E
                  VAL2=AANG
                  VAL3=PANG
                  NDIM=3
               ENDIF
               FRAC=DFRAC(HISEDT(SEHIS(I)))
               CALL HISTOG(NDIM,SEHIS(I),FRAC*VLUE,VAL1,VAL2,VAL3)
 249        CONTINUE
         ELSE IF (LAE.AND.IETYP.EQ.5) THEN
*        IF A AE HISTOGRAM SHOULD BE UPDATED AND CURRENT ELECTRON IS AN
*        AUGER ELECTRON THEN DO:
*        (LTRAN=TRUE AND LAUGER=TRUE MEANS THAT AUGER ELECTRON LEFT
*        BACK OF SAMPLE)
            IF (LAANG) AANG=ANGLE(DIRX,DIRY)
            IF (LPANG) PANG=180/PI*DACOS(-DIRZ)
            DO 250 I=1,NAE_HIS(IESUBTYP)
      	       FRAC = 1.
               NDIM=1
*	          ELECTRON DETECTOR (DE? or AE?)
CS    	       IF (CVALUE(AE_HIS(IESUBTYP,I))(1:1).EQ.'D' .OR. 
CS   .		   CVALUE(AE_HIS(IESUBTYP,I))(1:1).EQ.'A') THEN
CS    		   FRAC = DFRAC
CS    	       ENDIF
               IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'E' .OR.
     .		   CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'B' .OR.
     .		   CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'S') THEN
*                 AUGER ELECTRON SPECTRUM/ENERGY (SECONDARIES INCLUDED)
                  VAL1=E
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'P') THEN
*                 POLAR ANGLE DISTRIBUTION OF BACKSCATTERED ELECTRONS
                  VAL1=PANG
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'W') THEN
*                 ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION OF
                  VAL1=AANG
                  VAL2=PANG
                  NDIM=2
               ELSE IF (CVALUE(AE_HIS(IESUBTYP,I))(3:3).EQ.'D') THEN
*                 ENERGY AND ANGLE (AZIMUTHAL AND POLAR) DISTRIBUTION
*                 OF BACKSCATTERED ELECTRONS
                  VAL1=E
                  VAL2=AANG
                  VAL3=PANG
                  NDIM=3
               ENDIF
               FRAC=DFRAC(HISEDT(AE_HIS(IESUBTYP,I)))
               CALL HISTOG(NDIM,AE_HIS(IESUBTYP,I),FRAC*VLUE,
     .		           VAL1,VAL2,VAL3)
 250        CONTINUE
         ENDIF
         GOTO 251
      ENDIF
            

251   CONTINUE

*     ---WRITE END POINT TO THE TRAJECTORY FILE
      IF (LINT) THEN
         WRITE (NTRAJ,*) XE,YE,ZE,DENSUM,DEESUM,
     1                ZERO,ZERO,ZERO,ZERO,ZERO,ZERO,.TRUE.
      ELSEIF (LTRA) THEN
         WRITE (NTRAJ,*) XE,YE,ZE,.TRUE.
      ELSEIF (LPOI) THEN
         WRITE (NTRAJ,*) XE,YE,ZE,DENSUM,DEESUM
      ENDIF

      RETURN

C OUTPUT
C ------

300   CONTINUE


*     ---REPORT ELECTRONS INSIDE, BACKSCATTERED, TRANSMITTED
*     ---WRITE TO GENERAL OUTPUT FILE (NFOUT)
CS      WRITE (NFOUT,'('' '')')
CS      WRITE (NFOUT,'(T2,''NUMBER OF ELECTRONS:'')')
CS      IF (NIN.GT.0)
CS     +     WRITE (NFOUT,'(T5,''INSIDE TARGET:'',I7)') NIN
CS      IF (NBACK.GT.0) THEN
CS          WRITE (NFOUT,'(T5,''BACKSCATTERED:'',I7)') NBACK
CS          WRITE (NFOUT,'(T5,''BACKSCATTERING YIELD'',F8.2)')
CS     +          DBLE(NBACK)/DBLE(NETRN)
CS      ENDIF 
CS      IF (NTRAN.GT.0)
CS     +     WRITE (NFOUT,'(T5,''TRANSMITTED  :'',I7)') NTRAN

*     ---CLOSE TRAJECTORY FILE
      IF (LPOI.OR.LTRA.OR.LINT) CLOSE (NTRAJ)

      RETURN

980   STOP 'ERROR DURING OPENING TRAJECTORY FILE'

      END
