c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
      SUBROUTINE TSTGEO (
     D                   NPTS, NFCS,
     V                   PCOORD, ILINES, IENDIX,
     I                   DISTNC,
     O                   IER)

      INCLUDE 'TYPE.INC'

***   TSTGEO - TESTS THE GEOMETRY TOWARDS ITS CORRECT DEFINITION.
*
**    ARGUMENTS
*
*        NPTS   - NUMBER OF POINTS
*        NFCS   - NUMBER OF FACES
*        PCOORD - COORDINATES OF THE POINTS
*        ILINES - INDICES OF THE FACES
*        IENDIX - ENDINDICES OF THE LINES
*        DISTNC - MINIMUM DISTANCE BETWEEN TWO POINTS
*        IER    - ERROR INDICATOR: 0 ... EVERYTHING OK
*
**    AUTHOR:  H.STIPPEL  15-MAR-1994
*
      LOGICAL   LDIFF, LFIRST, LLAST
      DIMENSION PCOORD(2*NPTS), ILINES(3*NPTS-6), IENDIX(NFCS)

      PARAMETER (PI=3.141592654,ZERO=0,ONE=1,TWO=2)

**    START OF EXECUTABLE CODE
*     ************************

      IER = 0

*     --- TEST THE POINTS: MINIMUM DISTANCE
      DISTSQ = DISTNC * DISTNC
      DO 20 IPT = 1, NPTS
         DO 10 JPT = IPT+1, NPTS
            DISTX = PCOORD(2*IPT-1) - PCOORD(2*JPT-1)
            DISTZ = PCOORD(2*IPT)   - PCOORD(2*JPT)
            DIST  = DISTX * DISTX + DISTZ * DISTZ
            IF (DIST.LT.DISTSQ) THEN
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               WRITE (*,*) '***** ERROR: POINT', IPT, ' AND', JPT,
     +                     ' TOO CLOSE! *****'
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               IER = 1
               RETURN
            ENDIF
 10      CONTINUE
 20   CONTINUE

*     --- TEST THE FACES

*     FACES ARE CONSTRUCTED ONLY BY INDICES FOR THE POINTS
*     ==> THEY MUST BE GREATER THEN ZERO AND LESS THEN OR
*     EQUAL TO NPTS
      DO 30 ILN = 1, IENDIX(NFCS)
         IF (ILINES(ILN).LT.1 .OR. ILINES(ILN).GT.NPTS) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            IF (ILINES(ILN).LT.1) THEN
               WRITE (*,*) '***** ERROR: REFERENCE .LT. 1 FOUND! *****'
            ELSE
               WRITE (*,*) '***** ERROR: REFERENCE .GT.', NPTS,
     +                     ' FOUND! *****'
            ENDIF
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            IER = 2
            RETURN
         ENDIF
 30   CONTINUE

*     EVERY POINT MUST ONLY BE REFERENCED ONCE IN A FACE
*     ONLY EXEPTION: LAST POINT; IF FIRST AND LAST POINT ARE
*                    EQUAL, THE LAST POINT IS ELIMINATED
      IBEGIN = 1
      IPACK = 0
      DO 60 IFC = 1, NFCS
         IF (ILINES(IENDIX(IFC)).EQ.ILINES(IBEGIN)) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** WARNING: FIRST AND LAST POINT OF FACE',
     +                  IFC, ' ARE EQUAL! --> ELIMINATING LAST POINT',
     +                  '! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            IPACK = IPACK + 1
            ILINES(IENDIX(IFC)) = 0
         ENDIF
         DO 50 ILN = IBEGIN, IENDIX(IFC)
            DO 40 ITST = ILN+1, IENDIX(IFC)
               IF (ILINES(ILN).EQ.ILINES(ITST)) THEN
                  WRITE (*,*) ' '
                  WRITE (*,*) ' '
                  WRITE (*,*) '***** ERROR: POINT', ILINES(ILN),
     +                        ' REFERENCED MORE THAN ONCE IN FACE',
     +                        IFC, '! *****'
                  WRITE (*,*) ' '
                  WRITE (*,*) ' '
                  IER = 3
                  RETURN
               ENDIF
 40         CONTINUE
 50      CONTINUE
         IBEGIN = IENDIX(IFC) + 1
 60   CONTINUE

*     IF POINTS HAVE TO BE ELIMINATED: PACK ARRAYS
      IF (IPACK.GT.0) THEN
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         WRITE (*,*) '***** INFO: PACK ARRAYS! *****'
         DO 90 IFC = 1, NFCS
            IF (ILINES(IENDIX(IFC)).EQ.0) THEN
*     SHIFT ALL FOLLOWING INDICES
               DO 70 ILN = IENDIX(IFC), IENDIX(NFCS)-1
                  ILINES(ILN) = ILINES(ILN+1)
 70            CONTINUE
*     ALTER THE END INDEX ARRAY
               DO 80 IEND = IFC, NFCS
                  IENDIX(IEND) = IENDIX(IEND) - 1
 80            CONTINUE
            ENDIF
 90      CONTINUE
         WRITE (*,*) '            PACKED', IPACK, ' POINTS.'
         WRITE (*,*) ' '
         WRITE (*,*) ' '
      ENDIF

*     TEST IF THE FACES HAVE AT LEAST THREE LINES
      IBEGIN = 0
      DO 100 IFC = 1, NFCS
         IF (IENDIX(IFC)-IBEGIN.LT.3) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: FACE', IFC, ' CONTAINS LESS',
     +           ' THAN TWO LINES! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            IER = 4
            RETURN
         ENDIF
         IBEGIN = IENDIX(IFC)
 100  CONTINUE

*     CHECK IF LINE IS REFERENCED MORE THAN TWICE
      IBEGIN = 1
      DO 140 IFC = 1, NFCS
         IFROM = ILINES(IENDIX(IFC))
         DO 130 ILN = IBEGIN, IENDIX(IFC)
            ITO = ILINES(ILN)
            NREFS = 1
            IFIRST = ILN + 1
            DO 120 IFACE = IFC, NFCS
               IF (IFACE.EQ.IFC) THEN
                  ITFROM = ILINES(ILN)
               ELSE
                  ITFROM = ILINES(IENDIX(IFACE))
               ENDIF
               DO 110 ITST = IFIRST, IENDIX(IFACE)
                  ITTO = ILINES(ITST)
                  IF ((IFROM.EQ.ITFROM .AND. ITO.EQ.ITTO) .OR.
     .                (IFROM.EQ.ITTO   .AND. ITO.EQ.ITFROM))
     +                 NREFS = NREFS + 1
                  ITFROM = ITTO
 110           CONTINUE
               IFIRST = IENDIX(IFACE) + 1
 120        CONTINUE
            IF (NREFS.GT.2) THEN
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               WRITE (*,*) '***** ERROR: LINE', IFROM, ' ,', ITO,
     +                     ' REFERENCED MORE THAN TWICE! *****'
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               IER = 5
               RETURN
            ENDIF
            IFROM = ITO
 130     CONTINUE
         IBEGIN = IENDIX(IFC) + 1
 140  CONTINUE

*     CHECK ON INTERSECTING LINES
      IBEGIN = 1
      DO 180 IFC = 1, NFCS
         IFROM = ILINES(IENDIX(IFC))
         DO 170 ILN = IBEGIN, IENDIX(IFC)
            ITO = ILINES(ILN)
            IFIRST = ILN
            DO 160 IFACE = IFC, NFCS
               ITFROM = ILINES(IENDIX(IFACE))
               IF (IFACE.EQ.IFC) THEN
                  ITTO   = ILINES(IBEGIN)
               ELSE
                  ITTO   = ILINES(IFIRST)
               ENDIF
               DO 150 ITST = IFIRST, IENDIX(IFACE)
                  IF (IFROM.NE.ITFROM .AND. ITO.NE.ITTO .AND.
     .                IFROM.NE.ITTO   .AND. ITO.NE.ITFROM) THEN
*     TEST ON AN INTERSECTION
                     V1X = PCOORD(2*ITO-1)  - PCOORD(2*IFROM-1)
                     V1Z = PCOORD(2*ITO)    - PCOORD(2*IFROM)
                     V2X = PCOORD(2*ITTO-1) - PCOORD(2*ITFROM-1)
                     V2Z = PCOORD(2*ITTO)   - PCOORD(2*ITFROM)
                     DET = V1Z * V2X - V1X * V2Z
                     IF (DET.NE.ZERO) THEN
*     LINES ARE NOT PARALLEL
                        DX = PCOORD(2*ITFROM-1) - PCOORD(2*IFROM-1)
                        DZ = PCOORD(2*ITFROM)   - PCOORD(2*IFROM)
                        DLBD = (V1X * DZ - V1Z * DX) / DET
                        IF (ZERO.LE.DLBD .AND. DLBD.LE.ONE) THEN
*     INTERSECTION IS POSSIBLE
                           DMUE = (V2X * DZ - V2Z * DX) / DET
                           IF (ZERO.LE.DMUE .AND. DMUE.LE.ONE) THEN
*     INTERSECTION FOUND
                              WRITE (*,*) ' '
                              WRITE (*,*) ' '
                              WRITE (*,*) '***** ERROR: INTERSECTING ',
     +                             'LINES FOUND: FACES=', IFC, ' AND',
     +                             IFACE, '! *****'
                              WRITE (*,*) '                          ',
     +                             '             LINES=', IFROM, ' TO',
     +                             ITO, ' AND', ITFROM, ' TO', ITTO
                              WRITE (*,*) ' '
                              WRITE (*,*) ' '
                              IER = 6
                              RETURN
                           ENDIF
                        ENDIF
                     ENDIF
                  ENDIF
                  ITFROM = ILINES(ITST)
                  IF (ITST.LT.IENDIX(IFACE)) ITTO = ILINES(ITST+1)
 150           CONTINUE
               IFIRST = IENDIX(IFACE) + 1
 160        CONTINUE
            IFROM = ITO
 170     CONTINUE
         IBEGIN = IENDIX(IFC) + 1
 180  CONTINUE

*     ---TEST IF A FACE IS INSIDE ANOTHER ONE:
*        FIRST TEST, WHETER A POINT OF A FACE LIES INSIDE ANOTHER ONE
      IBEGIN = 1
      DO 230 IFC = 1, NFCS
         LDIFF = .FALSE.
         DO 220 IPT = IBEGIN, IENDIX(IFC)
            IFIRST = 1
            DO 210 IFACE = 1, NFCS
               IF (IFACE.NE.IFC) THEN
                  DO 190 ILN = IFIRST, IENDIX(IFACE)
                     IF (ILINES(ILN).EQ.ILINES(IPT)) GOTO 220
 190              CONTINUE
                  LDIFF = .TRUE.
*     TEST IF THE POINT IS INSIDE THE FACE
                  LFIRST = .FALSE.
                  LLAST = .FALSE.
                  NINTER = 0
                  IPTST = ILINES(IPT)
                  IFROM = ILINES(IENDIX(IFACE))
                  DO 200 ITST = IFIRST, IENDIX(IFACE)
*     USE THE INTERSECTION METHOD FOR THE POINT LOCATION
                     ITO = ILINES(ITST)
                     VX = PCOORD(2*ITO-1) - PCOORD(2*IFROM-1)
                     VZ = PCOORD(2*ITO)   - PCOORD(2*IFROM)
                     IF (VX.NE.ZERO) THEN
*     LINES ARE NOT PARALLEL
                        DLBD = (PCOORD(2*IPTST-1) - PCOORD(2*IFROM-1))
     /                       / VX
                        IF (ZERO.LE.DLBD .AND. DLBD.LE.ONE) THEN
                           DMUE = PCOORD(2*IFROM) - PCOORD(2*IPTST)
     +                          + DLBD * VZ
                           IF (DMUE.GE.ZERO) THEN
                              NINTER = NINTER + 1
                              IF (ITST.EQ.IFIRST) THEN
                                 IF (DLBD.EQ.ZERO) THEN
                                    LFIRST = .TRUE.
                                    PXF = PCOORD(2*ITO-1)
                                 ELSE
                                    IF (DLBD.EQ.ONE) THEN
                                       LLAST = .TRUE.
                                       PXO = PCOORD(2*IFROM-1)
                                    ENDIF
                                 ENDIF
                              ELSEIF (ITST.EQ.IENDIX(IFACE)) THEN
                                 IF (LFIRST) THEN
                                    IF (DLBD.EQ.ONE) THEN
*     TEST, WHETHER THE INTERSECTION IS A "REAL" INTERSECTION OR NOT
                                       IF ((PXF.LT.
     .                                      PCOORD(2*IPTST-1)     .AND.
     .                                      PCOORD(2*IFROM-1).GE.
     .                                      PCOORD(2*IPTST-1))     .OR.
     .                                     (PXF.GE.
     .                                      PCOORD(2*IPTST-1)     .AND.
     .                                      PCOORD(2*IFROM-1).LT.
     .                                      PCOORD(2*IPTST-1)))
     +                                      NINTER = NINTER - 1
                                    ENDIF
                                 ELSEIF (LLAST) THEN
                                    LLAST = .FALSE.
                                    IF (DLBD.EQ.ZERO) THEN
*     TEST, WHETHER THE INTERSECTION IS A "REAL" INTERSECTION OR NOT
                                       IF ((PXO.LT.
     .                                      PCOORD(2*IPTST-1)   .AND.
     .                                      PCOORD(2*ITO-1).GE.
     .                                      PCOORD(2*IPTST-1))   .OR.
     .                                     (PXO.GE.
     .                                      PCOORD(2*IPTST-1)   .AND.
     .                                      PCOORD(2*ITO-1).LT.
     .                                      PCOORD(2*IPTST-1)))
     +                                      NINTER = NINTER - 1
                                    ENDIF
                                 ENDIF
                              ELSE
                                 IF (DLBD.EQ.ONE) THEN
                                    LLAST = .TRUE.
                                    PXO = PCOORD(2*IFROM-1)
                                 ELSEIF (LLAST) THEN
                                    LLAST = .FALSE.
                                    IF (DLBD.EQ.ZERO) THEN
*     TEST, WHETHER THE INTERSECTION IS A "REAL" INTERSECTION OR NOT
                                       IF ((PXO.LT.
     .                                      PCOORD(2*IPTST-1)   .AND.
     .                                      PCOORD(2*ITO-1).GE.
     .                                      PCOORD(2*IPTST-1))   .OR.
     .                                     (PXO.GE.
     .                                      PCOORD(2*IPTST-1)   .AND.
     .                                      PCOORD(2*ITO-1).LT.
     .                                      PCOORD(2*IPTST-1)))
     +                                      NINTER = NINTER - 1
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ELSE
                              LLAST = .FALSE.
                           ENDIF
                        ELSE
                           LLAST = .FALSE.
                        ENDIF
                     ENDIF
                     IFROM = ITO
 200              CONTINUE
                  IF (MOD(NINTER,2).EQ.1) THEN
                     WRITE (*,*) ' '
                     WRITE (*,*) ' '
                     WRITE (*,*) '***** ERROR: POINT', IPTST,
     +                    ' OF FACE', IFC, ' INSIDE FACE', IFACE,
     +                    '! *****'
                     WRITE (*,*) ' '
                     WRITE (*,*) ' '
                     IER = 7
                     RETURN
                  ENDIF
               ENDIF
               IFIRST = IENDIX(IFACE) + 1
 210        CONTINUE
 220     CONTINUE
         IF (.NOT.LDIFF .AND. NFCS.GT.1) THEN
            IF (IFC.EQ.1) THEN
               NPTS1 = IENDIX(1)
            ELSE
               NPTS1 = IENDIX(IFC) - IENDIX(IFC-1)
            ENDIF
            IF (IFACE.EQ.1) THEN
               NPTS2 = IENDIX(1)
            ELSE
               NPTS2 = IENDIX(IFACE) - IENDIX(IFACE-1)
            ENDIF
            IF (NPTS1.EQ.NPTS2) THEN
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               WRITE (*,*) '***** ERROR: FACE', IFC, ' AND', IFACE,
     ,              ' ARE IDENTICAL! *****'
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               IER = 8
               RETURN
            ELSEIF (NPTS1.GT.NPTS2) THEN
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               WRITE (*,*) '***** ERROR: FACE', IFACE, ' INSIDE', IFC,
     ,              '! *****'
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               IER = 9
               RETURN
            ELSE
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               WRITE (*,*) '***** ERROR: FACE', IFC, ' INSIDE', IFACE,
     ,              '! *****'
               WRITE (*,*) ' '
               WRITE (*,*) ' '
               IER = 10
               RETURN
            ENDIF
         ENDIF
         IBEGIN = IENDIX(IFC) + 1
 230  CONTINUE

*        SECOND, TEST IF A LINE IS INSIDE ANOTHER FACE
      IBEGIN = 1
      DO 280 IFC = 1, NFCS
         ITFROM = ILINES(IENDIX(IFC))
         DO 270 IPT = IBEGIN, IENDIX(IFC)
            ITTO = ILINES(IPT)
            IFIRST = 1
            DO 260 IFACE = 1, NFCS
               IF (IFACE.NE.IFC) THEN
                  IFROM = ILINES(IENDIX(IFACE))
                  DO 240 ILN = IFIRST, IENDIX(IFACE)
                     ITO = ILINES(ILN)
                     IF ((IFROM.EQ.ITFROM .AND. ITO.EQ.ITTO) .OR.
     .                   (IFROM.EQ.ITTO   .AND. ITO.EQ.ITFROM)) THEN
                        ITFROM = ITTO
                        GOTO 270
                     ENDIF
                     IFROM = ITO
 240              CONTINUE
*     NOW TEST THIS LINE --> TAKE THE MIDPOINT OF THE LINE
                  PX = (PCOORD(2*ITFROM-1) + PCOORD(2*ITTO-1)) / TWO
                  PZ = (PCOORD(2*ITFROM)   + PCOORD(2*ITTO))   / TWO
*     USE THE INTERSECTION METHOD FOR THE POINT LOCATION
                  LFIRST = .FALSE.
                  LLAST = .FALSE.
                  NINTER = 0
                  IFROM = ILINES(IENDIX(IFACE))
                  DO 250 ITST = IFIRST, IENDIX(IFACE)
                     ITO = ILINES(ITST)
                     VX = PCOORD(2*ITO-1) - PCOORD(2*IFROM-1)
                     VZ = PCOORD(2*ITO)   - PCOORD(2*IFROM)
                     IF (VX.NE.ZERO) THEN
*     LINES ARE NOT PARALLEL
                        DLBD = (PX - PCOORD(2*IFROM-1)) / VX
                        IF (ZERO.LE.DLBD .AND. DLBD.LE.ONE) THEN
                           DMUE = PCOORD(2*IFROM) - PZ + DLBD * VZ
                           IF (DMUE.GE.ZERO) THEN
                              NINTER = NINTER + 1
                              IF (ITST.EQ.IFIRST) THEN
                                 IF (DLBD.EQ.ZERO) THEN
                                    LFIRST = .TRUE.
                                    PXF = PCOORD(2*ITO-1)
                                 ELSE
                                    IF (DLBD.EQ.ONE) THEN
                                       LLAST = .TRUE.
                                       PXO = PCOORD(2*IFROM-1)
                                    ENDIF
                                 ENDIF
                              ELSEIF (ITST.EQ.IENDIX(IFACE)) THEN
                                 IF (LFIRST) THEN
                                    IF (DLBD.EQ.ONE) THEN
*     TEST, WHETHER THE INTERSECTION IS A "REAL" INTERSECTION OR NOT
                                       IF ((PXF.LT.PX .AND. PX.LT.
     .                                      PCOORD(2*IFROM-1))     .OR.
     .                                     (PXF.GE.PX .AND. PX.GE.
     .                                      PCOORD(2*IFROM-1)))
     +                                      NINTER = NINTER - 1
                                    ENDIF
                                 ELSEIF (LLAST) THEN
                                    LLAST = .FALSE.
                                    IF (DLBD.EQ.ZERO) THEN
*     TEST, WHETHER THE INTERSECTION IS A "REAL" INTERSECTION OR NOT
                                       IF ((PXO.LT.PX .AND. PX.LT.
     .                                      PCOORD(2*ITO-1))       .OR.
     .                                     (PXO.GE.PX .AND. PX.GE.
     .                                      PCOORD(2*ITO-1)))
     +                                      NINTER = NINTER - 1
                                    ENDIF
                                 ENDIF
                              ELSE
                                 IF (DLBD.EQ.ONE) THEN
                                    LLAST = .TRUE.
                                    PXO = PCOORD(2*IFROM-1)
                                 ELSEIF (LLAST) THEN
                                    LLAST = .FALSE.
                                    IF (DLBD.EQ.ZERO) THEN
*     TEST, WHETHER THE INTERSECTION IS A "REAL" INTERSECTION OR NOT
                                       IF ((PXO.LT.PX .AND. PX.LT.
     .                                      PCOORD(2*ITO-1))       .OR.
     .                                     (PXO.GE.PX .AND. PX.GE.
     .                                      PCOORD(2*ITO-1)))
     +                                      NINTER = NINTER - 1
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ELSE
                              LLAST = .FALSE.
                           ENDIF
                        ELSE
                           LLAST = .FALSE.
                        ENDIF
                     ENDIF
                     IFROM = ITO
 250              CONTINUE
                  IF (MOD(NINTER,2).EQ.1) THEN
                     WRITE (*,*) ' '
                     WRITE (*,*) ' '
                     WRITE (*,*) '***** ERROR: LINE', ITFROM, ' ,',
     +                    ITTO, ' OF FACE', IFC, ' INSIDE FACE', IFACE,
     +                    '! *****'
                     WRITE (*,*) ' '
                     WRITE (*,*) ' '
                     IER = 11
                     RETURN
                  ENDIF
               ENDIF
               IFIRST = IENDIX(IFACE) + 1
 260        CONTINUE
            ITFROM = ITTO
 270     CONTINUE
         IBEGIN = IENDIX(IFC) + 1
 280  CONTINUE

*     TEST THE ORIENTATION
      IBEGIN = 1
      DO 310 IFC = 1, NFCS
         AREA = ZERO
         PX = PCOORD(2*ILINES(IBEGIN)-1)
         PZ = PCOORD(2*ILINES(IBEGIN))
         IFIRST = ILINES(IBEGIN+1)
         DO 290 IPT = IBEGIN+2, IENDIX(IFC)
            ILAST = ILINES(IPT)
            V1X = PCOORD(2*IFIRST-1) - PX
            V1Z = PCOORD(2*IFIRST)   - PZ
            V2X = PCOORD(2*ILAST-1)  - PX
            V2Z = PCOORD(2*ILAST)    - PZ
            AREA = AREA + V1X * V2Z - V1Z * V2X
            IFIRST = ILAST
 290     CONTINUE
         IF (AREA.LT.ZERO) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** INFO: TURNING FACE', IFC, '! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
*     TURN THE FACE SO THAT IT IS ORIENTED COUNTER CLOCKWISE
            IDEST = IENDIX(IFC)
            DO 300 IPT = IBEGIN, (IBEGIN+IENDIX(IFC))/2
               ISWAP = ILINES(IPT)
               ILINES(IPT) = ILINES(IDEST)
               ILINES(IDEST) = ISWAP
               IDEST = IDEST - 1
 300        CONTINUE
         ELSEIF (AREA.EQ.ZERO) THEN
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            WRITE (*,*) '***** ERROR: FACE', IFC,
     ,           ' HAS ZERO AREA! *****'
            WRITE (*,*) ' '
            WRITE (*,*) ' '
            IER = 9
            RETURN
         ENDIF
         IBEGIN = IENDIX(IFC) + 1
 310  CONTINUE

      RETURN
      END
