c
c     Copyright (c) 2000, Compaq Computer Corporation
c
c     See the file "license.terms" for information on usage and redistribution
c     of this file, and for a DISCLAIMER OF ALL WARRANTIES.
c
      SUBROUTINE XSTART(
     D                  NPTS,
     I                  IETRN,NETRN,DIRX,DIRY,DIRZ,XC,ZS,PCOORD,
     V                  NMISS,
     O                  XE,YE,ZE,IR,INTERS)

***   XSTART - DETERMINE STARTING POINT OF ELECTRON TRAJECTORY
*
**    ARGUMENTS
*
*     NPTS   - NUMBER OF POINTS OF THE GEOMETRY
*     IETRN  - INDEX OF THE ELECTRON
*     NETRN  - TOTAL NUMBER OF ELECTRONS IN THE SIMULATION
*     DIRX   - X-COMPONENT OF THE DIRECTION OF MOTION
*     DIRY   - Y-COMPONENT OF THE DIRECTION OF MOTION
*     DIRZ   - Z-COMPONENT OF THE DIRECTION OF MOTION
*     XC     - X-COORDINATE OF THE STARTING POINT
*     ZS     - Z-COORDIANTE OF THE SURFACE
*     PCOORD - COORDINATES OF THE POINTS
*     NMISS  - NUMBER OF ELECTRONS WHICH MISSED THE SIMULATION AREA
*     XE     - X-COORDINATE OF THE STARTING POINT OF THE TRAJECTORY
*     YE     - Y-COORDINATE OF THE STARTING POINT OF THE TRAJECTORY
*     ZE     - Z-COORDINATE OF THE STARTING POINT OF THE TRAJECTORY
*              (EXTERNAL COORDINATES)
*     IR     - REGION WHICH IS ENTERED
*     INTERS - 1, IF THE ELECTRON ENTERS THE TARGET
*
**    AUTHOR:  G.HOBLER  12-AUG-1990
*              H.STIPPEL 21-FEB-1994
*
      INCLUDE 'TYPE.INC'
      INCLUDE 'PARAM.INC'

      DIMENSION PCOORD(2*NPTS)

      LOGICAL INTEXT,ONLYZ


**    START OF EXECUTABLE CODE
*     ************************


      IF (IETRN.LT.1 .OR. IETRN.GT.NETRN)
     1   STOP 'STRANGE VALUE OF IETRN IN SUBROUTINE XSTART'

C SELECT A POINT INSIDE THE UNIT CUBE

      N3=NETRN
      I3=IETRN

 10   CONTINUE

      N1=N3**THIRD+0.000001

      IF (I3.GT.N1**3) THEN
         N3=N3-N1**3
         I3=I3-N1**3
         GOTO 10
      ENDIF

      IY=MOD((I3-1)/N1,N1)
      X=XC
      Y=4*(IY+RANNUM())/N1
      Z=ZS

C ROTATE IT TO THE EXTERNAL COORDINATE SYSTEM
*************************************
CHW SOMETHING IS GOING ON HERE.....
*************************************
C      INTEXT=.TRUE.
C      ONLYZ=.FALSE.
C      CALL ROTATE(
C     I            X,Y,Z,
C     O            XE,YE,ZE,
C     I            INTEXT,ONLYZ)

C CALCULATE THE PROJECTION TO THE SURFACE

      CALL IMENTR (
     I             X, Z, DIRX, DIRZ, PCOORD,
     O             INTERS, XE, ZE, IR, IER)
      IF (IER.NE.0) THEN
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         WRITE (*,*) '***** ERROR: OCCURED IN IMENTR! *****'
         WRITE (*,*) ' '
         WRITE (*,*) ' '
         STOP
      ENDIF
      IF (INTERS.EQ.0 .OR. IR.EQ.-1 .OR. IR.EQ.0) THEN
         NMISS = NMISS + 1
         INTERS = 0
      ELSE
         IF (DIRX.NE.ZERO) THEN
            YE = Y + (XE - X) / DIRX * DIRY
         ELSEIF (DIRZ.NE.ZERO) THEN
            YE = Y + (ZE - Z) / DIRZ * DIRY
         ENDIF
      ENDIF

      END
